@@ -1,31 +1,35 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "base/files/file_path.h"
 
 #include <string.h>
+
 #include <algorithm>
 
 #include "base/check_op.h"
-#include "base/macros.h"
+#include "base/files/safe_base_name.h"
+#include "base/numerics/safe_conversions.h"
 #include "base/pickle.h"
+#include "base/ranges/algorithm.h"
 #include "base/strings/strcat.h"
 #include "base/strings/string_piece.h"
 #include "base/strings/string_util.h"
 #include "base/strings/sys_string_conversions.h"
+#include "base/strings/utf_ostream_operators.h"
 #include "base/strings/utf_string_conversions.h"
-#include "build/build_config.h"
+#include "base/trace_event/base_tracing.h"
 
-#if defined(OS_APPLE)
-#include "base/mac/scoped_cftyperef.h"
+#if BUILDFLAG(IS_APPLE)
+#include "base/apple/scoped_cftyperef.h"
 #include "base/third_party/icu/icu_utf.h"
 #endif
 
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
 #include <windows.h>
 #include "base/win/win_util.h"
-#elif defined(OS_APPLE)
+#elif BUILDFLAG(IS_APPLE)
 #include <CoreFoundation/CoreFoundation.h>
 #endif
 
@@ -36,9 +40,9 @@ using StringPieceType = FilePath::StringPieceType;
 
 namespace {
 
-const char* const kCommonDoubleExtensionSuffixes[] = {"gz", "xz", "bz2", "z",
-                                                      "bz"};
-const char* const kCommonDoubleExtensions[] = { "user.js" };
+const char* const kCommonDoubleExtensionSuffixes[] = {
+    "bz", "bz2", "gz", "lz", "lzma", "lzo", "xz", "z", "zst"};
+const char* const kCommonDoubleExtensions[] = {"user.js"};
 
 const FilePath::CharType kStringTerminator = FILE_PATH_LITERAL('\0');
 
@@ -141,13 +145,13 @@ StringType::size_type ExtensionSeparatorPosition(const StringType& path) {
 
   for (auto* i : kCommonDoubleExtensions) {
     StringType extension(path, penultimate_dot + 1);
-    if (LowerCaseEqualsASCII(extension, i))
+    if (EqualsCaseInsensitiveASCII(extension, i))
       return penultimate_dot;
   }
 
   StringType extension(path, last_dot + 1);
   for (auto* i : kCommonDoubleExtensionSuffixes) {
-    if (LowerCaseEqualsASCII(extension, i)) {
+    if (EqualsCaseInsensitiveASCII(extension, i)) {
       if ((last_dot - penultimate_dot) <= 5U &&
           (last_dot - penultimate_dot) > 1U) {
         return penultimate_dot;
@@ -219,15 +223,11 @@ bool FilePath::IsSeparator(CharType character) {
   return false;
 }
 
-void FilePath::GetComponents(std::vector<StringType>* components) const {
-  DCHECK(components);
-  if (!components)
-    return;
-  components->clear();
-  if (value().empty())
-    return;
-
+std::vector<FilePath::StringType> FilePath::GetComponents() const {
   std::vector<StringType> ret_val;
+  if (value().empty())
+    return ret_val;
+
   FilePath current = *this;
   FilePath base;
 
@@ -247,11 +247,11 @@ void FilePath::GetComponents(std::vector<StringType>* components) const {
   // Capture drive letter, if any.
   FilePath dir = current.DirName();
   StringType::size_type letter = FindDriveLetter(dir.value());
-  if (letter != StringType::npos) {
-    ret_val.push_back(StringType(dir.value(), 0, letter + 1));
-  }
+  if (letter != StringType::npos)
+    ret_val.emplace_back(dir.value(), 0, letter + 1);
 
-  *components = std::vector<StringType>(ret_val.rbegin(), ret_val.rend());
+  ranges::reverse(ret_val);
+  return ret_val;
 }
 
 bool FilePath::IsParent(const FilePath& child) const {
@@ -260,10 +260,8 @@ bool FilePath::IsParent(const FilePath& child) const {
 
 bool FilePath::AppendRelativePath(const FilePath& child,
                                   FilePath* path) const {
-  std::vector<StringType> parent_components;
-  std::vector<StringType> child_components;
-  GetComponents(&parent_components);
-  child.GetComponents(&child_components);
+  std::vector<StringType> parent_components = GetComponents();
+  std::vector<StringType> child_components = child.GetComponents();
 
   if (parent_components.empty() ||
       parent_components.size() >= child_components.size())
@@ -287,6 +285,16 @@ bool FilePath::AppendRelativePath(const FilePath& child,
   }
 #endif  // defined(FILE_PATH_USES_DRIVE_LETTERS)
 
+  // The first 2 components for network paths are [<2-Separators>, <hostname>].
+  // Use case-insensitive comparison for the hostname.
+  // https://tools.ietf.org/html/rfc3986#section-3.2.2
+  if (IsNetwork() && parent_components.size() > 1) {
+    if (*parent_comp++ != *child_comp++ ||
+        !base::EqualsCaseInsensitiveASCII(*parent_comp++, *child_comp++)) {
+      return false;
+    }
+  }
+
   while (parent_comp != parent_components.end()) {
     if (*parent_comp != *child_comp)
       return false;
@@ -331,8 +339,22 @@ FilePath FilePath::DirName() const {
     // separator intact indicating alternate root.
     new_path.path_.resize(letter + 3);
   } else if (last_separator != 0) {
-    // path_ is somewhere else, trim the basename.
-    new_path.path_.resize(last_separator);
+    bool trim_to_basename = true;
+#if BUILDFLAG(IS_POSIX)
+    // On Posix, more than two leading separators are always collapsed to one.
+    // See
+    // https://pubs.opengroup.org/onlinepubs/9699919799/basedefs/V1_chap04.html#tag_04_13
+    // So, do not strip any of the separators, let
+    // StripTrailingSeparatorsInternal() take care of the extra.
+    if (AreAllSeparators(new_path.path_.substr(0, last_separator + 1))) {
+      new_path.path_.resize(last_separator + 1);
+      trim_to_basename = false;
+    }
+#endif  // BUILDFLAG(IS_POSIX)
+    if (trim_to_basename) {
+      // path_ is somewhere else, trim the basename.
+      new_path.path_.resize(last_separator);
+    }
   }
 
   new_path.StripTrailingSeparatorsInternal();
@@ -419,9 +441,9 @@ FilePath FilePath::InsertBeforeExtension(StringPieceType suffix) const {
 FilePath FilePath::InsertBeforeExtensionASCII(StringPiece suffix)
     const {
   DCHECK(IsStringASCII(suffix));
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
   return InsertBeforeExtension(UTF8ToWide(suffix));
-#elif defined(OS_POSIX) || defined(OS_FUCHSIA)
+#elif BUILDFLAG(IS_POSIX) || BUILDFLAG(IS_FUCHSIA)
   return InsertBeforeExtension(suffix);
 #endif
 }
@@ -440,15 +462,15 @@ FilePath FilePath::AddExtension(StringPieceType extension) const {
       *(str.end() - 1) != kExtensionSeparator) {
     str.append(1, kExtensionSeparator);
   }
-  str.append(extension.data(), extension.size());
+  str.append(extension);
   return FilePath(str);
 }
 
 FilePath FilePath::AddExtensionASCII(StringPiece extension) const {
   DCHECK(IsStringASCII(extension));
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
   return AddExtension(UTF8ToWide(extension));
-#elif defined(OS_POSIX) || defined(OS_FUCHSIA)
+#elif BUILDFLAG(IS_POSIX) || BUILDFLAG(IS_FUCHSIA)
   return AddExtension(extension);
 #endif
 }
@@ -466,7 +488,7 @@ FilePath FilePath::ReplaceExtension(StringPieceType extension) const {
   StringType str = no_ext.value();
   if (extension[0] != kExtensionSeparator)
     str.append(1, kExtensionSeparator);
-  str.append(extension.data(), extension.size());
+  str.append(extension);
   return FilePath(str);
 }
 
@@ -481,6 +503,17 @@ bool FilePath::MatchesExtension(StringPieceType extension) const {
   return FilePath::CompareEqualIgnoreCase(extension, current_extension);
 }
 
+bool FilePath::MatchesFinalExtension(StringPieceType extension) const {
+  DCHECK(extension.empty() || extension[0] == kExtensionSeparator);
+
+  StringType current_final_extension = FinalExtension();
+
+  if (current_final_extension.length() != extension.length())
+    return false;
+
+  return FilePath::CompareEqualIgnoreCase(extension, current_final_extension);
+}
+
 FilePath FilePath::Append(StringPieceType component) const {
   StringPieceType appended = component;
   StringType without_nuls;
@@ -521,7 +554,7 @@ FilePath FilePath::Append(StringPieceType component) const {
     }
   }
 
-  new_path.path_.append(appended.data(), appended.size());
+  new_path.path_.append(appended);
   return new_path;
 }
 
@@ -529,11 +562,15 @@ FilePath FilePath::Append(const FilePath& component) const {
   return Append(component.value());
 }
 
+FilePath FilePath::Append(const SafeBaseName& component) const {
+  return Append(component.path().value());
+}
+
 FilePath FilePath::AppendASCII(StringPiece component) const {
   DCHECK(base::IsStringASCII(component));
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
   return Append(UTF8ToWide(component));
-#elif defined(OS_POSIX) || defined(OS_FUCHSIA)
+#elif BUILDFLAG(IS_POSIX) || BUILDFLAG(IS_FUCHSIA)
   return Append(component);
 #endif
 }
@@ -542,6 +579,11 @@ bool FilePath::IsAbsolute() const {
   return IsPathAbsolute(path_);
 }
 
+bool FilePath::IsNetwork() const {
+  return path_.length() > 1 && FilePath::IsSeparator(path_[0]) &&
+         FilePath::IsSeparator(path_[1]);
+}
+
 bool FilePath::EndsWithSeparator() const {
   if (empty())
     return false;
@@ -574,9 +616,7 @@ bool FilePath::ReferencesParent() const {
     return false;
   }
 
-  std::vector<StringType> components;
-  GetComponents(&components);
-
+  std::vector<StringType> components = GetComponents();
   std::vector<StringType>::const_iterator it = components.begin();
   for (; it != components.end(); ++it) {
     const StringType& component = *it;
@@ -593,9 +633,9 @@ bool FilePath::ReferencesParent() const {
   return false;
 }
 
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
 
-string16 FilePath::LossyDisplayName() const {
+std::u16string FilePath::LossyDisplayName() const {
   return AsString16(path_);
 }
 
@@ -607,10 +647,16 @@ std::string FilePath::AsUTF8Unsafe() const {
   return WideToUTF8(value());
 }
 
-string16 FilePath::AsUTF16Unsafe() const {
+std::u16string FilePath::AsUTF16Unsafe() const {
   return WideToUTF16(value());
 }
 
+// static
+FilePath FilePath::FromASCII(StringPiece ascii) {
+  DCHECK(base::IsStringASCII(ascii));
+  return FilePath(ASCIIToWide(ascii));
+}
+
 // static
 FilePath FilePath::FromUTF8Unsafe(StringPiece utf8) {
   return FilePath(UTF8ToWide(utf8));
@@ -618,15 +664,15 @@ FilePath FilePath::FromUTF8Unsafe(StringPiece utf8) {
 
 // static
 FilePath FilePath::FromUTF16Unsafe(StringPiece16 utf16) {
-  return FilePath(AsWStringPiece(utf16));
+  return FilePath(AsWStringView(utf16));
 }
 
-#elif defined(OS_POSIX) || defined(OS_FUCHSIA)
+#elif BUILDFLAG(IS_POSIX) || BUILDFLAG(IS_FUCHSIA)
 
 // See file_path.h for a discussion of the encoding of paths on POSIX
 // platforms.  These encoding conversion functions are not quite correct.
 
-string16 FilePath::LossyDisplayName() const {
+std::u16string FilePath::LossyDisplayName() const {
   return WideToUTF16(SysNativeMBToWide(path_));
 }
 
@@ -644,7 +690,7 @@ std::string FilePath::AsUTF8Unsafe() const {
 #endif
 }
 
-string16 FilePath::AsUTF16Unsafe() const {
+std::u16string FilePath::AsUTF16Unsafe() const {
 #if defined(SYSTEM_NATIVE_UTF8)
   return UTF8ToUTF16(value());
 #else
@@ -652,6 +698,12 @@ string16 FilePath::AsUTF16Unsafe() const {
 #endif
 }
 
+// static
+FilePath FilePath::FromASCII(StringPiece ascii) {
+  DCHECK(base::IsStringASCII(ascii));
+  return FilePath(ascii);
+}
+
 // static
 FilePath FilePath::FromUTF8Unsafe(StringPiece utf8) {
 #if defined(SYSTEM_NATIVE_UTF8)
@@ -670,12 +722,12 @@ FilePath FilePath::FromUTF16Unsafe(StringPiece16 utf16) {
 #endif
 }
 
-#endif  // defined(OS_WIN)
+#endif  // BUILDFLAG(IS_WIN)
 
 void FilePath::WriteToPickle(Pickle* pickle) const {
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
   pickle->WriteString16(AsStringPiece16(path_));
-#elif defined(OS_POSIX) || defined(OS_FUCHSIA)
+#elif BUILDFLAG(IS_POSIX) || BUILDFLAG(IS_FUCHSIA)
   pickle->WriteString(path_);
 #else
 #error Unsupported platform
@@ -683,12 +735,12 @@ void FilePath::WriteToPickle(Pickle* pickle) const {
 }
 
 bool FilePath::ReadFromPickle(PickleIterator* iter) {
-#if defined(OS_WIN)
-  base::string16 path;
+#if BUILDFLAG(IS_WIN)
+  std::u16string path;
   if (!iter->ReadString16(&path))
     return false;
   path_ = UTF16ToWide(path);
-#elif defined(OS_POSIX) || defined(OS_FUCHSIA)
+#elif BUILDFLAG(IS_POSIX) || BUILDFLAG(IS_FUCHSIA)
   if (!iter->ReadString(&path_))
     return false;
 #else
@@ -701,7 +753,7 @@ bool FilePath::ReadFromPickle(PickleIterator* iter) {
   return true;
 }
 
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
 // Windows specific implementation of file string comparisons.
 
 int FilePath::CompareIgnoreCase(StringPieceType string1,
@@ -735,10 +787,10 @@ int FilePath::CompareIgnoreCase(StringPieceType string1,
   return 0;
 }
 
-#elif defined(OS_APPLE)
+#elif BUILDFLAG(IS_APPLE)
 // Mac OS X specific implementation of file string comparisons.
 
-// cf. http://developer.apple.com/mac/library/technotes/tn/tn1150.html#UnicodeSubtleties
+// cf. https://developer.apple.com/library/archive/technotes/tn/tn1150.html#UnicodeSubtleties
 //
 // "When using CreateTextEncoding to create a text encoding, you should set
 // the TextEncodingBase to kTextEncodingUnicodeV2_0, set the
@@ -764,11 +816,12 @@ int FilePath::CompareIgnoreCase(StringPieceType string1,
 // Ignored characters are mapped to zero.
 //
 // cf. downloadable file linked in
-// http://developer.apple.com/mac/library/technotes/tn/tn1150.html#StringComparisonAlgorithm
+// https://developer.apple.com/library/archive/technotes/tn/tn1150.html#Downloads
 
 namespace {
 
-const UInt16 lower_case_table[] = {
+// clang-format off
+const UInt16 lower_case_table[11 * 256] = {
   // High-byte indices ( == 0 iff no case mapping and no ignorables )
 
   /* 0 */ 0x0100, 0x0200, 0x0000, 0x0300, 0x0400, 0x0500, 0x0000, 0x0000,
@@ -1154,26 +1207,32 @@ const UInt16 lower_case_table[] = {
   /* F */ 0xFFF0, 0xFFF1, 0xFFF2, 0xFFF3, 0xFFF4, 0xFFF5, 0xFFF6, 0xFFF7,
           0xFFF8, 0xFFF9, 0xFFFA, 0xFFFB, 0xFFFC, 0xFFFD, 0xFFFE, 0xFFFF,
 };
+// clang-format on
 
-// Returns the next non-ignorable codepoint within string starting from the
-// position indicated by index, or zero if there are no more.
-// The passed-in index is automatically advanced as the characters in the input
-// HFS-decomposed UTF-8 strings are read.
-inline int HFSReadNextNonIgnorableCodepoint(const char* string,
-                                            int length,
-                                            int* index) {
-  int codepoint = 0;
+// Returns the next non-ignorable codepoint within `string` starting from the
+// position indicated by `index`, or zero if there are no more.
+// The passed-in `index` is automatically advanced as the characters in the
+// input HFS-decomposed UTF-8 strings are read.
+inline base_icu::UChar32 HFSReadNextNonIgnorableCodepoint(const char* string,
+                                                          size_t length,
+                                                          size_t* index) {
+  base_icu::UChar32 codepoint = 0;
   while (*index < length && codepoint == 0) {
     // CBU8_NEXT returns a value < 0 in error cases. For purposes of string
     // comparison, we just use that value and flag it with DCHECK.
-    CBU8_NEXT(string, *index, length, codepoint);
+    CBU8_NEXT(reinterpret_cast<const uint8_t*>(string), *index, length,
+              codepoint);
     DCHECK_GT(codepoint, 0);
-    if (codepoint > 0) {
+
+    // Note: Here, there are no lower case conversion implemented in the
+    // Supplementary Multilingual Plane (codepoint > 0xFFFF).
+
+    if (codepoint > 0 && codepoint <= 0xFFFF) {
       // Check if there is a subtable for this upper byte.
       int lookup_offset = lower_case_table[codepoint >> 8];
       if (lookup_offset != 0)
         codepoint = lower_case_table[lookup_offset + (codepoint & 0x00FF)];
-      // Note: codepoint1 may be again 0 at this point if the character was
+      // Note: `codepoint` may be again 0 at this point if the character was
       // an ignorable.
     }
   }
@@ -1187,18 +1246,16 @@ inline int HFSReadNextNonIgnorableCodepoint(const char* string,
 // The input strings must be in the special HFS decomposed form.
 int FilePath::HFSFastUnicodeCompare(StringPieceType string1,
                                     StringPieceType string2) {
-  int length1 = string1.length();
-  int length2 = string2.length();
-  int index1 = 0;
-  int index2 = 0;
+  size_t length1 = string1.length();
+  size_t length2 = string2.length();
+  size_t index1 = 0;
+  size_t index2 = 0;
 
   for (;;) {
-    int codepoint1 = HFSReadNextNonIgnorableCodepoint(string1.data(),
-                                                      length1,
-                                                      &index1);
-    int codepoint2 = HFSReadNextNonIgnorableCodepoint(string2.data(),
-                                                      length2,
-                                                      &index2);
+    base_icu::UChar32 codepoint1 =
+        HFSReadNextNonIgnorableCodepoint(string1.data(), length1, &index1);
+    base_icu::UChar32 codepoint2 =
+        HFSReadNextNonIgnorableCodepoint(string2.data(), length2, &index2);
     if (codepoint1 != codepoint2)
       return (codepoint1 < codepoint2) ? -1 : 1;
     if (codepoint1 == 0) {
@@ -1211,28 +1268,24 @@ int FilePath::HFSFastUnicodeCompare(StringPieceType string1,
 
 StringType FilePath::GetHFSDecomposedForm(StringPieceType string) {
   StringType result;
-  ScopedCFTypeRef<CFStringRef> cfstring(
-      CFStringCreateWithBytesNoCopy(
-          NULL,
-          reinterpret_cast<const UInt8*>(string.data()),
-          string.length(),
-          kCFStringEncodingUTF8,
-          false,
-          kCFAllocatorNull));
+  apple::ScopedCFTypeRef<CFStringRef> cfstring(CFStringCreateWithBytesNoCopy(
+      NULL, reinterpret_cast<const UInt8*>(string.data()),
+      checked_cast<CFIndex>(string.length()), kCFStringEncodingUTF8, false,
+      kCFAllocatorNull));
   if (cfstring) {
     // Query the maximum length needed to store the result. In most cases this
     // will overestimate the required space. The return value also already
     // includes the space needed for a terminating 0.
-    CFIndex length = CFStringGetMaximumSizeOfFileSystemRepresentation(cfstring);
+    CFIndex length =
+        CFStringGetMaximumSizeOfFileSystemRepresentation(cfstring.get());
     DCHECK_GT(length, 0);  // should be at least 1 for the 0-terminator.
     // Reserve enough space for CFStringGetFileSystemRepresentation to write
     // into. Also set the length to the maximum so that we can shrink it later.
     // (Increasing rather than decreasing it would clobber the string contents!)
-    result.reserve(length);
-    result.resize(length - 1);
-    Boolean success = CFStringGetFileSystemRepresentation(cfstring,
-                                                          &result[0],
-                                                          length);
+    result.reserve(static_cast<size_t>(length));
+    result.resize(static_cast<size_t>(length) - 1);
+    Boolean success =
+        CFStringGetFileSystemRepresentation(cfstring.get(), &result[0], length);
     if (success) {
       // Reduce result.length() to actual string length.
       result.resize(strlen(result.c_str()));
@@ -1258,22 +1311,14 @@ int FilePath::CompareIgnoreCase(StringPieceType string1,
 
   // GetHFSDecomposedForm() returns an empty string in an error case.
   if (hfs1.empty() || hfs2.empty()) {
-    ScopedCFTypeRef<CFStringRef> cfstring1(
-        CFStringCreateWithBytesNoCopy(
-            NULL,
-            reinterpret_cast<const UInt8*>(string1.data()),
-            string1.length(),
-            kCFStringEncodingUTF8,
-            false,
-            kCFAllocatorNull));
-    ScopedCFTypeRef<CFStringRef> cfstring2(
-        CFStringCreateWithBytesNoCopy(
-            NULL,
-            reinterpret_cast<const UInt8*>(string2.data()),
-            string2.length(),
-            kCFStringEncodingUTF8,
-            false,
-            kCFAllocatorNull));
+    apple::ScopedCFTypeRef<CFStringRef> cfstring1(CFStringCreateWithBytesNoCopy(
+        NULL, reinterpret_cast<const UInt8*>(string1.data()),
+        checked_cast<CFIndex>(string1.length()), kCFStringEncodingUTF8, false,
+        kCFAllocatorNull));
+    apple::ScopedCFTypeRef<CFStringRef> cfstring2(CFStringCreateWithBytesNoCopy(
+        NULL, reinterpret_cast<const UInt8*>(string2.data()),
+        checked_cast<CFIndex>(string2.length()), kCFStringEncodingUTF8, false,
+        kCFAllocatorNull));
     // If neither GetHFSDecomposedForm nor CFStringCreateWithBytesNoCopy
     // succeed, fall back to strcmp. This can occur when the input string is
     // invalid UTF-8.
@@ -1287,15 +1332,14 @@ int FilePath::CompareIgnoreCase(StringPieceType string1,
       return 0;
     }
 
-    return CFStringCompare(cfstring1,
-                           cfstring2,
-                           kCFCompareCaseInsensitive);
+    return static_cast<int>(CFStringCompare(cfstring1.get(), cfstring2.get(),
+                                            kCFCompareCaseInsensitive));
   }
 
   return HFSFastUnicodeCompare(hfs1, hfs2);
 }
 
-#elif defined(OS_POSIX) || defined(OS_FUCHSIA)
+#elif BUILDFLAG(IS_POSIX) || BUILDFLAG(IS_FUCHSIA)
 
 // Generic Posix system comparisons.
 int FilePath::CompareIgnoreCase(StringPieceType string1,
@@ -1338,6 +1382,10 @@ FilePath FilePath::NormalizePathSeparators() const {
   return NormalizePathSeparatorsTo(kSeparators[0]);
 }
 
+void FilePath::WriteIntoTrace(perfetto::TracedValue context) const {
+  perfetto::WriteIntoTracedValue(std::move(context), value());
+}
+
 FilePath FilePath::NormalizePathSeparatorsTo(CharType separator) const {
 #if defined(FILE_PATH_USES_WIN_SEPARATORS)
   DCHECK_NE(kSeparators + kSeparatorsLength,
@@ -1352,7 +1400,7 @@ FilePath FilePath::NormalizePathSeparatorsTo(CharType separator) const {
 #endif
 }
 
-#if defined(OS_ANDROID)
+#if BUILDFLAG(IS_ANDROID)
 bool FilePath::IsContentUri() const {
   return StartsWith(path_, "content://", base::CompareCase::INSENSITIVE_ASCII);
 }
