@@ -1,4 +1,4 @@
-// Copyright 2016 The Chromium Authors. All rights reserved.
+// Copyright 2016 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,14 +8,15 @@
 #include <memory>
 #include <utility>
 
-#include "base/bind.h"
-#include "base/callback_helpers.h"
 #include "base/containers/queue.h"
+#include "base/functional/bind.h"
+#include "base/functional/callback_helpers.h"
 #include "base/location.h"
 #include "base/memory/ptr_util.h"
 #include "base/memory/ref_counted.h"
 #include "base/synchronization/lock.h"
 #include "base/synchronization/waitable_event.h"
+#include "base/task/sequenced_task_runner.h"
 #include "base/task/single_thread_task_runner.h"
 #include "base/test/bind.h"
 #include "base/test/gtest_util.h"
@@ -23,10 +24,8 @@
 #include "base/test/task_environment.h"
 #include "base/test/test_timeouts.h"
 #include "base/threading/platform_thread.h"
-#include "base/threading/sequenced_task_runner_handle.h"
 #include "base/threading/thread.h"
 #include "base/threading/thread_checker_impl.h"
-#include "base/threading/thread_task_runner_handle.h"
 #include "build/build_config.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
@@ -44,11 +43,11 @@ void RunNestedLoopTask(int* counter) {
   RunLoop nested_run_loop(RunLoop::Type::kNestableTasksAllowed);
 
   // This task should quit |nested_run_loop| but not the main RunLoop.
-  ThreadTaskRunnerHandle::Get()->PostTask(
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
       FROM_HERE, BindOnce(&QuitWhenIdleTask, Unretained(&nested_run_loop),
                           Unretained(counter)));
 
-  ThreadTaskRunnerHandle::Get()->PostDelayedTask(
+  SingleThreadTaskRunner::GetCurrentDefault()->PostDelayedTask(
       FROM_HERE, MakeExpectedNotRunClosure(FROM_HERE), Days(1));
 
   nested_run_loop.Run();
@@ -146,10 +145,11 @@ class TestBoundDelegate final : public InjectableTestDelegate {
   TestBoundDelegate() = default;
 
   // Makes this TestBoundDelegate become the RunLoop::Delegate and
-  // ThreadTaskRunnerHandle for this thread.
+  // SingleThreadTaskRunner::CurrentDefaultHandle for this thread.
   void BindToCurrentThread() {
     thread_task_runner_handle_ =
-        std::make_unique<ThreadTaskRunnerHandle>(simple_task_runner_);
+        std::make_unique<SingleThreadTaskRunner::CurrentDefaultHandle>(
+            simple_task_runner_);
     RunLoop::RegisterDelegateForCurrentThread(this);
   }
 
@@ -191,7 +191,8 @@ class TestBoundDelegate final : public InjectableTestDelegate {
   scoped_refptr<SimpleSingleThreadTaskRunner> simple_task_runner_ =
       MakeRefCounted<SimpleSingleThreadTaskRunner>();
 
-  std::unique_ptr<ThreadTaskRunnerHandle> thread_task_runner_handle_;
+  std::unique_ptr<SingleThreadTaskRunner::CurrentDefaultHandle>
+      thread_task_runner_handle_;
 
   bool should_quit_ = false;
 };
@@ -247,12 +248,12 @@ class RunLoopTest : public testing::TestWithParam<RunLoopTestType> {
 
 TEST_P(RunLoopTest, QuitWhenIdle) {
   int counter = 0;
-  ThreadTaskRunnerHandle::Get()->PostTask(
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
       FROM_HERE, BindOnce(&QuitWhenIdleTask, Unretained(&run_loop_),
                           Unretained(&counter)));
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                          MakeExpectedRunClosure(FROM_HERE));
-  ThreadTaskRunnerHandle::Get()->PostDelayedTask(
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, MakeExpectedRunClosure(FROM_HERE));
+  SingleThreadTaskRunner::GetCurrentDefault()->PostDelayedTask(
       FROM_HERE, MakeExpectedNotRunClosure(FROM_HERE), Days(1));
 
   run_loop_.Run();
@@ -261,14 +262,14 @@ TEST_P(RunLoopTest, QuitWhenIdle) {
 
 TEST_P(RunLoopTest, QuitWhenIdleNestedLoop) {
   int counter = 0;
-  ThreadTaskRunnerHandle::Get()->PostTask(
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
       FROM_HERE, BindOnce(&RunNestedLoopTask, Unretained(&counter)));
-  ThreadTaskRunnerHandle::Get()->PostTask(
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
       FROM_HERE, BindOnce(&QuitWhenIdleTask, Unretained(&run_loop_),
                           Unretained(&counter)));
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                          MakeExpectedRunClosure(FROM_HERE));
-  ThreadTaskRunnerHandle::Get()->PostDelayedTask(
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, MakeExpectedRunClosure(FROM_HERE));
+  SingleThreadTaskRunner::GetCurrentDefault()->PostDelayedTask(
       FROM_HERE, MakeExpectedNotRunClosure(FROM_HERE), Days(1));
 
   run_loop_.Run();
@@ -276,11 +277,11 @@ TEST_P(RunLoopTest, QuitWhenIdleNestedLoop) {
 }
 
 TEST_P(RunLoopTest, QuitWhenIdleClosure) {
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                          run_loop_.QuitWhenIdleClosure());
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                          MakeExpectedRunClosure(FROM_HERE));
-  ThreadTaskRunnerHandle::Get()->PostDelayedTask(
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, run_loop_.QuitWhenIdleClosure());
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, MakeExpectedRunClosure(FROM_HERE));
+  SingleThreadTaskRunner::GetCurrentDefault()->PostDelayedTask(
       FROM_HERE, MakeExpectedNotRunClosure(FROM_HERE), Days(1));
 
   run_loop_.Run();
@@ -306,8 +307,8 @@ TEST_P(RunLoopTest, QuitFromOtherSequence) {
       other_thread.task_runner();
 
   // Always expected to run before asynchronous Quit() kicks in.
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                          MakeExpectedRunClosure(FROM_HERE));
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, MakeExpectedRunClosure(FROM_HERE));
 
   WaitableEvent loop_was_quit(WaitableEvent::ResetPolicy::MANUAL,
                               WaitableEvent::InitialState::NOT_SIGNALED);
@@ -321,8 +322,8 @@ TEST_P(RunLoopTest, QuitFromOtherSequence) {
   // Anything that's posted after the Quit closure was posted back to this
   // sequence shouldn't get a chance to run.
   loop_was_quit.Wait();
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                          MakeExpectedNotRunClosure(FROM_HERE));
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, MakeExpectedNotRunClosure(FROM_HERE));
 
   run_loop_.Run();
 }
@@ -335,8 +336,8 @@ TEST_P(RunLoopTest, QuitFromOtherSequenceWithClosure) {
       other_thread.task_runner();
 
   // Always expected to run before asynchronous Quit() kicks in.
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                          MakeExpectedRunClosure(FROM_HERE));
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, MakeExpectedRunClosure(FROM_HERE));
 
   WaitableEvent loop_was_quit(WaitableEvent::ResetPolicy::MANUAL,
                               WaitableEvent::InitialState::NOT_SIGNALED);
@@ -348,8 +349,8 @@ TEST_P(RunLoopTest, QuitFromOtherSequenceWithClosure) {
   // Anything that's posted after the Quit closure was posted back to this
   // sequence shouldn't get a chance to run.
   loop_was_quit.Wait();
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                          MakeExpectedNotRunClosure(FROM_HERE));
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, MakeExpectedNotRunClosure(FROM_HERE));
 
   run_loop_.Run();
 }
@@ -363,8 +364,8 @@ TEST_P(RunLoopTest, QuitFromOtherSequenceRacy) {
       other_thread.task_runner();
 
   // Always expected to run before asynchronous Quit() kicks in.
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                          MakeExpectedRunClosure(FROM_HERE));
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, MakeExpectedRunClosure(FROM_HERE));
 
   other_sequence->PostTask(FROM_HERE, run_loop_.QuitClosure());
 
@@ -380,8 +381,8 @@ TEST_P(RunLoopTest, QuitFromOtherSequenceRacyWithClosure) {
       other_thread.task_runner();
 
   // Always expected to run before asynchronous Quit() kicks in.
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                          MakeExpectedRunClosure(FROM_HERE));
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, MakeExpectedRunClosure(FROM_HERE));
 
   other_sequence->PostTask(FROM_HERE, run_loop_.QuitClosure());
 
@@ -395,16 +396,16 @@ TEST_P(RunLoopTest, QuitWhenIdleFromOtherSequence) {
   scoped_refptr<SequencedTaskRunner> other_sequence =
       other_thread.task_runner();
 
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                          MakeExpectedRunClosure(FROM_HERE));
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, MakeExpectedRunClosure(FROM_HERE));
 
   other_sequence->PostTask(
       FROM_HERE,
       base::BindOnce([](RunLoop* run_loop) { run_loop->QuitWhenIdle(); },
                      Unretained(&run_loop_)));
 
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                          MakeExpectedRunClosure(FROM_HERE));
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, MakeExpectedRunClosure(FROM_HERE));
 
   run_loop_.Run();
 
@@ -419,13 +420,13 @@ TEST_P(RunLoopTest, QuitWhenIdleFromOtherSequenceWithClosure) {
   scoped_refptr<SequencedTaskRunner> other_sequence =
       other_thread.task_runner();
 
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                          MakeExpectedRunClosure(FROM_HERE));
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, MakeExpectedRunClosure(FROM_HERE));
 
   other_sequence->PostTask(FROM_HERE, run_loop_.QuitWhenIdleClosure());
 
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                          MakeExpectedRunClosure(FROM_HERE));
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, MakeExpectedRunClosure(FROM_HERE));
 
   run_loop_.Run();
 
@@ -435,72 +436,77 @@ TEST_P(RunLoopTest, QuitWhenIdleFromOtherSequenceWithClosure) {
 
 TEST_P(RunLoopTest, IsRunningOnCurrentThread) {
   EXPECT_FALSE(RunLoop::IsRunningOnCurrentThread());
-  ThreadTaskRunnerHandle::Get()->PostTask(
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
       FROM_HERE,
       BindOnce([]() { EXPECT_TRUE(RunLoop::IsRunningOnCurrentThread()); }));
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE, run_loop_.QuitClosure());
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, run_loop_.QuitClosure());
   run_loop_.Run();
 }
 
 TEST_P(RunLoopTest, IsNestedOnCurrentThread) {
   EXPECT_FALSE(RunLoop::IsNestedOnCurrentThread());
 
-  ThreadTaskRunnerHandle::Get()->PostTask(
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
       FROM_HERE, BindOnce([]() {
         EXPECT_FALSE(RunLoop::IsNestedOnCurrentThread());
 
         RunLoop nested_run_loop(RunLoop::Type::kNestableTasksAllowed);
 
-        ThreadTaskRunnerHandle::Get()->PostTask(
+        SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
             FROM_HERE, BindOnce([]() {
               EXPECT_TRUE(RunLoop::IsNestedOnCurrentThread());
             }));
-        ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                                nested_run_loop.QuitClosure());
+        SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+            FROM_HERE, nested_run_loop.QuitClosure());
 
         EXPECT_FALSE(RunLoop::IsNestedOnCurrentThread());
         nested_run_loop.Run();
         EXPECT_FALSE(RunLoop::IsNestedOnCurrentThread());
       }));
 
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE, run_loop_.QuitClosure());
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, run_loop_.QuitClosure());
   run_loop_.Run();
 }
 
 TEST_P(RunLoopTest, CannotRunMoreThanOnce) {
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE, run_loop_.QuitClosure());
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, run_loop_.QuitClosure());
   run_loop_.Run();
   EXPECT_DCHECK_DEATH({ run_loop_.Run(); });
 }
 
 TEST_P(RunLoopTest, CanRunUntilIdleMoreThanOnce) {
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE, DoNothing());
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(FROM_HERE, DoNothing());
   run_loop_.RunUntilIdle();
 
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE, DoNothing());
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(FROM_HERE, DoNothing());
   run_loop_.RunUntilIdle();
   run_loop_.RunUntilIdle();
 }
 
 TEST_P(RunLoopTest, CanRunUntilIdleThenRunIfNotQuit) {
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE, DoNothing());
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(FROM_HERE, DoNothing());
   run_loop_.RunUntilIdle();
 
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE, run_loop_.QuitClosure());
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, run_loop_.QuitClosure());
   run_loop_.Run();
 }
 
 TEST_P(RunLoopTest, CannotRunUntilIdleThenRunIfQuit) {
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE, run_loop_.QuitClosure());
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, run_loop_.QuitClosure());
   run_loop_.RunUntilIdle();
 
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE, DoNothing());
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(FROM_HERE, DoNothing());
   EXPECT_DCHECK_DEATH({ run_loop_.Run(); });
 }
 
 TEST_P(RunLoopTest, CannotRunAgainIfQuitWhenIdle) {
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                          run_loop_.QuitWhenIdleClosure());
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE, run_loop_.QuitWhenIdleClosure());
   run_loop_.RunUntilIdle();
 
   EXPECT_DCHECK_DEATH({ run_loop_.RunUntilIdle(); });
@@ -539,8 +545,8 @@ TEST_P(RunLoopTest, NestingObservers) {
 
   const RepeatingClosure run_nested_loop = BindRepeating([]() {
     RunLoop nested_run_loop(RunLoop::Type::kNestableTasksAllowed);
-    ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
-                                            nested_run_loop.QuitClosure());
+    SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
+        FROM_HERE, nested_run_loop.QuitClosure());
     nested_run_loop.Run();
   });
 
@@ -549,12 +555,14 @@ TEST_P(RunLoopTest, NestingObservers) {
   // when exiting each nesting depth. Each one of these tasks is ahead of the
   // QuitClosures as those are only posted at the end of the queue when
   // |run_nested_loop| is executed.
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE, run_nested_loop);
-  ThreadTaskRunnerHandle::Get()->PostTask(
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(FROM_HERE,
+                                                        run_nested_loop);
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
       FROM_HERE,
       base::BindOnce(&MockTask::Task, base::Unretained(&mock_task_a)));
-  ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE, run_nested_loop);
-  ThreadTaskRunnerHandle::Get()->PostTask(
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(FROM_HERE,
+                                                        run_nested_loop);
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
       FROM_HERE,
       base::BindOnce(&MockTask::Task, base::Unretained(&mock_task_b)));
 
@@ -591,7 +599,8 @@ INSTANTIATE_TEST_SUITE_P(Mock,
 
 TEST(RunLoopDeathTest, MustRegisterBeforeInstantiating) {
   TestBoundDelegate unbound_test_delegate_;
-  // RunLoop::RunLoop() should CHECK fetching the ThreadTaskRunnerHandle.
+  // RunLoop::RunLoop() should CHECK fetching the
+  // SingleThreadTaskRunner::CurrentDefaultHandle.
   EXPECT_DEATH_IF_SUPPORTED({ RunLoop(); }, "");
 }
 
@@ -611,13 +620,13 @@ TEST(RunLoopDelegateTest, NestableTasksDontRunInDefaultNestedLoops) {
   // The first task on the main loop will result in a nested run loop. Since
   // it's not kNestableTasksAllowed, no further task should be processed until
   // it's quit.
-  ThreadTaskRunnerHandle::Get()->PostTask(
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
       FROM_HERE,
       BindOnce([](RunLoop* nested_run_loop) { nested_run_loop->Run(); },
                Unretained(&nested_run_loop)));
 
   // Post a task that will fail if it runs inside the nested run loop.
-  ThreadTaskRunnerHandle::Get()->PostTask(
+  SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
       FROM_HERE,
       BindOnce(
           [](const bool& nested_run_loop_ended,
