@@ -1,4 +1,4 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,7 +9,7 @@
 #include <algorithm>
 #include <set>
 
-#include "base/bind.h"
+#include "base/functional/bind.h"
 #include "base/json/json_reader.h"
 #include "base/logging.h"
 #include "base/memory/ptr_util.h"
@@ -21,6 +21,7 @@
 #include "base/trace_event/trace_config.h"
 #include "base/trace_event/trace_log.h"
 #include "base/values.h"
+#include "third_party/abseil-cpp/absl/types/optional.h"
 
 namespace {
 void OnTraceDataCollected(base::OnceClosure quit_closure,
@@ -37,12 +38,7 @@ namespace trace_analyzer {
 
 // TraceEvent
 
-TraceEvent::TraceEvent()
-    : thread(0, 0),
-      timestamp(0),
-      duration(0),
-      phase(TRACE_EVENT_PHASE_BEGIN),
-      other_event(nullptr) {}
+TraceEvent::TraceEvent() : thread(0, 0) {}
 
 TraceEvent::TraceEvent(TraceEvent&& other) = default;
 
@@ -51,22 +47,19 @@ TraceEvent::~TraceEvent() = default;
 TraceEvent& TraceEvent::operator=(TraceEvent&& rhs) = default;
 
 bool TraceEvent::SetFromJSON(const base::Value* event_value) {
-  if (event_value->type() != base::Value::Type::DICTIONARY) {
-    LOG(ERROR) << "Value must be Type::DICTIONARY";
+  if (!event_value->is_dict()) {
+    LOG(ERROR) << "Value must be Type::DICT";
     return false;
   }
-  const base::DictionaryValue* dictionary =
-      static_cast<const base::DictionaryValue*>(event_value);
 
-  std::string phase_str;
-  const base::DictionaryValue* args = nullptr;
-
-  if (!dictionary->GetString("ph", &phase_str)) {
+  const base::Value::Dict& event_dict = event_value->GetDict();
+  const std::string* maybe_phase = event_dict.FindString("ph");
+  if (!maybe_phase) {
     LOG(ERROR) << "ph is missing from TraceEvent JSON";
     return false;
   }
 
-  phase = *phase_str.data();
+  phase = *maybe_phase->data();
 
   bool may_have_duration = (phase == TRACE_EVENT_PHASE_COMPLETE);
   bool require_origin = (phase != TRACE_EVENT_PHASE_METADATA);
@@ -74,85 +67,139 @@ bool TraceEvent::SetFromJSON(const base::Value* event_value) {
                      phase == TRACE_EVENT_PHASE_ASYNC_STEP_INTO ||
                      phase == TRACE_EVENT_PHASE_ASYNC_STEP_PAST ||
                      phase == TRACE_EVENT_PHASE_MEMORY_DUMP ||
-                     phase == TRACE_EVENT_PHASE_ENTER_CONTEXT ||
-                     phase == TRACE_EVENT_PHASE_LEAVE_CONTEXT ||
                      phase == TRACE_EVENT_PHASE_CREATE_OBJECT ||
                      phase == TRACE_EVENT_PHASE_DELETE_OBJECT ||
                      phase == TRACE_EVENT_PHASE_SNAPSHOT_OBJECT ||
-                     phase == TRACE_EVENT_PHASE_ASYNC_END);
+                     phase == TRACE_EVENT_PHASE_ASYNC_END ||
+                     phase == TRACE_EVENT_PHASE_NESTABLE_ASYNC_BEGIN ||
+                     phase == TRACE_EVENT_PHASE_NESTABLE_ASYNC_END);
 
-  if (require_origin && !dictionary->GetInteger("pid", &thread.process_id)) {
-    LOG(ERROR) << "pid is missing from TraceEvent JSON";
-    return false;
-  }
-  if (require_origin && !dictionary->GetInteger("tid", &thread.thread_id)) {
-    LOG(ERROR) << "tid is missing from TraceEvent JSON";
-    return false;
-  }
-  if (require_origin && !dictionary->GetDouble("ts", &timestamp)) {
-    LOG(ERROR) << "ts is missing from TraceEvent JSON";
-    return false;
+  if (require_origin) {
+    absl::optional<int> maybe_process_id = event_dict.FindInt("pid");
+    if (!maybe_process_id) {
+      LOG(ERROR) << "pid is missing from TraceEvent JSON";
+      return false;
+    }
+    thread.process_id = *maybe_process_id;
+
+    absl::optional<int> maybe_thread_id = event_dict.FindInt("tid");
+    if (!maybe_thread_id) {
+      LOG(ERROR) << "tid is missing from TraceEvent JSON";
+      return false;
+    }
+    thread.thread_id = *maybe_thread_id;
+
+    absl::optional<double> maybe_timestamp = event_dict.FindDouble("ts");
+    if (!maybe_timestamp) {
+      LOG(ERROR) << "ts is missing from TraceEvent JSON";
+      return false;
+    }
+    timestamp = *maybe_timestamp;
   }
   if (may_have_duration) {
-    dictionary->GetDouble("dur", &duration);
+    absl::optional<double> maybe_duration = event_dict.FindDouble("dur");
+    if (maybe_duration)
+      duration = *maybe_duration;
   }
-  if (!dictionary->GetString("cat", &category)) {
+  const std::string* maybe_category = event_dict.FindString("cat");
+  if (!maybe_category) {
     LOG(ERROR) << "cat is missing from TraceEvent JSON";
     return false;
   }
-  if (!dictionary->GetString("name", &name)) {
+  category = *maybe_category;
+  const std::string* maybe_name = event_dict.FindString("name");
+  if (!maybe_name) {
     LOG(ERROR) << "name is missing from TraceEvent JSON";
     return false;
   }
-  if (!dictionary->GetDictionary("args", &args)) {
-    std::string stripped_args;
+  name = *maybe_name;
+  const base::Value::Dict* maybe_args = event_dict.FindDict("args");
+  if (!maybe_args) {
     // If argument filter is enabled, the arguments field contains a string
     // value.
-    if (!dictionary->GetString("args", &stripped_args) ||
-        stripped_args != "__stripped__") {
+    const std::string* maybe_stripped_args = event_dict.FindString("args");
+    if (!maybe_stripped_args || *maybe_stripped_args != "__stripped__") {
       LOG(ERROR) << "args is missing from TraceEvent JSON";
       return false;
     }
   }
-  if (require_id && !dictionary->GetString("id", &id) &&
-      !dictionary->FindKey("id2")) {
-    LOG(ERROR)
-        << "id/id2 is missing from ASYNC_BEGIN/ASYNC_END TraceEvent JSON";
-    return false;
+  const base::Value::Dict* maybe_id2 = nullptr;
+  if (require_id) {
+    const std::string* maybe_id = event_dict.FindString("id");
+    maybe_id2 = event_dict.FindDict("id2");
+    if (!maybe_id && !maybe_id2) {
+      LOG(ERROR)
+          << "id/id2 is missing from ASYNC_BEGIN/ASYNC_END TraceEvent JSON";
+      return false;
+    }
+    if (maybe_id)
+      id = *maybe_id;
   }
 
-  dictionary->GetDouble("tdur", &thread_duration);
-  dictionary->GetDouble("tts", &thread_timestamp);
-  dictionary->GetString("scope", &scope);
-  dictionary->GetString("bind_id", &bind_id);
-  dictionary->GetBoolean("flow_out", &flow_out);
-  dictionary->GetBoolean("flow_in", &flow_in);
+  absl::optional<double> maybe_thread_duration = event_dict.FindDouble("tdur");
+  if (maybe_thread_duration) {
+    thread_duration = *maybe_thread_duration;
+  }
+  absl::optional<double> maybe_thread_timestamp = event_dict.FindDouble("tts");
+  if (maybe_thread_timestamp) {
+    thread_timestamp = *maybe_thread_timestamp;
+  }
+  const std::string* maybe_scope = event_dict.FindString("scope");
+  if (maybe_scope) {
+    scope = *maybe_scope;
+  }
+  const std::string* maybe_bind_id = event_dict.FindString("bind_id");
+  if (maybe_bind_id) {
+    bind_id = *maybe_bind_id;
+  }
+  absl::optional<bool> maybe_flow_out = event_dict.FindBool("flow_out");
+  if (maybe_flow_out) {
+    flow_out = *maybe_flow_out;
+  }
+  absl::optional<bool> maybe_flow_in = event_dict.FindBool("flow_in");
+  if (maybe_flow_in) {
+    flow_in = *maybe_flow_in;
+  }
 
-  const base::DictionaryValue* id2;
-  if (dictionary->GetDictionary("id2", &id2)) {
-    id2->GetString("global", &global_id2);
-    id2->GetString("local", &local_id2);
+  if (maybe_id2) {
+    const std::string* maybe_global_id2 = maybe_id2->FindString("global");
+    if (maybe_global_id2) {
+      global_id2 = *maybe_global_id2;
+    }
+    const std::string* maybe_local_id2 = maybe_id2->FindString("local");
+    if (maybe_local_id2) {
+      local_id2 = *maybe_local_id2;
+    }
   }
 
   // For each argument, copy the type and create a trace_analyzer::TraceValue.
-  if (args) {
-    for (base::DictionaryValue::Iterator it(*args); !it.IsAtEnd();
-         it.Advance()) {
-      std::string str;
-      bool boolean = false;
-      int int_num = 0;
-      double double_num = 0.0;
-      if (it.value().GetAsString(&str)) {
-        arg_strings[it.key()] = str;
-      } else if (it.value().GetAsInteger(&int_num)) {
-        arg_numbers[it.key()] = static_cast<double>(int_num);
-      } else if (it.value().GetAsBoolean(&boolean)) {
-        arg_numbers[it.key()] = static_cast<double>(boolean ? 1 : 0);
-      } else if (it.value().GetAsDouble(&double_num)) {
-        arg_numbers[it.key()] = double_num;
+  // TODO(crbug.com/1303874): Add BINARY and LIST arg types if needed.
+  if (maybe_args) {
+    for (auto pair : *maybe_args) {
+      switch (pair.second.type()) {
+        case base::Value::Type::STRING:
+          arg_strings[pair.first] = pair.second.GetString();
+          break;
+
+        case base::Value::Type::INTEGER:
+          arg_numbers[pair.first] = static_cast<double>(pair.second.GetInt());
+          break;
+
+        case base::Value::Type::BOOLEAN:
+          arg_numbers[pair.first] = pair.second.GetBool() ? 1.0 : 0.0;
+          break;
+
+        case base::Value::Type::DOUBLE:
+          arg_numbers[pair.first] = pair.second.GetDouble();
+          break;
+
+        case base::Value::Type::DICT:
+          arg_dicts[pair.first] = pair.second.GetDict().Clone();
+          break;
+
+        default:
+          break;
       }
-      // Record all arguments as values.
-      arg_values[it.key()] = it.value().CreateDeepCopy();
     }
   }
 
@@ -163,9 +210,9 @@ double TraceEvent::GetAbsTimeToOtherEvent() const {
   return fabs(other_event->timestamp - timestamp);
 }
 
-bool TraceEvent::GetArgAsString(const std::string& name,
+bool TraceEvent::GetArgAsString(const std::string& arg_name,
                                 std::string* arg) const {
-  const auto it = arg_strings.find(name);
+  const auto it = arg_strings.find(arg_name);
   if (it != arg_strings.end()) {
     *arg = it->second;
     return true;
@@ -173,9 +220,9 @@ bool TraceEvent::GetArgAsString(const std::string& name,
   return false;
 }
 
-bool TraceEvent::GetArgAsNumber(const std::string& name,
+bool TraceEvent::GetArgAsNumber(const std::string& arg_name,
                                 double* arg) const {
-  const auto it = arg_numbers.find(name);
+  const auto it = arg_numbers.find(arg_name);
   if (it != arg_numbers.end()) {
     *arg = it->second;
     return true;
@@ -183,62 +230,62 @@ bool TraceEvent::GetArgAsNumber(const std::string& name,
   return false;
 }
 
-bool TraceEvent::GetArgAsValue(const std::string& name,
-                               std::unique_ptr<base::Value>* arg) const {
-  const auto it = arg_values.find(name);
-  if (it != arg_values.end()) {
-    *arg = it->second->CreateDeepCopy();
+bool TraceEvent::GetArgAsDict(const std::string& arg_name,
+                              base::Value::Dict* arg) const {
+  const auto it = arg_dicts.find(arg_name);
+  if (it != arg_dicts.end()) {
+    *arg = it->second.Clone();
     return true;
   }
   return false;
 }
 
-bool TraceEvent::HasStringArg(const std::string& name) const {
-  return (arg_strings.find(name) != arg_strings.end());
+bool TraceEvent::HasStringArg(const std::string& arg_name) const {
+  return (arg_strings.find(arg_name) != arg_strings.end());
 }
 
-bool TraceEvent::HasNumberArg(const std::string& name) const {
-  return (arg_numbers.find(name) != arg_numbers.end());
+bool TraceEvent::HasNumberArg(const std::string& arg_name) const {
+  return (arg_numbers.find(arg_name) != arg_numbers.end());
 }
 
-bool TraceEvent::HasArg(const std::string& name) const {
-  return (arg_values.find(name) != arg_values.end());
+bool TraceEvent::HasDictArg(const std::string& arg_name) const {
+  return (arg_dicts.find(arg_name) != arg_dicts.end());
 }
 
-std::string TraceEvent::GetKnownArgAsString(const std::string& name) const {
+std::string TraceEvent::GetKnownArgAsString(const std::string& arg_name) const {
   std::string arg_string;
-  bool result = GetArgAsString(name, &arg_string);
+  bool result = GetArgAsString(arg_name, &arg_string);
   DCHECK(result);
   return arg_string;
 }
 
-double TraceEvent::GetKnownArgAsDouble(const std::string& name) const {
+double TraceEvent::GetKnownArgAsDouble(const std::string& arg_name) const {
   double arg_double = 0;
-  bool result = GetArgAsNumber(name, &arg_double);
+  bool result = GetArgAsNumber(arg_name, &arg_double);
   DCHECK(result);
   return arg_double;
 }
 
-int TraceEvent::GetKnownArgAsInt(const std::string& name) const {
+int TraceEvent::GetKnownArgAsInt(const std::string& arg_name) const {
   double arg_double = 0;
-  bool result = GetArgAsNumber(name, &arg_double);
+  bool result = GetArgAsNumber(arg_name, &arg_double);
   DCHECK(result);
   return static_cast<int>(arg_double);
 }
 
-bool TraceEvent::GetKnownArgAsBool(const std::string& name) const {
+bool TraceEvent::GetKnownArgAsBool(const std::string& arg_name) const {
   double arg_double = 0;
-  bool result = GetArgAsNumber(name, &arg_double);
+  bool result = GetArgAsNumber(arg_name, &arg_double);
   DCHECK(result);
   return (arg_double != 0.0);
 }
 
-std::unique_ptr<base::Value> TraceEvent::GetKnownArgAsValue(
-    const std::string& name) const {
-  std::unique_ptr<base::Value> arg_value;
-  bool result = GetArgAsValue(name, &arg_value);
+base::Value::Dict TraceEvent::GetKnownArgAsDict(
+    const std::string& arg_name) const {
+  base::Value::Dict arg_dict;
+  bool result = GetArgAsDict(arg_name, &arg_dict);
   DCHECK(result);
-  return arg_value;
+  return arg_dict;
 }
 
 // QueryNode
@@ -727,25 +774,21 @@ size_t FindMatchingEvents(const std::vector<TraceEvent>& events,
 
 bool ParseEventsFromJson(const std::string& json,
                          std::vector<TraceEvent>* output) {
-  base::Optional<base::Value> root = base::JSONReader::Read(json);
+  absl::optional<base::Value> root = base::JSONReader::Read(json);
 
   if (!root)
     return false;
 
-  base::Value::ListView list;
+  base::Value::List* list = nullptr;
   if (root->is_list()) {
-    list = root->GetList();
+    list = &root->GetList();
   } else if (root->is_dict()) {
-    base::Value* trace_events = root->FindListKey("traceEvents");
-    if (!trace_events)
-      return false;
-
-    list = trace_events->GetList();
-  } else {
-    return false;
+    list = root->GetDict().FindList("traceEvents");
   }
+  if (!list)
+    return false;
 
-  for (const auto& item : list) {
+  for (const auto& item : *list) {
     TraceEvent event;
     if (!event.SetFromJSON(&item))
       return false;
@@ -765,10 +808,11 @@ TraceAnalyzer::TraceAnalyzer()
 TraceAnalyzer::~TraceAnalyzer() = default;
 
 // static
-TraceAnalyzer* TraceAnalyzer::Create(const std::string& json_events) {
+std::unique_ptr<TraceAnalyzer> TraceAnalyzer::Create(
+    const std::string& json_events) {
   std::unique_ptr<TraceAnalyzer> analyzer(new TraceAnalyzer());
   if (analyzer->SetEvents(json_events))
-    return analyzer.release();
+    return analyzer;
   return nullptr;
 }
 
@@ -797,13 +841,14 @@ void TraceAnalyzer::AssociateBeginEndEvents() {
 void TraceAnalyzer::AssociateAsyncBeginEndEvents(bool match_pid) {
   using trace_analyzer::Query;
 
-  Query begin(
-      Query::EventPhaseIs(TRACE_EVENT_PHASE_ASYNC_BEGIN) ||
-      Query::EventPhaseIs(TRACE_EVENT_PHASE_ASYNC_STEP_INTO) ||
-      Query::EventPhaseIs(TRACE_EVENT_PHASE_ASYNC_STEP_PAST));
+  Query begin(Query::EventPhaseIs(TRACE_EVENT_PHASE_ASYNC_BEGIN) ||
+              Query::EventPhaseIs(TRACE_EVENT_PHASE_ASYNC_STEP_INTO) ||
+              Query::EventPhaseIs(TRACE_EVENT_PHASE_ASYNC_STEP_PAST) ||
+              Query::EventPhaseIs(TRACE_EVENT_PHASE_NESTABLE_ASYNC_BEGIN));
   Query end(Query::EventPhaseIs(TRACE_EVENT_PHASE_ASYNC_END) ||
             Query::EventPhaseIs(TRACE_EVENT_PHASE_ASYNC_STEP_INTO) ||
-            Query::EventPhaseIs(TRACE_EVENT_PHASE_ASYNC_STEP_PAST));
+            Query::EventPhaseIs(TRACE_EVENT_PHASE_ASYNC_STEP_PAST) ||
+            Query::EventPhaseIs(TRACE_EVENT_PHASE_NESTABLE_ASYNC_END));
   Query match(Query::EventCategory() == Query::OtherCategory() &&
               Query::EventId() == Query::OtherId());
 
@@ -938,7 +983,7 @@ std::unique_ptr<TraceAnalyzer> Stop() {
   run_loop.Run();
   buffer.Finish();
 
-  return base::WrapUnique(TraceAnalyzer::Create(trace_output.json_output));
+  return TraceAnalyzer::Create(trace_output.json_output);
 }
 
 // TraceEventVector utility functions.
