@@ -1,4 +1,4 @@
-// Copyright 2020 The Chromium Authors. All rights reserved.
+// Copyright 2020 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,15 +7,18 @@
 #include <memory>
 
 #include "base/barrier_closure.h"
-#include "base/bind.h"
-#include "base/callback.h"
-#include "base/callback_helpers.h"
+#include "base/functional/bind.h"
+#include "base/functional/callback.h"
+#include "base/functional/callback_helpers.h"
+#include "base/memory/raw_ptr.h"
+#include "base/metrics/field_trial_params.h"
 #include "base/run_loop.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/synchronization/lock.h"
 #include "base/synchronization/waitable_event.h"
 #include "base/test/bind.h"
 #include "base/test/metrics/histogram_tester.h"
+#include "base/test/power_monitor_test.h"
 #include "base/test/scoped_feature_list.h"
 #include "base/test/simple_test_tick_clock.h"
 #include "base/test/task_environment.h"
@@ -38,14 +41,16 @@ namespace {
 
 // Use with a FeatureList to activate crash dumping for threads marked as
 // threadpool threads.
-const std::vector<base::test::ScopedFeatureList::FeatureAndParams>
-    kFeatureAndParams{
-        {base::kEnableHangWatcher, {{"ui_thread_log_level", "2"}}}};
+const std::vector<base::test::FeatureRefAndParams> kFeatureAndParams{
+    {base::kEnableHangWatcher, {{"ui_thread_log_level", "2"}}}};
 
 // Use this value to mark things very far off in the future. Adding this
 // to TimeTicks::Now() gives a point that will never be reached during the
 // normal execution of a test.
-constexpr TimeDelta kVeryLongDelta{base::TimeDelta::FromDays(365)};
+constexpr TimeDelta kVeryLongDelta{base::Days(365)};
+
+// A relatively small time delta to ensure ordering of hung threads list.
+constexpr TimeDelta kSmallCPUQuantum{base::Milliseconds(1)};
 
 constexpr uint64_t kArbitraryDeadline = 0x0000C0FFEEC0FFEEu;
 constexpr uint64_t kAllOnes = 0xFFFFFFFFFFFFFFFFu;
@@ -53,6 +58,51 @@ constexpr uint64_t kAllZeros = 0x0000000000000000u;
 constexpr uint64_t kOnesThenZeroes = 0xAAAAAAAAAAAAAAAAu;
 constexpr uint64_t kZeroesThenOnes = 0x5555555555555555u;
 
+#if BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_CHROMEOS)
+class HangWatcherEnabledInZygoteChildTest
+    : public testing::TestWithParam<std::tuple<bool, bool>> {
+ public:
+  HangWatcherEnabledInZygoteChildTest() {
+    std::vector<base::test::FeatureRefAndParams> enabled_features =
+        kFeatureAndParams;
+    std::vector<test::FeatureRef> disabled_features;
+    if (std::get<0>(GetParam())) {
+      enabled_features.push_back(test::FeatureRefAndParams(
+          base::kEnableHangWatcherInZygoteChildren, {}));
+    } else {
+      disabled_features.push_back(base::kEnableHangWatcherInZygoteChildren);
+    }
+    feature_list_.InitWithFeaturesAndParameters(enabled_features,
+                                                disabled_features);
+    HangWatcher::InitializeOnMainThread(
+        HangWatcher::ProcessType::kUtilityProcess,
+        /*is_zygote_child=*/std::get<1>(GetParam()));
+  }
+
+  void TearDown() override { HangWatcher::UnitializeOnMainThreadForTesting(); }
+
+  HangWatcherEnabledInZygoteChildTest(
+      const HangWatcherEnabledInZygoteChildTest& other) = delete;
+  HangWatcherEnabledInZygoteChildTest& operator=(
+      const HangWatcherEnabledInZygoteChildTest& other) = delete;
+
+ protected:
+  base::test::ScopedFeatureList feature_list_;
+};
+
+TEST_P(HangWatcherEnabledInZygoteChildTest, IsEnabled) {
+  // If the kEnableHangWatcherInZygoteChildren feature is disabled and
+  // InitializeOnMainThread is called with is_zygote_child==true, IsEnabled()
+  // should return false. It should return true in all other situations.
+  ASSERT_EQ(std::get<0>(GetParam()) || !std::get<1>(GetParam()),
+            HangWatcher::IsEnabled());
+}
+
+INSTANTIATE_TEST_SUITE_P(HangWatcherZygoteTest,
+                         HangWatcherEnabledInZygoteChildTest,
+                         testing::Combine(testing::Bool(), testing::Bool()));
+#endif  // BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_CHROMEOS)
+
 // Waits on provided WaitableEvent before executing and signals when done.
 class BlockingThread : public DelegateSimpleThread::Delegate {
  public:
@@ -69,7 +119,7 @@ class BlockingThread : public DelegateSimpleThread::Delegate {
     // happens on the right thread.
     base::ScopedClosureRunner unregister_closure =
         base::HangWatcher::RegisterThread(
-            base::HangWatcher::ThreadType::kUIThread);
+            base::HangWatcher::ThreadType::kMainThread);
 
     WatchHangsInScope scope(timeout_);
     wait_until_entered_scope_.Signal();
@@ -101,19 +151,20 @@ class BlockingThread : public DelegateSimpleThread::Delegate {
   // Will be signaled once ThreadMain has run.
   WaitableEvent run_event_;
 
-  base::WaitableEvent* const unblock_thread_;
+  const raw_ptr<base::WaitableEvent> unblock_thread_;
 
   base::TimeDelta timeout_;
 };
 
 class HangWatcherTest : public testing::Test {
  public:
-  const base::TimeDelta kTimeout = base::TimeDelta::FromSeconds(10);
-  const base::TimeDelta kHangTime = kTimeout + base::TimeDelta::FromSeconds(1);
+  const base::TimeDelta kTimeout = base::Seconds(10);
+  const base::TimeDelta kHangTime = kTimeout + base::Seconds(1);
 
   HangWatcherTest() {
     feature_list_.InitWithFeaturesAndParameters(kFeatureAndParams, {});
-    hang_watcher_.InitializeOnMainThread();
+    HangWatcher::InitializeOnMainThread(
+        HangWatcher::ProcessType::kBrowserProcess, false);
 
     hang_watcher_.SetAfterMonitorClosureForTesting(base::BindRepeating(
         &WaitableEvent::Signal, base::Unretained(&monitor_event_)));
@@ -129,7 +180,7 @@ class HangWatcherTest : public testing::Test {
     hang_watcher_.Start();
   }
 
-  void TearDown() override { hang_watcher_.UnitializeOnMainThreadForTesting(); }
+  void TearDown() override { HangWatcher::UnitializeOnMainThreadForTesting(); }
 
   HangWatcherTest(const HangWatcherTest& other) = delete;
   HangWatcherTest& operator=(const HangWatcherTest& other) = delete;
@@ -209,7 +260,7 @@ class HangWatcherBlockingThreadTest : public HangWatcherTest {
 TEST_F(HangWatcherTest, InvalidatingExpectationsPreventsCapture) {
   // Register the main test thread for hang watching.
   auto unregister_thread_closure =
-      HangWatcher::RegisterThread(base::HangWatcher::ThreadType::kUIThread);
+      HangWatcher::RegisterThread(base::HangWatcher::ThreadType::kMainThread);
 
   // Create a hang.
   WatchHangsInScope expires_instantly(base::TimeDelta{});
@@ -228,7 +279,7 @@ TEST_F(HangWatcherTest, InvalidatingExpectationsPreventsCapture) {
 TEST_F(HangWatcherTest, MultipleInvalidateExpectationsDoNotCancelOut) {
   // Register the main test thread for hang watching.
   auto unregister_thread_closure =
-      HangWatcher::RegisterThread(base::HangWatcher::ThreadType::kUIThread);
+      HangWatcher::RegisterThread(base::HangWatcher::ThreadType::kMainThread);
 
   // Create a hang.
   WatchHangsInScope expires_instantly(base::TimeDelta{});
@@ -250,7 +301,7 @@ TEST_F(HangWatcherTest, MultipleInvalidateExpectationsDoNotCancelOut) {
 TEST_F(HangWatcherTest, NewInnerWatchHangsInScopeAfterInvalidationDetectsHang) {
   // Register the main test thread for hang watching.
   auto unregister_thread_closure =
-      HangWatcher::RegisterThread(base::HangWatcher::ThreadType::kUIThread);
+      HangWatcher::RegisterThread(base::HangWatcher::ThreadType::kMainThread);
 
   WatchHangsInScope expires_instantly(base::TimeDelta{});
   task_environment_.FastForwardBy(kHangTime);
@@ -290,7 +341,7 @@ TEST_F(HangWatcherTest,
        NewSeparateWatchHangsInScopeAfterInvalidationDetectsHang) {
   // Register the main test thread for hang watching.
   auto unregister_thread_closure =
-      HangWatcher::RegisterThread(base::HangWatcher::ThreadType::kUIThread);
+      HangWatcher::RegisterThread(base::HangWatcher::ThreadType::kMainThread);
 
   {
     WatchHangsInScope expires_instantly(base::TimeDelta{});
@@ -318,7 +369,7 @@ TEST_F(HangWatcherTest,
 TEST_F(HangWatcherTest, ScopeDisabledObjectInnerScope) {
   // Register the main test thread for hang watching.
   auto unregister_thread_closure =
-      HangWatcher::RegisterThread(base::HangWatcher::ThreadType::kUIThread);
+      HangWatcher::RegisterThread(base::HangWatcher::ThreadType::kMainThread);
 
   // Start a WatchHangsInScope that expires right away. Then advance
   // time to make sure no hang is detected.
@@ -344,7 +395,7 @@ TEST_F(HangWatcherTest, ScopeDisabledObjectInnerScope) {
 TEST_F(HangWatcherTest, NewScopeAfterDisabling) {
   // Register the main test thread for hang watching.
   auto unregister_thread_closure =
-      HangWatcher::RegisterThread(base::HangWatcher::ThreadType::kUIThread);
+      HangWatcher::RegisterThread(base::HangWatcher::ThreadType::kMainThread);
 
   // Start a WatchHangsInScope that expires right away. Then advance
   // time to make sure no hang is detected.
@@ -375,17 +426,15 @@ TEST_F(HangWatcherTest, NestedScopes) {
   // threaded.
   auto current_hang_watch_state =
       base::internal::HangWatchState::CreateHangWatchStateForCurrentThread(
-          HangWatcher::ThreadType::kUIThread);
+          HangWatcher::ThreadType::kMainThread);
 
   ASSERT_FALSE(current_hang_watch_state->IsOverDeadline());
   base::TimeTicks original_deadline = current_hang_watch_state->GetDeadline();
 
-  constexpr base::TimeDelta kFirstTimeout(
-      base::TimeDelta::FromMilliseconds(500));
+  constexpr base::TimeDelta kFirstTimeout(base::Milliseconds(500));
   base::TimeTicks first_deadline = base::TimeTicks::Now() + kFirstTimeout;
 
-  constexpr base::TimeDelta kSecondTimeout(
-      base::TimeDelta::FromMilliseconds(250));
+  constexpr base::TimeDelta kSecondTimeout(base::Milliseconds(250));
   base::TimeTicks second_deadline = base::TimeTicks::Now() + kSecondTimeout;
 
   // At this point we have not set any timeouts.
@@ -512,11 +561,17 @@ namespace {
 class HangWatcherSnapshotTest : public testing::Test {
  public:
   void SetUp() override {
+    feature_list_.InitWithFeaturesAndParameters(kFeatureAndParams, {});
+    HangWatcher::InitializeOnMainThread(
+        HangWatcher::ProcessType::kBrowserProcess, false);
+
     // The monitoring loop behavior is not verified in this test so we want to
     // trigger monitoring manually.
     hang_watcher_.SetMonitoringPeriodForTesting(kVeryLongDelta);
   }
 
+  void TearDown() override { HangWatcher::UnitializeOnMainThreadForTesting(); }
+
   HangWatcherSnapshotTest() = default;
   HangWatcherSnapshotTest(const HangWatcherSnapshotTest& other) = delete;
   HangWatcherSnapshotTest& operator=(const HangWatcherSnapshotTest& other) =
@@ -533,6 +588,7 @@ class HangWatcherSnapshotTest : public testing::Test {
   // list of hung thread ids is correct.
   void TestIDList(const std::string& id_list) {
     list_of_hung_thread_ids_during_capture_ = id_list;
+    task_environment_.AdvanceClock(kSmallCPUQuantum);
     TriggerMonitorAndWaitForCompletion();
     ASSERT_EQ(++reference_capture_count_, hang_capture_count_);
   }
@@ -540,6 +596,7 @@ class HangWatcherSnapshotTest : public testing::Test {
   // Verify that even if hang monitoring takes place no hangs are detected.
   void ExpectNoCapture() {
     int old_capture_count = hang_capture_count_;
+    task_environment_.AdvanceClock(kSmallCPUQuantum);
     TriggerMonitorAndWaitForCompletion();
     ASSERT_EQ(old_capture_count, hang_capture_count_);
   }
@@ -576,6 +633,14 @@ class HangWatcherSnapshotTest : public testing::Test {
   // actually took place.
   int reference_capture_count_ = 0;
 
+  std::string seconds_since_last_power_resume_crash_key_;
+
+  base::test::ScopedFeatureList feature_list_;
+
+  // Used exclusively for MOCK_TIME.
+  test::SingleThreadTaskEnvironment task_environment_{
+      test::TaskEnvironment::TimeSource::MOCK_TIME};
+
   HangWatcher hang_watcher_;
 };
 }  // namespace
@@ -594,14 +659,14 @@ TEST_F(HangWatcherSnapshotTest, NonActionableReport) {
 
   // Register the main test thread for hang watching.
   auto unregister_thread_closure =
-      HangWatcher::RegisterThread(base::HangWatcher::ThreadType::kUIThread);
+      HangWatcher::RegisterThread(base::HangWatcher::ThreadType::kMainThread);
   {
     // Start a WatchHangsInScope that expires right away. Ensures that
     // the first monitor will detect a hang.
     WatchHangsInScope expires_instantly(base::TimeDelta{});
 
     internal::HangWatchState* current_hang_watch_state =
-        internal::HangWatchState::GetHangWatchStateForCurrentThread()->Get();
+        internal::HangWatchState::GetHangWatchStateForCurrentThread();
 
     // Simulate the deadline changing concurrently during the capture. This
     // makes the capture fail since marking of the deadline fails.
@@ -622,8 +687,16 @@ TEST_F(HangWatcherSnapshotTest, NonActionableReport) {
   }
 }
 
-// Disabled for being flaky. crbug.com/1078828
-TEST_F(HangWatcherSnapshotTest, DISABLED_HungThreadIDs) {
+// TODO(crbug.com/1223033): On MAC, the base::PlatformThread::CurrentId(...)
+// should return the system wide IDs. The HungThreadIDs test fails because the
+// reported process ids do not match.
+#if BUILDFLAG(IS_MAC)
+#define MAYBE_HungThreadIDs DISABLED_HungThreadIDs
+#else
+#define MAYBE_HungThreadIDs HungThreadIDs
+#endif
+
+TEST_F(HangWatcherSnapshotTest, MAYBE_HungThreadIDs) {
   // During hang capture the list of hung threads should be populated.
   hang_watcher_.SetOnHangClosureForTesting(base::BindLambdaForTesting([this]() {
     EXPECT_EQ(hang_watcher_.GrabWatchStateSnapshotForTesting()
@@ -642,11 +715,15 @@ TEST_F(HangWatcherSnapshotTest, DISABLED_HungThreadIDs) {
 
   // Register the main test thread for hang watching.
   auto unregister_thread_closure =
-      HangWatcher::RegisterThread(base::HangWatcher::ThreadType::kUIThread);
+      HangWatcher::RegisterThread(base::HangWatcher::ThreadType::kMainThread);
 
   BlockingThread blocking_thread(&monitor_event_, base::TimeDelta{});
   blocking_thread.StartAndWaitForScopeEntered();
   {
+    // Ensure the blocking thread entered the scope before the main thread. This
+    // will guarantee an ordering while reporting the list of hung threads.
+    task_environment_.AdvanceClock(kSmallCPUQuantum);
+
     // Start a WatchHangsInScope that expires right away. Ensures that
     // the first monitor will detect a hang. This scope will naturally have a
     // later deadline than the one in |blocking_thread_| since it was created
@@ -681,18 +758,75 @@ TEST_F(HangWatcherSnapshotTest, DISABLED_HungThreadIDs) {
   TestIDList(ConcatenateThreadIds({test_thread_id_}));
 }
 
+TEST_F(HangWatcherSnapshotTest, TimeSinceLastSystemPowerResumeCrashKey) {
+  // Override the capture of hangs. Simulate a crash key capture.
+  hang_watcher_.SetOnHangClosureForTesting(base::BindLambdaForTesting([this]() {
+    ++hang_capture_count_;
+    seconds_since_last_power_resume_crash_key_ =
+        hang_watcher_.GetTimeSinceLastSystemPowerResumeCrashKeyValue();
+  }));
+
+  // When hang capture is over, unblock the main thread.
+  hang_watcher_.SetAfterMonitorClosureForTesting(
+      base::BindLambdaForTesting([this]() { monitor_event_.Signal(); }));
+
+  hang_watcher_.Start();
+
+  // Register the main test thread for hang watching.
+  auto unregister_thread_closure =
+      HangWatcher::RegisterThread(base::HangWatcher::ThreadType::kMainThread);
+
+  {
+    WatchHangsInScope expires_instantly(base::TimeDelta{});
+    task_environment_.AdvanceClock(kSmallCPUQuantum);
+
+    TriggerMonitorAndWaitForCompletion();
+    EXPECT_EQ(1, hang_capture_count_);
+    EXPECT_EQ("Never suspended", seconds_since_last_power_resume_crash_key_);
+  }
+
+  {
+    test::ScopedPowerMonitorTestSource power_monitor_source;
+    power_monitor_source.Suspend();
+    task_environment_.AdvanceClock(kSmallCPUQuantum);
+
+    {
+      WatchHangsInScope expires_instantly(base::TimeDelta{});
+      task_environment_.AdvanceClock(kSmallCPUQuantum);
+      TriggerMonitorAndWaitForCompletion();
+      EXPECT_EQ(2, hang_capture_count_);
+      EXPECT_EQ("Power suspended", seconds_since_last_power_resume_crash_key_);
+    }
+
+    power_monitor_source.Resume();
+    constexpr TimeDelta kAfterResumeTime{base::Seconds(5)};
+    task_environment_.AdvanceClock(kAfterResumeTime);
+
+    {
+      WatchHangsInScope expires_instantly(base::TimeDelta{});
+      TriggerMonitorAndWaitForCompletion();
+      EXPECT_EQ(3, hang_capture_count_);
+      EXPECT_EQ(base::NumberToString(kAfterResumeTime.InSeconds()),
+                seconds_since_last_power_resume_crash_key_);
+    }
+  }
+}
+
 namespace {
 
 // Determines how long the HangWatcher will wait between calls to
 // Monitor(). Choose a low value so that that successive invocations happens
 // fast. This makes tests that wait for monitoring run fast and makes tests that
 // expect no monitoring fail fast.
-const base::TimeDelta kMonitoringPeriod = base::TimeDelta::FromMilliseconds(1);
+const base::TimeDelta kMonitoringPeriod = base::Milliseconds(1);
 
 // Test if and how often the HangWatcher periodically monitors for hangs.
 class HangWatcherPeriodicMonitoringTest : public testing::Test {
  public:
   HangWatcherPeriodicMonitoringTest() {
+    hang_watcher_.InitializeOnMainThread(
+        HangWatcher::ProcessType::kBrowserProcess, false);
+
     hang_watcher_.SetMonitoringPeriodForTesting(kMonitoringPeriod);
     hang_watcher_.SetOnHangClosureForTesting(base::BindRepeating(
         &WaitableEvent::Signal, base::Unretained(&hang_event_)));
@@ -708,6 +842,8 @@ class HangWatcherPeriodicMonitoringTest : public testing::Test {
   HangWatcherPeriodicMonitoringTest& operator=(
       const HangWatcherPeriodicMonitoringTest& other) = delete;
 
+  void TearDown() override { hang_watcher_.UnitializeOnMainThreadForTesting(); }
+
  protected:
   // Setup the callback invoked after waiting in HangWatcher to advance the
   // tick clock by the desired time delta.
@@ -780,9 +916,9 @@ TEST_F(HangWatcherPeriodicMonitoringTest, PeriodicCallsTakePlace) {
   // Setup the HangWatcher to unblock run_loop when the Monitor() has been
   // invoked enough times.
   hang_watcher_.SetAfterMonitorClosureForTesting(BarrierClosure(
-      kMinimumMonitorCount, base::BindLambdaForTesting([&run_loop, this]() {
+      kMinimumMonitorCount, base::BindLambdaForTesting([&run_loop]() {
         // Test condition are confirmed, stop monitoring.
-        hang_watcher_.StopMonitoringForTesting();
+        HangWatcher::StopMonitoringForTesting();
 
         // Unblock the test main thread.
         run_loop.Quit();
@@ -797,7 +933,7 @@ TEST_F(HangWatcherPeriodicMonitoringTest, PeriodicCallsTakePlace) {
 
   // Register a thread,
   unregister_thread_closure_ =
-      HangWatcher::RegisterThread(base::HangWatcher::ThreadType::kUIThread);
+      HangWatcher::RegisterThread(base::HangWatcher::ThreadType::kMainThread);
 
   run_loop.Run();
 
@@ -820,13 +956,13 @@ TEST_F(HangWatcherPeriodicMonitoringTest, NoMonitorOnOverSleep) {
 
   // Make the HangWatcher tick clock advance so much after waiting that it will
   // detect oversleeping every time. This will keep it from monitoring.
-  InstallAfterWaitCallback(base::TimeDelta::FromMinutes(1));
+  InstallAfterWaitCallback(base::Minutes(1));
 
   hang_watcher_.Start();
 
   // Register a thread.
   unregister_thread_closure_ =
-      HangWatcher::RegisterThread(base::HangWatcher::ThreadType::kUIThread);
+      HangWatcher::RegisterThread(base::HangWatcher::ThreadType::kMainThread);
 
   // Unblock the test thread. All waits were perceived as oversleeping so all
   // monitoring was inhibited.
@@ -845,12 +981,13 @@ class WatchHangsInScopeBlockingTest : public testing::Test {
  public:
   WatchHangsInScopeBlockingTest() {
     feature_list_.InitWithFeaturesAndParameters(kFeatureAndParams, {});
-    hang_watcher_.InitializeOnMainThread();
+    HangWatcher::InitializeOnMainThread(
+        HangWatcher::ProcessType::kBrowserProcess, false);
 
     hang_watcher_.SetOnHangClosureForTesting(base::BindLambdaForTesting([&] {
       capture_started_.Signal();
       // Simulate capturing that takes a long time.
-      PlatformThread::Sleep(base::TimeDelta::FromMilliseconds(500));
+      PlatformThread::Sleep(base::Milliseconds(500));
 
       continue_capture_.Wait();
       completed_capture_ = true;
@@ -859,7 +996,7 @@ class WatchHangsInScopeBlockingTest : public testing::Test {
     hang_watcher_.SetAfterMonitorClosureForTesting(
         base::BindLambdaForTesting([&] {
           // Simulate monitoring that takes a long time.
-          PlatformThread::Sleep(base::TimeDelta::FromMilliseconds(500));
+          PlatformThread::Sleep(base::Milliseconds(500));
           completed_monitoring_.Signal();
         }));
 
@@ -870,10 +1007,10 @@ class WatchHangsInScopeBlockingTest : public testing::Test {
 
     // Register the test main thread for hang watching.
     unregister_thread_closure_ =
-        HangWatcher::RegisterThread(base::HangWatcher::ThreadType::kUIThread);
+        HangWatcher::RegisterThread(base::HangWatcher::ThreadType::kMainThread);
   }
 
-  void TearDown() override { hang_watcher_.UnitializeOnMainThreadForTesting(); }
+  void TearDown() override { HangWatcher::UnitializeOnMainThreadForTesting(); }
 
   WatchHangsInScopeBlockingTest(const WatchHangsInScopeBlockingTest& other) =
       delete;
@@ -965,7 +1102,7 @@ TEST_F(WatchHangsInScopeBlockingTest, ScopeBlocksDuringCapture) {
   VerifyScopesDontBlock();
 }
 
-#if defined(OS_MAC) && defined(ARCH_CPU_ARM64)
+#if BUILDFLAG(IS_MAC) && defined(ARCH_CPU_ARM64)
 // Flaky hangs on arm64 Macs: https://crbug.com/1140207
 #define MAYBE_NewScopeDoesNotBlockDuringCapture \
   DISABLED_NewScopeDoesNotBlockDuringCapture
@@ -1038,17 +1175,13 @@ TEST_F(HangWatchDeadlineTest, SetAndClearPersistentFlag) {
   AssertNoFlagsSet();
 
   // Grab the original values for flags and deadline.
-  uint64_t old_flags;
-  base::TimeTicks old_deadline;
-  std::tie(old_flags, old_deadline) = deadline_.GetFlagsAndDeadline();
+  auto [old_flags, old_deadline] = deadline_.GetFlagsAndDeadline();
 
   // Set the flag. Operation cannot fail.
   deadline_.SetIgnoreCurrentWatchHangsInScope();
 
   // Get new flags and deadline.
-  uint64_t new_flags;
-  base::TimeTicks new_deadline;
-  std::tie(new_flags, new_deadline) = deadline_.GetFlagsAndDeadline();
+  auto [new_flags, new_deadline] = deadline_.GetFlagsAndDeadline();
 
   // Flag was set properly.
   ASSERT_TRUE(HangWatchDeadline::IsFlagSet(
@@ -1097,9 +1230,7 @@ TEST_F(HangWatchDeadlineTest, SetDeadline) {
 TEST_F(HangWatchDeadlineTest, SetShouldBlockOnHangDeadlineChanged) {
   AssertNoFlagsSet();
 
-  uint64_t flags;
-  base::TimeTicks deadline;
-  std::tie(flags, deadline) = deadline_.GetFlagsAndDeadline();
+  auto [flags, deadline] = deadline_.GetFlagsAndDeadline();
 
   // Simulate value change. Flags are constant.
   const base::TimeTicks new_deadline =
@@ -1124,9 +1255,7 @@ TEST_F(HangWatchDeadlineTest, SetShouldBlockOnHangDeadlineChanged) {
 TEST_F(HangWatchDeadlineTest, ClearIgnoreHangsDeadlineChanged) {
   AssertNoFlagsSet();
 
-  uint64_t flags;
-  base::TimeTicks deadline;
-  std::tie(flags, deadline) = deadline_.GetFlagsAndDeadline();
+  auto [flags, deadline] = deadline_.GetFlagsAndDeadline();
 
   deadline_.SetIgnoreCurrentWatchHangsInScope();
   std::tie(flags, deadline) = deadline_.GetFlagsAndDeadline();
@@ -1142,7 +1271,7 @@ TEST_F(HangWatchDeadlineTest, ClearIgnoreHangsDeadlineChanged) {
            kArbitraryDeadline;
   }));
 
-  // Clearing kIgnoreHang is unafected by deadline or flags change.
+  // Clearing kIgnoreHang is unaffected by deadline or flags change.
   deadline_.UnsetIgnoreCurrentWatchHangsInScope();
   ASSERT_FALSE(deadline_.IsFlagSet(
       HangWatchDeadline::Flag::kIgnoreCurrentWatchHangsInScope));
@@ -1158,9 +1287,7 @@ TEST_F(HangWatchDeadlineTest,
        SetIgnoreCurrentHangWatchScopeEnableDeadlineChangedd) {
   AssertNoFlagsSet();
 
-  uint64_t flags;
-  base::TimeTicks deadline;
-  std::tie(flags, deadline) = deadline_.GetFlagsAndDeadline();
+  auto [flags, deadline] = deadline_.GetFlagsAndDeadline();
 
   // Simulate deadline change. Flags are constant.
   const base::TimeTicks new_deadline =
@@ -1185,9 +1312,7 @@ TEST_F(HangWatchDeadlineTest,
 // Setting a new deadline should wipe flags that a not persistent.
 // Persistent flags should not be disturbed.
 TEST_F(HangWatchDeadlineTest, SetDeadlineWipesFlags) {
-  uint64_t flags;
-  base::TimeTicks deadline;
-  std::tie(flags, deadline) = deadline_.GetFlagsAndDeadline();
+  auto [flags, deadline] = deadline_.GetFlagsAndDeadline();
 
   ASSERT_TRUE(deadline_.SetShouldBlockOnHang(flags, deadline));
   ASSERT_TRUE(deadline_.IsFlagSet(HangWatchDeadline::Flag::kShouldBlockOnHang));
