@@ -1,4 +1,4 @@
-// Copyright 2014 The Chromium Authors. All rights reserved.
+// Copyright 2014 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -19,18 +19,21 @@
 #include "cc/paint/paint_record.h"
 #include "cc/paint/render_surface_filters.h"
 #include "cc/paint/skia_paint_canvas.h"
-#include "cc/test/geometry_test_utils.h"
 #include "cc/test/pixel_test_utils.h"
 #include "cc/test/skia_common.h"
 #include "cc/test/test_skcanvas.h"
+#include "skia/ext/font_utils.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "third_party/skia/include/core/SkBitmap.h"
 #include "third_party/skia/include/core/SkColor.h"
+#include "third_party/skia/include/core/SkFont.h"
 #include "third_party/skia/include/core/SkSurface.h"
+#include "third_party/skia/include/core/SkTextBlob.h"
 #include "ui/gfx/geometry/rect.h"
 #include "ui/gfx/geometry/rect_conversions.h"
-#include "ui/gfx/skia_util.h"
+#include "ui/gfx/geometry/skia_conversions.h"
+#include "ui/gfx/geometry/transform.h"
 
 namespace cc {
 
@@ -76,16 +79,11 @@ class DisplayItemListTest : public testing::Test {
 
 #define EXPECT_TRACED_RECT(x, y, width, height, rect_list) \
   do {                                                     \
-    ASSERT_EQ(4u, rect_list->GetSize());                   \
-    double d;                                              \
-    EXPECT_TRUE((rect_list)->GetDouble(0, &d));            \
-    EXPECT_EQ(x, d);                                       \
-    EXPECT_TRUE((rect_list)->GetDouble(1, &d));            \
-    EXPECT_EQ(y, d);                                       \
-    EXPECT_TRUE((rect_list)->GetDouble(2, &d));            \
-    EXPECT_EQ(width, d);                                   \
-    EXPECT_TRUE((rect_list)->GetDouble(3, &d));            \
-    EXPECT_EQ(height, d);                                  \
+    ASSERT_EQ(4u, rect_list->size());                      \
+    EXPECT_EQ(x, (*rect_list)[0].GetIfDouble());           \
+    EXPECT_EQ(y, (*rect_list)[1].GetIfDouble());           \
+    EXPECT_EQ(width, (*rect_list)[2].GetIfDouble());       \
+    EXPECT_EQ(height, (*rect_list)[3].GetIfDouble());      \
   } while (false)
 
 // AddToValue should not crash if there are different numbers of visual_rect
@@ -110,29 +108,35 @@ TEST_F(DisplayItemListTest, TraceEmptyVisualRect) {
   // Pass: we don't crash
   std::unique_ptr<base::Value> root = ToBaseValue(list.get(), true);
 
-  const base::DictionaryValue* root_dict;
-  ASSERT_TRUE(root->GetAsDictionary(&root_dict));
-  const base::DictionaryValue* params_dict;
-  ASSERT_TRUE(root_dict->GetDictionary("params", &params_dict));
-  const base::ListValue* items;
-  ASSERT_TRUE(params_dict->GetList("items", &items));
-  ASSERT_EQ(2u, items->GetSize());
+  const base::Value::Dict* root_dict = root->GetIfDict();
+  ASSERT_NE(nullptr, root_dict);
+  const base::Value::Dict* params_dict = root_dict->FindDict("params");
+  ASSERT_NE(nullptr, params_dict);
+  const base::Value::List* items = params_dict->FindList("items");
+  ASSERT_NE(nullptr, items);
+  ASSERT_EQ(2u, items->size());
 
-  const base::DictionaryValue* item_dict;
-  const base::ListValue* visual_rect;
-  std::string name;
+  const base::Value::Dict* item_dict;
+  const base::Value::List* visual_rect;
+  const std::string* name;
 
-  ASSERT_TRUE(items->GetDictionary(0, &item_dict));
-  ASSERT_TRUE(item_dict->GetList("visual_rect", &visual_rect));
+  item_dict = ((*items)[0]).GetIfDict();
+  ASSERT_NE(nullptr, item_dict);
+  visual_rect = item_dict->FindList("visual_rect");
+  ASSERT_NE(nullptr, visual_rect);
   EXPECT_TRACED_RECT(0, 0, 0, 0, visual_rect);
-  EXPECT_TRUE(item_dict->GetString("name", &name));
-  EXPECT_EQ("DrawRect", name);
+  name = item_dict->FindString("name");
+  ASSERT_NE(nullptr, name);
+  EXPECT_EQ("DrawRectOp", *name);
 
-  ASSERT_TRUE(items->GetDictionary(1, &item_dict));
-  ASSERT_TRUE(item_dict->GetList("visual_rect", &visual_rect));
+  item_dict = ((*items)[1]).GetIfDict();
+  ASSERT_NE(nullptr, item_dict);
+  visual_rect = item_dict->FindList("visual_rect");
+  ASSERT_NE(nullptr, visual_rect);
   EXPECT_TRACED_RECT(8, 9, 10, 10, visual_rect);
-  EXPECT_TRUE(item_dict->GetString("name", &name));
-  EXPECT_EQ("DrawRect", name);
+  name = item_dict->FindString("name");
+  ASSERT_NE(nullptr, name);
+  EXPECT_EQ("DrawRectOp", *name);
 }
 
 TEST_F(DisplayItemListTest, SingleUnpairedRange) {
@@ -297,7 +301,7 @@ TEST_F(DisplayItemListTest, TransformPairedRange) {
   {
     list->StartPaint();
     list->push<SaveOp>();
-    list->push<ConcatOp>(transform.GetMatrixAsSkM44());
+    list->push<ConcatOp>(gfx::TransformToSkM44(transform));
     list->EndPaintOfPairedBegin();
   }
 
@@ -334,7 +338,7 @@ TEST_F(DisplayItemListTest, TransformPairedRange) {
       SkRect::MakeLTRB(0.f + first_offset.x(), 0.f + first_offset.y(),
                        60.f + first_offset.x(), 60.f + first_offset.y()),
       red_paint);
-  expected_canvas.setMatrix(SkMatrix(transform.matrix()));
+  expected_canvas.setMatrix(gfx::TransformToSkM44(transform));
   expected_canvas.drawRect(
       SkRect::MakeLTRB(50.f + second_offset.x(), 50.f + second_offset.y(),
                        75.f + second_offset.x(), 75.f + second_offset.y()),
@@ -349,7 +353,8 @@ TEST_F(DisplayItemListTest, FilterPairedRange) {
   unsigned char pixels[4 * 100 * 100] = {0};
   auto list = base::MakeRefCounted<DisplayItemList>();
 
-  sk_sp<SkSurface> source_surface = SkSurface::MakeRasterN32Premul(50, 50);
+  sk_sp<SkSurface> source_surface =
+      SkSurfaces::Raster(SkImageInfo::MakeN32Premul(50, 50));
   SkCanvas* source_canvas = source_surface->getCanvas();
   source_canvas->clear(SkColorSetRGB(128, 128, 128));
   PaintImage source_image = PaintImageBuilder::WithDefault()
@@ -371,7 +376,7 @@ TEST_F(DisplayItemListTest, FilterPairedRange) {
   // below.
   SkRect rect = SkRect::MakeWH(source_image.width(), source_image.height());
   sk_sp<PaintFilter> image_filter = sk_make_sp<ImagePaintFilter>(
-      source_image, rect, rect, kHigh_SkFilterQuality);
+      source_image, rect, rect, PaintFlags::FilterQuality::kHigh);
   filters.Append(FilterOperation::CreateReferenceFilter(image_filter));
   filters.Append(FilterOperation::CreateBrightnessFilter(0.5f));
   gfx::RectF filter_bounds(10.f, 10.f, 50.f, 50.f);
@@ -381,12 +386,11 @@ TEST_F(DisplayItemListTest, FilterPairedRange) {
     list->push<TranslateOp>(filter_bounds.x(), filter_bounds.y());
 
     PaintFlags flags;
-    flags.setImageFilter(
-        RenderSurfaceFilters::BuildImageFilter(filters, filter_bounds.size()));
+    flags.setImageFilter(RenderSurfaceFilters::BuildImageFilter(filters));
 
     SkRect layer_bounds = gfx::RectFToSkRect(filter_bounds);
     layer_bounds.offset(-filter_bounds.x(), -filter_bounds.y());
-    list->push<SaveLayerOp>(&layer_bounds, &flags);
+    list->push<SaveLayerOp>(layer_bounds, flags);
     list->push<TranslateOp>(-filter_bounds.x(), -filter_bounds.y());
 
     list->EndPaintOfPairedBegin();
@@ -459,45 +463,50 @@ TEST_F(DisplayItemListTest, AsValueWithNoOps) {
 
   // Pass |true| to ask for PaintOps even though there are none.
   std::unique_ptr<base::Value> root = ToBaseValue(list.get(), true);
-  const base::DictionaryValue* root_dict;
-  ASSERT_TRUE(root->GetAsDictionary(&root_dict));
+  const base::Value::Dict* root_dict = root->GetIfDict();
+  ASSERT_NE(nullptr, root_dict);
   // The traced value has a params dictionary as its root.
   {
-    const base::DictionaryValue* params_dict;
-    ASSERT_TRUE(root_dict->GetDictionary("params", &params_dict));
+    const base::Value::Dict* params_dict = root_dict->FindDict("params");
+    ASSERT_NE(nullptr, params_dict);
 
     // The real contents of the traced value is in here.
     {
-      const base::ListValue* params_list;
+      const base::Value::List* params_list;
 
       // The layer_rect field is present by empty.
-      ASSERT_TRUE(params_dict->GetList("layer_rect", &params_list));
+      params_list = params_dict->FindList("layer_rect");
+      ASSERT_NE(nullptr, params_list);
       EXPECT_TRACED_RECT(0, 0, 0, 0, params_list);
 
       // The items list is there but empty.
-      ASSERT_TRUE(params_dict->GetList("items", &params_list));
-      EXPECT_EQ(0u, params_list->GetSize());
+      params_list = params_dict->FindList("items");
+      ASSERT_NE(nullptr, params_list);
+      EXPECT_EQ(0u, params_list->size());
     }
   }
 
   // Pass |false| to not include PaintOps.
   root = ToBaseValue(list.get(), false);
-  ASSERT_TRUE(root->GetAsDictionary(&root_dict));
+  root_dict = root->GetIfDict();
+  ASSERT_NE(nullptr, root_dict);
   // The traced value has a params dictionary as its root.
   {
-    const base::DictionaryValue* params_dict;
-    ASSERT_TRUE(root_dict->GetDictionary("params", &params_dict));
+    const base::Value::Dict* params_dict = root_dict->FindDict("params");
+    ASSERT_NE(nullptr, params_dict);
 
     // The real contents of the traced value is in here.
     {
-      const base::ListValue* params_list;
+      const base::Value::List* params_list;
 
       // The layer_rect field is present by empty.
-      ASSERT_TRUE(params_dict->GetList("layer_rect", &params_list));
+      params_list = params_dict->FindList("layer_rect");
+      ASSERT_NE(nullptr, params_list);
       EXPECT_TRACED_RECT(0, 0, 0, 0, params_list);
 
       // The items list is not there since we asked for no ops.
-      ASSERT_FALSE(params_dict->GetList("items", &params_list));
+      params_list = params_dict->FindList("items");
+      ASSERT_EQ(nullptr, params_list);
     }
   }
 }
@@ -511,7 +520,7 @@ TEST_F(DisplayItemListTest, AsValueWithOps) {
   {
     list->StartPaint();
     list->push<SaveOp>();
-    list->push<ConcatOp>(transform.GetMatrixAsSkM44());
+    list->push<ConcatOp>(gfx::TransformToSkM44(transform));
     list->EndPaintOfPairedBegin();
   }
 
@@ -523,7 +532,7 @@ TEST_F(DisplayItemListTest, AsValueWithOps) {
     PaintFlags red_paint;
     red_paint.setColor(SK_ColorRED);
 
-    list->push<SaveLayerOp>(nullptr, &red_paint);
+    list->push<SaveLayerOp>(red_paint);
     list->push<TranslateOp>(static_cast<float>(offset.x()),
                             static_cast<float>(offset.y()));
     list->push<DrawRectOp>(SkRect::MakeWH(4, 4), red_paint);
@@ -541,66 +550,71 @@ TEST_F(DisplayItemListTest, AsValueWithOps) {
 
   // Pass |true| to ask for PaintOps to be included.
   std::unique_ptr<base::Value> root = ToBaseValue(list.get(), true);
-  const base::DictionaryValue* root_dict;
-  ASSERT_TRUE(root->GetAsDictionary(&root_dict));
+  const base::Value::Dict* root_dict = root->GetIfDict();
+  ASSERT_NE(nullptr, root_dict);
   // The traced value has a params dictionary as its root.
   {
-    const base::DictionaryValue* params_dict;
-    ASSERT_TRUE(root_dict->GetDictionary("params", &params_dict));
+    const base::Value::Dict* params_dict = root_dict->FindDict("params");
+    ASSERT_NE(nullptr, params_dict);
 
     // The real contents of the traced value is in here.
     {
-      const base::ListValue* layer_rect_list;
+      const base::Value::List* layer_rect_list =
+          params_dict->FindList("layer_rect");
       // The layer_rect field is present and has the bounds of the rtree.
-      ASSERT_TRUE(params_dict->GetList("layer_rect", &layer_rect_list));
+      ASSERT_NE(nullptr, layer_rect_list);
       EXPECT_TRACED_RECT(2, 3, 8, 9, layer_rect_list);
 
       // The items list has 3 things in it since we built 3 visual rects.
-      const base::ListValue* items;
-      ASSERT_TRUE(params_dict->GetList("items", &items));
-      ASSERT_EQ(7u, items->GetSize());
+      const base::Value::List* items = params_dict->FindList("items");
+      ASSERT_NE(nullptr, items);
+      ASSERT_EQ(7u, items->size());
 
-      const char* expected_names[] = {"Save",      "Concat",   "SaveLayer",
-                                      "Translate", "DrawRect", "Restore",
-                                      "Restore"};
+      const char* expected_names[] = {
+          "SaveOp",     "ConcatOp",  "SaveLayerOp", "TranslateOp",
+          "DrawRectOp", "RestoreOp", "RestoreOp"};
       bool expected_has_skp[] = {false, true, true, true, true, false, false};
 
       for (int i = 0; i < 7; ++i) {
-        const base::DictionaryValue* item_dict;
-        ASSERT_TRUE(items->GetDictionary(i, &item_dict));
+        const base::Value& item_value = (*items)[i];
+        ASSERT_TRUE(item_value.is_dict());
+        const base::Value::Dict& item_dict = item_value.GetDict();
 
-        const base::ListValue* visual_rect;
-        ASSERT_TRUE(item_dict->GetList("visual_rect", &visual_rect));
+        const base::Value::List* visual_rect =
+            item_dict.FindList("visual_rect");
+        ASSERT_NE(nullptr, visual_rect);
         EXPECT_TRACED_RECT(2, 3, 8, 9, visual_rect);
 
-        std::string name;
-        EXPECT_TRUE(item_dict->GetString("name", &name));
-        EXPECT_EQ(expected_names[i], name);
+        const std::string* name = item_dict.FindString("name");
+        EXPECT_NE(nullptr, name);
+        EXPECT_EQ(expected_names[i], *name);
 
-        EXPECT_EQ(
-            expected_has_skp[i],
-            item_dict->GetString("skp64", static_cast<std::string*>(nullptr)));
+        EXPECT_EQ(expected_has_skp[i],
+                  item_dict.FindString("skp64") != nullptr);
       }
     }
   }
 
   // Pass |false| to not include PaintOps.
   root = ToBaseValue(list.get(), false);
-  ASSERT_TRUE(root->GetAsDictionary(&root_dict));
+  root_dict = root->GetIfDict();
+  ASSERT_NE(nullptr, root_dict);
   // The traced value has a params dictionary as its root.
   {
-    const base::DictionaryValue* params_dict;
-    ASSERT_TRUE(root_dict->GetDictionary("params", &params_dict));
+    const base::Value::Dict* params_dict = root_dict->FindDict("params");
+    ASSERT_NE(nullptr, params_dict);
 
     // The real contents of the traced value is in here.
     {
-      const base::ListValue* params_list;
+      const base::Value::List* params_list;
       // The layer_rect field is present and has the bounds of the rtree.
-      ASSERT_TRUE(params_dict->GetList("layer_rect", &params_list));
+      params_list = params_dict->FindList("layer_rect");
+      ASSERT_NE(nullptr, params_list);
       EXPECT_TRACED_RECT(2, 3, 8, 9, params_list);
 
       // The items list is not present since we asked for no ops.
-      ASSERT_FALSE(params_dict->GetList("items", &params_list));
+      params_list = params_dict->FindList("items");
+      ASSERT_EQ(nullptr, params_list);
     }
   }
 }
@@ -652,7 +666,7 @@ TEST_F(DisplayItemListTest, AppendVisualRectSimple) {
   }
 
   EXPECT_EQ(1u, list->TotalOpCount());
-  EXPECT_RECT_EQ(drawing_bounds, list->VisualRectForTesting(0));
+  EXPECT_EQ(drawing_bounds, list->VisualRectForTesting(0));
 }
 
 TEST_F(DisplayItemListTest, AppendVisualRectEmptyBlock) {
@@ -675,9 +689,9 @@ TEST_F(DisplayItemListTest, AppendVisualRectEmptyBlock) {
   }
 
   EXPECT_EQ(3u, list->TotalOpCount());
-  EXPECT_RECT_EQ(gfx::Rect(), list->VisualRectForTesting(0));
-  EXPECT_RECT_EQ(gfx::Rect(), list->VisualRectForTesting(1));
-  EXPECT_RECT_EQ(gfx::Rect(), list->VisualRectForTesting(2));
+  EXPECT_EQ(gfx::Rect(), list->VisualRectForTesting(0));
+  EXPECT_EQ(gfx::Rect(), list->VisualRectForTesting(1));
+  EXPECT_EQ(gfx::Rect(), list->VisualRectForTesting(2));
 }
 
 TEST_F(DisplayItemListTest, AppendVisualRectEmptyBlockContainingEmptyBlock) {
@@ -712,11 +726,11 @@ TEST_F(DisplayItemListTest, AppendVisualRectEmptyBlockContainingEmptyBlock) {
   }
 
   EXPECT_EQ(5u, list->TotalOpCount());
-  EXPECT_RECT_EQ(gfx::Rect(), list->VisualRectForTesting(0));
-  EXPECT_RECT_EQ(gfx::Rect(), list->VisualRectForTesting(1));
-  EXPECT_RECT_EQ(gfx::Rect(), list->VisualRectForTesting(2));
-  EXPECT_RECT_EQ(gfx::Rect(), list->VisualRectForTesting(3));
-  EXPECT_RECT_EQ(gfx::Rect(), list->VisualRectForTesting(4));
+  EXPECT_EQ(gfx::Rect(), list->VisualRectForTesting(0));
+  EXPECT_EQ(gfx::Rect(), list->VisualRectForTesting(1));
+  EXPECT_EQ(gfx::Rect(), list->VisualRectForTesting(2));
+  EXPECT_EQ(gfx::Rect(), list->VisualRectForTesting(3));
+  EXPECT_EQ(gfx::Rect(), list->VisualRectForTesting(4));
 }
 
 TEST_F(DisplayItemListTest, AppendVisualRectBlockContainingDrawing) {
@@ -747,10 +761,10 @@ TEST_F(DisplayItemListTest, AppendVisualRectBlockContainingDrawing) {
   }
 
   EXPECT_EQ(4u, list->TotalOpCount());
-  EXPECT_RECT_EQ(drawing_bounds, list->VisualRectForTesting(0));
-  EXPECT_RECT_EQ(drawing_bounds, list->VisualRectForTesting(1));
-  EXPECT_RECT_EQ(drawing_bounds, list->VisualRectForTesting(2));
-  EXPECT_RECT_EQ(drawing_bounds, list->VisualRectForTesting(3));
+  EXPECT_EQ(drawing_bounds, list->VisualRectForTesting(0));
+  EXPECT_EQ(drawing_bounds, list->VisualRectForTesting(1));
+  EXPECT_EQ(drawing_bounds, list->VisualRectForTesting(2));
+  EXPECT_EQ(drawing_bounds, list->VisualRectForTesting(3));
 }
 
 TEST_F(DisplayItemListTest, AppendVisualRectBlockContainingEscapedDrawing) {
@@ -781,10 +795,10 @@ TEST_F(DisplayItemListTest, AppendVisualRectBlockContainingEscapedDrawing) {
   }
 
   EXPECT_EQ(4u, list->TotalOpCount());
-  EXPECT_RECT_EQ(drawing_bounds, list->VisualRectForTesting(0));
-  EXPECT_RECT_EQ(drawing_bounds, list->VisualRectForTesting(1));
-  EXPECT_RECT_EQ(drawing_bounds, list->VisualRectForTesting(2));
-  EXPECT_RECT_EQ(drawing_bounds, list->VisualRectForTesting(3));
+  EXPECT_EQ(drawing_bounds, list->VisualRectForTesting(0));
+  EXPECT_EQ(drawing_bounds, list->VisualRectForTesting(1));
+  EXPECT_EQ(drawing_bounds, list->VisualRectForTesting(2));
+  EXPECT_EQ(drawing_bounds, list->VisualRectForTesting(3));
 }
 
 TEST_F(DisplayItemListTest,
@@ -824,11 +838,11 @@ TEST_F(DisplayItemListTest,
   }
 
   EXPECT_EQ(5u, list->TotalOpCount());
-  EXPECT_RECT_EQ(drawing_a_bounds, list->VisualRectForTesting(0));
-  EXPECT_RECT_EQ(drawing_b_bounds, list->VisualRectForTesting(1));
-  EXPECT_RECT_EQ(drawing_b_bounds, list->VisualRectForTesting(2));
-  EXPECT_RECT_EQ(drawing_b_bounds, list->VisualRectForTesting(3));
-  EXPECT_RECT_EQ(drawing_b_bounds, list->VisualRectForTesting(4));
+  EXPECT_EQ(drawing_a_bounds, list->VisualRectForTesting(0));
+  EXPECT_EQ(drawing_b_bounds, list->VisualRectForTesting(1));
+  EXPECT_EQ(drawing_b_bounds, list->VisualRectForTesting(2));
+  EXPECT_EQ(drawing_b_bounds, list->VisualRectForTesting(3));
+  EXPECT_EQ(drawing_b_bounds, list->VisualRectForTesting(4));
 }
 
 TEST_F(DisplayItemListTest, AppendVisualRectTwoBlocksTwoDrawings) {
@@ -882,14 +896,14 @@ TEST_F(DisplayItemListTest, AppendVisualRectTwoBlocksTwoDrawings) {
   EXPECT_EQ(8u, list->TotalOpCount());
   gfx::Rect merged_drawing_bounds = gfx::Rect(drawing_a_bounds);
   merged_drawing_bounds.Union(drawing_b_bounds);
-  EXPECT_RECT_EQ(merged_drawing_bounds, list->VisualRectForTesting(0));
-  EXPECT_RECT_EQ(merged_drawing_bounds, list->VisualRectForTesting(1));
-  EXPECT_RECT_EQ(drawing_a_bounds, list->VisualRectForTesting(2));
-  EXPECT_RECT_EQ(drawing_b_bounds, list->VisualRectForTesting(3));
-  EXPECT_RECT_EQ(drawing_b_bounds, list->VisualRectForTesting(4));
-  EXPECT_RECT_EQ(drawing_b_bounds, list->VisualRectForTesting(5));
-  EXPECT_RECT_EQ(drawing_b_bounds, list->VisualRectForTesting(6));
-  EXPECT_RECT_EQ(merged_drawing_bounds, list->VisualRectForTesting(7));
+  EXPECT_EQ(merged_drawing_bounds, list->VisualRectForTesting(0));
+  EXPECT_EQ(merged_drawing_bounds, list->VisualRectForTesting(1));
+  EXPECT_EQ(drawing_a_bounds, list->VisualRectForTesting(2));
+  EXPECT_EQ(drawing_b_bounds, list->VisualRectForTesting(3));
+  EXPECT_EQ(drawing_b_bounds, list->VisualRectForTesting(4));
+  EXPECT_EQ(drawing_b_bounds, list->VisualRectForTesting(5));
+  EXPECT_EQ(drawing_b_bounds, list->VisualRectForTesting(6));
+  EXPECT_EQ(merged_drawing_bounds, list->VisualRectForTesting(7));
 }
 
 TEST_F(DisplayItemListTest,
@@ -945,14 +959,14 @@ TEST_F(DisplayItemListTest,
   EXPECT_EQ(8u, list->TotalOpCount());
   gfx::Rect merged_drawing_bounds = gfx::Rect(drawing_a_bounds);
   merged_drawing_bounds.Union(drawing_b_bounds);
-  EXPECT_RECT_EQ(merged_drawing_bounds, list->VisualRectForTesting(0));
-  EXPECT_RECT_EQ(merged_drawing_bounds, list->VisualRectForTesting(1));
-  EXPECT_RECT_EQ(drawing_a_bounds, list->VisualRectForTesting(2));
-  EXPECT_RECT_EQ(drawing_b_bounds, list->VisualRectForTesting(3));
-  EXPECT_RECT_EQ(drawing_b_bounds, list->VisualRectForTesting(4));
-  EXPECT_RECT_EQ(drawing_b_bounds, list->VisualRectForTesting(5));
-  EXPECT_RECT_EQ(drawing_b_bounds, list->VisualRectForTesting(6));
-  EXPECT_RECT_EQ(merged_drawing_bounds, list->VisualRectForTesting(7));
+  EXPECT_EQ(merged_drawing_bounds, list->VisualRectForTesting(0));
+  EXPECT_EQ(merged_drawing_bounds, list->VisualRectForTesting(1));
+  EXPECT_EQ(drawing_a_bounds, list->VisualRectForTesting(2));
+  EXPECT_EQ(drawing_b_bounds, list->VisualRectForTesting(3));
+  EXPECT_EQ(drawing_b_bounds, list->VisualRectForTesting(4));
+  EXPECT_EQ(drawing_b_bounds, list->VisualRectForTesting(5));
+  EXPECT_EQ(drawing_b_bounds, list->VisualRectForTesting(6));
+  EXPECT_EQ(merged_drawing_bounds, list->VisualRectForTesting(7));
 }
 
 TEST_F(DisplayItemListTest,
@@ -1008,14 +1022,14 @@ TEST_F(DisplayItemListTest,
   EXPECT_EQ(8u, list->TotalOpCount());
   gfx::Rect merged_drawing_bounds = gfx::Rect(drawing_a_bounds);
   merged_drawing_bounds.Union(drawing_b_bounds);
-  EXPECT_RECT_EQ(merged_drawing_bounds, list->VisualRectForTesting(0));
-  EXPECT_RECT_EQ(merged_drawing_bounds, list->VisualRectForTesting(1));
-  EXPECT_RECT_EQ(drawing_a_bounds, list->VisualRectForTesting(2));
-  EXPECT_RECT_EQ(drawing_b_bounds, list->VisualRectForTesting(3));
-  EXPECT_RECT_EQ(drawing_b_bounds, list->VisualRectForTesting(4));
-  EXPECT_RECT_EQ(drawing_b_bounds, list->VisualRectForTesting(5));
-  EXPECT_RECT_EQ(drawing_b_bounds, list->VisualRectForTesting(6));
-  EXPECT_RECT_EQ(merged_drawing_bounds, list->VisualRectForTesting(7));
+  EXPECT_EQ(merged_drawing_bounds, list->VisualRectForTesting(0));
+  EXPECT_EQ(merged_drawing_bounds, list->VisualRectForTesting(1));
+  EXPECT_EQ(drawing_a_bounds, list->VisualRectForTesting(2));
+  EXPECT_EQ(drawing_b_bounds, list->VisualRectForTesting(3));
+  EXPECT_EQ(drawing_b_bounds, list->VisualRectForTesting(4));
+  EXPECT_EQ(drawing_b_bounds, list->VisualRectForTesting(5));
+  EXPECT_EQ(drawing_b_bounds, list->VisualRectForTesting(6));
+  EXPECT_EQ(merged_drawing_bounds, list->VisualRectForTesting(7));
 }
 
 TEST_F(DisplayItemListTest,
@@ -1071,14 +1085,14 @@ TEST_F(DisplayItemListTest,
   EXPECT_EQ(8u, list->TotalOpCount());
   gfx::Rect merged_drawing_bounds = gfx::Rect(drawing_a_bounds);
   merged_drawing_bounds.Union(drawing_b_bounds);
-  EXPECT_RECT_EQ(merged_drawing_bounds, list->VisualRectForTesting(0));
-  EXPECT_RECT_EQ(merged_drawing_bounds, list->VisualRectForTesting(1));
-  EXPECT_RECT_EQ(drawing_a_bounds, list->VisualRectForTesting(2));
-  EXPECT_RECT_EQ(drawing_b_bounds, list->VisualRectForTesting(3));
-  EXPECT_RECT_EQ(drawing_b_bounds, list->VisualRectForTesting(4));
-  EXPECT_RECT_EQ(drawing_b_bounds, list->VisualRectForTesting(5));
-  EXPECT_RECT_EQ(drawing_b_bounds, list->VisualRectForTesting(6));
-  EXPECT_RECT_EQ(merged_drawing_bounds, list->VisualRectForTesting(7));
+  EXPECT_EQ(merged_drawing_bounds, list->VisualRectForTesting(0));
+  EXPECT_EQ(merged_drawing_bounds, list->VisualRectForTesting(1));
+  EXPECT_EQ(drawing_a_bounds, list->VisualRectForTesting(2));
+  EXPECT_EQ(drawing_b_bounds, list->VisualRectForTesting(3));
+  EXPECT_EQ(drawing_b_bounds, list->VisualRectForTesting(4));
+  EXPECT_EQ(drawing_b_bounds, list->VisualRectForTesting(5));
+  EXPECT_EQ(drawing_b_bounds, list->VisualRectForTesting(6));
+  EXPECT_EQ(merged_drawing_bounds, list->VisualRectForTesting(7));
 }
 
 TEST_F(DisplayItemListTest, VisualRectForPairsEnclosingEmptyPainting) {
@@ -1107,9 +1121,9 @@ TEST_F(DisplayItemListTest, VisualRectForPairsEnclosingEmptyPainting) {
   }
 
   EXPECT_EQ(3u, list->TotalOpCount());
-  EXPECT_RECT_EQ(visual_rect, list->VisualRectForTesting(0));
-  EXPECT_RECT_EQ(visual_rect, list->VisualRectForTesting(1));
-  EXPECT_RECT_EQ(visual_rect, list->VisualRectForTesting(2));
+  EXPECT_EQ(visual_rect, list->VisualRectForTesting(0));
+  EXPECT_EQ(visual_rect, list->VisualRectForTesting(1));
+  EXPECT_EQ(visual_rect, list->VisualRectForTesting(2));
 }
 
 TEST_F(DisplayItemListTest, TotalOpCount) {
@@ -1127,7 +1141,7 @@ TEST_F(DisplayItemListTest, TotalOpCount) {
   list->StartPaint();
   list->push<SaveOp>();
   list->push<TranslateOp>(10.f, 20.f);
-  list->push<DrawRecordOp>(sub_list->ReleaseAsRecord());
+  list->push<DrawRecordOp>(sub_list->FinalizeAndReleaseAsRecord());
   list->push<RestoreOp>();
   list->EndPaintOfUnpaired(gfx::Rect());
   EXPECT_EQ(8u, list->TotalOpCount());
@@ -1137,40 +1151,41 @@ TEST_F(DisplayItemListTest, AreaOfDrawText) {
   auto list = base::MakeRefCounted<DisplayItemList>();
   auto sub_list = base::MakeRefCounted<DisplayItemList>();
 
-  auto text_blob1 = SkTextBlob::MakeFromString("ABCD", SkFont());
+  SkFont font = skia::DefaultFont();
+  auto text_blob1 = SkTextBlob::MakeFromString("ABCD", font);
   gfx::Size text_blob1_size(ceilf(text_blob1->bounds().width()),
                             ceilf(text_blob1->bounds().height()));
   auto text_blob1_area = text_blob1_size.width() * text_blob1_size.height();
-  auto text_blob2 = SkTextBlob::MakeFromString("EFG", SkFont());
+  auto text_blob2 = SkTextBlob::MakeFromString("EFG", font);
   gfx::Size text_blob2_size(ceilf(text_blob2->bounds().width()),
                             ceilf(text_blob2->bounds().height()));
   auto text_blob2_area = text_blob2_size.width() * text_blob2_size.height();
 
   sub_list->StartPaint();
-  sub_list->push<DrawTextBlobOp>(text_blob1, 0, 0, PaintFlags());
+  sub_list->push<DrawTextBlobOp>(text_blob1, 0.0f, 0.0f, PaintFlags());
   sub_list->EndPaintOfUnpaired(gfx::Rect());
-  auto record = sub_list->ReleaseAsRecord();
+  auto record = sub_list->FinalizeAndReleaseAsRecord();
 
   list->StartPaint();
   list->push<SaveOp>();
-  list->push<TranslateOp>(100, 100);
+  list->push<TranslateOp>(100.0f, 100.0f);
   list->push<DrawRecordOp>(record);
   list->push<RestoreOp>();
   list->EndPaintOfUnpaired(gfx::Rect(gfx::Point(100, 100), text_blob1_size));
 
   list->StartPaint();
   list->push<SaveOp>();
-  list->push<TranslateOp>(100, 400);
+  list->push<TranslateOp>(100.0f, 400.0f);
   list->push<DrawRecordOp>(record);
   list->push<RestoreOp>();
   list->EndPaintOfUnpaired(gfx::Rect(gfx::Point(100, 400), text_blob1_size));
 
   list->StartPaint();
-  list->push<DrawTextBlobOp>(text_blob2, 10, 20, PaintFlags());
+  list->push<DrawTextBlobOp>(text_blob2, 10.0f, 20.0f, PaintFlags());
   list->EndPaintOfUnpaired(gfx::Rect(text_blob2_size));
 
   list->StartPaint();
-  list->push<DrawTextBlobOp>(text_blob2, 400, 100, PaintFlags());
+  list->push<DrawTextBlobOp>(text_blob2, 400.0f, 100.0f, PaintFlags());
   list->EndPaintOfUnpaired(gfx::Rect(gfx::Point(400, 100), text_blob2_size));
 
   list->StartPaint();
