@@ -1,4 +1,4 @@
-// Copyright 2013 The Chromium Authors. All rights reserved.
+// Copyright 2013 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,8 +9,10 @@
 #include "base/timer/lap_timer.h"
 #include "cc/paint/paint_op_buffer.h"
 #include "cc/paint/paint_op_buffer_serializer.h"
+#include "cc/paint/paint_op_writer.h"
 #include "cc/paint/paint_shader.h"
 #include "cc/test/test_options_provider.h"
+#include "skia/ext/font_utils.h"
 #include "testing/perf/perf_result_reporter.h"
 #include "third_party/skia/include/core/SkMaskFilter.h"
 #include "third_party/skia/include/effects/SkColorMatrixFilter.h"
@@ -32,12 +34,8 @@ class PaintOpPerfTest : public testing::Test {
       : timer_(kNumWarmupRuns,
                base::Milliseconds(kTimeLimitMillis),
                kTimeCheckInterval),
-        serialized_data_(static_cast<char*>(
-            base::AlignedAlloc(kMaxSerializedBufferBytes,
-                               PaintOpBuffer::PaintOpAlign))),
-        deserialized_data_(static_cast<char*>(
-            base::AlignedAlloc(sizeof(LargestPaintOp),
-                               PaintOpBuffer::PaintOpAlign))) {}
+        serialized_data_(
+            PaintOpWriter::AllocateAlignedBuffer(kMaxSerializedBufferBytes)) {}
 
   void RunTest(const std::string& name, const PaintOpBuffer& buffer) {
     TestOptionsProvider test_options_provider;
@@ -50,7 +48,7 @@ class PaintOpPerfTest : public testing::Test {
       SimpleBufferSerializer serializer(
           serialized_data_.get(), kMaxSerializedBufferBytes,
           test_options_provider.serialize_options());
-      serializer.Serialize(&buffer, nullptr, preamble);
+      serializer.Serialize(buffer, nullptr, preamble);
       bytes_written = serializer.written();
 
       // Force client paint cache entries to be written every time.
@@ -73,8 +71,8 @@ class PaintOpPerfTest : public testing::Test {
 
       while (true) {
         PaintOp* deserialized_op = PaintOp::Deserialize(
-            to_read, remaining_read_bytes, deserialized_data_.get(),
-            sizeof(LargestPaintOp), &bytes_read,
+            to_read, remaining_read_bytes, deserialized_data_,
+            kLargestPaintOpAlignedSize, &bytes_read,
             test_options_provider.deserialize_options());
         CHECK(deserialized_op);
         deserialized_op->DestroyThis();
@@ -98,7 +96,8 @@ class PaintOpPerfTest : public testing::Test {
  protected:
   base::LapTimer timer_;
   std::unique_ptr<char, base::AlignedFreeDeleter> serialized_data_;
-  std::unique_ptr<char, base::AlignedFreeDeleter> deserialized_data_;
+  alignas(PaintOpBuffer::kPaintOpAlign) char deserialized_data_
+      [kLargestPaintOpAlignedSize];
 };
 
 // Ops that can be memcopied both when serializing and deserializing.
@@ -127,8 +126,7 @@ TEST_F(PaintOpPerfTest, ManyFlagsOps) {
   flags.setPathEffect(SkDashPathEffect::Make(intervals, 2, 0));
   flags.setMaskFilter(SkMaskFilter::MakeBlur(
       SkBlurStyle::kOuter_SkBlurStyle, 4.3));
-  flags.setColorFilter(
-      SkColorMatrixFilter::MakeLightingFilter(SK_ColorYELLOW, SK_ColorGREEN));
+  flags.setColorFilter(ColorFilter::MakeLuma());
 
   SkLayerDrawLooper::Builder looper_builder;
   looper_builder.addLayer();
@@ -137,7 +135,7 @@ TEST_F(PaintOpPerfTest, ManyFlagsOps) {
   looper_builder.addLayer(layer_info);
   flags.setLooper(looper_builder.detach());
 
-  sk_sp<PaintShader> shader = PaintShader::MakeColor(SK_ColorTRANSPARENT);
+  sk_sp<PaintShader> shader = PaintShader::MakeColor(SkColors::kTransparent);
   flags.setShader(std::move(shader));
 
   SkPath path;
@@ -154,7 +152,7 @@ TEST_F(PaintOpPerfTest, ManyFlagsOps) {
 TEST_F(PaintOpPerfTest, TextOps) {
   PaintOpBuffer buffer;
 
-  auto typeface = SkTypeface::MakeDefault();
+  auto typeface = skia::DefaultTypeface();
 
   SkFont font;
   font.setTypeface(typeface);
