@@ -1,4 +1,4 @@
-// Copyright 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -15,10 +15,13 @@
 #include <utility>
 #include <vector>
 
+#include "base/memory/raw_ptr.h"
+#include "base/memory/raw_ptr_exclusion.h"
 #include "cc/base/region.h"
 #include "cc/base/tiling_data.h"
 #include "cc/cc_export.h"
 #include "cc/paint/paint_worklet_input.h"
+#include "cc/raster/raster_source.h"
 #include "cc/tiles/tile.h"
 #include "cc/tiles/tile_priority.h"
 #include "cc/trees/occlusion.h"
@@ -33,7 +36,6 @@ class TracedValue;
 
 namespace cc {
 
-class RasterSource;
 class PictureLayerTiling;
 class PrioritizedTile;
 
@@ -53,7 +55,7 @@ class CC_EXPORT PictureLayerTilingClient {
   virtual const PaintWorkletRecordMap& GetPaintWorkletRecords() const = 0;
   virtual bool IsDirectlyCompositedImage() const = 0;
   virtual bool ScrollInteractionInProgress() const = 0;
-  virtual bool CurrentScrollDidCheckerboardLargeArea() const = 0;
+  virtual bool CurrentScrollCheckerboardsDueToNoRecording() const = 0;
 
  protected:
   virtual ~PictureLayerTilingClient() {}
@@ -107,14 +109,24 @@ class CC_EXPORT PictureLayerTiling {
 
   PictureLayerTilingClient* client() const { return client_; }
 
-  void SetRasterSourceAndResize(scoped_refptr<RasterSource> raster_source);
+  // Returns true if the current tiling needs to update tile priority rects and
+  // tiles.
+  bool SetRasterSourceAndResize(scoped_refptr<RasterSource> raster_source);
   void Invalidate(const Region& layer_invalidation);
   void CreateMissingTilesInLiveTilesRect();
   void TakeTilesAndPropertiesFrom(PictureLayerTiling* pending_twin,
                                   const Region& layer_invalidation);
 
-  bool IsTileRequiredForActivation(const Tile* tile) const;
-  bool IsTileRequiredForDraw(const Tile* tile) const;
+  bool IsTileRequiredForActivation(const Tile* tile) const {
+    return IsTileRequiredForActivation(
+        tile, [this](const Tile* tile) { return IsTileVisible(tile); },
+        IsTileOccluded(tile));
+  }
+
+  bool IsTileRequiredForDraw(const Tile* tile) const {
+    return IsTileRequiredForDraw(
+        tile, [this](const Tile* tile) { return IsTileVisible(tile); });
+  }
 
   // Returns true if the tile should be processed for decoding images skipped
   // during rasterization.
@@ -226,6 +238,25 @@ class CC_EXPORT PictureLayerTiling {
                          soon_border_rect, eventually_rect, Occlusion());
   }
 
+  using TileMap =
+      std::unordered_map<TileMapKey, std::unique_ptr<Tile>, TileMapKeyHash>;
+
+  // Iterates over the tiles of a PictureLayerTiling. Order of iteration is not
+  // defined.
+  class CC_EXPORT TileIterator {
+   public:
+    explicit TileIterator(PictureLayerTiling* tiling);
+    ~TileIterator();
+
+    Tile* GetCurrent();
+    void Next();
+    bool AtEnd() const;
+
+   private:
+    raw_ptr<PictureLayerTiling> tiling_;
+    PictureLayerTiling::TileMap::iterator iter_;
+  };
+
   // Iterate over all tiles to fill content_rect.  Even if tiles are invalid
   // (i.e. no valid resource) this tiling should still iterate over them.
   // The union of all geometry_rect calls for each element iterated over should
@@ -258,12 +289,18 @@ class CC_EXPORT PictureLayerTiling {
    private:
     gfx::Rect ComputeGeometryRect() const;
 
-    const PictureLayerTiling* tiling_ = nullptr;
+    // RAW_PTR_EXCLUSION: Performance reasons: based on analysis of sampling
+    // profiler data and tab_search:top100:2020.
+    RAW_PTR_EXCLUSION const PictureLayerTiling* tiling_ = nullptr;
+
     gfx::Size coverage_rect_max_bounds_;
     gfx::Rect coverage_rect_;
     gfx::AxisTransform2d coverage_to_content_;
 
-    Tile* current_tile_ = nullptr;
+    // RAW_PTR_EXCLUSION: Performance reasons: based on analysis of sampling
+    // profiler data and tab_search:top100:2020.
+    RAW_PTR_EXCLUSION Tile* current_tile_ = nullptr;
+
     gfx::Rect current_geometry_rect_;
     int tile_i_ = 0;
     int tile_j_ = 0;
@@ -295,9 +332,11 @@ class CC_EXPORT PictureLayerTiling {
  protected:
   friend class CoverageIterator;
   friend class PrioritizedTile;
+  friend class TileIterator;
   friend class TilingSetRasterQueueAll;
   friend class TilingSetRasterQueueRequired;
   friend class TilingSetEvictionQueue;
+  friend class TilesWithResourceIterator;
 
   // PENDING VISIBLE RECT refers to the visible rect that will become current
   // upon activation (ie, the pending tree's visible rect). Tiles in this
@@ -312,8 +351,73 @@ class CC_EXPORT PictureLayerTiling {
     EVENTUALLY_RECT
   };
 
-  using TileMap =
-      std::unordered_map<TileMapKey, std::unique_ptr<Tile>, TileMapKeyHash>;
+  bool IsTileVisible(const Tile* tile) const {
+    gfx::Rect tile_bounds =
+        tiling_data_.TileBounds(tile->tiling_i_index(), tile->tiling_j_index());
+    return tile_bounds.Intersects(current_visible_rect_);
+  }
+
+  template <typename VisibilityChecker>
+  bool IsTileRequiredForActivation(const Tile* tile,
+                                   VisibilityChecker is_visible,
+                                   bool is_tile_occluded) const {
+    if (tree_ == PENDING_TREE) {
+      if (!can_require_tiles_for_activation_ ||
+          resolution_ != HIGH_RESOLUTION || is_tile_occluded) {
+        return false;
+      }
+
+      // We may be checking the active tree tile here (since this function is
+      // also called for active trees below, ensure that this is at all a valid
+      // tile on the pending tree.
+      if (tile->tiling_i_index() >= tiling_data_.num_tiles_x() ||
+          tile->tiling_j_index() >= tiling_data_.num_tiles_y()) {
+        return false;
+      }
+
+      if (!is_visible(tile))
+        return false;
+
+      if (client_->RequiresHighResToDraw())
+        return true;
+
+      const PictureLayerTiling* active_twin =
+          client_->GetPendingOrActiveTwinTiling(this);
+      if (!active_twin || !TilingMatchesTileIndices(active_twin))
+        return true;
+
+      if (active_twin->raster_source()->GetSize() != raster_source()->GetSize())
+        return true;
+
+      if (active_twin->current_visible_rect_ != current_visible_rect_)
+        return true;
+
+      Tile* twin_tile =
+          active_twin->TileAt(tile->tiling_i_index(), tile->tiling_j_index());
+      if (!twin_tile)
+        return false;
+      return true;
+    }
+
+    DCHECK_EQ(tree_, ACTIVE_TREE);
+    const PictureLayerTiling* pending_twin =
+        client_->GetPendingOrActiveTwinTiling(this);
+    // If we don't have a pending tree, or the pending tree will overwrite the
+    // given tile, then it is not required for activation.
+    if (!pending_twin || !TilingMatchesTileIndices(pending_twin) ||
+        pending_twin->TileAt(tile->tiling_i_index(), tile->tiling_j_index())) {
+      return false;
+    }
+    // Otherwise, ask the pending twin if this tile is required for activation.
+    return pending_twin->IsTileRequiredForActivation(tile);
+  }
+
+  template <typename VisibilityChecker>
+  bool IsTileRequiredForDraw(const Tile* tile,
+                             VisibilityChecker is_visible) const {
+    return tree_ == ACTIVE_TREE && resolution_ == HIGH_RESOLUTION &&
+           is_visible(tile) && !IsTileOccludedOnCurrentTree(tile);
+  }
 
   void SetLiveTilesRect(const gfx::Rect& live_tiles_rect);
   void VerifyLiveTilesRect() const;
@@ -334,13 +438,42 @@ class CC_EXPORT PictureLayerTiling {
   bool IsTileOccludedOnCurrentTree(const Tile* tile) const;
   Tile::CreateInfo CreateInfoForTile(int i, int j) const;
   bool ShouldCreateTileAt(const Tile::CreateInfo& info) const;
-  bool IsTileOccluded(const Tile* tile) const;
-  PrioritizedTile MakePrioritizedTile(
-      Tile* tile,
-      PriorityRectType priority_rect_type) const;
-  TilePriority ComputePriorityForTile(
-      const Tile* tile,
-      PriorityRectType priority_rect_type) const;
+
+  bool IsTileOccluded(const Tile* tile) const {
+    // If this tile is not occluded on this tree, then it is not occluded.
+    if (!IsTileOccludedOnCurrentTree(tile))
+      return false;
+
+    // Otherwise, if this is the pending tree, we're done and the tile is
+    // occluded.
+    if (tree_ == PENDING_TREE)
+      return true;
+
+    // On the active tree however, we need to check if this tile will be
+    // unoccluded upon activation, in which case it has to be considered
+    // unoccluded.
+    const PictureLayerTiling* pending_twin =
+        client_->GetPendingOrActiveTwinTiling(this);
+    if (pending_twin) {
+      // If there's a pending tile in the same position. Or if the pending twin
+      // would have to be creating all tiles, then we don't need to worry about
+      // occlusion on the twin.
+      if (!TilingMatchesTileIndices(pending_twin) ||
+          pending_twin->TileAt(tile->tiling_i_index(),
+                               tile->tiling_j_index())) {
+        return true;
+      }
+      return pending_twin->IsTileOccludedOnCurrentTree(tile);
+    }
+    return true;
+  }
+
+  PrioritizedTile MakePrioritizedTile(Tile* tile,
+                                      PriorityRectType priority_rect_type,
+                                      bool is_tile_occluded) const;
+  TilePriority ComputePriorityForTile(const Tile* tile,
+                                      PriorityRectType priority_rect_type,
+                                      bool is_tile_occluded) const;
   PriorityRectType ComputePriorityRectTypeForTile(const Tile* tile) const;
   bool has_visible_rect_tiles() const { return has_visible_rect_tiles_; }
   bool has_skewport_rect_tiles() const { return has_skewport_rect_tiles_; }
@@ -376,9 +509,11 @@ class CC_EXPORT PictureLayerTiling {
   gfx::Rect EnclosingLayerRectFromContentsRect(
       const gfx::Rect& contents_rect) const;
 
+  void SetTilingSize(const gfx::Size& tiling_size);
+
   // Given properties.
   const gfx::AxisTransform2d raster_transform_;
-  PictureLayerTilingClient* const client_;
+  const raw_ptr<PictureLayerTilingClient> client_;
   const WhichTree tree_;
   scoped_refptr<RasterSource> raster_source_;
   const float min_preraster_distance_;
