@@ -206,6 +206,8 @@ void UkmManager::RecordEventLatencyUKM(
       switch (dispatch_stage) {
         case EventMetrics::DispatchStage::kGenerated:
           switch (end_stage) {
+            case EventMetrics::DispatchStage::
+                kScrollsBlockingTouchDispatchedToRenderer:
             case EventMetrics::DispatchStage::kArrivedInBrowserMain:
               // Will build the `GenerationToRendererCompositor` metric on the
               // `kArrivedInBrowserMain` stage.
@@ -218,6 +220,9 @@ void UkmManager::RecordEventLatencyUKM(
               break;
           }
           break;
+        case EventMetrics::DispatchStage::
+            kScrollsBlockingTouchDispatchedToRenderer:
+          break;
         case EventMetrics::DispatchStage::kArrivedInBrowserMain:
           DCHECK_EQ(end_stage,
                     EventMetrics::DispatchStage::kArrivedInRendererCompositor);
