@@ -1,4 +1,4 @@
-// Copyright 2013 The Chromium Authors. All rights reserved.
+// Copyright 2013 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,29 +7,28 @@
 #include <stddef.h>
 
 #include <iterator>
+#include <memory>
+#include <set>
 
-#include "base/bind.h"
-#include "base/callback.h"
 #include "base/check_op.h"
 #include "base/files/file_enumerator.h"
 #include "base/files/file_util.h"
+#include "base/functional/bind.h"
+#include "base/functional/callback.h"
 #include "base/lazy_instance.h"
-#include "base/macros.h"
+#include "base/ranges/algorithm.h"
 #include "base/sequence_checker.h"
 #include "base/stl_util.h"
 #include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/task/lazy_thread_pool_task_runner.h"
-#include "content/public/browser/browser_task_traits.h"
+#include "content/public/browser/browser_thread.h"
 
 #include "content/public/browser/browser_thread.h"
 
-using base::Bind;
-using base::Callback;
 using base::FileEnumerator;
 using base::FilePath;
 using base::Time;
-using base::TimeDelta;
 using base::TimeTicks;
 using content::BrowserThread;
 using std::map;
@@ -66,6 +65,10 @@ const Trigram kUndefinedTrigram = -1;
 class Index {
  public:
   Index();
+
+  Index(const Index&) = delete;
+  Index& operator=(const Index&) = delete;
+
   // Index is only instantiated as a leak LazyInstance, so the destructor is
   // never called.
   ~Index() = delete;
@@ -91,8 +94,6 @@ class Index {
   IndexedFilesMap index_times_;
   vector<bool> is_normalized_;
   SEQUENCE_CHECKER(sequence_checker_);
-
-  DISALLOW_COPY_AND_ASSIGN(Index);
 };
 
 base::LazyInstance<Index>::Leaky g_trigram_index = LAZY_INSTANCE_INITIALIZER;
@@ -216,9 +217,8 @@ vector<FilePath> Index::Search(const string& query) {
   for (; it != trigrams.end(); ++it) {
     Trigram trigram = *it;
     if (first) {
-      std::copy(index_[trigram].begin(),
-                index_[trigram].end(),
-                std::inserter(file_ids, file_ids.begin()));
+      base::ranges::copy(index_[trigram],
+                         std::inserter(file_ids, file_ids.begin()));
       first = false;
       continue;
     }
@@ -259,8 +259,6 @@ void Index::NormalizeVectors() {
   }
 }
 
-typedef Callback<void(bool, const vector<bool>&)> IndexerCallback;
-
 }  // namespace
 
 DevToolsFileSystemIndexer::FileSystemIndexingJob::FileSystemIndexingJob(
@@ -304,16 +302,16 @@ void DevToolsFileSystemIndexer::FileSystemIndexingJob::CollectFilesToIndex() {
   if (stopped_)
     return;
   if (!file_enumerator_) {
-    file_enumerator_.reset(new FileEnumerator(
+    file_enumerator_ = std::make_unique<FileEnumerator>(
         pending_folders_.back(), false,
-        FileEnumerator::FILES | FileEnumerator::DIRECTORIES));
+        FileEnumerator::FILES | FileEnumerator::DIRECTORIES);
     pending_folders_.pop_back();
   }
   FilePath file_path = file_enumerator_->Next();
   if (file_path.empty() && !pending_folders_.empty()) {
-    file_enumerator_.reset(new FileEnumerator(
+    file_enumerator_ = std::make_unique<FileEnumerator>(
         pending_folders_.back(), false,
-        FileEnumerator::FILES | FileEnumerator::DIRECTORIES));
+        FileEnumerator::FILES | FileEnumerator::DIRECTORIES);
     pending_folders_.pop_back();
     impl_task_runner()->PostTask(
         FROM_HERE, BindOnce(&FileSystemIndexingJob::CollectFilesToIndex, this));
@@ -345,7 +343,7 @@ void DevToolsFileSystemIndexer::FileSystemIndexingJob::CollectFilesToIndex() {
       g_trigram_index.Get().LastModifiedTimeForFile(file_path);
   FileEnumerator::FileInfo file_info = file_enumerator_->GetInfo();
   Time current_last_modified_time = file_info.GetLastModifiedTime();
-  if (current_last_modified_time > saved_last_modified_time) {
+  if (current_last_modified_time >= saved_last_modified_time) {
     file_path_times_[file_path] = current_last_modified_time;
   }
   impl_task_runner()->PostTask(
@@ -443,7 +441,7 @@ void DevToolsFileSystemIndexer::FileSystemIndexingJob::ReportWorked() {
   TimeTicks current_time = TimeTicks::Now();
   bool should_send_worked_nitification = true;
   if (!last_worked_notification_time_.is_null()) {
-    TimeDelta delta = current_time - last_worked_notification_time_;
+    base::TimeDelta delta = current_time - last_worked_notification_time_;
     if (delta.InMilliseconds() < kMinTimeoutBetweenWorkedNitification)
       should_send_worked_nitification = false;
   }
