@@ -1,4 +1,4 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,7 +8,8 @@
 #include <memory>
 #include <string>
 
-#include "base/macros.h"
+#include "base/memory/weak_ptr.h"
+#include "base/time/time.h"
 #include "chrome/browser/devtools/devtools_contents_resizing_strategy.h"
 #include "chrome/browser/devtools/devtools_toggle_action.h"
 #include "chrome/browser/devtools/devtools_ui_bindings.h"
@@ -33,7 +34,7 @@ namespace user_prefs {
 class PrefRegistrySyncable;
 }
 
-// Values that represent different actions to open DevTools window.
+// Values that represent different actions to open and close DevTools window.
 // These values are written to logs. New enum values can be added, but existing
 // enums must never be renumbered or deleted and reused.
 enum class DevToolsOpenedByAction {
@@ -49,25 +50,39 @@ enum class DevToolsOpenedByAction {
   kInspectorModeShortcut = 4,
   // Toggle-open via F12
   kToggleShortcut = 5,
+  // Link on chrome://inspect
+  kInspectLink = 6,
+  // Via --auto-open-devtools-for-tabs or "Auto-open DevTools for popups"
+  kAutomaticForNewTarget = 7,
+  // Re-open when some targets (like apps) reload
+  kTargetReload = 8,
+  // Open Node DevTools button in a regular app
+  kOpenForNodeFromAnotherTarget = 9,
   // Add values above this line with a corresponding label in
   // tools/metrics/histograms/enums.xml
-  kMaxValue = kToggleShortcut,
+  kMaxValue = kOpenForNodeFromAnotherTarget,
+};
+
+enum class DevToolsClosedByAction {
+  kUnknown = 0,
+  // Main menu -> More Tools -> Developer Tools
+  // or Ctrl+Shift+I shortcut
+  kMainMenuOrMainShortcut = 1,
+  // Toggle-open via F12
+  kToggleShortcut = 2,
+  kCloseButton = 3,
+  kTargetDetach = 4,
+  kMaxValue = kTargetDetach,
 };
 
 class DevToolsWindow : public DevToolsUIBindings::Delegate,
                        public content::WebContentsDelegate {
  public:
-  class ObserverWithAccessor : public content::WebContentsObserver {
-   public:
-    explicit ObserverWithAccessor(content::WebContents* web_contents);
-    ~ObserverWithAccessor() override;
-
-   private:
-    DISALLOW_COPY_AND_ASSIGN(ObserverWithAccessor);
-  };
-
   static const char kDevToolsApp[];
 
+  DevToolsWindow(const DevToolsWindow&) = delete;
+  DevToolsWindow& operator=(const DevToolsWindow&) = delete;
+
   ~DevToolsWindow() override;
 
   static void RegisterProfilePrefs(user_prefs::PrefRegistrySyncable* registry);
@@ -103,22 +118,28 @@ class DevToolsWindow : public DevToolsUIBindings::Delegate,
   // How to get pointer to the created window see comments for
   // ToggleDevToolsWindow().
   static void OpenDevToolsWindow(content::WebContents* inspected_web_contents,
-                                 const DevToolsToggleAction& action);
+                                 const DevToolsToggleAction& action,
+                                 DevToolsOpenedByAction opened_by);
 
   // Open or reveal DevTools window, with no special action.
   // How to get pointer to the created window see comments for
   // ToggleDevToolsWindow().
-  static void OpenDevToolsWindow(content::WebContents* inspected_web_contents);
+  static void OpenDevToolsWindow(content::WebContents* inspected_web_contents,
+                                 DevToolsOpenedByAction opened_by);
+  static void OpenDevToolsWindow(content::WebContents* inspected_web_contents,
+                                 Profile* profile,
+                                 DevToolsOpenedByAction opened_by);
 
   // Open or reveal DevTools window, with no special action. Use |profile| to
   // open client window in, default to |host|'s profile if none given.
-  static void OpenDevToolsWindow(
-      scoped_refptr<content::DevToolsAgentHost> host,
-      Profile* profile);
+  static void OpenDevToolsWindow(scoped_refptr<content::DevToolsAgentHost> host,
+                                 Profile* profile,
+                                 DevToolsOpenedByAction opened_by);
   // Similar to previous one, but forces the bundled frontend to be used.
   static void OpenDevToolsWindowWithBundledFrontend(
       scoped_refptr<content::DevToolsAgentHost> host,
-      Profile* profile);
+      Profile* profile,
+      DevToolsOpenedByAction opened_by);
 
   // Perform specified action for current WebContents inside a |browser|.
   // This may close currently open DevTools window.
@@ -139,7 +160,9 @@ class DevToolsWindow : public DevToolsUIBindings::Delegate,
       DevToolsOpenedByAction opened_by = DevToolsOpenedByAction::kUnknown);
 
   // Node frontend is always undocked.
-  static DevToolsWindow* OpenNodeFrontendWindow(Profile* profile);
+  static DevToolsWindow* OpenNodeFrontendWindow(
+      Profile* profile,
+      DevToolsOpenedByAction opened_by);
 
   static void InspectElement(content::RenderFrameHost* inspected_frame_host,
                              int x,
@@ -147,6 +170,9 @@ class DevToolsWindow : public DevToolsUIBindings::Delegate,
 
   static void LogDevToolsOpenedByAction(DevToolsOpenedByAction opened_by);
 
+  // Logs UKM event when DevTools is opened.
+  static void LogDevToolsOpenedUKM(content::WebContents* web_contents);
+
   static std::unique_ptr<content::NavigationThrottle>
   MaybeCreateNavigationThrottle(content::NavigationHandle* handle);
 
@@ -202,7 +228,7 @@ class DevToolsWindow : public DevToolsUIBindings::Delegate,
   //    to BeforeUnloadFired is proxied through HandleBeforeUnload() rather
   //    than getting called directly.
   // 3a. If |DevToolsWindow::BeforeUnloadFired| is called with |proceed|=false
-  //     it calls throught to the content's BeforeUnloadFired(), which from the
+  //     it calls through to the content's BeforeUnloadFired(), which from the
   //     WebContents perspective looks the same as the |content|'s own
   //     beforeunload dialog having had it's 'stay on this page' button clicked.
   // 3b. If |proceed| = true, then it fires beforeunload event on |contents|
@@ -232,7 +258,7 @@ class DevToolsWindow : public DevToolsUIBindings::Delegate,
 
   // Returns true if this contents beforeunload event was intercepted by
   // devtools and false otherwise. If the event was intercepted, caller should
-  // not fire beforeunlaod event on |contents| itself as devtools window will
+  // not fire beforeunload event on |contents| itself as devtools window will
   // take care of it, otherwise caller should continue handling the event as
   // usual.
   static bool InterceptPageBeforeUnload(content::WebContents* contents);
@@ -262,16 +288,18 @@ class DevToolsWindow : public DevToolsUIBindings::Delegate,
 
   static void OpenDevToolsWindowForFrame(
       Profile* profile,
-      const scoped_refptr<content::DevToolsAgentHost>& agent_host);
+      const scoped_refptr<content::DevToolsAgentHost>& agent_host,
+      DevToolsOpenedByAction opened_by);
   static void OpenDevToolsWindowForWorker(
       Profile* profile,
-      const scoped_refptr<content::DevToolsAgentHost>& worker_agent);
+      const scoped_refptr<content::DevToolsAgentHost>& worker_agent,
+      DevToolsOpenedByAction opened_by);
 
   // DevTools lifecycle typically follows this way:
   // - Toggle/Open: client call;
   // - Create;
   // - ScheduleShow: setup window to be functional, but not yet show;
-  // - DocumentOnLoadCompletedInMainFrame: frontend loaded;
+  // - DocumentOnLoadCompletedInPrimaryMainFrame: frontend loaded;
   // - SetIsDocked: frontend decided on docking state;
   // - OnLoadCompleted: ready to present frontend;
   // - Show: actually placing frontend WebContents to a Browser or docked place;
@@ -302,17 +330,21 @@ class DevToolsWindow : public DevToolsUIBindings::Delegate,
                  std::unique_ptr<content::WebContents> main_web_contents,
                  DevToolsUIBindings* bindings,
                  content::WebContents* inspected_web_contents,
-                 bool can_dock);
+                 bool can_dock,
+                 DevToolsOpenedByAction opened_by);
 
   // External frontend is always undocked.
   static void OpenExternalFrontend(
       Profile* profile,
       const std::string& frontend_uri,
       const scoped_refptr<content::DevToolsAgentHost>& agent_host,
-      bool use_bundled_frontend);
+      bool use_bundled_frontend,
+      DevToolsOpenedByAction opened_by);
+
   static void OpenDevToolsWindow(scoped_refptr<content::DevToolsAgentHost> host,
                                  Profile* profile,
-                                 bool use_bundled_frontend);
+                                 bool use_bundled_frontend,
+                                 DevToolsOpenedByAction opened_by);
 
   static DevToolsWindow* Create(Profile* profile,
                                 content::WebContents* inspected_web_contents,
@@ -321,16 +353,20 @@ class DevToolsWindow : public DevToolsUIBindings::Delegate,
                                 bool can_dock,
                                 const std::string& settings,
                                 const std::string& panel,
-                                bool has_other_clients);
+                                bool has_other_clients,
+                                bool browser_connection,
+                                DevToolsOpenedByAction opened_by);
   static GURL GetDevToolsURL(Profile* profile,
                              FrontendType frontend_type,
                              const std::string& frontend_url,
                              bool can_dock,
                              const std::string& panel,
-                             bool has_other_clients);
+                             bool has_other_clients,
+                             bool browser_connection);
 
   static void ToggleDevToolsWindow(
       content::WebContents* web_contents,
+      Profile* profile,
       bool force_open,
       const DevToolsToggleAction& action,
       const std::string& settings,
@@ -344,7 +380,7 @@ class DevToolsWindow : public DevToolsUIBindings::Delegate,
                       std::unique_ptr<content::WebContents> new_contents,
                       const GURL& target_url,
                       WindowOpenDisposition disposition,
-                      const gfx::Rect& initial_rect,
+                      const blink::mojom::WindowFeatures& window_features,
                       bool user_gesture,
                       bool* was_blocked) override;
   void WebContentsCreated(content::WebContents* source_contents,
@@ -366,16 +402,15 @@ class DevToolsWindow : public DevToolsUIBindings::Delegate,
       const content::NativeWebKeyboardEvent& event) override;
   content::JavaScriptDialogManager* GetJavaScriptDialogManager(
       content::WebContents* source) override;
-  content::ColorChooser* OpenColorChooser(
-      content::WebContents* web_contents,
-      SkColor color,
-      const std::vector<blink::mojom::ColorSuggestionPtr>& suggestions)
-      override;
+  std::unique_ptr<content::EyeDropper> OpenEyeDropper(
+      content::RenderFrameHost* render_frame_host,
+      content::EyeDropperListener* listener) override;
   void RunFileChooser(content::RenderFrameHost* render_frame_host,
                       scoped_refptr<content::FileSelectListener> listener,
                       const blink::mojom::FileChooserParams& params) override;
   bool PreHandleGestureEvent(content::WebContents* source,
                              const blink::WebGestureEvent& event) override;
+  void Close(DevToolsClosedByAction closed_by);
 
   // content::DevToolsUIBindings::Delegate overrides
   void ActivateWindow() override;
@@ -393,9 +428,12 @@ class DevToolsWindow : public DevToolsUIBindings::Delegate,
   void ReadyForTest() override;
   void ConnectionReady() override;
   void SetOpenNewWindowForPopups(bool value) override;
-  InfoBarService* GetInfoBarService() override;
+  infobars::ContentInfoBarManager* GetInfoBarManager() override;
   void RenderProcessGone(bool crashed) override;
   void ShowCertificateViewer(const std::string& cert_viewer) override;
+  int GetDockStateForLogging() override;
+  int GetOpenedByForLogging() override;
+  int GetClosedByForLogging() override;
 
   void ColorPickedInEyeDropper(int r, int g, int b, int a);
 
@@ -425,7 +463,7 @@ class DevToolsWindow : public DevToolsUIBindings::Delegate,
   void OnLocaleChanged();
   void OverrideAndSyncDevToolsRendererPrefs();
 
-  std::unique_ptr<ObserverWithAccessor> inspected_contents_observer_;
+  base::WeakPtr<content::WebContents> inspected_web_contents_;
 
   FrontendType frontend_type_;
   Profile* profile_;
@@ -467,6 +505,10 @@ class DevToolsWindow : public DevToolsUIBindings::Delegate,
   std::unique_ptr<DevToolsEventForwarder> event_forwarder_;
   std::unique_ptr<DevToolsEyeDropper> eye_dropper_;
 
+  const DevToolsOpenedByAction opened_by_;
+  DevToolsClosedByAction closed_by_;
+  const base::UnguessableToken session_id_for_logging_;
+
   class Throttle;
   Throttle* throttle_ = nullptr;
   bool open_new_window_for_popups_ = false;
@@ -475,8 +517,9 @@ class DevToolsWindow : public DevToolsUIBindings::Delegate,
 
   PrefChangeRegistrar pref_change_registrar_;
 
+  base::ScopedClosureRunner capture_handle_;
+
   friend class DevToolsEventForwarder;
-  DISALLOW_COPY_AND_ASSIGN(DevToolsWindow);
 };
 
 #endif  // CHROME_BROWSER_DEVTOOLS_DEVTOOLS_WINDOW_H_
