@@ -1,4 +1,4 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,51 +8,40 @@
 
 #include <limits>
 #include <memory>
+#include <string>
 #include <utility>
 
-#include "base/bind.h"
 #include "base/files/file_path.h"
-#include "base/i18n/file_util_icu.h"
-#include "base/i18n/time_formatting.h"
+#include "base/functional/bind.h"
 #include "base/lazy_instance.h"
 #include "base/path_service.h"
-#include "base/stl_util.h"
-#include "base/strings/string16.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
-#include "base/task/post_task.h"
-#include "base/task/thread_pool.h"
-#include "base/time/time.h"
 #include "build/build_config.h"
-#include "chrome/browser/bookmarks/bookmark_html_writer.h"
 #include "chrome/browser/bookmarks/bookmark_model_factory.h"
 #include "chrome/browser/bookmarks/managed_bookmark_service_factory.h"
 #include "chrome/browser/extensions/api/bookmarks/bookmark_api_constants.h"
 #include "chrome/browser/extensions/api/bookmarks/bookmark_api_helpers.h"
-#include "chrome/browser/importer/external_process_importer_host.h"
-#include "chrome/browser/importer/importer_uma.h"
+#include "chrome/browser/extensions/api/bookmarks/bookmarks_api_watcher.h"
 #include "chrome/browser/platform_util.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/ui/chrome_select_file_policy.h"
-#include "chrome/common/chrome_paths.h"
 #include "chrome/common/extensions/api/bookmarks.h"
 #include "chrome/common/importer/importer_data_types.h"
 #include "chrome/common/pref_names.h"
-#include "chrome/grit/generated_resources.h"
 #include "components/bookmarks/browser/bookmark_model.h"
 #include "components/bookmarks/browser/bookmark_utils.h"
+#include "components/bookmarks/common/bookmark_metrics.h"
 #include "components/bookmarks/common/bookmark_pref_names.h"
 #include "components/bookmarks/managed/managed_bookmark_service.h"
 #include "components/prefs/pref_service.h"
 #include "components/user_prefs/user_prefs.h"
 #include "content/public/browser/browser_context.h"
 #include "content/public/browser/browser_thread.h"
-#include "content/public/browser/notification_service.h"
 #include "content/public/browser/web_contents.h"
 #include "extensions/browser/event_router.h"
 #include "extensions/browser/extension_function_dispatcher.h"
-#include "extensions/browser/notification_types.h"
 #include "ui/base/l10n/l10n_util.h"
 
 using bookmarks::BookmarkModel;
@@ -67,29 +56,6 @@ using content::BrowserContext;
 using content::BrowserThread;
 using content::WebContents;
 
-namespace {
-
-// Generates a default path (including a default filename) that will be
-// used for pre-populating the "Export Bookmarks" file chooser dialog box.
-base::FilePath GetDefaultFilepathForBookmarkExport() {
-  base::Time time = base::Time::Now();
-
-  // Concatenate a date stamp to the filename.
-  std::string filename =
-      l10n_util::GetStringFUTF8(IDS_EXPORT_BOOKMARKS_DEFAULT_FILENAME,
-                                base::TimeFormatShortDateNumeric(time));
-  base::FilePath path = base::FilePath::FromUTF8Unsafe(filename);
-  base::FilePath::StringType path_str = path.value();
-  base::i18n::ReplaceIllegalCharactersInPath(&path_str, '_');
-  path = base::FilePath(path_str);
-
-  base::FilePath default_path;
-  base::PathService::Get(chrome::DIR_USER_DOCUMENTS, &default_path);
-  return default_path.Append(path);
-}
-
-}  // namespace
-
 ExtensionFunction::ResponseAction BookmarksFunction::Run() {
   BookmarkModel* model =
       BookmarkModelFactory::GetForBrowserContext(GetProfile());
@@ -101,7 +67,7 @@ ExtensionFunction::ResponseAction BookmarksFunction::Run() {
   }
 
   ResponseValue response = RunOnReady();
-  return response ? RespondNow(std::move(response)) : RespondLater();
+  return RespondNow(std::move(response));
 }
 
 BookmarkModel* BookmarksFunction::GetBookmarkModel() {
@@ -132,11 +98,10 @@ const BookmarkNode* BookmarksFunction::GetBookmarkNodeFromId(
 const BookmarkNode* BookmarksFunction::CreateBookmarkNode(
     BookmarkModel* model,
     const CreateDetails& details,
-    const BookmarkNode::MetaInfoMap* meta_info,
     std::string* error) {
   int64_t parent_id;
 
-  if (!details.parent_id.get()) {
+  if (!details.parent_id) {
     // Optional, default to "other bookmarks".
     parent_id = model->other_node()->id();
   } else if (!base::StringToInt64(*details.parent_id, &parent_id)) {
@@ -144,27 +109,32 @@ const BookmarkNode* BookmarksFunction::CreateBookmarkNode(
     return nullptr;
   }
   const BookmarkNode* parent = bookmarks::GetBookmarkNodeByID(model, parent_id);
-  if (!CanBeModified(parent, error))
+  if (!CanBeModified(parent, error)) {
     return nullptr;
+  }
+  if (!parent->is_folder()) {
+    *error = bookmark_api_constants::kInvalidParentError;
+    return nullptr;
+  }
 
   size_t index;
-  if (!details.index.get()) {  // Optional (defaults to end).
+  if (!details.index) {  // Optional (defaults to end).
     index = parent->children().size();
   } else {
     if (*details.index < 0 ||
-        size_t{*details.index} > parent->children().size()) {
+        static_cast<size_t>(*details.index) > parent->children().size()) {
       *error = bookmark_api_constants::kInvalidIndexError;
       return nullptr;
     }
-    index = size_t{*details.index};
+    index = static_cast<size_t>(*details.index);
   }
 
-  base::string16 title;  // Optional.
-  if (details.title.get())
+  std::u16string title;  // Optional.
+  if (details.title)
     title = base::UTF8ToUTF16(*details.title);
 
   std::string url_string;  // Optional.
-  if (details.url.get())
+  if (details.url)
     url_string = *details.url;
 
   GURL url(url_string);
@@ -175,9 +145,9 @@ const BookmarkNode* BookmarksFunction::CreateBookmarkNode(
 
   const BookmarkNode* node;
   if (url_string.length()) {
-    node = model->AddURL(parent, index, title, url, meta_info);
+    node = model->AddNewURL(parent, index, title, url);
   } else {
-    node = model->AddFolder(parent, index, title, meta_info);
+    node = model->AddFolder(parent, index, title);
     model->SetDateFolderModified(parent, base::Time::Now());
   }
 
@@ -216,10 +186,8 @@ Profile* BookmarksFunction::GetProfile() {
 void BookmarksFunction::OnResponded() {
   DCHECK(response_type());
   if (*response_type() == ExtensionFunction::SUCCEEDED) {
-    content::NotificationService::current()->Notify(
-        extensions::NOTIFICATION_EXTENSION_BOOKMARKS_API_INVOKED,
-        content::Source<const Extension>(extension()),
-        content::Details<const BookmarksFunction>(this));
+    BookmarksApiWatcher::GetForBrowserContext(browser_context())
+        ->NotifyApiInvoked(extension(), this);
   }
 }
 
@@ -231,9 +199,7 @@ void BookmarksFunction::BookmarkModelLoaded(BookmarkModel* model,
   model->RemoveObserver(this);
 
   ResponseValue response = RunOnReady();
-  if (response)
-    Respond(std::move(response));
-  // else, the function will Respond() on its own later.
+  Respond(std::move(response));
 
   Release();  // Balanced in Run().
 }
@@ -251,10 +217,9 @@ BookmarkEventRouter::~BookmarkEventRouter() {
   }
 }
 
-void BookmarkEventRouter::DispatchEvent(
-    events::HistogramValue histogram_value,
-    const std::string& event_name,
-    std::unique_ptr<base::ListValue> event_args) {
+void BookmarkEventRouter::DispatchEvent(events::HistogramValue histogram_value,
+                                        const std::string& event_name,
+                                        base::Value::List event_args) {
   EventRouter* event_router = EventRouter::Get(browser_context_);
   if (event_router) {
     event_router->BroadcastEvent(std::make_unique<extensions::Event>(
@@ -280,9 +245,9 @@ void BookmarkEventRouter::BookmarkNodeMoved(BookmarkModel* model,
   const BookmarkNode* node = new_parent->children()[new_index].get();
   api::bookmarks::OnMoved::MoveInfo move_info;
   move_info.parent_id = base::NumberToString(new_parent->id());
-  move_info.index = int{new_index};
+  move_info.index = static_cast<int>(new_index);
   move_info.old_parent_id = base::NumberToString(old_parent->id());
-  move_info.old_index = int{old_index};
+  move_info.old_index = static_cast<int>(old_index);
 
   DispatchEvent(events::BOOKMARKS_ON_MOVED, api::bookmarks::OnMoved::kEventName,
                 api::bookmarks::OnMoved::Create(
@@ -291,7 +256,8 @@ void BookmarkEventRouter::BookmarkNodeMoved(BookmarkModel* model,
 
 void BookmarkEventRouter::BookmarkNodeAdded(BookmarkModel* model,
                                             const BookmarkNode* parent,
-                                            size_t index) {
+                                            size_t index,
+                                            bool added_by_user) {
   const BookmarkNode* node = parent->children()[index].get();
   BookmarkTreeNode tree_node =
       bookmark_api_helpers::GetBookmarkTreeNode(managed_, node, false, false);
@@ -309,7 +275,7 @@ void BookmarkEventRouter::BookmarkNodeRemoved(
     const std::set<GURL>& removed_urls) {
   api::bookmarks::OnRemoved::RemoveInfo remove_info;
   remove_info.parent_id = base::NumberToString(parent->id());
-  remove_info.index = int{index};
+  remove_info.index = static_cast<int>(index);
   bookmark_api_helpers::PopulateBookmarkTreeNode(managed_, node, true, false,
                                                  &remove_info.node);
 
@@ -322,10 +288,8 @@ void BookmarkEventRouter::BookmarkNodeRemoved(
 void BookmarkEventRouter::BookmarkAllUserNodesRemoved(
     BookmarkModel* model,
     const std::set<GURL>& removed_urls) {
-  NOTREACHED();
-  // TODO(shashishekhar) Currently this notification is only used on Android,
-  // which does not support extensions. If Desktop needs to support this, add
-  // a new event to the extensions api.
+  // TODO(crbug.com/1468324): This used to be used only on Android, but that's
+  // no longer the case. We need to implement a new event to handle this.
 }
 
 void BookmarkEventRouter::BookmarkNodeChanged(BookmarkModel* model,
@@ -338,7 +302,7 @@ void BookmarkEventRouter::BookmarkNodeChanged(BookmarkModel* model,
   api::bookmarks::OnChanged::ChangeInfo change_info;
   change_info.title = base::UTF16ToUTF8(node->GetTitle());
   if (node->is_url())
-    change_info.url.reset(new std::string(node->url().spec()));
+    change_info.url = node->url().spec();
 
   DispatchEvent(events::BOOKMARKS_ON_CHANGED,
                 api::bookmarks::OnChanged::kEventName,
@@ -409,14 +373,14 @@ BookmarksAPI::GetFactoryInstance() {
 }
 
 void BookmarksAPI::OnListenerAdded(const EventListenerInfo& details) {
-  bookmark_event_router_.reset(
-      new BookmarkEventRouter(Profile::FromBrowserContext(browser_context_)));
+  bookmark_event_router_ = std::make_unique<BookmarkEventRouter>(
+      Profile::FromBrowserContext(browser_context_));
   EventRouter::Get(browser_context_)->UnregisterObserver(this);
 }
 
 ExtensionFunction::ResponseValue BookmarksGetFunction::RunOnReady() {
-  std::unique_ptr<api::bookmarks::Get::Params> params(
-      api::bookmarks::Get::Params::Create(*args_));
+  std::optional<api::bookmarks::Get::Params> params =
+      api::bookmarks::Get::Params::Create(args());
   if (!params)
     return BadMessage();
 
@@ -447,8 +411,8 @@ ExtensionFunction::ResponseValue BookmarksGetFunction::RunOnReady() {
 }
 
 ExtensionFunction::ResponseValue BookmarksGetChildrenFunction::RunOnReady() {
-  std::unique_ptr<api::bookmarks::GetChildren::Params> params(
-      api::bookmarks::GetChildren::Params::Create(*args_));
+  std::optional<api::bookmarks::GetChildren::Params> params =
+      api::bookmarks::GetChildren::Params::Create(args());
   if (!params)
     return BadMessage();
 
@@ -467,8 +431,8 @@ ExtensionFunction::ResponseValue BookmarksGetChildrenFunction::RunOnReady() {
 }
 
 ExtensionFunction::ResponseValue BookmarksGetRecentFunction::RunOnReady() {
-  std::unique_ptr<api::bookmarks::GetRecent::Params> params(
-      api::bookmarks::GetRecent::Params::Create(*args_));
+  std::optional<api::bookmarks::GetRecent::Params> params =
+      api::bookmarks::GetRecent::Params::Create(args());
   if (!params)
     return BadMessage();
   if (params->number_of_items < 1) {
@@ -501,8 +465,8 @@ ExtensionFunction::ResponseValue BookmarksGetTreeFunction::RunOnReady() {
 }
 
 ExtensionFunction::ResponseValue BookmarksGetSubTreeFunction::RunOnReady() {
-  std::unique_ptr<api::bookmarks::GetSubTree::Params> params(
-      api::bookmarks::GetSubTree::Params::Create(*args_));
+  std::optional<api::bookmarks::GetSubTree::Params> params =
+      api::bookmarks::GetSubTree::Params::Create(args());
   if (!params)
     return BadMessage();
 
@@ -518,16 +482,16 @@ ExtensionFunction::ResponseValue BookmarksGetSubTreeFunction::RunOnReady() {
 }
 
 ExtensionFunction::ResponseValue BookmarksSearchFunction::RunOnReady() {
-  std::unique_ptr<api::bookmarks::Search::Params> params(
-      api::bookmarks::Search::Params::Create(*args_));
+  std::optional<api::bookmarks::Search::Params> params =
+      api::bookmarks::Search::Params::Create(args());
   if (!params)
     return BadMessage();
 
   std::vector<const BookmarkNode*> nodes;
   if (params->query.as_string) {
     bookmarks::QueryFields query;
-    query.word_phrase_query.reset(
-        new base::string16(base::UTF8ToUTF16(*params->query.as_string)));
+    query.word_phrase_query = std::make_unique<std::u16string>(
+        base::UTF8ToUTF16(*params->query.as_string));
     bookmarks::GetBookmarksMatchingProperties(
         BookmarkModelFactory::GetForBrowserContext(GetProfile()), query,
         std::numeric_limits<int>::max(), &nodes);
@@ -537,13 +501,15 @@ ExtensionFunction::ResponseValue BookmarksSearchFunction::RunOnReady() {
         *params->query.as_object;
     bookmarks::QueryFields query;
     if (object.query) {
-      query.word_phrase_query.reset(
-          new base::string16(base::UTF8ToUTF16(*object.query)));
+      query.word_phrase_query =
+          std::make_unique<std::u16string>(base::UTF8ToUTF16(*object.query));
     }
     if (object.url)
-      query.url.reset(new base::string16(base::UTF8ToUTF16(*object.url)));
+      query.url =
+          std::make_unique<std::u16string>(base::UTF8ToUTF16(*object.url));
     if (object.title)
-      query.title.reset(new base::string16(base::UTF8ToUTF16(*object.title)));
+      query.title =
+          std::make_unique<std::u16string>(base::UTF8ToUTF16(*object.title));
     bookmarks::GetBookmarksMatchingProperties(
         BookmarkModelFactory::GetForBrowserContext(GetProfile()), query,
         std::numeric_limits<int>::max(), &nodes);
@@ -561,8 +527,8 @@ ExtensionFunction::ResponseValue BookmarksRemoveFunctionBase::RunOnReady() {
   if (!EditBookmarksEnabled())
     return Error(bookmark_api_constants::kEditBookmarksDisabled);
 
-  std::unique_ptr<api::bookmarks::Remove::Params> params(
-      api::bookmarks::Remove::Params::Create(*args_));
+  std::optional<api::bookmarks::Remove::Params> params =
+      api::bookmarks::Remove::Params::Create(args());
   if (!params)
     return BadMessage();
 
@@ -593,8 +559,8 @@ ExtensionFunction::ResponseValue BookmarksCreateFunction::RunOnReady() {
   if (!EditBookmarksEnabled())
     return Error(bookmark_api_constants::kEditBookmarksDisabled);
 
-  std::unique_ptr<api::bookmarks::Create::Params> params(
-      api::bookmarks::Create::Params::Create(*args_));
+  std::optional<api::bookmarks::Create::Params> params =
+      api::bookmarks::Create::Params::Create(args());
   if (!params)
     return BadMessage();
 
@@ -602,7 +568,7 @@ ExtensionFunction::ResponseValue BookmarksCreateFunction::RunOnReady() {
   BookmarkModel* model =
       BookmarkModelFactory::GetForBrowserContext(GetProfile());
   const BookmarkNode* node =
-      CreateBookmarkNode(model, params->bookmark, nullptr, &error);
+      CreateBookmarkNode(model, params->bookmark, &error);
   if (!node)
     return Error(error);
 
@@ -615,8 +581,8 @@ ExtensionFunction::ResponseValue BookmarksMoveFunction::RunOnReady() {
   if (!EditBookmarksEnabled())
     return Error(bookmark_api_constants::kEditBookmarksDisabled);
 
-  std::unique_ptr<api::bookmarks::Move::Params> params(
-      api::bookmarks::Move::Params::Create(*args_));
+  std::optional<api::bookmarks::Move::Params> params =
+      api::bookmarks::Move::Params::Create(args());
   if (!params)
     return BadMessage();
 
@@ -631,7 +597,7 @@ ExtensionFunction::ResponseValue BookmarksMoveFunction::RunOnReady() {
     return Error(bookmark_api_constants::kModifySpecialError);
 
   const BookmarkNode* parent = nullptr;
-  if (!params->destination.parent_id.get()) {
+  if (!params->destination.parent_id) {
     // Optional, defaults to current parent.
     parent = node->parent();
   } else {
@@ -641,16 +607,23 @@ ExtensionFunction::ResponseValue BookmarksMoveFunction::RunOnReady() {
 
     parent = bookmarks::GetBookmarkNodeByID(model, parent_id);
   }
-  if (!CanBeModified(parent, &error) || !CanBeModified(node, &error))
+
+  if (!CanBeModified(parent, &error) || !CanBeModified(node, &error)) {
     return Error(error);
+  }
+
+  if (!parent->is_folder()) {
+    return Error(bookmark_api_constants::kInvalidParentError);
+  }
 
   size_t index;
-  if (params->destination.index.get()) {  // Optional (defaults to end).
+  if (params->destination.index) {  // Optional (defaults to end).
     if (*params->destination.index < 0 ||
-        size_t{*params->destination.index} > parent->children().size()) {
+        static_cast<size_t>(*params->destination.index) >
+            parent->children().size()) {
       return Error(bookmark_api_constants::kInvalidIndexError);
     }
-    index = size_t{*params->destination.index};
+    index = static_cast<size_t>(*params->destination.index);
   } else {
     index = parent->children().size();
   }
@@ -666,22 +639,22 @@ ExtensionFunction::ResponseValue BookmarksUpdateFunction::RunOnReady() {
   if (!EditBookmarksEnabled())
     return Error(bookmark_api_constants::kEditBookmarksDisabled);
 
-  std::unique_ptr<api::bookmarks::Update::Params> params(
-      api::bookmarks::Update::Params::Create(*args_));
+  std::optional<api::bookmarks::Update::Params> params =
+      api::bookmarks::Update::Params::Create(args());
   if (!params)
     return BadMessage();
 
   // Optional but we need to distinguish non present from an empty title.
-  base::string16 title;
+  std::u16string title;
   bool has_title = false;
-  if (params->changes.title.get()) {
+  if (params->changes.title) {
     title = base::UTF8ToUTF16(*params->changes.title);
     has_title = true;
   }
 
   // Optional.
   std::string url_string;
-  if (params->changes.url.get())
+  if (params->changes.url)
     url_string = *params->changes.url;
   GURL url(url_string);
   if (!url_string.empty() && !url.is_valid())
@@ -701,118 +674,15 @@ ExtensionFunction::ResponseValue BookmarksUpdateFunction::RunOnReady() {
     return Error(bookmark_api_constants::kCannotSetUrlOfFolderError);
 
   if (has_title)
-    model->SetTitle(node, title);
+    model->SetTitle(node, title,
+                    bookmarks::metrics::BookmarkEditSource::kExtension);
   if (!url.is_empty())
-    model->SetURL(node, url);
+    model->SetURL(node, url,
+                  bookmarks::metrics::BookmarkEditSource::kExtension);
 
   BookmarkTreeNode tree_node = bookmark_api_helpers::GetBookmarkTreeNode(
       GetManagedBookmarkService(), node, false, false);
   return ArgumentList(api::bookmarks::Update::Results::Create(tree_node));
 }
 
-BookmarksIOFunction::BookmarksIOFunction() {}
-
-BookmarksIOFunction::~BookmarksIOFunction() {
-  // There may be pending file dialogs, we need to tell them that we've gone
-  // away so they don't try and call back to us.
-  if (select_file_dialog_.get())
-    select_file_dialog_->ListenerDestroyed();
-}
-
-void BookmarksIOFunction::ShowSelectFileDialog(
-    ui::SelectFileDialog::Type type,
-    const base::FilePath& default_path) {
-  if (!dispatcher())
-    return;  // Extension was unloaded.
-
-  DCHECK_CURRENTLY_ON(content::BrowserThread::UI);
-
-  // Balanced in one of the three callbacks of SelectFileDialog:
-  // either FileSelectionCanceled, MultiFilesSelected, or FileSelected
-  AddRef();
-
-  WebContents* web_contents = GetSenderWebContents();
-
-  select_file_dialog_ = ui::SelectFileDialog::Create(
-      this, std::make_unique<ChromeSelectFilePolicy>(web_contents));
-  ui::SelectFileDialog::FileTypeInfo file_type_info;
-  file_type_info.extensions.resize(1);
-  file_type_info.extensions[0].push_back(FILE_PATH_LITERAL("html"));
-  gfx::NativeWindow owning_window =
-      web_contents ? platform_util::GetTopLevel(web_contents->GetNativeView())
-                   : gfx::kNullNativeWindow;
-  // |web_contents| can be nullptr (for background pages), which is fine. In
-  // such a case if file-selection dialogs are forbidden by policy, we will not
-  // show an InfoBar, which is better than letting one appear out of the blue.
-  select_file_dialog_->SelectFile(
-      type, base::string16(), default_path, &file_type_info, 0,
-      base::FilePath::StringType(), owning_window, nullptr);
-}
-
-void BookmarksIOFunction::FileSelectionCanceled(void* params) {
-  Release();  // Balanced in BookmarksIOFunction::SelectFile()
-}
-
-void BookmarksIOFunction::MultiFilesSelected(
-    const std::vector<base::FilePath>& files, void* params) {
-  Release();  // Balanced in BookmarsIOFunction::SelectFile()
-  NOTREACHED() << "Should not be able to select multiple files";
-}
-
-ExtensionFunction::ResponseValue BookmarksImportFunction::RunOnReady() {
-  if (!EditBookmarksEnabled())
-    return Error(bookmark_api_constants::kEditBookmarksDisabled);
-  ShowSelectFileDialog(ui::SelectFileDialog::SELECT_OPEN_FILE,
-                       base::FilePath());
-  // TODO(crbug.com/1073255): This will respond before a file is selected, which
-  // seems incorrect. Waiting and responding until after
-  // ui::SelectFileDialog::Listener is fired should be right thing to do, but
-  // that requires auditing bookmark page callsites.
-  return NoArguments();
-}
-
-void BookmarksImportFunction::FileSelected(const base::FilePath& path,
-                                           int index,
-                                           void* params) {
-  // Deletes itself.
-  ExternalProcessImporterHost* importer_host = new ExternalProcessImporterHost;
-  importer::SourceProfile source_profile;
-  source_profile.importer_type = importer::TYPE_BOOKMARKS_FILE;
-  source_profile.source_path = path;
-  importer_host->StartImportSettings(source_profile,
-                                     GetProfile(),
-                                     importer::FAVORITES,
-                                     new ProfileWriter(GetProfile()));
-
-  importer::LogImporterUseToMetrics("BookmarksAPI",
-                                    importer::TYPE_BOOKMARKS_FILE);
-  Release();  // Balanced in BookmarksIOFunction::SelectFile()
-}
-
-ExtensionFunction::ResponseValue BookmarksExportFunction::RunOnReady() {
-  // "bookmarks.export" is exposed to a small number of extensions. These
-  // extensions use user gesture for export, so use USER_VISIBLE priority.
-  // GetDefaultFilepathForBookmarkExport() might have to touch filesystem
-  // (stat or access, for example), so this requires IO.
-  base::ThreadPool::PostTaskAndReplyWithResult(
-      FROM_HERE,
-      {base::MayBlock(), base::TaskPriority::USER_VISIBLE,
-       base::TaskShutdownBehavior::SKIP_ON_SHUTDOWN},
-      base::BindOnce(&GetDefaultFilepathForBookmarkExport),
-      base::BindOnce(&BookmarksIOFunction::ShowSelectFileDialog, this,
-                     ui::SelectFileDialog::SELECT_SAVEAS_FILE));
-  // TODO(crbug.com/1073255): This will respond before a file is selected, which
-  // seems incorrect. Waiting and responding until after
-  // ui::SelectFileDialog::Listener is fired should be right thing to do, but
-  // that requires auditing bookmark page callsites.
-  return NoArguments();
-}
-
-void BookmarksExportFunction::FileSelected(const base::FilePath& path,
-                                           int index,
-                                           void* params) {
-  bookmark_html_writer::WriteBookmarks(GetProfile(), path, nullptr);
-  Release();  // Balanced in BookmarksIOFunction::SelectFile()
-}
-
 }  // namespace extensions
