@@ -1,4 +1,4 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,37 +10,39 @@
 
 #include <map>
 #include <memory>
+#include <optional>
 #include <set>
 #include <utility>
 
-#include "base/bind.h"
 #include "base/command_line.h"
+#include "base/functional/bind.h"
 #include "base/json/json_reader.h"
 #include "base/json/json_writer.h"
 #include "base/lazy_instance.h"
-#include "base/macros.h"
+#include "base/memory/raw_ptr.h"
 #include "base/memory/singleton.h"
-#include "base/scoped_observer.h"
-#include "base/stl_util.h"
+#include "base/metrics/histogram_functions.h"
+#include "base/ranges/algorithm.h"
+#include "base/scoped_observation.h"
 #include "base/strings/string_number_conversions.h"
-#include "base/task/post_task.h"
+#include "base/types/optional_util.h"
 #include "base/values.h"
-#include "chrome/browser/chrome_notification_types.h"
 #include "chrome/browser/devtools/chrome_devtools_manager_delegate.h"
 #include "chrome/browser/extensions/api/debugger/debugger_api_constants.h"
 #include "chrome/browser/extensions/api/debugger/extension_dev_tools_infobar_delegate.h"
 #include "chrome/browser/extensions/extension_service.h"
 #include "chrome/browser/extensions/extension_tab_util.h"
-#include "chrome/browser/extensions/extension_util.h"
+#include "chrome/browser/lifetime/termination_notification.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/webui/chrome_web_ui_controller_factory.h"
 #include "chrome/common/chrome_switches.h"
+#include "chrome/common/pref_names.h"
+#include "components/security_interstitials/content/security_interstitial_tab_helper.h"
 #include "content/public/browser/browser_task_traits.h"
 #include "content/public/browser/browser_thread.h"
 #include "content/public/browser/devtools_agent_host.h"
-#include "content/public/browser/notification_service.h"
-#include "content/public/browser/notification_source.h"
+#include "content/public/browser/navigation_entry.h"
 #include "content/public/browser/render_process_host.h"
 #include "content/public/browser/render_widget_host.h"
 #include "content/public/browser/web_contents.h"
@@ -51,12 +53,15 @@
 #include "extensions/browser/extension_host.h"
 #include "extensions/browser/extension_registry.h"
 #include "extensions/browser/extension_registry_observer.h"
+#include "extensions/browser/extension_util.h"
+#include "extensions/browser/guest_view/mime_handler_view/mime_handler_view_guest.h"
 #include "extensions/common/constants.h"
 #include "extensions/common/error_utils.h"
 #include "extensions/common/extension.h"
 #include "extensions/common/manifest_constants.h"
 #include "extensions/common/permissions/permissions_data.h"
 #include "extensions/common/switches.h"
+#include "url/origin.h"
 
 using content::DevToolsAgentHost;
 using content::RenderProcessHost;
@@ -78,19 +83,28 @@ namespace {
 // Helpers --------------------------------------------------------------------
 
 void CopyDebuggee(Debuggee* dst, const Debuggee& src) {
-  if (src.tab_id)
-    dst->tab_id.reset(new int(*src.tab_id));
-  if (src.extension_id)
-    dst->extension_id.reset(new std::string(*src.extension_id));
-  if (src.target_id)
-    dst->target_id.reset(new std::string(*src.target_id));
+  dst->tab_id = src.tab_id;
+  dst->extension_id = src.extension_id;
+  dst->target_id = src.target_id;
+}
+
+bool ExtensionMayAttachToTargetProfile(Profile* extension_profile,
+                                       bool allow_incognito_access,
+                                       DevToolsAgentHost& agent_host) {
+  Profile* profile =
+      Profile::FromBrowserContext(agent_host.GetBrowserContext());
+  if (!profile)
+    return false;
+  if (!extension_profile->IsSameOrParent(profile))
+    return false;
+  return profile == extension_profile || allow_incognito_access;
 }
 
 // Returns true if the given |Extension| is allowed to attach to the specified
 // |url|.
 bool ExtensionMayAttachToURL(const Extension& extension,
+                             Profile* extension_profile,
                              const GURL& url,
-                             Profile* profile,
                              std::string* error) {
   // Allow the extension to attach to about:blank and empty URLs.
   if (url.is_empty() || url == "about:")
@@ -109,7 +123,8 @@ bool ExtensionMayAttachToURL(const Extension& extension,
   if (extension.permissions_data()->IsPolicyBlockedHost(url))
     return false;
 
-  if (url.SchemeIsFile() && !util::AllowFileAccess(extension.id(), profile)) {
+  if (url.SchemeIsFile() &&
+      !util::AllowFileAccess(extension.id(), extension_profile)) {
     *error = debugger_api_constants::kRestrictedError;
     return false;
   }
@@ -117,44 +132,117 @@ bool ExtensionMayAttachToURL(const Extension& extension,
   return true;
 }
 
-constexpr char kBrowserTargetId[] = "browser";
-
-constexpr char kPerfettoUIExtensionId[] = "lfmkphfpdbjijhpomgecfikhfohaoine";
-
-bool ExtensionMayAttachToBrowser(const Extension& extension) {
-  return extension.id() == kPerfettoUIExtensionId;
-}
-
-bool ExtensionMayAttachToWebContents(const Extension& extension,
-                                     WebContents& web_contents,
-                                     Profile* profile,
-                                     std::string* error) {
-  // This is *not* redundant to the checks below, as
-  // web_contents.GetLastCommittedURL() may be different from
-  // web_contents.GetMainFrame()->GetLastCommittedURL(), with the
-  // former being a 'virtual' URL as obtained from NavigationEntry.
-  if (!ExtensionMayAttachToURL(extension, web_contents.GetLastCommittedURL(),
-                               profile, error)) {
+bool ExtensionMayAttachToURLOrInnerURL(const Extension& extension,
+                                       Profile* extension_profile,
+                                       const GURL& url,
+                                       std::string* error) {
+  if (!ExtensionMayAttachToURL(extension, extension_profile, url, error))
+    return false;
+  // For nested URLs, make sure ExtensionMayAttachToURL() allows both
+  // the outer and the inner URLs.
+  if (url.inner_url() && !ExtensionMayAttachToURL(extension, extension_profile,
+                                                  *url.inner_url(), error)) {
     return false;
-  }
-
-  for (content::RenderFrameHost* rfh : web_contents.GetAllFrames()) {
-    if (!ExtensionMayAttachToURL(extension, rfh->GetLastCommittedURL(), profile,
-                                 error))
-      return false;
   }
   return true;
 }
 
-bool ExtensionMayAttachToAgentHost(const Extension& extension,
-                                   DevToolsAgentHost& agent_host,
-                                   Profile* profile,
-                                   std::string* error) {
-  if (WebContents* wc = agent_host.GetWebContents())
-    return ExtensionMayAttachToWebContents(extension, *wc, profile, error);
+constexpr char kBrowserTargetId[] = "browser";
 
-  return ExtensionMayAttachToURL(extension, agent_host.GetURL(), profile,
-                                 error);
+constexpr char kPerfettoUIExtensionId[] = "lfmkphfpdbjijhpomgecfikhfohaoine";
+
+bool ExtensionIsTrusted(const Extension& extension) {
+  return extension.id() == kPerfettoUIExtensionId;
+}
+
+bool ExtensionMayAttachToRenderFrameHost(
+    const Extension& extension,
+    Profile* extension_profile,
+    content::RenderFrameHost* render_frame_host,
+    std::string* error) {
+  bool result = true;
+  render_frame_host->ForEachRenderFrameHostWithAction(
+      [&extension, extension_profile, error,
+       &result](content::RenderFrameHost* render_frame_host) {
+        // If |render_frame_host| is attached to an inner MimeHandlerViewGuest
+        // skip it. This is done to fix crbug.com/1293856 because an extension
+        // cannot inspect another extension.
+        if (MimeHandlerViewGuest::FromRenderFrameHost(render_frame_host)) {
+          return content::RenderFrameHost::FrameIterationAction::kSkipChildren;
+        }
+
+        if (render_frame_host->GetWebUI()) {
+          *error = debugger_api_constants::kRestrictedError;
+          result = false;
+          return content::RenderFrameHost::FrameIterationAction::kStop;
+        }
+
+        // We check both the last committed URL and the SiteURL because this
+        // method may be called in the middle of a navigation where the SiteURL
+        // has been updated but navigation hasn't committed yet.
+        if (!ExtensionMayAttachToURLOrInnerURL(
+                extension, extension_profile,
+                render_frame_host->GetLastCommittedURL(), error) ||
+            !ExtensionMayAttachToURLOrInnerURL(
+                extension, extension_profile,
+                render_frame_host->GetSiteInstance()->GetSiteURL(), error)) {
+          result = false;
+          return content::RenderFrameHost::FrameIterationAction::kStop;
+        }
+
+        return content::RenderFrameHost::FrameIterationAction::kContinue;
+      });
+  return result;
+}
+
+bool ExtensionMayAttachToWebContents(const Extension& extension,
+                                     Profile* extension_profile,
+                                     WebContents& web_contents,
+                                     std::string* error) {
+  security_interstitials::SecurityInterstitialTabHelper*
+      security_interstitial_tab_helper = security_interstitials::
+          SecurityInterstitialTabHelper::FromWebContents(&web_contents);
+  if (security_interstitial_tab_helper &&
+      security_interstitial_tab_helper->IsDisplayingInterstitial()) {
+    *error = debugger_api_constants::kRestrictedError;
+    return false;
+  }
+  // This is *not* redundant to the checks below, as
+  // web_contents.GetLastCommittedURL() may be different from
+  // web_contents.GetPrimaryMainFrame()->GetLastCommittedURL(), with the
+  // former being a 'virtual' URL as obtained from NavigationEntry.
+  if (!ExtensionMayAttachToURL(extension, extension_profile,
+                               web_contents.GetLastCommittedURL(), error)) {
+    return false;
+  }
+  if (web_contents.GetController().GetPendingEntry() &&
+      !ExtensionMayAttachToURL(
+          extension, extension_profile,
+          web_contents.GetController().GetPendingEntry()->GetURL(), error)) {
+    return false;
+  }
+
+  return ExtensionMayAttachToRenderFrameHost(
+      extension, extension_profile, web_contents.GetPrimaryMainFrame(), error);
+}
+
+bool ExtensionMayAttachToAgentHost(const Extension& extension,
+                                   bool allow_incognito_access,
+                                   Profile* extension_profile,
+                                   DevToolsAgentHost& agent_host,
+                                   std::string* error) {
+  if (!ExtensionMayAttachToTargetProfile(extension_profile,
+                                         allow_incognito_access, agent_host)) {
+    *error = debugger_api_constants::kRestrictedError;
+    return false;
+  }
+  if (WebContents* wc = agent_host.GetWebContents()) {
+    return ExtensionMayAttachToWebContents(extension, extension_profile, *wc,
+                                           error);
+  }
+
+  return ExtensionMayAttachToURL(extension, extension_profile,
+                                 agent_host.GetURL(), error);
 }
 
 }  // namespace
@@ -166,16 +254,23 @@ base::LazyInstance<AttachedClientHosts>::Leaky g_attached_client_hosts =
     LAZY_INSTANCE_INITIALIZER;
 
 class ExtensionDevToolsClientHost : public content::DevToolsAgentHostClient,
-                                    public content::NotificationObserver,
                                     public ExtensionRegistryObserver {
  public:
-  ExtensionDevToolsClientHost(Profile* profile,
-                              DevToolsAgentHost* agent_host,
-                              scoped_refptr<const Extension> extension,
-                              const Debuggee& debuggee);
+  ExtensionDevToolsClientHost(
+      Profile* profile,
+      DevToolsAgentHost* agent_host,
+      scoped_refptr<const Extension> extension,
+      std::optional<WorkerId> extension_service_worker_id,
+      const Debuggee& debuggee);
+
+  ExtensionDevToolsClientHost(const ExtensionDevToolsClientHost&) = delete;
+  ExtensionDevToolsClientHost& operator=(const ExtensionDevToolsClientHost&) =
+      delete;
 
   ~ExtensionDevToolsClientHost() override;
 
+  std::string GetTypeForMetrics() override { return "Extension"; }
+
   bool Attach();
   const std::string& extension_id() { return extension_->id(); }
   DevToolsAgentHost* agent_host() { return agent_host_.get(); }
@@ -192,10 +287,13 @@ class ExtensionDevToolsClientHost : public content::DevToolsAgentHostClient,
   void AgentHostClosed(DevToolsAgentHost* agent_host) override;
   void DispatchProtocolMessage(DevToolsAgentHost* agent_host,
                                base::span<const uint8_t> message) override;
+  bool MayAttachToRenderFrameHost(
+      content::RenderFrameHost* render_frame_host) override;
   bool MayAttachToURL(const GURL& url, bool is_webui) override;
-  bool MayAttachToBrowser() override;
+  bool IsTrusted() override;
   bool MayReadLocalFiles() override;
   bool MayWriteLocalFiles() override;
+  std::optional<url::Origin> GetNavigationInitiatorOrigin() override;
 
  private:
   using PendingRequests =
@@ -203,55 +301,63 @@ class ExtensionDevToolsClientHost : public content::DevToolsAgentHostClient,
 
   void SendDetachedEvent();
 
-  // content::NotificationObserver implementation.
-  void Observe(int type,
-               const content::NotificationSource& source,
-               const content::NotificationDetails& details) override;
+  void OnAppTerminating();
 
   // ExtensionRegistryObserver implementation.
   void OnExtensionUnloaded(content::BrowserContext* browser_context,
                            const Extension* extension,
                            UnloadedExtensionReason reason) override;
 
-  Profile* profile_;
+  raw_ptr<Profile> profile_;
   scoped_refptr<DevToolsAgentHost> agent_host_;
   scoped_refptr<const Extension> extension_;
+  // The WorkerId of the extension service worker that called attach() for this
+  // client host, if any.
+  const std::optional<WorkerId> extension_service_worker_id_;
+
   Debuggee debuggee_;
-  content::NotificationRegistrar registrar_;
+  base::CallbackListSubscription on_app_terminating_subscription_;
   int last_request_id_ = 0;
   PendingRequests pending_requests_;
   base::CallbackListSubscription subscription_;
   api::debugger::DetachReason detach_reason_ =
-      api::debugger::DETACH_REASON_TARGET_CLOSED;
+      api::debugger::DetachReason::kTargetClosed;
+
+  // A service worker keepalive used to keep the associated worker alive while
+  // this client is attached. Only used if `extension_service_worker_id_` has a
+  // value.
+  std::optional<base::Uuid> service_worker_keepalive_;
 
   // Listen to extension unloaded notification.
-  ScopedObserver<ExtensionRegistry, ExtensionRegistryObserver>
-      extension_registry_observer_{this};
-
-  DISALLOW_COPY_AND_ASSIGN(ExtensionDevToolsClientHost);
+  base::ScopedObservation<ExtensionRegistry, ExtensionRegistryObserver>
+      extension_registry_observation_{this};
 };
 
 ExtensionDevToolsClientHost::ExtensionDevToolsClientHost(
     Profile* profile,
     DevToolsAgentHost* agent_host,
     scoped_refptr<const Extension> extension,
+    std::optional<WorkerId> extension_service_worker_id,
     const Debuggee& debuggee)
     : profile_(profile),
       agent_host_(agent_host),
-      extension_(std::move(extension)) {
+      extension_(std::move(extension)),
+      extension_service_worker_id_(std::move(extension_service_worker_id)) {
   CopyDebuggee(&debuggee_, debuggee);
 
   g_attached_client_hosts.Get().insert(this);
 
   // ExtensionRegistryObserver listen extension unloaded and detach debugger
   // from there.
-  extension_registry_observer_.Add(ExtensionRegistry::Get(profile_));
+  extension_registry_observation_.Observe(ExtensionRegistry::Get(profile_));
 
   // RVH-based agents disconnect from their clients when the app is terminating
   // but shared worker-based agents do not.
   // Disconnect explicitly to make sure that |this| observer is not leaked.
-  registrar_.Add(this, chrome::NOTIFICATION_APP_TERMINATING,
-                 content::NotificationService::AllSources());
+  on_app_terminating_subscription_ =
+      browser_shutdown::AddAppTerminatingCallback(
+          base::BindOnce(&ExtensionDevToolsClientHost::OnAppTerminating,
+                         base::Unretained(this)));
 }
 
 bool ExtensionDevToolsClientHost::Attach() {
@@ -273,12 +379,37 @@ bool ExtensionDevToolsClientHost::Attach() {
       extension_id(), extension_->name(),
       base::BindOnce(&ExtensionDevToolsClientHost::InfoBarDestroyed,
                      base::Unretained(this)));
+  if (extension_service_worker_id_) {
+    ProcessManager* process_manager = ProcessManager::Get(profile_);
+    CHECK(process_manager);
+    // The service worker should definitely be registered at this point.
+    CHECK(process_manager->HasServiceWorker(*extension_service_worker_id_));
+    service_worker_keepalive_ =
+        process_manager->IncrementServiceWorkerKeepaliveCount(
+            *extension_service_worker_id_,
+            content::ServiceWorkerExternalRequestTimeoutType::kDoesNotTimeout,
+            Activity::DEBUGGER, /*extra_data=*/std::string());
+  }
+
   return true;
 }
 
 ExtensionDevToolsClientHost::~ExtensionDevToolsClientHost() {
-  ExtensionDevToolsInfoBarDelegate::NotifyExtensionDetached(extension_id());
   g_attached_client_hosts.Get().erase(this);
+
+  // Decrement the associated worker keepalive, if any.
+  if (service_worker_keepalive_) {
+    CHECK(extension_service_worker_id_);
+    ProcessManager* process_manager = ProcessManager::Get(profile_);
+    CHECK(process_manager);
+    // The worker may have terminated for other reasons. Only decrement the
+    // keepalive if it's still around.
+    if (process_manager->HasServiceWorker(*extension_service_worker_id_)) {
+      process_manager->DecrementServiceWorkerKeepaliveCount(
+          *extension_service_worker_id_, *service_worker_keepalive_,
+          Activity::DEBUGGER, /*extra_data=*/std::string());
+    }
+  }
 }
 
 // DevToolsAgentHostClient implementation.
@@ -299,14 +430,14 @@ void ExtensionDevToolsClientHost::SendMessageToBackend(
     DebuggerSendCommandFunction* function,
     const std::string& method,
     SendCommand::Params::CommandParams* command_params) {
-  base::DictionaryValue protocol_request;
+  base::Value::Dict protocol_request;
   int request_id = ++last_request_id_;
   pending_requests_[request_id] = function;
-  protocol_request.SetInteger("id", request_id);
-  protocol_request.SetString("method", method);
+  protocol_request.Set("id", request_id);
+  protocol_request.Set("method", method);
   if (command_params) {
-    protocol_request.Set(
-        "params", command_params->additional_properties.CreateDeepCopy());
+    protocol_request.Set("params",
+                         command_params->additional_properties.Clone());
   }
 
   std::string json;
@@ -317,7 +448,7 @@ void ExtensionDevToolsClientHost::SendMessageToBackend(
 }
 
 void ExtensionDevToolsClientHost::InfoBarDestroyed() {
-  detach_reason_ = api::debugger::DETACH_REASON_CANCELED_BY_USER;
+  detach_reason_ = api::debugger::DetachReason::kCanceledByUser;
   RespondDetachedToPendingRequests();
   SendDetachedEvent();
   Close();
@@ -333,8 +464,7 @@ void ExtensionDevToolsClientHost::SendDetachedEvent() {
   if (!EventRouter::Get(profile_))
     return;
 
-  std::unique_ptr<base::ListValue> args(
-      OnDetach::Create(debuggee_, detach_reason_));
+  auto args(OnDetach::Create(debuggee_, detach_reason_));
   auto event =
       std::make_unique<Event>(events::DEBUGGER_ON_DETACH, OnDetach::kEventName,
                               std::move(args), profile_);
@@ -350,11 +480,7 @@ void ExtensionDevToolsClientHost::OnExtensionUnloaded(
     Close();
 }
 
-void ExtensionDevToolsClientHost::Observe(
-    int type,
-    const content::NotificationSource& source,
-    const content::NotificationDetails& details) {
-  DCHECK_EQ(chrome::NOTIFICATION_APP_TERMINATING, type);
+void ExtensionDevToolsClientHost::OnAppTerminating() {
   Close();
 }
 
@@ -367,61 +493,58 @@ void ExtensionDevToolsClientHost::DispatchProtocolMessage(
 
   base::StringPiece message_str(reinterpret_cast<const char*>(message.data()),
                                 message.size());
-  std::unique_ptr<base::Value> result = base::JSONReader::ReadDeprecated(
+  std::optional<base::Value> result = base::JSONReader::Read(
       message_str, base::JSON_REPLACE_INVALID_CHARACTERS);
   if (!result || !result->is_dict()) {
     LOG(ERROR) << "Tried to send invalid message to extension: " << message_str;
     return;
   }
-  base::DictionaryValue* dictionary =
-      static_cast<base::DictionaryValue*>(result.get());
+  base::Value::Dict& dictionary = result->GetDict();
 
-  int id;
-  if (!dictionary->GetInteger("id", &id)) {
-    std::string method_name;
-    if (!dictionary->GetString("method", &method_name))
+  std::optional<int> id = dictionary.FindInt("id");
+  if (!id) {
+    std::string* method_name = dictionary.FindString("method");
+    if (!method_name)
       return;
 
     OnEvent::Params params;
-    base::DictionaryValue* params_value;
-    if (dictionary->GetDictionary("params", &params_value))
-      params.additional_properties.Swap(params_value);
+    if (base::Value::Dict* params_value = dictionary.FindDict("params")) {
+      params.additional_properties = std::move(*params_value);
+    }
 
-    std::unique_ptr<base::ListValue> args(
-        OnEvent::Create(debuggee_, method_name, params));
+    auto args(OnEvent::Create(debuggee_, *method_name, params));
     auto event =
         std::make_unique<Event>(events::DEBUGGER_ON_EVENT, OnEvent::kEventName,
                                 std::move(args), profile_);
     EventRouter::Get(profile_)->DispatchEventToExtension(extension_id(),
                                                          std::move(event));
   } else {
-    auto it = pending_requests_.find(id);
+    auto it = pending_requests_.find(*id);
     if (it == pending_requests_.end())
       return;
 
-    it->second->SendResponseBody(dictionary);
+    it->second->SendResponseBody(base::Value(std::move(dictionary)));
     pending_requests_.erase(it);
   }
 }
 
+bool ExtensionDevToolsClientHost::MayAttachToRenderFrameHost(
+    content::RenderFrameHost* render_frame_host) {
+  std::string error;
+  return ExtensionMayAttachToRenderFrameHost(*extension_, profile_,
+                                             render_frame_host, &error);
+}
+
 bool ExtensionDevToolsClientHost::MayAttachToURL(const GURL& url,
                                                  bool is_webui) {
   if (is_webui)
     return false;
   std::string error;
-  if (!ExtensionMayAttachToURL(*extension_, url, profile_, &error))
-    return false;
-  // For nested URLs, make sure ExtensionMayAttachToURL() allows both
-  // the outer and the inner URLs.
-  if (url.inner_url() && !ExtensionMayAttachToURL(*extension_, *url.inner_url(),
-                                                  profile_, &error)) {
-    return false;
-  }
-  return true;
+  return ExtensionMayAttachToURLOrInnerURL(*extension_, profile_, url, &error);
 }
 
-bool ExtensionDevToolsClientHost::MayAttachToBrowser() {
-  return ExtensionMayAttachToBrowser(*extension_);
+bool ExtensionDevToolsClientHost::IsTrusted() {
+  return ExtensionIsTrusted(*extension_);
 }
 
 bool ExtensionDevToolsClientHost::MayReadLocalFiles() {
@@ -432,6 +555,14 @@ bool ExtensionDevToolsClientHost::MayWriteLocalFiles() {
   return false;
 }
 
+std::optional<url::Origin>
+ExtensionDevToolsClientHost::GetNavigationInitiatorOrigin() {
+  // Ensure that navigations started by debugger API are treated as
+  // renderer-initiated by this extension, so that URL spoof defenses are in
+  // effect.
+  return extension_->origin();
+}
+
 // DebuggerFunction -----------------------------------------------------------
 
 DebuggerFunction::DebuggerFunction() : client_host_(nullptr) {}
@@ -462,8 +593,8 @@ bool DebuggerFunction::InitAgentHost(std::string* error) {
         &web_contents);
     if (result && web_contents) {
       if (!ExtensionMayAttachToWebContents(
-              *extension(), *web_contents,
-              Profile::FromBrowserContext(browser_context()), error)) {
+              *extension(), Profile::FromBrowserContext(browser_context()),
+              *web_contents, error)) {
         return false;
       }
 
@@ -487,21 +618,21 @@ bool DebuggerFunction::InitAgentHost(std::string* error) {
         DevToolsAgentHost::GetForId(*debuggee_.target_id);
     if (agent_host) {
       if (!ExtensionMayAttachToAgentHost(
-              *extension(), *agent_host,
-              Profile::FromBrowserContext(browser_context()), error)) {
+              *extension(), include_incognito_information(),
+              Profile::FromBrowserContext(browser_context()), *agent_host,
+              error)) {
         return false;
       }
       agent_host_ = std::move(agent_host);
     } else if (*debuggee_.target_id == kBrowserTargetId &&
-               ExtensionMayAttachToBrowser(*extension())) {
+               ExtensionIsTrusted(*extension())) {
       // TODO(caseq): get rid of the below code, browser agent host should
       // really be a singleton.
       // Re-use existing browser agent hosts.
       const std::string& extension_id = extension()->id();
       AttachedClientHosts& hosts = g_attached_client_hosts.Get();
-      auto it = std::find_if(
-          hosts.begin(), hosts.end(),
-          [&extension_id](ExtensionDevToolsClientHost* client_host) {
+      auto it = base::ranges::find_if(
+          hosts, [&extension_id](ExtensionDevToolsClientHost* client_host) {
             return client_host->extension_id() == extension_id &&
                    client_host->agent_host() &&
                    client_host->agent_host()->GetType() ==
@@ -545,8 +676,8 @@ ExtensionDevToolsClientHost* DebuggerFunction::FindClientHost() {
   const std::string& extension_id = extension()->id();
   DevToolsAgentHost* agent_host = agent_host_.get();
   AttachedClientHosts& hosts = g_attached_client_hosts.Get();
-  auto it = std::find_if(
-      hosts.begin(), hosts.end(),
+  auto it = base::ranges::find_if(
+      hosts,
       [&agent_host, &extension_id](ExtensionDevToolsClientHost* client_host) {
         return client_host->agent_host() == agent_host &&
                client_host->extension_id() == extension_id;
@@ -562,8 +693,8 @@ DebuggerAttachFunction::DebuggerAttachFunction() = default;
 DebuggerAttachFunction::~DebuggerAttachFunction() = default;
 
 ExtensionFunction::ResponseAction DebuggerAttachFunction::Run() {
-  std::unique_ptr<Attach::Params> params(Attach::Params::Create(*args_));
-  EXTENSION_FUNCTION_VALIDATE(params.get());
+  std::optional<Attach::Params> params = Attach::Params::Create(args());
+  EXTENSION_FUNCTION_VALIDATE(params);
 
   CopyDebuggee(&debuggee_, params->target);
   std::string error;
@@ -582,15 +713,24 @@ ExtensionFunction::ResponseAction DebuggerAttachFunction::Run() {
         FormatErrorMessage(debugger_api_constants::kAlreadyAttachedError)));
   }
 
+  Profile* profile = Profile::FromBrowserContext(browser_context());
   auto host = std::make_unique<ExtensionDevToolsClientHost>(
-      Profile::FromBrowserContext(browser_context()), agent_host_.get(),
-      extension(), debuggee_);
+      profile, agent_host_.get(), extension(), worker_id(), debuggee_);
 
   if (!host->Attach()) {
     return RespondNow(Error(debugger_api_constants::kRestrictedError));
   }
 
   host.release();  // An attached client host manages its own lifetime.
+
+  if (!(Manifest::IsPolicyLocation(extension()->location()) ||
+        Manifest::IsComponentLocation(extension()->location()))) {
+    bool is_developer_mode =
+        profile->GetPrefs()->GetBoolean(prefs::kExtensionsUIDeveloperMode);
+    base::UmaHistogramBoolean("Extensions.Debugger.UserIsInDeveloperMode",
+                              is_developer_mode);
+  }
+
   return RespondNow(NoArguments());
 }
 
@@ -601,8 +741,8 @@ DebuggerDetachFunction::DebuggerDetachFunction() = default;
 DebuggerDetachFunction::~DebuggerDetachFunction() = default;
 
 ExtensionFunction::ResponseAction DebuggerDetachFunction::Run() {
-  std::unique_ptr<Detach::Params> params(Detach::Params::Create(*args_));
-  EXTENSION_FUNCTION_VALIDATE(params.get());
+  std::optional<Detach::Params> params = Detach::Params::Create(args());
+  EXTENSION_FUNCTION_VALIDATE(params);
 
   CopyDebuggee(&debuggee_, params->target);
   std::string error;
@@ -621,36 +761,34 @@ DebuggerSendCommandFunction::DebuggerSendCommandFunction() = default;
 DebuggerSendCommandFunction::~DebuggerSendCommandFunction() = default;
 
 ExtensionFunction::ResponseAction DebuggerSendCommandFunction::Run() {
-  std::unique_ptr<SendCommand::Params> params(
-      SendCommand::Params::Create(*args_));
-  EXTENSION_FUNCTION_VALIDATE(params.get());
+  std::optional<SendCommand::Params> params =
+      SendCommand::Params::Create(args());
+  EXTENSION_FUNCTION_VALIDATE(params);
 
   CopyDebuggee(&debuggee_, params->target);
   std::string error;
   if (!InitClientHost(&error))
     return RespondNow(Error(std::move(error)));
 
-  client_host_->SendMessageToBackend(this, params->method,
-      params->command_params.get());
+  client_host_->SendMessageToBackend(
+      this, params->method, base::OptionalToPtr(params->command_params));
   if (did_respond())
     return AlreadyResponded();
   return RespondLater();
 }
 
-void DebuggerSendCommandFunction::SendResponseBody(
-    base::DictionaryValue* response) {
-  base::Value* error_body;
-  if (response->Get("error", &error_body)) {
+void DebuggerSendCommandFunction::SendResponseBody(base::Value response) {
+  if (base::Value* error_body = response.GetDict().Find("error")) {
     std::string error;
     base::JSONWriter::Write(*error_body, &error);
     Respond(Error(std::move(error)));
     return;
   }
 
-  base::DictionaryValue* result_body;
   SendCommand::Results::Result result;
-  if (response->GetDictionary("result", &result_body))
-    result.additional_properties.Swap(result_body);
+  if (base::Value::Dict* result_body = response.GetDict().FindDict("result")) {
+    result.additional_properties = std::move(*result_body);
+  }
 
   Respond(ArgumentList(SendCommand::Results::Create(result)));
 }
@@ -676,35 +814,37 @@ const char kTargetTypeBackgroundPage[] = "background_page";
 const char kTargetTypeWorker[] = "worker";
 const char kTargetTypeOther[] = "other";
 
-std::unique_ptr<base::DictionaryValue> SerializeTarget(
-    scoped_refptr<DevToolsAgentHost> host) {
-  std::unique_ptr<base::DictionaryValue> dictionary(
-      new base::DictionaryValue());
-  dictionary->SetString(kTargetIdField, host->GetId());
-  dictionary->SetString(kTargetTitleField, host->GetTitle());
-  dictionary->SetBoolean(kTargetAttachedField, host->IsAttached());
-  dictionary->SetString(kTargetUrlField, host->GetURL().spec());
+base::Value::Dict SerializeTarget(scoped_refptr<DevToolsAgentHost> host) {
+  base::Value::Dict dictionary;
+  dictionary.Set(kTargetIdField, host->GetId());
+  dictionary.Set(kTargetTitleField, host->GetTitle());
+  dictionary.Set(kTargetAttachedField, host->IsAttached());
+  dictionary.Set(kTargetUrlField, host->GetURL().spec());
 
   std::string type = host->GetType();
   std::string target_type = kTargetTypeOther;
   if (type == DevToolsAgentHost::kTypePage) {
     int tab_id =
         extensions::ExtensionTabUtil::GetTabId(host->GetWebContents());
-    dictionary->SetInteger(kTargetTabIdField, tab_id);
+    if (tab_id != api::tabs::TAB_ID_NONE) {
+      dictionary.Set(kTargetTabIdField, tab_id);
+    } else {
+      dictionary.Set(kTargetExtensionIdField, host->GetURL().host());
+    }
     target_type = kTargetTypePage;
   } else if (type == ChromeDevToolsManagerDelegate::kTypeBackgroundPage) {
-    dictionary->SetString(kTargetExtensionIdField, host->GetURL().host());
+    dictionary.Set(kTargetExtensionIdField, host->GetURL().host());
     target_type = kTargetTypeBackgroundPage;
   } else if (type == DevToolsAgentHost::kTypeServiceWorker ||
              type == DevToolsAgentHost::kTypeSharedWorker) {
     target_type = kTargetTypeWorker;
   }
 
-  dictionary->SetString(kTargetTypeField, target_type);
+  dictionary.Set(kTargetTypeField, target_type);
 
   GURL favicon_url = host->GetFaviconURL();
   if (favicon_url.is_valid())
-    dictionary->SetString(kTargetFaviconUrlField, favicon_url.spec());
+    dictionary.Set(kTargetFaviconUrlField, favicon_url.spec());
 
   return dictionary;
 }
@@ -717,12 +857,22 @@ DebuggerGetTargetsFunction::~DebuggerGetTargetsFunction() = default;
 
 ExtensionFunction::ResponseAction DebuggerGetTargetsFunction::Run() {
   content::DevToolsAgentHost::List list = DevToolsAgentHost::GetOrCreateAll();
-  std::unique_ptr<base::ListValue> result(new base::ListValue());
-  for (size_t i = 0; i < list.size(); ++i)
-    result->Append(SerializeTarget(list[i]));
+  base::Value::List result;
+  Profile* profile = Profile::FromBrowserContext(browser_context());
+  for (auto& host : list) {
+    // TODO(crbug.com/1348385): hide all Tab targets for now to avoid
+    // compatibility problems. Consider exposing them later when they're fully
+    // supported, and compatibility considerations are better understood.
+    if (host->GetType() == DevToolsAgentHost::kTypeTab)
+      continue;
+    if (!ExtensionMayAttachToTargetProfile(
+            profile, include_incognito_information(), *host)) {
+      continue;
+    }
+    result.Append(SerializeTarget(host));
+  }
 
-  return RespondNow(
-      OneArgument(base::Value::FromUniquePtrValue(std::move(result))));
+  return RespondNow(WithArguments(std::move(result)));
 }
 
 }  // namespace extensions
