@@ -1,4 +1,4 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,28 +10,46 @@
 #include <utility>
 #include <vector>
 
-#include "ash/public/cpp/ash_pref_names.h"
-#include "base/bind.h"
+#include "ash/constants/ash_features.h"
+#include "ash/constants/ash_pref_names.h"
+#include "ash/webui/settings/public/constants/routes.mojom.h"
 #include "base/command_line.h"
+#include "base/containers/contains.h"
 #include "base/containers/flat_set.h"
+#include "base/containers/span.h"
+#include "base/functional/bind.h"
+#include "base/functional/callback.h"
 #include "base/json/json_writer.h"
 #include "base/lazy_instance.h"
 #include "base/memory/scoped_refptr.h"
+#include "base/no_destructor.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_util.h"
-#include "base/strings/stringprintf.h"
 #include "base/system/sys_info.h"
 #include "base/values.h"
-#include "chrome/browser/chromeos/crostini/crostini_features.h"
-#include "chrome/browser/chromeos/crostini/crostini_manager.h"
-#include "chrome/browser/chromeos/crostini/crostini_pref_names.h"
-#include "chrome/browser/chromeos/crostini/crostini_terminal.h"
-#include "chrome/browser/chromeos/crostini/crostini_util.h"
-#include "chrome/browser/extensions/api/terminal/crostini_startup_status.h"
+#include "chrome/browser/ash/crostini/crostini_pref_names.h"
+#include "chrome/browser/ash/crostini/crostini_util.h"
+#include "chrome/browser/ash/guest_os/guest_id.h"
+#include "chrome/browser/ash/guest_os/guest_os_pref_names.h"
+#include "chrome/browser/ash/guest_os/guest_os_session_tracker.h"
+#include "chrome/browser/ash/guest_os/guest_os_terminal.h"
+#include "chrome/browser/ash/guest_os/public/guest_os_service.h"
+#include "chrome/browser/ash/guest_os/public/guest_os_terminal_provider.h"
+#include "chrome/browser/ash/guest_os/public/guest_os_terminal_provider_registry.h"
+#include "chrome/browser/ash/guest_os/public/types.h"
+#include "chrome/browser/ash/guest_os/virtual_machines/virtual_machines_util.h"
+#include "chrome/browser/browser_process.h"
+#include "chrome/browser/extensions/api/terminal/startup_status.h"
 #include "chrome/browser/extensions/extension_service.h"
 #include "chrome/browser/extensions/extension_tab_util.h"
+#include "chrome/browser/policy/system_features_disable_list_policy_handler.h"
+#include "chrome/browser/profiles/profile_manager.h"
+#include "chrome/browser/ui/browser_finder.h"
+#include "chrome/browser/ui/browser_tabstrip.h"
+#include "chrome/browser/ui/settings_window_manager_chromeos.h"
 #include "chrome/common/chrome_switches.h"
 #include "chrome/common/extensions/api/terminal_private.h"
+#include "chromeos/ash/components/dbus/cicerone/cicerone_client.h"
 #include "chromeos/process_proxy/process_proxy_registry.h"
 #include "components/prefs/pref_service.h"
 #include "content/public/browser/browser_context.h"
@@ -42,7 +60,10 @@
 #include "extensions/browser/app_window/app_window.h"
 #include "extensions/browser/app_window/app_window_registry.h"
 #include "extensions/browser/event_router.h"
+#include "extensions/browser/extension_registry.h"
 #include "extensions/browser/extensions_browser_client.h"
+#include "extensions/common/constants.h"
+#include "ui/display/types/display_constants.h"
 
 namespace terminal_private = extensions::api::terminal_private;
 namespace OnTerminalResize =
@@ -55,9 +76,9 @@ namespace CloseTerminalProcess =
     extensions::api::terminal_private::CloseTerminalProcess;
 namespace SendInput = extensions::api::terminal_private::SendInput;
 namespace AckOutput = extensions::api::terminal_private::AckOutput;
-namespace SetSettings = extensions::api::terminal_private::SetSettings;
-
-using crostini::mojom::InstallerState;
+namespace OpenWindow = extensions::api::terminal_private::OpenWindow;
+namespace GetPrefs = extensions::api::terminal_private::GetPrefs;
+namespace SetPrefs = extensions::api::terminal_private::SetPrefs;
 
 namespace {
 
@@ -74,21 +95,54 @@ const char kSwitchVmName[] = "vm_name";
 const char kSwitchTargetContainer[] = "target_container";
 const char kSwitchStartupId[] = "startup_id";
 const char kSwitchCurrentWorkingDir[] = "cwd";
+const char kSwitchContainerFeatures[] = "container_features";
 
 const char kCwdTerminalIdPrefix[] = "terminal_id:";
 
+// Prefs that we read and observe.
+static const base::NoDestructor<std::vector<std::string>> kPrefsReadAllowList{{
+    ash::prefs::kAccessibilitySpokenFeedbackEnabled,
+    crostini::prefs::kCrostiniEnabled,
+    guest_os::prefs::kGuestOsTerminalSettings,
+    crostini::prefs::kTerminalSshAllowedByPolicy,
+    guest_os::prefs::kGuestOsContainers,
+}};
+
+void CloseTerminal(const std::string& terminal_id,
+                   base::OnceCallback<void(bool)> callback) {
+  chromeos::ProcessProxyRegistry::GetTaskRunner()->PostTaskAndReplyWithResult(
+      FROM_HERE,
+      base::BindOnce(
+          [](const std::string& terminal_id) {
+            return chromeos::ProcessProxyRegistry::Get()->CloseProcess(
+                terminal_id);
+          },
+          terminal_id),
+      std::move(callback));
+}
+
 class TerminalTabHelper
     : public content::WebContentsUserData<TerminalTabHelper> {
  public:
+  ~TerminalTabHelper() override {
+    // The web contents object is being destructed. We should close all
+    // terminals that haven't been closed already. This can happen when the JS
+    // code didn't have a chance to do that (e.g. memory stress causes the web
+    // contents to be killed directly).
+    for (const std::string& terminal_id : terminal_ids_) {
+      CloseTerminal(terminal_id, base::DoNothing());
+    }
+  }
+
   void AddTerminalId(const std::string& terminal_id) {
     if (!terminal_ids_.insert(terminal_id).second) {
-      LOG(ERROR) << "terminal id already exists" << terminal_id;
+      LOG(ERROR) << "Terminal id already exists: " << terminal_id;
     }
   }
 
   void RemoveTerminalId(const std::string& terminal_id) {
     if (terminal_ids_.erase(terminal_id) == 0) {
-      LOG(ERROR) << "terminal id does not exists" << terminal_id;
+      LOG(ERROR) << "Terminal id does not exist: " << terminal_id;
     }
   }
 
@@ -104,7 +158,8 @@ class TerminalTabHelper
   }
 
  private:
-  explicit TerminalTabHelper(content::WebContents* contents) {}
+  explicit TerminalTabHelper(content::WebContents* contents)
+      : content::WebContentsUserData<TerminalTabHelper>(*contents) {}
 
   friend class content::WebContentsUserData<TerminalTabHelper>;
   WEB_CONTENTS_USER_DATA_KEY_DECL();
@@ -112,7 +167,7 @@ class TerminalTabHelper
   base::flat_set<std::string> terminal_ids_;
 };
 
-WEB_CONTENTS_USER_DATA_KEY_IMPL(TerminalTabHelper)
+WEB_CONTENTS_USER_DATA_KEY_IMPL(TerminalTabHelper);
 
 // Copies the value of |switch_name| if present from |src| to |dst|.  If not
 // present, uses |default_value| if nonempty.  Returns the value set into |dst|.
@@ -129,6 +184,19 @@ std::string GetSwitch(const base::CommandLine& src,
   return result;
 }
 
+std::string GetContainerFeaturesArg() {
+  std::string result;
+  // There are only a few available features so concatenating strings is
+  // sufficient.
+  for (vm_tools::cicerone::ContainerFeature feature :
+       crostini::GetContainerFeatures()) {
+    if (!result.empty())
+      result += ",";
+    result += base::NumberToString(static_cast<int>(feature));
+  }
+  return result;
+}
+
 void NotifyProcessOutput(content::BrowserContext* browser_context,
                          const std::string& terminal_id,
                          const std::string& output_type,
@@ -140,10 +208,11 @@ void NotifyProcessOutput(content::BrowserContext* browser_context,
     return;
   }
 
-  std::unique_ptr<base::ListValue> args(new base::ListValue());
-  args->AppendString(terminal_id);
-  args->AppendString(output_type);
-  args->AppendString(output);
+  base::Value::List args;
+  args.Append(terminal_id);
+  args.Append(output_type);
+  args.Append(base::Value(base::make_span(
+      reinterpret_cast<const uint8_t*>(&output[0]), output.size())));
 
   extensions::EventRouter* event_router =
       extensions::EventRouter::Get(browser_context);
@@ -155,18 +224,19 @@ void NotifyProcessOutput(content::BrowserContext* browser_context,
   }
 }
 
-void PreferenceChanged(Profile* profile,
-                       const std::string& pref_name,
-                       extensions::events::HistogramValue histogram,
-                       const char* eventName) {
-  auto args = std::make_unique<base::ListValue>();
-  args->Append(profile->GetPrefs()->Get(pref_name)->CreateDeepCopy());
+void PrefChanged(Profile* profile, const std::string& pref_name) {
   extensions::EventRouter* event_router = extensions::EventRouter::Get(profile);
-  if (event_router) {
-    auto event = std::make_unique<extensions::Event>(histogram, eventName,
-                                                     std::move(args));
-    event_router->BroadcastEvent(std::move(event));
+  if (!event_router) {
+    return;
   }
+  base::Value::List args;
+  base::Value::Dict prefs;
+  prefs.Set(pref_name, profile->GetPrefs()->GetValue(pref_name).Clone());
+  args.Append(std::move(prefs));
+  auto event = std::make_unique<extensions::Event>(
+      extensions::events::TERMINAL_PRIVATE_ON_PREF_CHANGED,
+      terminal_private::OnPrefChanged::kEventName, std::move(args));
+  event_router->BroadcastEvent(std::move(event));
 }
 
 }  // namespace
@@ -178,20 +248,10 @@ TerminalPrivateAPI::TerminalPrivateAPI(content::BrowserContext* context)
       pref_change_registrar_(std::make_unique<PrefChangeRegistrar>()) {
   Profile* profile = Profile::FromBrowserContext(context);
   pref_change_registrar_->Init(profile->GetPrefs());
-  pref_change_registrar_->Add(
-      crostini::prefs::kCrostiniTerminalSettings,
-      base::BindRepeating(
-          &PreferenceChanged, profile,
-          crostini::prefs::kCrostiniTerminalSettings,
-          extensions::events::TERMINAL_PRIVATE_ON_SETTINGS_CHANGED,
-          terminal_private::OnSettingsChanged::kEventName));
-  pref_change_registrar_->Add(
-      ash::prefs::kAccessibilitySpokenFeedbackEnabled,
-      base::BindRepeating(
-          &PreferenceChanged, profile,
-          ash::prefs::kAccessibilitySpokenFeedbackEnabled,
-          extensions::events::TERMINAL_PRIVATE_ON_A11Y_STATUS_CHANGED,
-          terminal_private::OnA11yStatusChanged::kEventName));
+  for (const auto& pref : *kPrefsReadAllowList) {
+    pref_change_registrar_->Add(pref,
+                                base::BindRepeating(&PrefChanged, profile));
+  }
 }
 
 TerminalPrivateAPI::~TerminalPrivateAPI() = default;
@@ -205,14 +265,17 @@ TerminalPrivateAPI::GetFactoryInstance() {
   return g_factory.Pointer();
 }
 
+TerminalPrivateOpenTerminalProcessFunction::
+    TerminalPrivateOpenTerminalProcessFunction() = default;
+
 TerminalPrivateOpenTerminalProcessFunction::
     ~TerminalPrivateOpenTerminalProcessFunction() = default;
 
 ExtensionFunction::ResponseAction
 TerminalPrivateOpenTerminalProcessFunction::Run() {
-  std::unique_ptr<OpenTerminalProcess::Params> params(
-      OpenTerminalProcess::Params::Create(*args_));
-  EXTENSION_FUNCTION_VALIDATE(params.get());
+  std::optional<OpenTerminalProcess::Params> params =
+      OpenTerminalProcess::Params::Create(args());
+  EXTENSION_FUNCTION_VALIDATE(params);
 
   return OpenProcess(params->process_name, std::move(params->args));
 }
@@ -220,7 +283,7 @@ TerminalPrivateOpenTerminalProcessFunction::Run() {
 ExtensionFunction::ResponseAction
 TerminalPrivateOpenTerminalProcessFunction::OpenProcess(
     const std::string& process_name,
-    std::unique_ptr<std::vector<std::string>> args) {
+    std::optional<std::vector<std::string>> args) {
   const std::string& user_id_hash =
       extensions::ExtensionsBrowserClient::Get()->GetUserIdHashFromContext(
           browser_context());
@@ -237,6 +300,12 @@ TerminalPrivateOpenTerminalProcessFunction::OpenProcess(
             command_line->GetSwitchValueASCII(switches::kCroshCommand))));
 
   } else if (process_name == kCroshName) {
+    // Ensure crosh is allowed before starting terminal.
+    if (policy::SystemFeaturesDisableListPolicyHandler::IsSystemFeatureDisabled(
+            policy::SystemFeature::kCrosh, g_browser_process->local_state())) {
+      return RespondNow(Error("crosh not allowed"));
+    }
+
     // command=crosh: use '/usr/bin/crosh' on a device, 'cat' otherwise.
     if (base::SysInfo::IsRunningOnChromeOS()) {
       OpenProcess(user_id_hash,
@@ -245,47 +314,83 @@ TerminalPrivateOpenTerminalProcessFunction::OpenProcess(
       OpenProcess(user_id_hash,
                   base::CommandLine(base::FilePath(kStubbedCroshCommand)));
     }
-
   } else if (process_name == kVmShellName) {
-    // Ensure crostini is allowed before starting terminal.
-    Profile* profile = Profile::FromBrowserContext(browser_context());
-    if (!crostini::CrostiniFeatures::Get()->IsAllowedNow(profile))
+    // Ensure vms are allowed before starting terminal.
+    if (!virtual_machines::AreVirtualMachinesAllowedByPolicy()) {
       return RespondNow(Error("vmshell not allowed"));
-
+    }
     // command=vmshell: ensure --owner_id, --vm_name, --target_container, --cwd
     // are set, and the specified vm/container is running.
     base::CommandLine cmdline((base::FilePath(kVmShellCommand)));
     if (!args)
-      args = std::make_unique<std::vector<std::string>>();
+      args.emplace();
     args->insert(args->begin(), kVmShellCommand);
     base::CommandLine params_args(*args);
+    VLOG(1) << "Original cmdline= " << params_args.GetCommandLineString();
     std::string owner_id =
         GetSwitch(params_args, &cmdline, kSwitchOwnerId, user_id_hash);
     std::string vm_name = GetSwitch(params_args, &cmdline, kSwitchVmName,
                                     crostini::kCrostiniDefaultVmName);
     std::string container_name =
-        GetSwitch(params_args, &cmdline, kSwitchTargetContainer,
-                  crostini::kCrostiniDefaultContainerName);
+        GetSwitch(params_args, &cmdline, kSwitchTargetContainer, "");
     GetSwitch(params_args, &cmdline, kSwitchCurrentWorkingDir, "");
     std::string startup_id = params_args.GetSwitchValueASCII(kSwitchStartupId);
-    crostini::ContainerId container_id(vm_name, container_name);
+    guest_id_ = std::make_unique<guest_os::GuestId>(guest_os::VmType::UNKNOWN,
+                                                    vm_name, container_name);
 
-    auto* mgr = crostini::CrostiniManager::GetForProfile(profile);
-    bool verbose = !mgr->GetContainerInfo(container_id).has_value();
-    auto observer = std::make_unique<CrostiniStartupStatus>(
+    // Unlike the other switches, this is computed here directly rather than
+    // taken from |args|.
+    std::string container_features = GetContainerFeaturesArg();
+    if (!container_features.empty())
+      cmdline.AppendSwitchASCII(kSwitchContainerFeatures, container_features);
+
+    // Append trailing passthrough args if any.  E.g. `-- vim file.txt`
+    auto passthrough_args = params_args.GetArgs();
+    if (!passthrough_args.empty()) {
+      cmdline.AppendArg("--");
+      for (const auto& arg : passthrough_args) {
+        cmdline.AppendArg(arg);
+      }
+    }
+    VLOG(1) << "Starting " << *guest_id_
+            << ", cmdline=" << cmdline.GetCommandLineString();
+
+    Profile* profile = Profile::FromBrowserContext(browser_context());
+    auto* service = guest_os::GuestOsService::GetForProfile(profile);
+    guest_os::GuestOsTerminalProvider* provider = nullptr;
+    if (service) {
+      provider = service->TerminalProviderRegistry()->Get(*guest_id_);
+    }
+    auto* tracker = guest_os::GuestOsSessionTracker::GetForProfile(profile);
+    bool verbose = !(tracker && tracker->GetInfo(*guest_id_).has_value());
+    auto status_printer = std::make_unique<StartupStatusPrinter>(
         base::BindRepeating(&NotifyProcessOutput, browser_context(), startup_id,
                             api::terminal_private::ToString(
-                                api::terminal_private::OUTPUT_TYPE_STDOUT)),
+                                api::terminal_private::OutputType::kStdout)),
         verbose);
-    // Save copy of pointer for RestartObserver before moving object.
-    CrostiniStartupStatus* observer_ptr = observer.get();
-    observer->ShowProgressAtInterval();
-    mgr->RestartCrostini(
-        container_id,
-        base::BindOnce(
-            &TerminalPrivateOpenTerminalProcessFunction::OnCrostiniRestarted,
-            this, std::move(observer), user_id_hash, std::move(cmdline)),
-        observer_ptr);
+    if (provider) {
+      startup_status_ =
+          provider->CreateStartupStatus(std::move(status_printer));
+      startup_status_->StartShowingSpinner();
+      provider->EnsureRunning(
+          startup_status_.get(),
+          base::BindOnce(
+              &TerminalPrivateOpenTerminalProcessFunction::OnGuestRunning, this,
+              user_id_hash, std::move(cmdline)));
+    } else {
+      // Don't recognise the guest. Options include:
+      // * It doesn't exist but the terminal app thinks it does e.g. out-of-sync
+      //   prefs, race between uninstalling and launching terminal.
+      // * We're installing Bruschetta, and it's using the terminal to finish
+      //   installing. We don't show Bruschetta until it's installed, but it's
+      //   running and users need to connect to it.
+      // Given this, try and connect directly to it, if it succeeds, great, if
+      // it fails, then report failure to the user.
+      startup_status_ =
+          std::make_unique<StartupStatus>(std::move(status_printer), 2);
+      startup_status_->StartShowingSpinner();
+      OpenVmshellProcess(user_id_hash, std::move(cmdline));
+    }
   } else {
     // command=[unrecognized].
     return RespondNow(Error("Invalid process name: " + process_name));
@@ -293,37 +398,30 @@ TerminalPrivateOpenTerminalProcessFunction::OpenProcess(
   return RespondLater();
 }
 
-void TerminalPrivateOpenTerminalProcessFunction::OnCrostiniRestarted(
-    std::unique_ptr<CrostiniStartupStatus> startup_status,
+void TerminalPrivateOpenTerminalProcessFunction::OnGuestRunning(
     const std::string& user_id_hash,
     base::CommandLine cmdline,
-    crostini::CrostiniResult result) {
-  if (crostini::MaybeShowCrostiniDialogBeforeLaunch(
-          Profile::FromBrowserContext(browser_context()), result)) {
-    const std::string msg = "Waiting for component update dialog response";
-    LOG(ERROR) << msg;
-    Respond(Error(msg));
-    return;
-  }
-  startup_status->OnCrostiniRestarted(result);
-  if (result == crostini::CrostiniResult::SUCCESS) {
+    bool success,
+    std::string failure_reason) {
+  if (success) {
     OpenVmshellProcess(user_id_hash, std::move(cmdline));
   } else {
-    const std::string msg =
-        base::StringPrintf("Error starting crostini for terminal: %d", result);
-    LOG(ERROR) << msg;
-    Respond(Error(msg));
+    startup_status_->OnFinished(success, failure_reason);
+    LOG(ERROR) << failure_reason;
+    Respond(Error(failure_reason));
   }
 }
 
 void TerminalPrivateOpenTerminalProcessFunction::OpenVmshellProcess(
     const std::string& user_id_hash,
     base::CommandLine cmdline) {
+  startup_status_->OnConnectingToVsh();
   const std::string cwd = cmdline.GetSwitchValueASCII(kSwitchCurrentWorkingDir);
 
   if (!base::StartsWith(cwd, kCwdTerminalIdPrefix)) {
     return OpenProcess(user_id_hash, std::move(cmdline));
   }
+  cmdline.RemoveSwitch(kSwitchCurrentWorkingDir);
 
   // The cwd has this format `terminal_id:<terminal_id>`. We need to convert the
   // terminal id to the pid of the shell process inside the container.
@@ -331,28 +429,31 @@ void TerminalPrivateOpenTerminalProcessFunction::OpenVmshellProcess(
       cwd.substr(sizeof(kCwdTerminalIdPrefix) - 1));
 
   // Lookup container shell pid from cicierone to use for cwd.
-  crostini::CrostiniManager::GetForProfile(
-      Profile::FromBrowserContext(browser_context()))
-      ->GetVshSession(
-          crostini::ContainerId::GetDefault(), host_pid,
-          base::BindOnce(
-              &TerminalPrivateOpenTerminalProcessFunction::OnGetVshSession,
-              this, user_id_hash, std::move(cmdline), /*terminal_id=*/cwd));
+  vm_tools::cicerone::GetVshSessionRequest request;
+  request.set_vm_name(guest_id_->vm_name);
+  request.set_container_name(guest_id_->container_name);
+  request.set_owner_id(crostini::CryptohomeIdForProfile(
+      Profile::FromBrowserContext(browser_context())));
+  request.set_host_vsh_pid(host_pid);
+  ash::CiceroneClient::Get()->GetVshSession(
+      request,
+      base::BindOnce(
+          &TerminalPrivateOpenTerminalProcessFunction::OnGetVshSession, this,
+          user_id_hash, std::move(cmdline), /*terminal_id=*/cwd));
 }
 
 void TerminalPrivateOpenTerminalProcessFunction::OnGetVshSession(
     const std::string& user_id_hash,
     base::CommandLine cmdline,
     const std::string& terminal_id,
-    bool success,
-    const std::string& failure_reason,
-    int32_t container_shell_pid) {
-  if (!success) {
-    LOG(WARNING) << "Failed to get vsh session for " << terminal_id << ". "
-                 << failure_reason;
+    std::optional<vm_tools::cicerone::GetVshSessionResponse> response) {
+  if (!response || !response->success()) {
+    LOG(WARNING) << "Failed to get vsh session for " << terminal_id << ": "
+                 << (response ? response->failure_reason() : "empty response");
   } else {
-    cmdline.AppendSwitchASCII(kSwitchCurrentWorkingDir,
-                              base::NumberToString(container_shell_pid));
+    cmdline.AppendSwitchASCII(
+        kSwitchCurrentWorkingDir,
+        base::NumberToString(response->container_shell_pid()));
   }
   OpenProcess(user_id_hash, std::move(cmdline));
 }
@@ -392,9 +493,12 @@ void TerminalPrivateOpenTerminalProcessFunction::OpenOnRegistryTaskRunner(
 void TerminalPrivateOpenTerminalProcessFunction::RespondOnUIThread(
     bool success,
     const std::string& terminal_id) {
+  if (startup_status_) {
+    startup_status_->OnFinished(
+        success, success ? "" : "Error connecting shell to guest");
+  }
   auto* contents = GetSenderWebContents();
   if (!contents) {
-    LOG(WARNING) << "content is closed before returning opened process";
     chromeos::ProcessProxyRegistry::GetTaskRunner()->PostTask(
         FROM_HERE,
         base::BindOnce(
@@ -405,6 +509,9 @@ void TerminalPrivateOpenTerminalProcessFunction::RespondOnUIThread(
               }
             },
             terminal_id));
+    const std::string msg = "Web contents closed during OpenProcess";
+    LOG(WARNING) << msg;
+    Respond(Error(msg));
     return;
   }
 
@@ -412,7 +519,7 @@ void TerminalPrivateOpenTerminalProcessFunction::RespondOnUIThread(
     Respond(Error("Failed to open process."));
     return;
   }
-  Respond(OneArgument(base::Value(terminal_id)));
+  Respond(WithArguments(terminal_id));
 
   TerminalTabHelper::CreateForWebContents(contents);
   TerminalTabHelper::FromWebContents(contents)->AddTerminalId(terminal_id);
@@ -423,9 +530,9 @@ TerminalPrivateOpenVmshellProcessFunction::
 
 ExtensionFunction::ResponseAction
 TerminalPrivateOpenVmshellProcessFunction::Run() {
-  std::unique_ptr<OpenVmshellProcess::Params> params(
-      OpenVmshellProcess::Params::Create(*args_));
-  EXTENSION_FUNCTION_VALIDATE(params.get());
+  std::optional<OpenVmshellProcess::Params> params =
+      OpenVmshellProcess::Params::Create(args());
+  EXTENSION_FUNCTION_VALIDATE(params);
 
   // Only opens 'vmshell'.
   return OpenProcess(kVmShellName, std::move(params->args));
@@ -434,8 +541,8 @@ TerminalPrivateOpenVmshellProcessFunction::Run() {
 TerminalPrivateSendInputFunction::~TerminalPrivateSendInputFunction() = default;
 
 ExtensionFunction::ResponseAction TerminalPrivateSendInputFunction::Run() {
-  std::unique_ptr<SendInput::Params> params(SendInput::Params::Create(*args_));
-  EXTENSION_FUNCTION_VALIDATE(params.get());
+  std::optional<SendInput::Params> params = SendInput::Params::Create(args());
+  EXTENSION_FUNCTION_VALIDATE(params);
 
   if (!TerminalTabHelper::ValidateTerminalId(GetSenderWebContents(),
                                              params->id)) {
@@ -456,9 +563,12 @@ ExtensionFunction::ResponseAction TerminalPrivateSendInputFunction::Run() {
 void TerminalPrivateSendInputFunction::SendInputOnRegistryTaskRunner(
     const std::string& terminal_id,
     const std::string& text) {
-  bool success =
-      chromeos::ProcessProxyRegistry::Get()->SendInput(terminal_id, text);
+  chromeos::ProcessProxyRegistry::Get()->SendInput(
+      terminal_id, text,
+      base::BindOnce(&TerminalPrivateSendInputFunction::OnSendInput, this));
+}
 
+void TerminalPrivateSendInputFunction::OnSendInput(bool success) {
   content::GetUIThreadTaskRunner({})->PostTask(
       FROM_HERE,
       base::BindOnce(&TerminalPrivateSendInputFunction::RespondOnUIThread, this,
@@ -466,7 +576,7 @@ void TerminalPrivateSendInputFunction::SendInputOnRegistryTaskRunner(
 }
 
 void TerminalPrivateSendInputFunction::RespondOnUIThread(bool success) {
-  Respond(OneArgument(base::Value(success)));
+  Respond(WithArguments(success));
 }
 
 TerminalPrivateCloseTerminalProcessFunction::
@@ -474,9 +584,9 @@ TerminalPrivateCloseTerminalProcessFunction::
 
 ExtensionFunction::ResponseAction
 TerminalPrivateCloseTerminalProcessFunction::Run() {
-  std::unique_ptr<CloseTerminalProcess::Params> params(
-      CloseTerminalProcess::Params::Create(*args_));
-  EXTENSION_FUNCTION_VALIDATE(params.get());
+  std::optional<CloseTerminalProcess::Params> params =
+      CloseTerminalProcess::Params::Create(args());
+  EXTENSION_FUNCTION_VALIDATE(params);
 
   if (!TerminalTabHelper::ValidateTerminalId(GetSenderWebContents(),
                                              params->id)) {
@@ -486,30 +596,18 @@ TerminalPrivateCloseTerminalProcessFunction::Run() {
   TerminalTabHelper::FromWebContents(GetSenderWebContents())
       ->RemoveTerminalId(params->id);
 
-  // Registry lives on its own task runner.
-  chromeos::ProcessProxyRegistry::GetTaskRunner()->PostTask(
-      FROM_HERE, base::BindOnce(&TerminalPrivateCloseTerminalProcessFunction::
-                                    CloseOnRegistryTaskRunner,
-                                this, params->id));
+  CloseTerminal(
+      params->id,
+      base::BindOnce(
+          &TerminalPrivateCloseTerminalProcessFunction::RespondOnUIThread,
+          this));
 
   return RespondLater();
 }
 
-void TerminalPrivateCloseTerminalProcessFunction::CloseOnRegistryTaskRunner(
-    const std::string& terminal_id) {
-  bool success =
-      chromeos::ProcessProxyRegistry::Get()->CloseProcess(terminal_id);
-
-  content::GetUIThreadTaskRunner({})->PostTask(
-      FROM_HERE,
-      base::BindOnce(
-          &TerminalPrivateCloseTerminalProcessFunction::RespondOnUIThread, this,
-          success));
-}
-
 void TerminalPrivateCloseTerminalProcessFunction::RespondOnUIThread(
     bool success) {
-  Respond(OneArgument(base::Value(success)));
+  Respond(WithArguments(success));
 }
 
 TerminalPrivateOnTerminalResizeFunction::
@@ -517,9 +615,9 @@ TerminalPrivateOnTerminalResizeFunction::
 
 ExtensionFunction::ResponseAction
 TerminalPrivateOnTerminalResizeFunction::Run() {
-  std::unique_ptr<OnTerminalResize::Params> params(
-      OnTerminalResize::Params::Create(*args_));
-  EXTENSION_FUNCTION_VALIDATE(params.get());
+  std::optional<OnTerminalResize::Params> params =
+      OnTerminalResize::Params::Create(args());
+  EXTENSION_FUNCTION_VALIDATE(params);
 
   if (!TerminalTabHelper::ValidateTerminalId(GetSenderWebContents(),
                                              params->id)) {
@@ -552,14 +650,14 @@ void TerminalPrivateOnTerminalResizeFunction::OnResizeOnRegistryTaskRunner(
 }
 
 void TerminalPrivateOnTerminalResizeFunction::RespondOnUIThread(bool success) {
-  Respond(OneArgument(base::Value(success)));
+  Respond(WithArguments(success));
 }
 
 TerminalPrivateAckOutputFunction::~TerminalPrivateAckOutputFunction() = default;
 
 ExtensionFunction::ResponseAction TerminalPrivateAckOutputFunction::Run() {
-  std::unique_ptr<AckOutput::Params> params(AckOutput::Params::Create(*args_));
-  EXTENSION_FUNCTION_VALIDATE(params.get());
+  std::optional<AckOutput::Params> params = AckOutput::Params::Create(args());
+  EXTENSION_FUNCTION_VALIDATE(params);
 
   // Every running terminal page will call ackOutput(), but we should only react
   // for the one who actually owns the output.
@@ -585,7 +683,35 @@ TerminalPrivateOpenWindowFunction::~TerminalPrivateOpenWindowFunction() =
     default;
 
 ExtensionFunction::ResponseAction TerminalPrivateOpenWindowFunction::Run() {
-  crostini::LaunchTerminal(Profile::FromBrowserContext(browser_context()));
+  std::optional<OpenWindow::Params> params = OpenWindow::Params::Create(args());
+  EXTENSION_FUNCTION_VALIDATE(params);
+
+  const std::string* url = &guest_os::GetTerminalHomeUrl();
+  bool as_tab = false;
+
+  auto& data = params->data;
+  if (data) {
+    if (data->url) {
+      url = &*data->url;
+    }
+    if (data->as_tab) {
+      as_tab = *data->as_tab;
+    }
+  }
+
+  if (as_tab) {
+    auto* browser = chrome::FindBrowserWithTab(GetSenderWebContents());
+    if (browser) {
+      chrome::AddTabAt(browser, GURL(*url), -1, true);
+    } else {
+      LOG(ERROR) << "cannot find the browser";
+    }
+  } else {
+    guest_os::LaunchTerminalWithUrl(
+        Profile::FromBrowserContext(browser_context()),
+        display::kInvalidDisplayId, /*restore_id=*/0, GURL(*url));
+  }
+
   return RespondNow(NoArguments());
 }
 
@@ -594,48 +720,86 @@ TerminalPrivateOpenOptionsPageFunction::
 
 ExtensionFunction::ResponseAction
 TerminalPrivateOpenOptionsPageFunction::Run() {
-  crostini::LaunchTerminalSettings(
+  guest_os::LaunchTerminalSettings(
       Profile::FromBrowserContext(browser_context()));
   return RespondNow(NoArguments());
 }
 
-TerminalPrivateGetSettingsFunction::~TerminalPrivateGetSettingsFunction() =
-    default;
+TerminalPrivateOpenSettingsSubpageFunction::
+    ~TerminalPrivateOpenSettingsSubpageFunction() = default;
 
-ExtensionFunction::ResponseAction TerminalPrivateGetSettingsFunction::Run() {
-  crostini::RecordTerminalSettingsChangesUMAs(
-      Profile::FromBrowserContext(browser_context()));
-  PrefService* service =
-      Profile::FromBrowserContext(browser_context())->GetPrefs();
-  const base::DictionaryValue* value =
-      service->GetDictionary(crostini::prefs::kCrostiniTerminalSettings);
-  return RespondNow(OneArgument(value->Clone()));
-}
-
-TerminalPrivateSetSettingsFunction::~TerminalPrivateSetSettingsFunction() =
-    default;
-
-ExtensionFunction::ResponseAction TerminalPrivateSetSettingsFunction::Run() {
-  std::unique_ptr<SetSettings::Params> params(
-      SetSettings::Params::Create(*args_));
-  EXTENSION_FUNCTION_VALIDATE(params.get());
-
-  PrefService* service =
-      Profile::FromBrowserContext(browser_context())->GetPrefs();
-  service->Set(crostini::prefs::kCrostiniTerminalSettings,
-               params->settings.additional_properties);
+ExtensionFunction::ResponseAction
+TerminalPrivateOpenSettingsSubpageFunction::Run() {
+  // Ignore params->subpage for now, and always open crostini.
+  chrome::SettingsWindowManager::GetInstance()->ShowOSSettings(
+      ProfileManager::GetActiveUserProfile(),
+      chromeos::settings::mojom::kCrostiniSectionPath);
   return RespondNow(NoArguments());
 }
 
-TerminalPrivateGetA11yStatusFunction::~TerminalPrivateGetA11yStatusFunction() =
-    default;
+TerminalPrivateGetOSInfoFunction::~TerminalPrivateGetOSInfoFunction() = default;
 
-ExtensionFunction::ResponseAction TerminalPrivateGetA11yStatusFunction::Run() {
-  return RespondNow(
-      OneArgument(Profile::FromBrowserContext(browser_context())
-                      ->GetPrefs()
-                      ->Get(ash::prefs::kAccessibilitySpokenFeedbackEnabled)
-                      ->Clone()));
+ExtensionFunction::ResponseAction TerminalPrivateGetOSInfoFunction::Run() {
+  base::Value::Dict info;
+  info.Set("alternative_emulator",
+           base::FeatureList::IsEnabled(
+               ash::features::kTerminalAlternativeEmulator));
+  info.Set("tast", extensions::ExtensionRegistry::Get(browser_context())
+                       ->enabled_extensions()
+                       .Contains(extension_misc::kGuestModeTestExtensionId));
+  return RespondNow(WithArguments(std::move(info)));
+}
+
+TerminalPrivateGetPrefsFunction::~TerminalPrivateGetPrefsFunction() = default;
+
+ExtensionFunction::ResponseAction TerminalPrivateGetPrefsFunction::Run() {
+  std::optional<GetPrefs::Params> params = GetPrefs::Params::Create(args());
+  EXTENSION_FUNCTION_VALIDATE(params);
+  PrefService* service =
+      Profile::FromBrowserContext(browser_context())->GetPrefs();
+  base::Value::Dict result;
+
+  for (const auto& path : params->paths) {
+    // Ignore non-allowed paths.
+    if (!base::Contains(*kPrefsReadAllowList, path)) {
+      LOG(WARNING) << "Ignoring non-allowed GetPrefs path=" << path;
+      continue;
+    }
+    if (path == guest_os::prefs::kGuestOsTerminalSettings) {
+      guest_os::RecordTerminalSettingsChangesUMAs(
+          Profile::FromBrowserContext(browser_context()));
+    }
+    result.Set(path, service->GetValue(path).Clone());
+  }
+  return RespondNow(WithArguments(std::move(result)));
+}
+
+TerminalPrivateSetPrefsFunction::~TerminalPrivateSetPrefsFunction() = default;
+
+ExtensionFunction::ResponseAction TerminalPrivateSetPrefsFunction::Run() {
+  std::optional<SetPrefs::Params> params = SetPrefs::Params::Create(args());
+  EXTENSION_FUNCTION_VALIDATE(params);
+
+  PrefService* service =
+      Profile::FromBrowserContext(browser_context())->GetPrefs();
+
+  static const base::NoDestructor<
+      base::flat_map<std::string, base::Value::Type>>
+      kAllowList{{{guest_os::prefs::kGuestOsTerminalSettings,
+                   base::Value::Type::DICT}}};
+
+  for (const auto item : params->prefs.additional_properties) {
+    // Write prefs if they are allowed, and match expected type, else ignore.
+    auto allow_it = kAllowList->find(item.first);
+    if (allow_it == kAllowList->end() ||
+        allow_it->second != item.second.type()) {
+      LOG(WARNING) << "Ignoring non-allowed SetPrefs path=" << item.first
+                   << ", type=" << item.second.type();
+      continue;
+    }
+    service->Set(item.first, item.second);
+  }
+  return RespondNow(NoArguments());
 }
 
 }  // namespace extensions
