@@ -1,4 +1,4 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,7 +8,6 @@
 
 #include <memory>
 
-#include "base/no_destructor.h"
 #include "chrome/browser/extensions/api/web_navigation/web_navigation_api_constants.h"
 #include "chrome/browser/extensions/api/web_navigation/web_navigation_api_helpers.h"
 #include "chrome/browser/extensions/extension_tab_util.h"
@@ -25,6 +24,7 @@
 #include "extensions/browser/event_router.h"
 #include "extensions/browser/extension_api_frame_id_map.h"
 #include "extensions/browser/view_type_utils.h"
+#include "extensions/common/mojom/view_type.mojom.h"
 #include "net/base/net_errors.h"
 
 namespace GetFrame = extensions::api::web_navigation::GetFrame;
@@ -34,18 +34,6 @@ namespace extensions {
 
 namespace web_navigation = api::web_navigation;
 
-namespace {
-
-using TabObserverMap =
-    std::map<content::WebContents*, WebNavigationTabObserver*>;
-
-TabObserverMap& GetTabObserverMap() {
-  static base::NoDestructor<TabObserverMap> s;
-  return *s;
-}
-
-}  // namespace
-
 // WebNavigtionEventRouter -------------------------------------------
 
 WebNavigationEventRouter::PendingWebContents::PendingWebContents() = default;
@@ -68,7 +56,7 @@ void WebNavigationEventRouter::PendingWebContents::Set(
 }
 
 void WebNavigationEventRouter::PendingWebContents::WebContentsDestroyed() {
-  std::move(on_destroy_).Run(target_web_contents_);
+  std::move(on_destroy_).Run(target_web_contents_.get());
   // |this| is deleted!
 }
 
@@ -95,11 +83,14 @@ void WebNavigationEventRouter::OnTabStripModelChanged(
     if (!tab_observer) {
       // If you hit this DCHECK(), please add reproduction steps to
       // http://crbug.com/109464.
-      DCHECK(GetViewType(replace->old_contents) != VIEW_TYPE_TAB_CONTENTS);
+      DCHECK(GetViewType(replace->old_contents) !=
+             mojom::ViewType::kTabContents);
       return;
     }
-    if (!FrameNavigationState::IsValidUrl(replace->old_contents->GetURL()) ||
-        !FrameNavigationState::IsValidUrl(replace->new_contents->GetURL()))
+    if (!FrameNavigationState::IsValidUrl(
+            replace->old_contents->GetLastCommittedURL()) ||
+        !FrameNavigationState::IsValidUrl(
+            replace->new_contents->GetLastCommittedURL()))
       return;
 
     web_navigation_api_helpers::DispatchOnTabReplaced(
@@ -124,15 +115,16 @@ void WebNavigationEventRouter::RecordNewWebContents(
   if (!tab_observer) {
     // If you hit this DCHECK(), please add reproduction steps to
     // http://crbug.com/109464.
-    DCHECK(GetViewType(source_web_contents) != VIEW_TYPE_TAB_CONTENTS);
+    DCHECK(GetViewType(source_web_contents) != mojom::ViewType::kTabContents);
     return;
   }
-  const FrameNavigationState& frame_navigation_state =
-      tab_observer->frame_navigation_state();
 
-  content::RenderFrameHost* frame_host = content::RenderFrameHost::FromID(
-      source_render_process_id, source_render_frame_id);
-  if (!frame_navigation_state.CanSendEvents(frame_host))
+  auto* frame_host = content::RenderFrameHost::FromID(source_render_process_id,
+                                                      source_render_frame_id);
+  auto* frame_navigation_state =
+      FrameNavigationState::GetForCurrentDocument(frame_host);
+
+  if (!frame_navigation_state || !frame_navigation_state->CanSendEvents())
     return;
 
   int source_extension_frame_id =
@@ -178,42 +170,35 @@ void WebNavigationEventRouter::PendingWebContentsDestroyed(
 
 WebNavigationTabObserver::WebNavigationTabObserver(
     content::WebContents* web_contents)
-    : WebContentsObserver(web_contents) {
-  GetTabObserverMap().insert(TabObserverMap::value_type(web_contents, this));
-  navigation_state_.FrameHostCreated(web_contents->GetMainFrame());
-}
+    : WebContentsObserver(web_contents),
+      content::WebContentsUserData<WebNavigationTabObserver>(*web_contents) {}
 
 WebNavigationTabObserver::~WebNavigationTabObserver() {}
 
 // static
 WebNavigationTabObserver* WebNavigationTabObserver::Get(
     content::WebContents* web_contents) {
-  auto i = GetTabObserverMap().find(web_contents);
-  return i == GetTabObserverMap().end() ? NULL : i->second;
+  return FromWebContents(web_contents);
 }
 
 void WebNavigationTabObserver::RenderFrameDeleted(
     content::RenderFrameHost* render_frame_host) {
-  if (navigation_state_.CanSendEvents(render_frame_host) &&
-      !navigation_state_.GetDocumentLoadCompleted(render_frame_host)) {
+  auto* navigation_state =
+      FrameNavigationState::GetForCurrentDocument(render_frame_host);
+  if (navigation_state && navigation_state->CanSendEvents() &&
+      !navigation_state->GetDocumentLoadCompleted()) {
     web_navigation_api_helpers::DispatchOnErrorOccurred(
-        web_contents(), render_frame_host,
-        navigation_state_.GetUrl(render_frame_host), net::ERR_ABORTED);
-    navigation_state_.SetErrorOccurredInFrame(render_frame_host);
+        web_contents(), render_frame_host, navigation_state->GetUrl(),
+        net::ERR_ABORTED);
+    navigation_state->SetErrorOccurredInFrame();
   }
 }
 
-void WebNavigationTabObserver::FrameDeleted(
-    content::RenderFrameHost* render_frame_host) {
-  navigation_state_.FrameHostDeleted(render_frame_host);
-}
-
 void WebNavigationTabObserver::RenderFrameHostChanged(
     content::RenderFrameHost* old_host,
     content::RenderFrameHost* new_host) {
   if (old_host)
     RenderFrameHostPendingDeletion(old_host);
-  navigation_state_.FrameHostCreated(new_host);
 }
 
 void WebNavigationTabObserver::DidStartNavigation(
@@ -262,67 +247,70 @@ void WebNavigationTabObserver::DidFinishNavigation(
 
 void WebNavigationTabObserver::DOMContentLoaded(
     content::RenderFrameHost* render_frame_host) {
-  if (!navigation_state_.CanSendEvents(render_frame_host))
+  auto* navigation_state =
+      FrameNavigationState::GetForCurrentDocument(render_frame_host);
+  if (!navigation_state || !navigation_state->CanSendEvents())
     return;
 
-  navigation_state_.SetParsingFinished(render_frame_host);
+  navigation_state->SetParsingFinished();
   web_navigation_api_helpers::DispatchOnDOMContentLoaded(
-      web_contents(), render_frame_host,
-      navigation_state_.GetUrl(render_frame_host));
+      web_contents(), render_frame_host, navigation_state->GetUrl());
 
-  if (!navigation_state_.GetDocumentLoadCompleted(render_frame_host))
+  if (!navigation_state->GetDocumentLoadCompleted())
     return;
 
   // The load might already have finished by the time we finished parsing. For
   // compatibility reasons, we artifically delay the load completed signal until
   // after parsing was completed.
   web_navigation_api_helpers::DispatchOnCompleted(
-      web_contents(), render_frame_host,
-      navigation_state_.GetUrl(render_frame_host));
+      web_contents(), render_frame_host, navigation_state->GetUrl());
 }
 
 void WebNavigationTabObserver::DidFinishLoad(
     content::RenderFrameHost* render_frame_host,
     const GURL& validated_url) {
+  auto* navigation_state =
+      FrameNavigationState::GetForCurrentDocument(render_frame_host);
   // When showing replacement content, we might get load signals for frames
-  // that weren't reguarly loaded.
-  if (!navigation_state_.IsValidFrame(render_frame_host))
+  // that weren't regularly loaded.
+  if (!navigation_state)
     return;
 
-  navigation_state_.SetDocumentLoadCompleted(render_frame_host);
-  if (!navigation_state_.CanSendEvents(render_frame_host))
+  navigation_state->SetDocumentLoadCompleted();
+  if (!navigation_state->CanSendEvents())
     return;
 
   // A new navigation might have started before the old one completed.
   // Ignore the old navigation completion in that case.
-  if (navigation_state_.GetUrl(render_frame_host) != validated_url)
+  if (navigation_state->GetUrl() != validated_url)
     return;
 
   // The load might already have finished by the time we finished parsing. For
   // compatibility reasons, we artifically delay the load completed signal until
   // after parsing was completed.
-  if (!navigation_state_.GetParsingFinished(render_frame_host))
+  if (!navigation_state->GetParsingFinished())
     return;
   web_navigation_api_helpers::DispatchOnCompleted(
-      web_contents(), render_frame_host,
-      navigation_state_.GetUrl(render_frame_host));
+      web_contents(), render_frame_host, navigation_state->GetUrl());
 }
 
 void WebNavigationTabObserver::DidFailLoad(
     content::RenderFrameHost* render_frame_host,
     const GURL& validated_url,
     int error_code) {
+  auto* navigation_state =
+      FrameNavigationState::GetForCurrentDocument(render_frame_host);
   // When showing replacement content, we might get load signals for frames
   // that weren't regularly loaded.
-  if (!navigation_state_.IsValidFrame(render_frame_host))
+  if (!navigation_state)
     return;
 
-  if (navigation_state_.CanSendEvents(render_frame_host)) {
+  if (navigation_state->CanSendEvents()) {
     web_navigation_api_helpers::DispatchOnErrorOccurred(
-        web_contents(), render_frame_host,
-        navigation_state_.GetUrl(render_frame_host), error_code);
+        web_contents(), render_frame_host, navigation_state->GetUrl(),
+        error_code);
   }
-  navigation_state_.SetErrorOccurredInFrame(render_frame_host);
+  navigation_state->SetErrorOccurredInFrame();
 }
 
 void WebNavigationTabObserver::DidOpenRequestedURL(
@@ -334,7 +322,9 @@ void WebNavigationTabObserver::DidOpenRequestedURL(
     ui::PageTransition transition,
     bool started_from_context_menu,
     bool renderer_initiated) {
-  if (!navigation_state_.CanSendEvents(source_render_frame_host))
+  auto* navigation_state =
+      FrameNavigationState::GetForCurrentDocument(source_render_frame_host);
+  if (!navigation_state || !navigation_state->CanSendEvents())
     return;
 
   // We only send the onCreatedNavigationTarget if we end up creating a new
@@ -365,10 +355,6 @@ void WebNavigationTabObserver::DidOpenRequestedURL(
       !new_contents_is_present_in_tabstrip);
 }
 
-void WebNavigationTabObserver::WebContentsDestroyed() {
-  GetTabObserverMap().erase(web_contents());
-}
-
 void WebNavigationTabObserver::DispatchCachedOnBeforeNavigate() {
   if (!pending_on_before_navigate_event_)
     return;
@@ -387,10 +373,12 @@ void WebNavigationTabObserver::HandleCommit(
       IsReferenceFragmentNavigation(navigation_handle->GetRenderFrameHost(),
                                     navigation_handle->GetURL());
 
-  navigation_state_.StartTrackingDocumentLoad(
-      navigation_handle->GetRenderFrameHost(), navigation_handle->GetURL(),
-      navigation_handle->IsSameDocument(),
-      false);  // is_error_page
+  FrameNavigationState::GetOrCreateForCurrentDocument(
+      navigation_handle->GetRenderFrameHost())
+      ->StartTrackingDocumentLoad(
+          navigation_handle->GetURL(), navigation_handle->IsSameDocument(),
+          navigation_handle->IsServedFromBackForwardCache(),
+          /*is_error_page=*/false);
 
   events::HistogramValue histogram_value = events::UNKNOWN;
   std::string event_name;
@@ -406,133 +394,227 @@ void WebNavigationTabObserver::HandleCommit(
   }
   web_navigation_api_helpers::DispatchOnCommitted(histogram_value, event_name,
                                                   navigation_handle);
+
+  if (navigation_handle->IsServedFromBackForwardCache()) {
+    web_navigation_api_helpers::DispatchOnCompleted(
+        navigation_handle->GetWebContents(),
+        navigation_handle->GetRenderFrameHost(), navigation_handle->GetURL());
+  }
 }
 
 void WebNavigationTabObserver::HandleError(
     content::NavigationHandle* navigation_handle) {
   if (navigation_handle->HasCommitted()) {
-    navigation_state_.StartTrackingDocumentLoad(
-        navigation_handle->GetRenderFrameHost(), navigation_handle->GetURL(),
-        navigation_handle->IsSameDocument(),
-        true);  // is_error_page
+    FrameNavigationState::GetOrCreateForCurrentDocument(
+        navigation_handle->GetRenderFrameHost())
+        ->StartTrackingDocumentLoad(navigation_handle->GetURL(),
+                                    navigation_handle->IsSameDocument(),
+                                    /*is_from_back_forward_cache=*/false,
+                                    /*is_error_page=*/true);
   }
 
   web_navigation_api_helpers::DispatchOnErrorOccurred(navigation_handle);
 }
 
-// See also NavigationController::IsURLSameDocumentNavigation.
 bool WebNavigationTabObserver::IsReferenceFragmentNavigation(
     content::RenderFrameHost* render_frame_host,
     const GURL& url) {
-  GURL existing_url = navigation_state_.GetUrl(render_frame_host);
+  auto* navigation_state =
+      FrameNavigationState::GetForCurrentDocument(render_frame_host);
+
+  GURL existing_url = navigation_state ? navigation_state->GetUrl() : GURL();
   if (existing_url == url)
     return false;
 
-  url::Replacements<char> replacements;
-  replacements.ClearRef();
-  return existing_url.ReplaceComponents(replacements) ==
-      url.ReplaceComponents(replacements);
+  return existing_url.EqualsIgnoringRef(url);
 }
 
 void WebNavigationTabObserver::RenderFrameHostPendingDeletion(
-    content::RenderFrameHost* pending_delete_rfh) {
-  // The |pending_delete_rfh| and its children are now pending deletion.
-  // Stop tracking them.
+    content::RenderFrameHost* pending_delete_render_frame_host) {
+  // The |pending_delete_render_frame_host| and its children are now pending
+  // deletion. Stop tracking them.
 
-  // 1) Collect them.
-  std::vector<content::RenderFrameHost*> to_be_deleted;
-  for (content::RenderFrameHost* render_frame_host : navigation_state_) {
-    if (render_frame_host == pending_delete_rfh ||
-        render_frame_host->IsDescendantOf(pending_delete_rfh)) {
-      to_be_deleted.push_back(render_frame_host);
-    }
-  }
-
-  // 2) Delete them.
-  for (content::RenderFrameHost* render_frame_host : to_be_deleted) {
-    // The RenderFrame may still be loading. Call RenderFrameDeleted()
-    // immediately to properly dispatch a load error occurred.
-    RenderFrameDeleted(render_frame_host);
-
-    navigation_state_.FrameHostDeleted(render_frame_host);
-  }
+  pending_delete_render_frame_host->ForEachRenderFrameHost(
+      [this](content::RenderFrameHost* render_frame_host) {
+        auto* navigation_state =
+            FrameNavigationState::GetForCurrentDocument(render_frame_host);
+        if (navigation_state) {
+          RenderFrameDeleted(render_frame_host);
+          FrameNavigationState::DeleteForCurrentDocument(render_frame_host);
+        }
+      });
 }
 
 ExtensionFunction::ResponseAction WebNavigationGetFrameFunction::Run() {
-  std::unique_ptr<GetFrame::Params> params(GetFrame::Params::Create(*args_));
-  EXTENSION_FUNCTION_VALIDATE(params.get());
-  int tab_id = params->details.tab_id;
-  int frame_id = params->details.frame_id;
+  std::optional<GetFrame::Params> params = GetFrame::Params::Create(args());
+  EXTENSION_FUNCTION_VALIDATE(params);
 
-  content::WebContents* web_contents;
-  if (!ExtensionTabUtil::GetTabById(tab_id, browser_context(),
-                                    include_incognito_information(),
-                                    &web_contents) ||
-      !web_contents) {
-    return RespondNow(OneArgument(base::Value()));
+  int tab_id = api::tabs::TAB_ID_NONE;
+  int frame_id = -1;
+
+  content::RenderFrameHost* render_frame_host = nullptr;
+  if (params->details.document_id) {
+    ExtensionApiFrameIdMap::DocumentId document_id =
+        ExtensionApiFrameIdMap::DocumentIdFromString(
+            *params->details.document_id);
+    if (!document_id)
+      return RespondNow(Error("Invalid documentId."));
+
+    // Note that we will globally find a RenderFrameHost but validate that
+    // we are in the right context still as we may be in the wrong profile
+    // or in incognito mode.
+    render_frame_host =
+        ExtensionApiFrameIdMap::Get()->GetRenderFrameHostByDocumentId(
+            document_id);
+
+    if (!render_frame_host)
+      return RespondNow(WithArguments(base::Value()));
+
+    content::WebContents* web_contents =
+        content::WebContents::FromRenderFrameHost(render_frame_host);
+    // We found the RenderFrameHost through a generic lookup so we must test to
+    // see if the WebContents is actually in our BrowserContext.
+    if (!ExtensionTabUtil::IsWebContentsInContext(
+            web_contents, browser_context(), include_incognito_information())) {
+      return RespondNow(WithArguments(base::Value()));
+    }
+
+    tab_id = ExtensionTabUtil::GetTabId(web_contents);
+    frame_id = ExtensionApiFrameIdMap::GetFrameId(render_frame_host);
+
+    // If the provided tab_id and frame_id do not match the calculated ones
+    // return.
+    if ((params->details.tab_id && *params->details.tab_id != tab_id) ||
+        (params->details.frame_id && *params->details.frame_id != frame_id)) {
+      return RespondNow(WithArguments(base::Value()));
+    }
+  } else {
+    // If documentId is not provided, tab_id and frame_id must be. Return early
+    // if not.
+    if (!params->details.tab_id || !params->details.frame_id) {
+      return RespondNow(Error(
+          "Either documentId or both tabId and frameId must be specified."));
+    }
+
+    tab_id = *params->details.tab_id;
+    frame_id = *params->details.frame_id;
+
+    content::WebContents* web_contents = nullptr;
+    if (!ExtensionTabUtil::GetTabById(tab_id, browser_context(),
+                                      include_incognito_information(),
+                                      &web_contents) ||
+        !web_contents) {
+      return RespondNow(WithArguments(base::Value()));
+    }
+
+    render_frame_host = ExtensionApiFrameIdMap::Get()->GetRenderFrameHostById(
+        web_contents, frame_id);
   }
 
-  WebNavigationTabObserver* observer =
-      WebNavigationTabObserver::Get(web_contents);
-  DCHECK(observer);
+  auto* frame_navigation_state =
+      render_frame_host
+          ? FrameNavigationState::GetForCurrentDocument(render_frame_host)
+          : nullptr;
+  if (!frame_navigation_state)
+    return RespondNow(WithArguments(base::Value()));
 
-  const FrameNavigationState& frame_navigation_state =
-      observer->frame_navigation_state();
-
-  content::RenderFrameHost* render_frame_host =
-      ExtensionApiFrameIdMap::Get()->GetRenderFrameHostById(web_contents,
-                                                            frame_id);
-  if (!frame_navigation_state.IsValidFrame(render_frame_host))
-    return RespondNow(OneArgument(base::Value()));
-
-  GURL frame_url = frame_navigation_state.GetUrl(render_frame_host);
-  if (!frame_navigation_state.IsValidUrl(frame_url))
-    return RespondNow(OneArgument(base::Value()));
+  GURL frame_url = frame_navigation_state->GetUrl();
+  if (!FrameNavigationState::IsValidUrl(frame_url))
+    return RespondNow(WithArguments(base::Value()));
 
   GetFrame::Results::Details frame_details;
   frame_details.url = frame_url.spec();
   frame_details.error_occurred =
-      frame_navigation_state.GetErrorOccurredInFrame(render_frame_host);
+      frame_navigation_state->GetErrorOccurredInFrame();
   frame_details.parent_frame_id =
-      ExtensionApiFrameIdMap::GetFrameId(render_frame_host->GetParent());
+      ExtensionApiFrameIdMap::GetParentFrameId(render_frame_host);
+  frame_details.document_id =
+      ExtensionApiFrameIdMap::GetDocumentId(render_frame_host).ToString();
+  // Only set the parentDocumentId value if we have a parent.
+  if (content::RenderFrameHost* parent_frame_host =
+          render_frame_host->GetParentOrOuterDocument()) {
+    frame_details.parent_document_id =
+        ExtensionApiFrameIdMap::GetDocumentId(parent_frame_host).ToString();
+  }
+  frame_details.frame_type =
+      ExtensionApiFrameIdMap::GetFrameType(render_frame_host);
+  frame_details.document_lifecycle =
+      ExtensionApiFrameIdMap::GetDocumentLifecycle(render_frame_host);
+
   return RespondNow(ArgumentList(GetFrame::Results::Create(frame_details)));
 }
 
 ExtensionFunction::ResponseAction WebNavigationGetAllFramesFunction::Run() {
-  std::unique_ptr<GetAllFrames::Params> params(
-      GetAllFrames::Params::Create(*args_));
-  EXTENSION_FUNCTION_VALIDATE(params.get());
+  std::optional<GetAllFrames::Params> params =
+      GetAllFrames::Params::Create(args());
+  EXTENSION_FUNCTION_VALIDATE(params);
   int tab_id = params->details.tab_id;
 
-  content::WebContents* web_contents;
+  content::WebContents* web_contents = nullptr;
   if (!ExtensionTabUtil::GetTabById(tab_id, browser_context(),
                                     include_incognito_information(),
                                     &web_contents) ||
       !web_contents) {
-    return RespondNow(OneArgument(base::Value()));
+    return RespondNow(WithArguments(base::Value()));
   }
 
-  WebNavigationTabObserver* observer =
-      WebNavigationTabObserver::Get(web_contents);
-  DCHECK(observer);
-
-  const FrameNavigationState& navigation_state =
-      observer->frame_navigation_state();
-
   std::vector<GetAllFrames::Results::DetailsType> result_list;
-  for (auto it = navigation_state.begin(); it != navigation_state.end(); ++it) {
-    GURL frame_url = navigation_state.GetUrl(*it);
-    if (!navigation_state.IsValidUrl(frame_url))
-      continue;
-    GetAllFrames::Results::DetailsType frame;
-    frame.url = frame_url.spec();
-    frame.frame_id = ExtensionApiFrameIdMap::GetFrameId(*it);
-    frame.parent_frame_id =
-        ExtensionApiFrameIdMap::GetFrameId((*it)->GetParent());
-    frame.process_id = (*it)->GetProcess()->GetID();
-    frame.error_occurred = navigation_state.GetErrorOccurredInFrame(*it);
-    result_list.push_back(std::move(frame));
-  }
+
+  // We currently do not expose back/forward cached frames in the GetAllFrames
+  // API, but we do explicitly include prerendered frames.
+  web_contents
+      ->ForEachRenderFrameHostWithAction(
+          [web_contents,
+           &result_list](content::RenderFrameHost* render_frame_host) {
+            // Don't expose inner WebContents for the getFrames API.
+            if (content::WebContents::FromRenderFrameHost(render_frame_host) !=
+                web_contents) {
+              return content::RenderFrameHost::FrameIterationAction::
+                  kSkipChildren;
+            }
+
+            auto* navigation_state =
+                FrameNavigationState::GetForCurrentDocument(render_frame_host);
+
+            if (!navigation_state ||
+                !FrameNavigationState::IsValidUrl(navigation_state->GetUrl())) {
+              return content::RenderFrameHost::FrameIterationAction::kContinue;
+            }
+
+            // Skip back/forward cached frames.
+            if (render_frame_host->IsInLifecycleState(
+                    content::RenderFrameHost::LifecycleState::
+                        kInBackForwardCache)) {
+              return content::RenderFrameHost::FrameIterationAction::
+                  kSkipChildren;
+            }
+
+            GetAllFrames::Results::DetailsType frame;
+            frame.url = navigation_state->GetUrl().spec();
+            frame.frame_id =
+                ExtensionApiFrameIdMap::GetFrameId(render_frame_host);
+            frame.parent_frame_id =
+                ExtensionApiFrameIdMap::GetParentFrameId(render_frame_host);
+            frame.document_id =
+                ExtensionApiFrameIdMap::GetDocumentId(render_frame_host)
+                    .ToString();
+            // Only set the parentDocumentId value if we have a parent.
+            if (content::RenderFrameHost* parent_frame_host =
+                    render_frame_host->GetParentOrOuterDocument()) {
+              frame.parent_document_id =
+                  ExtensionApiFrameIdMap::GetDocumentId(parent_frame_host)
+                      .ToString();
+            }
+            frame.frame_type =
+                ExtensionApiFrameIdMap::GetFrameType(render_frame_host);
+            frame.document_lifecycle =
+                ExtensionApiFrameIdMap::GetDocumentLifecycle(render_frame_host);
+            frame.process_id = render_frame_host->GetProcess()->GetID();
+            frame.error_occurred = navigation_state->GetErrorOccurredInFrame();
+            result_list.push_back(std::move(frame));
+            return content::RenderFrameHost::FrameIterationAction::kContinue;
+          });
+
   return RespondNow(ArgumentList(GetAllFrames::Results::Create(result_list)));
 }
 
@@ -557,8 +639,7 @@ WebNavigationAPI::WebNavigationAPI(content::BrowserContext* context)
                                  web_navigation::OnTabReplaced::kEventName);
 }
 
-WebNavigationAPI::~WebNavigationAPI() {
-}
+WebNavigationAPI::~WebNavigationAPI() {}
 
 void WebNavigationAPI::Shutdown() {
   EventRouter::Get(browser_context_)->UnregisterObserver(this);
@@ -574,11 +655,11 @@ WebNavigationAPI::GetFactoryInstance() {
 }
 
 void WebNavigationAPI::OnListenerAdded(const EventListenerInfo& details) {
-  web_navigation_event_router_.reset(new WebNavigationEventRouter(
-      Profile::FromBrowserContext(browser_context_)));
+  web_navigation_event_router_ = std::make_unique<WebNavigationEventRouter>(
+      Profile::FromBrowserContext(browser_context_));
   EventRouter::Get(browser_context_)->UnregisterObserver(this);
 }
 
-WEB_CONTENTS_USER_DATA_KEY_IMPL(WebNavigationTabObserver)
+WEB_CONTENTS_USER_DATA_KEY_IMPL(WebNavigationTabObserver);
 
 }  // namespace extensions
