@@ -1,37 +1,40 @@
-// Copyright 2018 The Chromium Authors. All rights reserved.
+// Copyright 2018 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/media/webrtc/webrtc_event_log_manager_common.h"
 
-#include <cctype>
 #include <limits>
 
 #include "base/files/file_util.h"
 #include "base/logging.h"
 #include "base/memory/scoped_refptr.h"
 #include "base/metrics/histogram_functions.h"
-#include "base/stl_util.h"
+#include "base/ranges/algorithm.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_piece.h"
+#include "base/strings/string_util.h"
 #include "base/strings/stringprintf.h"
-#include "base/threading/sequenced_task_runner_handle.h"
+#include "base/task/sequenced_task_runner.h"
 #include "base/unguessable_token.h"
 #include "build/chromeos_buildflags.h"
-#if !defined(TOOLKIT_QT)
+#if !BUILDFLAG(IS_QTWEBENGINE)
 #include "chrome/browser/policy/profile_policy_connector.h"
 #endif
 #include "chrome/browser/profiles/profile.h"
-#if !defined(TOOLKIT_QT)
+#if !BUILDFLAG(IS_QTWEBENGINE)
 #include "components/policy/core/common/policy_service.h"
 #endif
 #include "content/public/browser/browser_context.h"
 #include "content/public/browser/browser_thread.h"
 #include "content/public/browser/render_process_host.h"
+#include "third_party/abseil-cpp/absl/strings/ascii.h"
 #include "third_party/zlib/zlib.h"
 
 #if BUILDFLAG(IS_CHROMEOS_ASH)
 #include "chrome/browser/ash/profiles/profile_helper.h"
+#include "components/user_manager/user.h"
+#include "components/user_manager/user_type.h"
 #endif
 
 namespace webrtc_event_logging {
@@ -109,7 +112,7 @@ constexpr size_t kWebAppIdLength = 2;
 class Budget {
  public:
   // If !max.has_value(), the budget is unlimited.
-  explicit Budget(base::Optional<size_t> max) : max_(max), current_(0) {}
+  explicit Budget(std::optional<size_t> max) : max_(max), current_(0) {}
 
   // Check whether the budget allows consuming an additional |consumed| of
   // the resource.
@@ -141,7 +144,7 @@ class Budget {
   }
 
  private:
-  const base::Optional<size_t> max_;
+  const std::optional<size_t> max_;
   size_t current_;
 };
 
@@ -151,7 +154,7 @@ class BaseLogFileWriter : public LogFileWriter {
   // If !max_file_size_bytes.has_value(), an unlimited writer is created.
   // If it has a value, it must be at least MinFileSizeBytes().
   BaseLogFileWriter(const base::FilePath& path,
-                    base::Optional<size_t> max_file_size_bytes);
+                    std::optional<size_t> max_file_size_bytes);
 
   ~BaseLogFileWriter() override;
 
@@ -211,8 +214,8 @@ class BaseLogFileWriter : public LogFileWriter {
 };
 
 BaseLogFileWriter::BaseLogFileWriter(const base::FilePath& path,
-                                     base::Optional<size_t> max_file_size_bytes)
-    : task_runner_(base::SequencedTaskRunnerHandle::Get()),
+                                     std::optional<size_t> max_file_size_bytes)
+    : task_runner_(base::SequencedTaskRunner::GetCurrentDefault()),
       path_(path),
       state_(State::PRE_INIT),
       budget_(max_file_size_bytes) {}
@@ -222,7 +225,7 @@ BaseLogFileWriter::~BaseLogFileWriter() {
     // Chrome shut-down. The original task_runner_ is no longer running, so
     // no risk of concurrent access or races.
     DCHECK_CURRENTLY_ON(content::BrowserThread::UI);
-    task_runner_ = base::SequencedTaskRunnerHandle::Get();
+    task_runner_ = base::SequencedTaskRunner::GetCurrentDefault();
   }
 
   if (state() != State::CLOSED && state() != State::DELETED) {
@@ -240,7 +243,7 @@ bool BaseLogFileWriter::Init() {
 
   // Attempt to create the file.
   constexpr int file_flags = base::File::FLAG_CREATE | base::File::FLAG_WRITE |
-                             base::File::FLAG_EXCLUSIVE_WRITE;
+                             base::File::FLAG_WIN_EXCLUSIVE_WRITE;
   file_.Initialize(path_, file_flags);
   if (!file_.IsValid() || !file_.created()) {
     LOG(WARNING) << "Couldn't create remote-bound WebRTC event log file.";
@@ -369,7 +372,7 @@ bool BaseLogFileWriter::Finalize() {
 class GzippedLogFileWriter : public BaseLogFileWriter {
  public:
   GzippedLogFileWriter(const base::FilePath& path,
-                       base::Optional<size_t> max_file_size_bytes,
+                       std::optional<size_t> max_file_size_bytes,
                        std::unique_ptr<LogCompressor> compressor);
 
   ~GzippedLogFileWriter() override = default;
@@ -389,7 +392,7 @@ class GzippedLogFileWriter : public BaseLogFileWriter {
 
 GzippedLogFileWriter::GzippedLogFileWriter(
     const base::FilePath& path,
-    base::Optional<size_t> max_file_size_bytes,
+    std::optional<size_t> max_file_size_bytes,
     std::unique_ptr<LogCompressor> compressor)
     : BaseLogFileWriter(path, max_file_size_bytes),
       compressor_(std::move(compressor)) {
@@ -453,8 +456,7 @@ bool GzippedLogFileWriter::Write(const std::string& input) {
     }
   }
 
-  NOTREACHED();
-  return false;  // Appease compiler.
+  NOTREACHED_NORETURN();
 }
 
 bool GzippedLogFileWriter::Finalize() {
@@ -483,7 +485,7 @@ bool GzippedLogFileWriter::Finalize() {
 class GzipLogCompressor : public LogCompressor {
  public:
   GzipLogCompressor(
-      base::Optional<size_t> max_size_bytes,
+      std::optional<size_t> max_size_bytes,
       std::unique_ptr<CompressedSizeEstimator> compressed_size_estimator);
 
   ~GzipLogCompressor() override;
@@ -509,8 +511,8 @@ class GzipLogCompressor : public LogCompressor {
   // Returns the budget left after reserving the GZIP overhead.
   // Optionals without a value, both in the parameters as well as in the
   // return value of the function, signal an unlimited amount.
-  static base::Optional<size_t> SizeAfterOverheadReservation(
-      base::Optional<size_t> max_size_bytes);
+  static std::optional<size_t> SizeAfterOverheadReservation(
+      std::optional<size_t> max_size_bytes);
 
   // Compresses |input| into |output|, while observing the budget (unless
   // !budgeted). If |last|, also closes the stream.
@@ -529,7 +531,7 @@ class GzipLogCompressor : public LogCompressor {
 };
 
 GzipLogCompressor::GzipLogCompressor(
-    base::Optional<size_t> max_size_bytes,
+    std::optional<size_t> max_size_bytes,
     std::unique_ptr<CompressedSizeEstimator> compressed_size_estimator)
     : state_(State::PRE_HEADER),
       budget_(SizeAfterOverheadReservation(max_size_bytes)),
@@ -586,8 +588,7 @@ LogCompressor::Result GzipLogCompressor::Compress(const std::string& input,
       return result;
   }
 
-  NOTREACHED();
-  return Result::ERROR_ENCOUNTERED;  // Appease compiler.
+  NOTREACHED_NORETURN();
 }
 
 bool GzipLogCompressor::CreateFooter(std::string* output) {
@@ -615,10 +616,10 @@ bool GzipLogCompressor::CreateFooter(std::string* output) {
   return true;
 }
 
-base::Optional<size_t> GzipLogCompressor::SizeAfterOverheadReservation(
-    base::Optional<size_t> max_size_bytes) {
+std::optional<size_t> GzipLogCompressor::SizeAfterOverheadReservation(
+    std::optional<size_t> max_size_bytes) {
   if (!max_size_bytes.has_value()) {
-    return base::Optional<size_t>();
+    return std::optional<size_t>();
   } else {
     DCHECK_GE(max_size_bytes.value(), kGzipHeaderBytes + kGzipFooterBytes);
     return max_size_bytes.value() - (kGzipHeaderBytes + kGzipFooterBytes);
@@ -744,10 +745,8 @@ size_t ExtractWebAppId(base::StringPiece str) {
   DCHECK_EQ(str.length(), kWebAppIdLength);
 
   // Avoid leading '+', etc.
-  for (size_t i = 0; i < str.length(); i++) {
-    if (!std::isdigit(str[i])) {
-      return kInvalidWebRtcEventLogWebAppId;
-    }
+  if (!base::ranges::all_of(str, absl::ascii_isdigit)) {
+    return kInvalidWebRtcEventLogWebAppId;
   }
 
   size_t result;
@@ -781,7 +780,7 @@ base::FilePath::StringPieceType BaseLogFileWriterFactory::Extension() const {
 
 std::unique_ptr<LogFileWriter> BaseLogFileWriterFactory::Create(
     const base::FilePath& path,
-    base::Optional<size_t> max_file_size_bytes) const {
+    std::optional<size_t> max_file_size_bytes) const {
   if (max_file_size_bytes.has_value() &&
       max_file_size_bytes.value() < MinFileSizeBytes()) {
     LOG(WARNING) << "Max size (" << max_file_size_bytes.value()
@@ -824,7 +823,7 @@ size_t GzipLogCompressorFactory::MinSizeBytes() const {
 }
 
 std::unique_ptr<LogCompressor> GzipLogCompressorFactory::Create(
-    base::Optional<size_t> max_size_bytes) const {
+    std::optional<size_t> max_size_bytes) const {
   if (max_size_bytes.has_value() && max_size_bytes.value() < MinSizeBytes()) {
     LOG(WARNING) << "Max size (" << max_size_bytes.value()
                  << ") below minimum size (" << MinSizeBytes() << ").";
@@ -851,7 +850,7 @@ base::FilePath::StringPieceType GzippedLogFileWriterFactory::Extension() const {
 
 std::unique_ptr<LogFileWriter> GzippedLogFileWriterFactory::Create(
     const base::FilePath& path,
-    base::Optional<size_t> max_file_size_bytes) const {
+    std::optional<size_t> max_file_size_bytes) const {
   if (max_file_size_bytes.has_value() &&
       max_file_size_bytes.value() < MinFileSizeBytes()) {
     LOG(WARNING) << "Size below allowed minimum.";
@@ -933,7 +932,7 @@ base::FilePath WebRtcEventLogPath(
 bool IsValidRemoteBoundLogFilename(const std::string& filename) {
   // The -1 is because of the implict \0.
   const size_t kPrefixLength =
-      base::size(kRemoteBoundWebRtcEventLogFileNamePrefix) - 1;
+      std::size(kRemoteBoundWebRtcEventLogFileNamePrefix) - 1;
 
   // [prefix]_[web_app_id]_[log_id]
   const size_t expected_length =
@@ -973,12 +972,7 @@ bool IsValidRemoteBoundLogFilename(const std::string& filename) {
   // Expect log ID.
   const std::string log_id = filename.substr(index);
   DCHECK_EQ(log_id.length(), kWebRtcEventLogIdLength);
-  const char* const log_id_chars = "0123456789ABCDEF";
-  if (filename.find_first_not_of(log_id_chars, index) != std::string::npos) {
-    return false;
-  }
-
-  return true;
+  return base::ContainsOnlyChars(log_id, "0123456789ABCDEF");
 }
 
 bool IsValidRemoteBoundLogFilePath(const base::FilePath& path) {
@@ -1013,7 +1007,7 @@ size_t ExtractRemoteBoundWebRtcEventLogWebAppIdFromPath(
 
   // The -1 is because of the implict \0.
   const size_t kPrefixLength =
-      base::size(kRemoteBoundWebRtcEventLogFileNamePrefix) - 1;
+      std::size(kRemoteBoundWebRtcEventLogFileNamePrefix) - 1;
 
   // The +1 is for the underscore between the prefix and the web-app ID.
   // Length verified by above call to IsValidRemoteBoundLogFilename().
@@ -1027,7 +1021,7 @@ bool DoesProfileDefaultToLoggingEnabled(const Profile* const profile) {
 // For Chrome OS, exclude special profiles and users.
 #if BUILDFLAG(IS_CHROMEOS_ASH)
   const user_manager::User* user =
-      chromeos::ProfileHelper::Get()->GetUserByProfile(profile);
+      ash::ProfileHelper::Get()->GetUserByProfile(profile);
   // We do not log an error here since this can happen in several cases,
   // e.g. for signin profiles or lock screen app profiles.
   if (!user) {
@@ -1037,19 +1031,19 @@ bool DoesProfileDefaultToLoggingEnabled(const Profile* const profile) {
   if (user_type != user_manager::USER_TYPE_REGULAR) {
     return false;
   }
-  if (chromeos::ProfileHelper::IsEphemeralUserProfile(profile)) {
+  if (ash::ProfileHelper::IsEphemeralUserProfile(profile)) {
     return false;
   }
 #endif
 
-#if !defined(TOOLKIT_QT)
+#if !BUILDFLAG(IS_QTWEBENGINE)
   // We only want a default of true for regular (i.e. logged-in) profiles
   // receiving cloud-based user-level enterprise policies. Supervised (child)
   // profiles are considered regular and can also receive cloud policies in some
   // cases (e.g. on Chrome OS). Although currently this should be covered by the
   // other checks, let's explicitly check to anticipate edge cases and make the
   // requirement explicit.
-  if (!profile->IsRegularProfile() || profile->IsSupervised()) {
+  if (profile->IsOffTheRecord() || profile->IsChild()) {
     return false;
   }
 
