@@ -1,4 +1,4 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,13 +10,15 @@
 #include <utility>
 #include <vector>
 
-#include "base/bind.h"
+#include "base/command_line.h"
 #include "base/compiler_specific.h"
 #include "base/files/file_path.h"
+#include "base/functional/bind.h"
 #include "base/metrics/field_trial.h"
 #include "base/metrics/histogram_macros.h"
-#include "base/stl_util.h"
-#include "base/threading/thread_task_runner_handle.h"
+#include "base/path_service.h"
+#include "base/task/sequenced_task_runner.h"
+#include "base/task/single_thread_task_runner.h"
 #include "base/time/time.h"
 #include "base/trace_event/trace_event.h"
 #include "build/branding_buildflags.h"
@@ -29,12 +31,13 @@
 #include "chrome/browser/prefs/profile_pref_store_manager.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/sync/glue/sync_start_util.h"
-#include "chrome/browser/ui/profile_error_dialog.h"
+#include "chrome/browser/ui/profiles/profile_error_dialog.h"
 #include "chrome/common/buildflags.h"
 #include "chrome/common/chrome_constants.h"
+#include "chrome/common/chrome_paths.h"
 #include "chrome/common/pref_names.h"
+#include "chrome/grit/branded_strings.h"
 #include "chrome/grit/browser_resources.h"
-#include "chrome/grit/chromium_strings.h"
 #include "chrome/grit/generated_resources.h"
 #include "components/component_updater/pref_names.h"
 #include "components/policy/core/browser/configuration_policy_pref_store.h"
@@ -52,6 +55,8 @@
 #include "components/search_engines/default_search_manager.h"
 #include "components/search_engines/search_engines_pref_names.h"
 #include "components/signin/public/base/signin_pref_names.h"
+#include "components/supervised_user/core/common/buildflags.h"
+#include "components/sync/base/features.h"
 #include "components/sync/base/model_type.h"
 #include "components/sync/base/pref_names.h"
 #include "components/sync_preferences/pref_model_associator.h"
@@ -67,20 +72,24 @@
 #include "sql/error_delegate_util.h"
 #include "ui/base/resource/resource_bundle.h"
 
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+#include "base/files/file_util.h"
+#endif
+
 #if BUILDFLAG(ENABLE_EXTENSIONS)
 #include "extensions/browser/pref_names.h"
 #endif
 
 #if BUILDFLAG(ENABLE_SUPERVISED_USERS)
-#include "chrome/browser/supervised_user/supervised_user_pref_store.h"
+#include "components/supervised_user/core/browser/supervised_user_pref_store.h"
 #endif
 
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
 #include "base/enterprise_util.h"
 #if BUILDFLAG(ENABLE_RLZ)
 #include "rlz/lib/machine_id.h"  // nogncheck crbug.com/1125897
 #endif  // BUILDFLAG(ENABLE_RLZ)
-#endif  // defined(OS_WIN)
+#endif  // BUILDFLAG(IS_WIN)
 
 using content::BrowserContext;
 using content::BrowserThread;
@@ -93,12 +102,12 @@ using ValueType = prefs::mojom::TrackedPreferenceMetadata::ValueType;
 
 namespace {
 
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
 // Whether we are in testing mode; can be enabled via
 // DisableDomainCheckForTesting(). Forces startup checks to ignore the presence
 // of a domain when determining the active SettingsEnforcement group.
 bool g_disable_domain_check_for_testing = false;
-#endif  // OS_WIN
+#endif  // BUILDFLAG(IS_WIN)
 
 // These preferences must be kept in sync with the TrackedPreference enum in
 // tools/metrics/histograms/enums.xml. To add a new preference, append it to the
@@ -121,11 +130,12 @@ const prefs::TrackedPreferenceMetadata kTrackedPrefs[] = {
     {5, extensions::pref_names::kExtensions, EnforcementLevel::NO_ENFORCEMENT,
      PrefTrackingStrategy::SPLIT, ValueType::IMPERSONAL},
 #endif
-    {6, prefs::kGoogleServicesLastUsername, EnforcementLevel::ENFORCE_ON_LOAD,
-     PrefTrackingStrategy::ATOMIC, ValueType::PERSONAL},
+    {6, prefs::kGoogleServicesLastSyncingUsername,
+     EnforcementLevel::ENFORCE_ON_LOAD, PrefTrackingStrategy::ATOMIC,
+     ValueType::PERSONAL},
     {7, prefs::kSearchProviderOverrides, EnforcementLevel::ENFORCE_ON_LOAD,
      PrefTrackingStrategy::ATOMIC, ValueType::IMPERSONAL},
-#if !defined(OS_ANDROID)
+#if !BUILDFLAG(IS_ANDROID)
     {11, prefs::kPinnedTabs, EnforcementLevel::ENFORCE_ON_LOAD,
      PrefTrackingStrategy::ATOMIC, ValueType::IMPERSONAL},
 #endif
@@ -148,43 +158,25 @@ const prefs::TrackedPreferenceMetadata kTrackedPrefs[] = {
     // releases after M50.
     {18, prefs::kSafeBrowsingIncidentsSent, EnforcementLevel::ENFORCE_ON_LOAD,
      PrefTrackingStrategy::ATOMIC, ValueType::IMPERSONAL},
-#if defined(OS_WIN)
-    {19, prefs::kSwReporterPromptVersion, EnforcementLevel::ENFORCE_ON_LOAD,
-     PrefTrackingStrategy::ATOMIC, ValueType::IMPERSONAL},
-#endif
-#if defined(OS_WIN)
-    {22, prefs::kSwReporterPromptSeed, EnforcementLevel::ENFORCE_ON_LOAD,
-     PrefTrackingStrategy::ATOMIC, ValueType::IMPERSONAL},
-#endif
     {23, prefs::kGoogleServicesAccountId, EnforcementLevel::ENFORCE_ON_LOAD,
      PrefTrackingStrategy::ATOMIC, ValueType::PERSONAL},
-    {24, prefs::kGoogleServicesLastAccountId, EnforcementLevel::ENFORCE_ON_LOAD,
-     PrefTrackingStrategy::ATOMIC, ValueType::PERSONAL},
-#if defined(OS_WIN)
-    {25, prefs::kSettingsResetPromptPromptWave,
+    // This is being migrated to `kGoogleServicesLastSyncingGaiaId` since
+    // 2022/10, and should move to `CleanupDeprecatedTrackedPreferences()` in
+    // pref_hash_filter.cc when that migration completes.
+    {24, prefs::kGoogleServicesLastSyncingAccountIdDeprecated,
      EnforcementLevel::ENFORCE_ON_LOAD, PrefTrackingStrategy::ATOMIC,
-     ValueType::IMPERSONAL},
-    {26, prefs::kSettingsResetPromptLastTriggeredForDefaultSearch,
-     EnforcementLevel::ENFORCE_ON_LOAD, PrefTrackingStrategy::ATOMIC,
-     ValueType::IMPERSONAL},
-    {27, prefs::kSettingsResetPromptLastTriggeredForStartupUrls,
-     EnforcementLevel::ENFORCE_ON_LOAD, PrefTrackingStrategy::ATOMIC,
-     ValueType::IMPERSONAL},
-    {28, prefs::kSettingsResetPromptLastTriggeredForHomepage,
-     EnforcementLevel::ENFORCE_ON_LOAD, PrefTrackingStrategy::ATOMIC,
-     ValueType::IMPERSONAL},
-#endif  // defined(OS_WIN)
+     ValueType::PERSONAL},
     {29, prefs::kMediaStorageIdSalt, EnforcementLevel::ENFORCE_ON_LOAD,
      PrefTrackingStrategy::ATOMIC, ValueType::IMPERSONAL},
-#if defined(OS_WIN) && BUILDFLAG(GOOGLE_CHROME_BRANDING)
+#if BUILDFLAG(IS_WIN) && BUILDFLAG(GOOGLE_CHROME_BRANDING)
     {30, prefs::kModuleBlocklistCacheMD5Digest,
      EnforcementLevel::ENFORCE_ON_LOAD, PrefTrackingStrategy::ATOMIC,
      ValueType::IMPERSONAL},
 #endif
-#if defined(OS_WIN)
-    {31, prefs::kSwReporterReportingEnabled, EnforcementLevel::ENFORCE_ON_LOAD,
+#if BUILDFLAG(IS_WIN)
+    {32, prefs::kMediaCdmOriginData, EnforcementLevel::ENFORCE_ON_LOAD,
      PrefTrackingStrategy::ATOMIC, ValueType::IMPERSONAL},
-#endif  // defined(OS_WIN)
+#endif  // BUILDFLAG(IS_WIN)
 
     // See note at top, new items added here also need to be added to
     // histograms.xml's TrackedPreference enum.
@@ -192,7 +184,7 @@ const prefs::TrackedPreferenceMetadata kTrackedPrefs[] = {
 
 // One more than the last tracked preferences ID above.
 const size_t kTrackedPrefsReportingIDsCount =
-    kTrackedPrefs[base::size(kTrackedPrefs) - 1].reporting_id + 1;
+    kTrackedPrefs[std::size(kTrackedPrefs) - 1].reporting_id + 1;
 
 // Each group enforces a superset of the protection provided by the previous
 // one.
@@ -209,16 +201,10 @@ enum SettingsEnforcementGroup {
 };
 
 SettingsEnforcementGroup GetSettingsEnforcementGroup() {
-# if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
   if (!g_disable_domain_check_for_testing) {
-    static bool first_call = true;
-    static const bool is_managed = base::IsMachineExternallyManaged();
-    if (first_call) {
-      UMA_HISTOGRAM_BOOLEAN("Settings.TrackedPreferencesNoEnforcementOnDomain",
-                            is_managed);
-      first_call = false;
-    }
-    if (is_managed)
+    static const bool is_domain_joined = base::IsEnterpriseDevice();
+    if (is_domain_joined)
       return GROUP_NO_ENFORCEMENT;
   }
 #endif
@@ -226,7 +212,7 @@ SettingsEnforcementGroup GetSettingsEnforcementGroup() {
   // Use the strongest enforcement setting on Windows and MacOS. Remember to
   // update the OFFICIAL_BUILD section of extension_startup_browsertest.cc and
   // pref_hash_browsertest.cc when updating the default value below.
-#if defined(OS_WIN) || defined(OS_MAC)
+#if BUILDFLAG(IS_WIN) || BUILDFLAG(IS_MAC)
   return GROUP_ENFORCE_DEFAULT;
 #else
   return GROUP_NO_ENFORCEMENT;
@@ -240,7 +226,7 @@ GetTrackingConfiguration() {
       GetSettingsEnforcementGroup();
 
   std::vector<prefs::mojom::TrackedPreferenceMetadataPtr> result;
-  for (size_t i = 0; i < base::size(kTrackedPrefs); ++i) {
+  for (size_t i = 0; i < std::size(kTrackedPrefs); ++i) {
     prefs::mojom::TrackedPreferenceMetadataPtr data =
         prefs::ConstructTrackedMetadata(kTrackedPrefs[i]);
 
@@ -272,34 +258,61 @@ GetTrackingConfiguration() {
 std::unique_ptr<ProfilePrefStoreManager> CreateProfilePrefStoreManager(
     const base::FilePath& profile_path) {
   std::string legacy_device_id;
-#if defined(OS_WIN) && BUILDFLAG(ENABLE_RLZ)
-  // This is used by
-  // chrome/browser/apps/platform_apps/api/music_manager_private/device_id_win.cc
-  // but that API is private (http://crbug.com/276485) and other platforms are
-  // not available synchronously.
-  // As part of improving pref metrics on other platforms we may want to find
-  // ways to defer preference loading until the device ID can be used.
+#if BUILDFLAG(IS_WIN) && BUILDFLAG(ENABLE_RLZ)
+  // This was used by the musicManagerPrivate API, and remains here for backward
+  // compatibility so ProfilePrefStoreManager can continue to calculate the same
+  // hashes as before.
   rlz_lib::GetMachineId(&legacy_device_id);
 #endif
   std::string seed;
   CHECK(ui::ResourceBundle::HasSharedInstance());
 #if BUILDFLAG(GOOGLE_CHROME_BRANDING)
-  seed = ui::ResourceBundle::GetSharedInstance()
-             .GetRawDataResource(IDR_PREF_HASH_SEED_BIN)
-             .as_string();
+  seed = std::string(ui::ResourceBundle::GetSharedInstance().GetRawDataResource(
+      IDR_PREF_HASH_SEED_BIN));
 #endif
   return std::make_unique<ProfilePrefStoreManager>(profile_path, seed,
                                                    legacy_device_id);
 }
 
-void PrepareFactory(sync_preferences::PrefServiceSyncableFactory* factory,
-                    const base::FilePath& pref_filename,
-                    policy::PolicyService* policy_service,
-                    SupervisedUserSettingsService* supervised_user_settings,
-                    scoped_refptr<PersistentPrefStore> user_pref_store,
-                    scoped_refptr<PrefStore> extension_prefs,
-                    bool async,
-                    policy::BrowserPolicyConnector* policy_connector) {
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+// The initial implementation of the standalone browser store
+// saved the preferences associated with a user under /home/chronos, which made
+// the prefs available for all users which subsequently log in on the device.
+// Currently the preferences are stored under the user profile directory. This
+// method cleans up the standalone browser preferences file under /home/chronos.
+// Note that the standalone browser preferences can't be migrated from the
+// /home/chronos directory to the profile directory because it's not possible to
+// know which profile has configured the settings. Once the user signs into the
+// user session and starts the Lacros browser, the standalone browser settings
+// will be restored in Ash and saved to the correct location.
+// TODO(b/304685319): Remove this cleanup method.
+void CleanupObsoleteStandaloneBrowserPrefsFile() {
+  base::FilePath user_data_dir;
+  CHECK(base::PathService::Get(chrome::DIR_USER_DATA, &user_data_dir));
+
+  base::FilePath obsolete_standalone_browser_file = user_data_dir.Append(
+      FILE_PATH_LITERAL("standalone_browser_preferences.json"));
+
+  if (!base::PathExists(obsolete_standalone_browser_file)) {
+    return;
+  }
+
+  LOG(WARNING) << "Removing obsolete " << obsolete_standalone_browser_file
+               << " file: "
+               << base::DeleteFile(obsolete_standalone_browser_file);
+}
+#endif
+
+void PrepareFactory(
+    sync_preferences::PrefServiceSyncableFactory* factory,
+    const base::FilePath& pref_filename,
+    policy::PolicyService* policy_service,
+    supervised_user::SupervisedUserSettingsService* supervised_user_settings,
+    scoped_refptr<PersistentPrefStore> user_pref_store,
+    scoped_refptr<PrefStore> extension_prefs,
+    scoped_refptr<PersistentPrefStore> standalone_browser_prefs,
+    bool async,
+    policy::BrowserPolicyConnector* policy_connector) {
   factory->SetManagedPolicies(policy_service, policy_connector);
   factory->SetRecommendedPolicies(policy_service, policy_connector);
 
@@ -312,6 +325,10 @@ void PrepareFactory(sync_preferences::PrefServiceSyncableFactory* factory,
   }
 #endif
 
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+  factory->set_standalone_browser_prefs(std::move(standalone_browser_prefs));
+#endif
+
   factory->set_async(async);
   factory->set_extension_prefs(std::move(extension_prefs));
   factory->set_command_line_prefs(
@@ -321,7 +338,7 @@ void PrepareFactory(sync_preferences::PrefServiceSyncableFactory* factory,
       &chrome_prefs::HandlePersistentPrefStoreReadError, pref_filename));
   factory->set_user_prefs(std::move(user_pref_store));
   factory->SetPrefModelAssociatorClient(
-      ChromePrefModelAssociatorClient::GetInstance());
+      base::MakeRefCounted<ChromePrefModelAssociatorClient>());
 }
 
 class ResetOnLoadObserverImpl : public prefs::mojom::ResetOnLoadObserver {
@@ -329,6 +346,9 @@ class ResetOnLoadObserverImpl : public prefs::mojom::ResetOnLoadObserver {
   explicit ResetOnLoadObserverImpl(const base::FilePath& profile_path)
       : profile_path_(profile_path) {}
 
+  ResetOnLoadObserverImpl(const ResetOnLoadObserverImpl&) = delete;
+  ResetOnLoadObserverImpl& operator=(const ResetOnLoadObserverImpl&) = delete;
+
   void OnResetOnLoad() override {
     // A StartSyncFlare used to kick sync early in case of a reset event. This
     // is done since sync may bring back the user's server value post-reset
@@ -342,41 +362,25 @@ class ResetOnLoadObserverImpl : public prefs::mojom::ResetOnLoadObserver {
 
  private:
   const base::FilePath profile_path_;
-
-  DISALLOW_COPY_AND_ASSIGN(ResetOnLoadObserverImpl);
 };
 
 }  // namespace
 
 namespace chrome_prefs {
 
-namespace internals {
-
-// Group modifications should be reflected in first_run_browsertest.cc and
-// pref_hash_browsertest.cc.
-const char kSettingsEnforcementTrialName[] = "SettingsEnforcement";
-const char kSettingsEnforcementGroupNoEnforcement[] = "no_enforcement";
-const char kSettingsEnforcementGroupEnforceAlways[] = "enforce_always";
-const char kSettingsEnforcementGroupEnforceAlwaysWithDSE[] =
-    "enforce_always_with_dse";
-const char kSettingsEnforcementGroupEnforceAlwaysWithExtensionsAndDSE[] =
-    "enforce_always_with_extensions_and_dse";
-
-}  // namespace internals
-
 std::unique_ptr<PrefService> CreateLocalState(
     const base::FilePath& pref_filename,
+    scoped_refptr<PersistentPrefStore> pref_store,
     policy::PolicyService* policy_service,
     scoped_refptr<PrefRegistry> pref_registry,
-    bool async,
     policy::BrowserPolicyConnector* policy_connector) {
   sync_preferences::PrefServiceSyncableFactory factory;
   PrepareFactory(&factory, pref_filename, policy_service,
                  nullptr,  // supervised_user_settings
-                 base::MakeRefCounted<JsonPrefStore>(
-                     pref_filename, std::unique_ptr<PrefFilter>()),
+                 pref_store,
                  nullptr,  // extension_prefs
-                 async, policy_connector);
+                 nullptr,  // standalone_browser_prefs
+                 /*async=*/false, policy_connector);
 
   return factory.Create(std::move(pref_registry));
 }
@@ -386,7 +390,7 @@ std::unique_ptr<sync_preferences::PrefServiceSyncable> CreateProfilePrefs(
     mojo::PendingRemote<prefs::mojom::TrackedPreferenceValidationDelegate>
         validation_delegate,
     policy::PolicyService* policy_service,
-    SupervisedUserSettingsService* supervised_user_settings,
+    supervised_user::SupervisedUserSettingsService* supervised_user_settings,
     scoped_refptr<PrefStore> extension_prefs,
     scoped_refptr<user_prefs::PrefRegistrySyncable> pref_registry,
     policy::BrowserPolicyConnector* connector,
@@ -399,27 +403,56 @@ std::unique_ptr<sync_preferences::PrefServiceSyncable> CreateProfilePrefs(
       std::make_unique<ResetOnLoadObserverImpl>(profile_path),
       reset_on_load_observer.InitWithNewPipeAndPassReceiver());
   sync_preferences::PrefServiceSyncableFactory factory;
+  scoped_refptr<JsonPrefStore> standalone_browser_prefs = nullptr;
+
   scoped_refptr<PersistentPrefStore> user_pref_store =
       CreateProfilePrefStoreManager(profile_path)
           ->CreateProfilePrefStore(
               GetTrackingConfiguration(), kTrackedPrefsReportingIDsCount,
-              std::move(io_task_runner), std::move(reset_on_load_observer),
+              io_task_runner, std::move(reset_on_load_observer),
               std::move(validation_delegate));
+
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+  // In ash, load the standalone_browser_prefs. This PrefStore
+  // contains data about prefs set by extensions in lacros where the feature
+  // lives in ash (for example, screen magnifier). The values are persisted in
+  // ash so they can be loaded on startup or when lacros is not running.
+  standalone_browser_prefs = base::MakeRefCounted<JsonPrefStore>(
+      profile_path.Append(
+          FILE_PATH_LITERAL("standalone_browser_preferences.json")),
+      std::unique_ptr<PrefFilter>(), io_task_runner);
+
+  io_task_runner->PostTask(
+      FROM_HERE, base::BindOnce(&CleanupObsoleteStandaloneBrowserPrefsFile));
+#endif
+
   PrepareFactory(&factory, profile_path, policy_service,
                  supervised_user_settings, std::move(user_pref_store),
-                 std::move(extension_prefs), async, connector);
+                 std::move(extension_prefs),
+                 std::move(standalone_browser_prefs), async, connector);
+
+  if (base::FeatureList::IsEnabled(syncer::kEnablePreferencesAccountStorage)) {
+    // Note: Only mobile platforms are targeted as part of the experiment,
+    // which do not require preference protection. Hence pref filters and
+    // ProfilePrefStoreManager::CreateProfilePrefStore() can be avoided.
+    factory.SetAccountPrefStore(base::MakeRefCounted<JsonPrefStore>(
+        /*pref_filename=*/profile_path.Append(
+            chrome::kAccountPreferencesFilename),
+        /*pref_filter=*/nullptr,
+        /*file_task_runner=*/io_task_runner));
+  }
+
   return factory.CreateSyncable(std::move(pref_registry));
 }
 
 void DisableDomainCheckForTesting() {
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
   g_disable_domain_check_for_testing = true;
-#endif  // OS_WIN
+#endif  // BUILDFLAG(IS_WIN)
 }
 
-bool InitializePrefsFromMasterPrefs(
-    const base::FilePath& profile_path,
-    std::unique_ptr<base::DictionaryValue> master_prefs) {
+bool InitializePrefsFromMasterPrefs(const base::FilePath& profile_path,
+                                    base::Value::Dict master_prefs) {
   return CreateProfilePrefStoreManager(profile_path)
       ->InitializePrefsFromMasterPrefs(GetTrackingConfiguration(),
                                        kTrackedPrefsReportingIDsCount,
@@ -447,11 +480,6 @@ void HandlePersistentPrefStoreReadError(
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI) ||
          !BrowserThread::IsThreadInitialized(BrowserThread::UI));
 
-  // Sample the histogram also for the successful case in order to get a
-  // baseline on the success rate in addition to the error distribution.
-  UMA_HISTOGRAM_ENUMERATION("PrefService.ReadError", error,
-                            PersistentPrefStore::PREF_READ_ERROR_MAX_ENUM);
-
   if (error != PersistentPrefStore::PREF_READ_ERROR_NONE) {
 #if !BUILDFLAG(IS_CHROMEOS_ASH)
     // Failing to load prefs on startup is a bad thing(TM). See bug 38352 for
@@ -465,10 +493,11 @@ void HandlePersistentPrefStoreReadError(
     }
 
     if (message_id) {
-      // Note: ThreadTaskRunnerHandle() is usually BrowserThread::UI but during
-      // early startup it can be ChromeBrowserMainParts::DeferringTaskRunner
-      // which will forward to BrowserThread::UI when it's initialized.
-      base::ThreadTaskRunnerHandle::Get()->PostTask(
+      // Note: SingleThreadTaskRunner::CurrentDefaultHandle is usually
+      // BrowserThread::UI but during early startup it can be
+      // ChromeBrowserMainParts::DeferringTaskRunner which will forward to
+      // BrowserThread::UI when it's initialized.
+      base::SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
           FROM_HERE,
           base::BindOnce(&ShowProfileErrorDialog, ProfileErrorType::PREFERENCES,
                          message_id,
