@@ -1,8 +1,9 @@
-// Copyright 2014 The Chromium Authors. All rights reserved.
+// Copyright 2014 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include <memory>
+#include <optional>
 #include <string>
 #include <utility>
 
@@ -27,6 +28,7 @@
 #include "chrome/browser/prefs/profile_pref_store_manager.h"
 #include "chrome/browser/prefs/session_startup_pref.h"
 #include "chrome/browser/profiles/profile.h"
+#include "chrome/browser/search_engine_choice/search_engine_choice_service_factory.h"
 #include "chrome/browser/ui/browser.h"
 #include "chrome/common/chrome_constants.h"
 #include "chrome/common/chrome_paths.h"
@@ -46,7 +48,7 @@
 #include "ash/constants/ash_switches.h"
 #endif
 
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
 #include "base/win/registry.h"
 #include "chrome/install_static/install_util.h"
 #endif
@@ -69,7 +71,7 @@ enum AllowedBuckets {
   ALLOW_ANY
 };
 
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
 std::wstring GetRegistryPathForTestProfile() {
   // Cleanup follow-up to http://crbug.com/721245 for the previous location of
   // this test key which had similar problems (to a lesser extent). It's
@@ -146,7 +148,7 @@ int GetTrackedPrefHistogramCount(const char* histogram_name,
 }
 
 #if !BUILDFLAG(IS_CHROMEOS_ASH)
-std::unique_ptr<base::DictionaryValue> ReadPrefsDictionary(
+std::optional<base::Value::Dict> ReadPrefsDictionary(
     const base::FilePath& pref_file) {
   JSONFileValueDeserializer deserializer(pref_file);
   int error_code = JSONFileValueDeserializer::JSON_NO_ERROR;
@@ -155,21 +157,20 @@ std::unique_ptr<base::DictionaryValue> ReadPrefsDictionary(
       deserializer.Deserialize(&error_code, &error_str);
   if (!prefs || error_code != JSONFileValueDeserializer::JSON_NO_ERROR) {
     ADD_FAILURE() << "Error #" << error_code << ": " << error_str;
-    return std::unique_ptr<base::DictionaryValue>();
+    return std::nullopt;
   }
   if (!prefs->is_dict()) {
     ADD_FAILURE();
-    return std::unique_ptr<base::DictionaryValue>();
+    return std::nullopt;
   }
-  return std::unique_ptr<base::DictionaryValue>(
-      static_cast<base::DictionaryValue*>(prefs.release()));
+  return std::move(*prefs).TakeDict();
 }
 #endif
 
 // Returns whether external validation is supported on the platform through
 // storing MACs in the registry.
 bool SupportsRegistryValidation() {
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
   return true;
 #else
   return false;
@@ -208,7 +209,7 @@ class PrefHashBrowserTestBase : public extensions::ExtensionBrowserTest {
     extensions::ExtensionBrowserTest::SetUpCommandLine(command_line);
 #if BUILDFLAG(IS_CHROMEOS_ASH)
     command_line->AppendSwitch(
-        chromeos::switches::kIgnoreUserProfileMappingForTests);
+        ash::switches::kIgnoreUserProfileMappingForTests);
 #endif
   }
 
@@ -249,12 +250,12 @@ class PrefHashBrowserTestBase : public extensions::ExtensionBrowserTest {
     EXPECT_EQ(protection_level_ > PROTECTION_DISABLED_ON_PLATFORM,
               base::PathExists(protected_pref_file));
 
-    std::unique_ptr<base::DictionaryValue> unprotected_preferences(
+    std::optional<base::Value::Dict> unprotected_preferences(
         ReadPrefsDictionary(unprotected_pref_file));
     if (!unprotected_preferences)
       return false;
 
-    std::unique_ptr<base::DictionaryValue> protected_preferences;
+    std::optional<base::Value::Dict> protected_preferences;
     if (protection_level_ > PROTECTION_DISABLED_ON_PLATFORM) {
       protected_preferences = ReadPrefsDictionary(protected_pref_file);
       if (!protected_preferences)
@@ -262,11 +263,11 @@ class PrefHashBrowserTestBase : public extensions::ExtensionBrowserTest {
     }
 
     // Let the underlying test modify the preferences.
-    AttackPreferencesOnDisk(unprotected_preferences.get(),
-                            protected_preferences.get());
+    AttackPreferencesOnDisk(
+        &unprotected_preferences.value(),
+        protected_preferences ? &protected_preferences.value() : nullptr);
 
     // Write the modified preferences back to disk.
-
     JSONFileValueSerializer unprotected_prefs_serializer(unprotected_pref_file);
     EXPECT_TRUE(
         unprotected_prefs_serializer.Serialize(*unprotected_preferences));
@@ -287,7 +288,7 @@ class PrefHashBrowserTestBase : public extensions::ExtensionBrowserTest {
     // order to be able to test all SettingsEnforcement groups.
     chrome_prefs::DisableDomainCheckForTesting();
 
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
     // Avoid polluting prefs for the user and the bots by writing to a specific
     // testing registry path.
     registry_key_for_external_validation_ = GetRegistryPathForTestProfile();
@@ -309,7 +310,7 @@ class PrefHashBrowserTestBase : public extensions::ExtensionBrowserTest {
   }
 
   void TearDown() override {
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
     // When done, delete the Registry key to avoid polluting the registry.
     if (!content::IsPreTest()) {
       std::wstring registry_key = GetRegistryPathForTestProfile();
@@ -364,12 +365,12 @@ class PrefHashBrowserTestBase : public extensions::ExtensionBrowserTest {
         EXPECT_EQ(protection_level_ > PROTECTION_DISABLED_ON_PLATFORM,
                   num_tracked_prefs_ > 0);
 
-        int num_split_tracked_prefs = GetTrackedPrefHistogramCount(
+        int split_tracked_prefs = GetTrackedPrefHistogramCount(
             user_prefs::tracked::kTrackedPrefHistogramUnchanged,
             user_prefs::tracked::kTrackedPrefRegistryValidationSuffix,
             BEGIN_ALLOW_SINGLE_BUCKET + 5);
         EXPECT_EQ(protection_level_ > PROTECTION_DISABLED_ON_PLATFORM ? 1 : 0,
-                  num_split_tracked_prefs);
+                  split_tracked_prefs);
       }
 
       num_tracked_prefs_ += num_split_tracked_prefs;
@@ -399,8 +400,8 @@ class PrefHashBrowserTestBase : public extensions::ExtensionBrowserTest {
   // main test. |unprotected_preferences| is never NULL, |protected_preferences|
   // may be NULL if in PROTECTION_DISABLED_ON_PLATFORM mode.
   virtual void AttackPreferencesOnDisk(
-      base::DictionaryValue* unprotected_preferences,
-      base::DictionaryValue* protected_preferences) = 0;
+      base::Value::Dict* unprotected_preferences,
+      base::Value::Dict* protected_preferences) = 0;
 
   // Called from the body of the main test. Overrides should use it to verify
   // that the browser had the desired reaction when faced when the attack
@@ -416,18 +417,18 @@ class PrefHashBrowserTestBase : public extensions::ExtensionBrowserTest {
     if (!ProfilePrefStoreManager::kPlatformSupportsPreferenceTracking)
       return PROTECTION_DISABLED_ON_PLATFORM;
 
-#if defined(OS_WIN) || defined(OS_MAC)
+#if BUILDFLAG(IS_WIN) || BUILDFLAG(IS_MAC)
     // The strongest mode is enforced on Windows and MacOS in the absence of a
     // field trial.
     return PROTECTION_ENABLED_ALL;
 #else
     return PROTECTION_DISABLED_FOR_GROUP;
-#endif  // defined(OS_WIN) || defined(OS_MAC)
+#endif  // BUILDFLAG(IS_WIN) || BUILDFLAG(IS_MAC)
   }
 
   int num_tracked_prefs_;
 
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
   std::wstring registry_key_for_external_validation_;
 #endif
 };
@@ -443,8 +444,8 @@ class PrefHashBrowserTestUnchangedDefault : public PrefHashBrowserTestBase {
   }
 
   void AttackPreferencesOnDisk(
-      base::DictionaryValue* unprotected_preferences,
-      base::DictionaryValue* protected_preferences) override {
+      base::Value::Dict* unprotected_preferences,
+      base::Value::Dict* protected_preferences) override {
     // No attack.
   }
 
@@ -535,15 +536,15 @@ class PrefHashBrowserTestClearedAtomic : public PrefHashBrowserTestBase {
   }
 
   void AttackPreferencesOnDisk(
-      base::DictionaryValue* unprotected_preferences,
-      base::DictionaryValue* protected_preferences) override {
-    base::DictionaryValue* selected_prefs =
+      base::Value::Dict* unprotected_preferences,
+      base::Value::Dict* protected_preferences) override {
+    base::Value::Dict* selected_prefs =
         protection_level_ >= PROTECTION_ENABLED_BASIC ? protected_preferences
                                                       : unprotected_preferences;
     // |selected_prefs| should never be NULL under the protection level picking
     // it.
     EXPECT_TRUE(selected_prefs);
-    EXPECT_TRUE(selected_prefs->Remove(prefs::kHomePage, NULL));
+    EXPECT_TRUE(selected_prefs->Remove(prefs::kHomePage));
   }
 
   void VerifyReactionToPrefAttack() override {
@@ -607,7 +608,15 @@ class PrefHashBrowserTestUntrustedInitialized : public PrefHashBrowserTestBase {
     // Explicitly set the DSE (it's otherwise NULL by default, preventing
     // thorough testing of the PROTECTION_ENABLED_DSE level).
     DefaultSearchManager default_search_manager(
-        profile()->GetPrefs(), DefaultSearchManager::ObserverCallback());
+        profile()->GetPrefs(),
+        search_engines::SearchEngineChoiceServiceFactory::GetForProfile(
+            profile()),
+        DefaultSearchManager::ObserverCallback()
+#if BUILDFLAG(IS_CHROMEOS_LACROS)
+            ,
+        profile()->IsMainProfile()
+#endif  // BUILDFLAG(IS_CHROMEOS_LACROS)
+    );
     DefaultSearchManager::Source dse_source =
         static_cast<DefaultSearchManager::Source>(-1);
 
@@ -628,11 +637,11 @@ class PrefHashBrowserTestUntrustedInitialized : public PrefHashBrowserTestBase {
   }
 
   void AttackPreferencesOnDisk(
-      base::DictionaryValue* unprotected_preferences,
-      base::DictionaryValue* protected_preferences) override {
-    unprotected_preferences->Remove("protection.macs", NULL);
+      base::Value::Dict* unprotected_preferences,
+      base::Value::Dict* protected_preferences) override {
+    unprotected_preferences->RemoveByDottedPath("protection.macs");
     if (protected_preferences)
-      protected_preferences->Remove("protection.macs", NULL);
+      protected_preferences->RemoveByDottedPath("protection.macs");
   }
 
   void VerifyReactionToPrefAttack() override {
@@ -663,13 +672,13 @@ class PrefHashBrowserTestUntrustedInitialized : public PrefHashBrowserTestBase {
       case PROTECTION_ENABLED_ALL:
       case PROTECTION_ENABLED_EXTENSIONS:
         ++num_protected_prefs;
-        FALLTHROUGH;
+        [[fallthrough]];
       case PROTECTION_ENABLED_DSE:
         ++num_protected_prefs;
-        FALLTHROUGH;
+        [[fallthrough]];
       case PROTECTION_ENABLED_BASIC:
         num_protected_prefs += num_tracked_prefs() - num_null_values - 2;
-        FALLTHROUGH;
+        [[fallthrough]];
       case PROTECTION_DISABLED_FOR_GROUP:
       case PROTECTION_DISABLED_ON_PLATFORM:
         // No protection.
@@ -687,7 +696,15 @@ class PrefHashBrowserTestUntrustedInitialized : public PrefHashBrowserTestBase {
     // Explicitly verify the result of reported resets.
 
     DefaultSearchManager default_search_manager(
-        profile()->GetPrefs(), DefaultSearchManager::ObserverCallback());
+        profile()->GetPrefs(),
+        search_engines::SearchEngineChoiceServiceFactory::GetForProfile(
+            profile()),
+        DefaultSearchManager::ObserverCallback()
+#if BUILDFLAG(IS_CHROMEOS_LACROS)
+            ,
+        profile()->IsMainProfile()
+#endif  // BUILDFLAG(IS_CHROMEOS_LACROS)
+    );
     DefaultSearchManager::Source dse_source =
         static_cast<DefaultSearchManager::Source>(-1);
     default_search_manager.GetDefaultSearchEngine(&dse_source);
@@ -740,26 +757,25 @@ class PrefHashBrowserTestChangedAtomic : public PrefHashBrowserTestBase {
     profile()->GetPrefs()->SetInteger(prefs::kRestoreOnStartup,
                                       SessionStartupPref::URLS);
 
-    ListPrefUpdate update(profile()->GetPrefs(),
-                          prefs::kURLsToRestoreOnStartup);
-    update->AppendString("http://example.com");
+    ScopedListPrefUpdate update(profile()->GetPrefs(),
+                                prefs::kURLsToRestoreOnStartup);
+    update->Append("http://example.com");
   }
 
   void AttackPreferencesOnDisk(
-      base::DictionaryValue* unprotected_preferences,
-      base::DictionaryValue* protected_preferences) override {
-    base::DictionaryValue* selected_prefs =
+      base::Value::Dict* unprotected_preferences,
+      base::Value::Dict* protected_preferences) override {
+    base::Value::Dict* selected_prefs =
         protection_level_ >= PROTECTION_ENABLED_BASIC ? protected_preferences
                                                       : unprotected_preferences;
-    // |selected_prefs| should never be NULL under the protection level picking
+    // `selected_prefs` should never be NULL under the protection level picking
     // it.
-    EXPECT_TRUE(selected_prefs);
-    base::ListValue* startup_urls;
-    EXPECT_TRUE(
-        selected_prefs->GetList(prefs::kURLsToRestoreOnStartup, &startup_urls));
-    EXPECT_TRUE(startup_urls);
-    EXPECT_EQ(1U, startup_urls->GetSize());
-    startup_urls->AppendString("http://example.org");
+    ASSERT_TRUE(selected_prefs);
+    base::Value::List* startup_urls =
+        selected_prefs->FindListByDottedPath(prefs::kURLsToRestoreOnStartup);
+    ASSERT_TRUE(startup_urls);
+    EXPECT_EQ(1U, startup_urls->size());
+    startup_urls->Append("http://example.org");
   }
 
   void VerifyReactionToPrefAttack() override {
@@ -791,11 +807,9 @@ class PrefHashBrowserTestChangedAtomic : public PrefHashBrowserTestBase {
 // Preferences.
 #if !BUILDFLAG(IS_CHROMEOS_ASH)
     // Explicitly verify the result of reported resets.
-    EXPECT_EQ(protection_level_ >= PROTECTION_ENABLED_BASIC ? 0U : 2U,
-              profile()
-                  ->GetPrefs()
-                  ->GetList(prefs::kURLsToRestoreOnStartup)
-                  ->GetSize());
+    EXPECT_EQ(
+        protection_level_ >= PROTECTION_ENABLED_BASIC ? 0U : 2U,
+        profile()->GetPrefs()->GetList(prefs::kURLsToRestoreOnStartup).size());
 #endif
 
     // Nothing else should have triggered.
@@ -840,32 +854,31 @@ class PrefHashBrowserTestChangedSplitPref : public PrefHashBrowserTestBase {
   }
 
   void AttackPreferencesOnDisk(
-      base::DictionaryValue* unprotected_preferences,
-      base::DictionaryValue* protected_preferences) override {
-    base::DictionaryValue* selected_prefs =
+      base::Value::Dict* unprotected_preferences,
+      base::Value::Dict* protected_preferences) override {
+    base::Value::Dict* selected_prefs =
         protection_level_ >= PROTECTION_ENABLED_EXTENSIONS
             ? protected_preferences
             : unprotected_preferences;
     // |selected_prefs| should never be NULL under the protection level picking
     // it.
     EXPECT_TRUE(selected_prefs);
-    base::DictionaryValue* extensions_dict;
-    EXPECT_TRUE(selected_prefs->GetDictionary(
-        extensions::pref_names::kExtensions, &extensions_dict));
+    base::Value::Dict* extensions_dict = selected_prefs->FindDictByDottedPath(
+        extensions::pref_names::kExtensions);
     EXPECT_TRUE(extensions_dict);
 
     // Tamper with any installed setting for good.crx
-    base::DictionaryValue* good_crx_dict;
-    EXPECT_TRUE(extensions_dict->GetDictionary(kGoodCrxId, &good_crx_dict));
-    int good_crx_state;
-    EXPECT_TRUE(good_crx_dict->GetInteger("state", &good_crx_state));
-    EXPECT_EQ(extensions::Extension::ENABLED, good_crx_state);
-    good_crx_dict->SetInteger("state", extensions::Extension::DISABLED);
+    base::Value::Dict* good_crx_dict = extensions_dict->FindDict(kGoodCrxId);
+    ASSERT_TRUE(good_crx_dict);
+    std::optional<int> good_crx_state = good_crx_dict->FindInt("state");
+    ASSERT_TRUE(good_crx_state);
+    EXPECT_EQ(extensions::Extension::ENABLED, *good_crx_state);
+    good_crx_dict->Set("state", extensions::Extension::DISABLED);
 
     // Drop a fake extension (for the purpose of this test, dropped settings
     // don't need to be valid extension settings).
-    auto fake_extension = std::make_unique<base::DictionaryValue>();
-    fake_extension->SetString("name", "foo");
+    base::Value::Dict fake_extension;
+    fake_extension.Set("name", "foo");
     extensions_dict->Set(std::string(32, 'a'), std::move(fake_extension));
   }
 
@@ -947,10 +960,10 @@ class PrefHashBrowserTestUntrustedAdditionToPrefs
   }
 
   void AttackPreferencesOnDisk(
-      base::DictionaryValue* unprotected_preferences,
-      base::DictionaryValue* protected_preferences) override {
-    unprotected_preferences->SetInteger(prefs::kRestoreOnStartup,
-                                        SessionStartupPref::LAST);
+      base::Value::Dict* unprotected_preferences,
+      base::Value::Dict* protected_preferences) override {
+    unprotected_preferences->SetByDottedPath(
+        prefs::kRestoreOnStartup, static_cast<int>(SessionStartupPref::LAST));
   }
 
   void VerifyReactionToPrefAttack() override {
@@ -1029,13 +1042,13 @@ class PrefHashBrowserTestUntrustedAdditionToPrefsAfterWipe
   }
 
   void AttackPreferencesOnDisk(
-      base::DictionaryValue* unprotected_preferences,
-      base::DictionaryValue* protected_preferences) override {
+      base::Value::Dict* unprotected_preferences,
+      base::Value::Dict* protected_preferences) override {
     // Set or change the value in Preferences to the attacker's choice.
-    unprotected_preferences->SetString(prefs::kHomePage, "http://example.net");
+    unprotected_preferences->Set(prefs::kHomePage, "http://example.net");
     // Clear the value in Secure Preferences, if any.
     if (protected_preferences)
-      protected_preferences->Remove(prefs::kHomePage, NULL);
+      protected_preferences->Remove(prefs::kHomePage);
   }
 
   void VerifyReactionToPrefAttack() override {
@@ -1105,7 +1118,7 @@ class PrefHashBrowserTestUntrustedAdditionToPrefsAfterWipe
 PREF_HASH_BROWSER_TEST(PrefHashBrowserTestUntrustedAdditionToPrefsAfterWipe,
                        UntrustedAdditionToPrefsAfterWipe);
 
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
 class PrefHashBrowserTestRegistryValidationFailure
     : public PrefHashBrowserTestBase {
  public:
@@ -1114,8 +1127,8 @@ class PrefHashBrowserTestRegistryValidationFailure
   }
 
   void AttackPreferencesOnDisk(
-      base::DictionaryValue* unprotected_preferences,
-      base::DictionaryValue* protected_preferences) override {
+      base::Value::Dict* unprotected_preferences,
+      base::Value::Dict* protected_preferences) override {
     std::wstring registry_key =
         GetRegistryPathForTestProfile() + L"\\PreferenceMACs\\Default";
     base::win::RegKey key;
@@ -1123,7 +1136,7 @@ class PrefHashBrowserTestRegistryValidationFailure
                                       KEY_SET_VALUE | KEY_WOW64_32KEY));
     // An incorrect hash should still have the correct size.
     ASSERT_EQ(ERROR_SUCCESS,
-              key.WriteValue(L"homepage", base::string16(64, 'A').c_str()));
+              key.WriteValue(L"homepage", std::wstring(64, 'A').c_str()));
   }
 
   void VerifyReactionToPrefAttack() override {
@@ -1157,28 +1170,36 @@ class PrefHashBrowserTestDefaultSearch : public PrefHashBrowserTestBase {
   void SetupPreferences() override {
     // Set user selected default search engine.
     DefaultSearchManager default_search_manager(
-        profile()->GetPrefs(), DefaultSearchManager::ObserverCallback());
+        profile()->GetPrefs(),
+        search_engines::SearchEngineChoiceServiceFactory::GetForProfile(
+            profile()),
+        DefaultSearchManager::ObserverCallback()
+#if BUILDFLAG(IS_CHROMEOS_LACROS)
+            ,
+        profile()->IsMainProfile()
+#endif  // BUILDFLAG(IS_CHROMEOS_LACROS)
+    );
     DefaultSearchManager::Source dse_source =
         static_cast<DefaultSearchManager::Source>(-1);
 
     TemplateURLData user_dse;
-    user_dse.SetKeyword(base::UTF8ToUTF16("userkeyword"));
-    user_dse.SetShortName(base::UTF8ToUTF16("username"));
+    user_dse.SetKeyword(u"userkeyword");
+    user_dse.SetShortName(u"username");
     user_dse.SetURL("http://user_default_engine/search?q=good_user_query");
     default_search_manager.SetUserSelectedDefaultSearchEngine(user_dse);
 
     const TemplateURLData* current_dse =
         default_search_manager.GetDefaultSearchEngine(&dse_source);
     EXPECT_EQ(DefaultSearchManager::FROM_USER, dse_source);
-    EXPECT_EQ(current_dse->keyword(), base::UTF8ToUTF16("userkeyword"));
-    EXPECT_EQ(current_dse->short_name(), base::UTF8ToUTF16("username"));
+    EXPECT_EQ(current_dse->keyword(), u"userkeyword");
+    EXPECT_EQ(current_dse->short_name(), u"username");
     EXPECT_EQ(current_dse->url(),
               "http://user_default_engine/search?q=good_user_query");
   }
 
   void AttackPreferencesOnDisk(
-      base::DictionaryValue* unprotected_preferences,
-      base::DictionaryValue* protected_preferences) override {
+      base::Value::Dict* unprotected_preferences,
+      base::Value::Dict* protected_preferences) override {
     static constexpr char default_search_provider_data[] = R"(
     {
       "default_search_provider_data" : {
@@ -1209,22 +1230,28 @@ class PrefHashBrowserTestDefaultSearch : public PrefHashBrowserTestBase {
     })";
 
     // Try to override default search in all three of available preferences.
-    auto attack1 = base::DictionaryValue::From(
-        base::JSONReader::ReadDeprecated(default_search_provider_data));
-    auto attack2 = base::DictionaryValue::From(
-        base::JSONReader::ReadDeprecated(search_provider_overrides));
-    unprotected_preferences->MergeDictionary(attack1.get());
-    unprotected_preferences->MergeDictionary(attack2.get());
+    base::Value attack1 = *base::JSONReader::Read(default_search_provider_data);
+    base::Value attack2 = *base::JSONReader::Read(search_provider_overrides);
+    unprotected_preferences->Merge(attack1.GetDict().Clone());
+    unprotected_preferences->Merge(attack2.GetDict().Clone());
     if (protected_preferences) {
       // Override here, too.
-      protected_preferences->MergeDictionary(attack1.get());
-      protected_preferences->MergeDictionary(attack2.get());
+      protected_preferences->Merge(attack1.GetDict().Clone());
+      protected_preferences->Merge(attack2.GetDict().Clone());
     }
   }
 
   void VerifyReactionToPrefAttack() override {
     DefaultSearchManager default_search_manager(
-        profile()->GetPrefs(), DefaultSearchManager::ObserverCallback());
+        profile()->GetPrefs(),
+        search_engines::SearchEngineChoiceServiceFactory::GetForProfile(
+            profile()),
+        DefaultSearchManager::ObserverCallback()
+#if BUILDFLAG(IS_CHROMEOS_LACROS)
+            ,
+        profile()->IsMainProfile()
+#endif  // BUILDFLAG(IS_CHROMEOS_LACROS)
+    );
     DefaultSearchManager::Source dse_source =
         static_cast<DefaultSearchManager::Source>(-1);
 
@@ -1236,16 +1263,16 @@ class PrefHashBrowserTestDefaultSearch : public PrefHashBrowserTestBase {
 #if !BUILDFLAG(IS_CHROMEOS_ASH)
       // Attack is successful.
       EXPECT_EQ(DefaultSearchManager::FROM_USER, dse_source);
-      EXPECT_EQ(current_dse->keyword(), base::UTF8ToUTF16("badkeyword"));
-      EXPECT_EQ(current_dse->short_name(), base::UTF8ToUTF16("badname"));
+      EXPECT_EQ(current_dse->keyword(), u"badkeyword");
+      EXPECT_EQ(current_dse->short_name(), u"badname");
       EXPECT_EQ(current_dse->url(),
                 "http://bad_default_engine/search?q=dirty_user_query");
 #endif
     } else {
       // Attack fails.
       EXPECT_EQ(DefaultSearchManager::FROM_FALLBACK, dse_source);
-      EXPECT_NE(current_dse->keyword(), base::UTF8ToUTF16("badkeyword"));
-      EXPECT_NE(current_dse->short_name(), base::UTF8ToUTF16("badname"));
+      EXPECT_NE(current_dse->keyword(), u"badkeyword");
+      EXPECT_NE(current_dse->short_name(), u"badname");
       EXPECT_NE(current_dse->url(),
                 "http://bad_default_engine/search?q=dirty_user_query");
     }
@@ -1253,3 +1280,51 @@ class PrefHashBrowserTestDefaultSearch : public PrefHashBrowserTestBase {
 };
 
 PREF_HASH_BROWSER_TEST(PrefHashBrowserTestDefaultSearch, SearchProtected);
+
+// Verifies that we handle a protected Dict preference being changed to an
+// unexpected type (int). See https://crbug.com/1512724.
+class PrefHashBrowserTestExtensionDictTypeChanged
+    : public PrefHashBrowserTestBase {
+ public:
+  void SetupPreferences() override {
+    InstallExtensionWithUIAutoConfirm(test_data_dir_.AppendASCII("good.crx"), 1,
+                                      browser());
+  }
+
+  void AttackPreferencesOnDisk(
+      base::Value::Dict* unprotected_preferences,
+      base::Value::Dict* protected_preferences) override {
+    base::Value::Dict* const selected_prefs =
+        protection_level_ >= PROTECTION_ENABLED_EXTENSIONS
+            ? protected_preferences
+            : unprotected_preferences;
+    // |selected_prefs| should never be NULL under the protection level picking
+    // it.
+    ASSERT_TRUE(selected_prefs);
+    EXPECT_TRUE(selected_prefs->FindDictByDottedPath(
+        extensions::pref_names::kExtensions));
+    // Overwrite with an int (wrong type).
+    selected_prefs->SetByDottedPath(extensions::pref_names::kExtensions, 13);
+    EXPECT_EQ(13, selected_prefs
+                      ->FindIntByDottedPath(extensions::pref_names::kExtensions)
+                      .value());
+  }
+
+  void VerifyReactionToPrefAttack() override {
+    // Setting the extensions dict to an invalid type gets noticed regardless
+    // of protection level. This implementation just happened to be easier and
+    // it doesn't seem important to not protect the kExtensions from being the
+    // wrong type at any protection level. PrefService will correct the type
+    // either way.
+    EXPECT_EQ(protection_level_ > PROTECTION_DISABLED_ON_PLATFORM ? 1 : 0,
+              GetTrackedPrefHistogramCount(
+                  user_prefs::tracked::kTrackedPrefHistogramCleared,
+                  BEGIN_ALLOW_SINGLE_BUCKET + 5));
+
+    // Expect a dictionary for extensions. This shouldn't somehow explode.
+    profile()->GetPrefs()->GetDict(extensions::pref_names::kExtensions);
+  }
+};
+
+PREF_HASH_BROWSER_TEST(PrefHashBrowserTestExtensionDictTypeChanged,
+                       ExtensionDictTypeChanged);
