@@ -1,11 +1,13 @@
-// Copyright 2021 The Chromium Authors. All rights reserved.
+// Copyright 2021 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #ifndef CHROME_BROWSER_PRINTING_PREFS_UTIL_H_
 #define CHROME_BROWSER_PRINTING_PREFS_UTIL_H_
 
-#include "third_party/abseil-cpp/absl/types/optional.h"
+#include <optional>
+
+#include "printing/buildflags/buildflags.h"
 #include "ui/gfx/geometry/size.h"
 
 class PrefService;
@@ -13,7 +15,15 @@ class PrefService;
 namespace printing {
 
 // Parse the printing.paper_size_default preference.
-absl::optional<gfx::Size> ParsePaperSizeDefault(const PrefService& prefs);
+std::optional<gfx::Size> ParsePaperSizeDefault(const PrefService& prefs);
+
+#if BUILDFLAG(ENABLE_OOP_PRINTING)
+// Determine if out-of-process printing support is enabled.
+bool IsOopPrintingEnabled();
+
+// Determine if printing a job should be done out-of-process.
+bool ShouldPrintJobOop();
+#endif
 
 }  // namespace printing
 
