@@ -6,6 +6,7 @@
 
 #include <algorithm>
 #include <memory>
+#include <optional>
 #include <utility>
 
 #include "base/cancelable_callback.h"
@@ -47,7 +48,8 @@ class PwgRasterConverterHelper
   PwgRasterConverterHelper(const PwgRasterConverterHelper&) = delete;
   PwgRasterConverterHelper& operator=(const PwgRasterConverterHelper&) = delete;
 
-  void Convert(const base::RefCountedMemory* data,
+  void Convert(const std::optional<bool>& use_skia,
+               const base::RefCountedMemory* data,
                PwgRasterConverter::ResultCallback callback);
 
  private:
@@ -77,6 +79,7 @@ PwgRasterConverterHelper::~PwgRasterConverterHelper() {
 }
 
 void PwgRasterConverterHelper::Convert(
+    const std::optional<bool>& use_skia,
     const base::RefCountedMemory* data,
     PwgRasterConverter::ResultCallback callback) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
@@ -96,6 +99,10 @@ void PwgRasterConverterHelper::Convert(
     return;
   }
 
+  if (use_skia) {
+    pdf_to_pwg_raster_converter_remote_->SetUseSkiaRendererPolicy(*use_skia);
+  }
+
   // TODO(thestig): Write `data` into shared memory in the first place, to avoid
   // this memcpy().
   memcpy(memory.mapping.memory(), data->front(), data->size());
@@ -132,7 +139,8 @@ class PwgRasterConverterImpl : public PwgRasterConverter {
 
   ~PwgRasterConverterImpl() override;
 
-  void Start(const base::RefCountedMemory* data,
+  void Start(const std::optional<bool>& use_skia,
+             const base::RefCountedMemory* data,
              const PdfRenderSettings& conversion_settings,
              const PwgRasterSettings& bitmap_settings,
              ResultCallback callback) override;
@@ -149,14 +157,15 @@ PwgRasterConverterImpl::PwgRasterConverterImpl() = default;
 
 PwgRasterConverterImpl::~PwgRasterConverterImpl() = default;
 
-void PwgRasterConverterImpl::Start(const base::RefCountedMemory* data,
+void PwgRasterConverterImpl::Start(const std::optional<bool>& use_skia,
+                                   const base::RefCountedMemory* data,
                                    const PdfRenderSettings& conversion_settings,
                                    const PwgRasterSettings& bitmap_settings,
                                    ResultCallback callback) {
   cancelable_callback_.Reset(std::move(callback));
   utility_client_ = base::MakeRefCounted<PwgRasterConverterHelper>(
       conversion_settings, bitmap_settings);
-  utility_client_->Convert(data, cancelable_callback_.callback());
+  utility_client_->Convert(use_skia, data, cancelable_callback_.callback());
 }
 
 }  // namespace
