@@ -1,26 +1,26 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-import {assert} from 'chrome://resources/js/assert_ts.js';
-import {EventTracker} from 'chrome://resources/js/event_tracker.m.js';
-import {loadTimeData} from 'chrome://resources/js/load_time_data.m.js';
+import {assert} from 'chrome://resources/js/assert.js';
+import {EventTracker} from 'chrome://resources/js/event_tracker.js';
+import {loadTimeData} from 'chrome://resources/js/load_time_data.js';
 
-import {MetricsContext, PrintPreviewInitializationEvents} from '../metrics.js';
 import {CapabilitiesResponse, NativeLayer, NativeLayerImpl} from '../native_layer.js';
-// <if expr="chromeos_ash or chromeos_lacros">
+// <if expr="is_chromeos">
 import {NativeLayerCros, NativeLayerCrosImpl, PrinterSetupResponse} from '../native_layer_cros.js';
 
 // </if>
 import {Cdd, MediaSizeOption} from './cdd.js';
 import {createDestinationKey, createRecentDestinationKey, Destination, DestinationOrigin, GooglePromotedDestinationId, isPdfPrinter, PDF_DESTINATION_KEY, PrinterType, RecentDestination} from './destination.js';
-// <if expr="chromeos_ash or chromeos_lacros">
+// <if expr="is_chromeos">
 import {DestinationProvisionalType} from './destination.js';
 // </if>
 import {DestinationMatch} from './destination_match.js';
 import {ExtensionDestinationInfo, LocalDestinationInfo, parseDestination} from './local_parsers.js';
-// <if expr="chromeos_ash or chromeos_lacros">
+// <if expr="is_chromeos">
 import {parseExtensionDestination} from './local_parsers.js';
+import {getStatusReasonFromPrinterStatus, PrinterStatusReason} from './printer_status_cros.js';
 // </if>
 
 /**
@@ -153,8 +153,10 @@ export enum DestinationStoreEventType {
   ERROR = 'DestinationStore.ERROR',
   SELECTED_DESTINATION_CAPABILITIES_READY = 'DestinationStore' +
       '.SELECTED_DESTINATION_CAPABILITIES_READY',
-  // <if expr="chromeos_ash or chromeos_lacros">
+  // <if expr="is_chromeos">
   DESTINATION_EULA_READY = 'DestinationStore.DESTINATION_EULA_READY',
+  DESTINATION_PRINTER_STATUS_UPDATE =
+      'DestinationStore.DESTINATION_PRINTER_STATUS_UPDATE',
   // </if>
 }
 
@@ -184,17 +186,12 @@ export class DestinationStore extends EventTarget {
 
   private initialDestinationSelected_: boolean = false;
 
-  /**
-   * Used to track metrics.
-   */
-  private metrics_: MetricsContext = MetricsContext.destinationSearch();
-
   /**
    * Used to fetch local print destinations.
    */
   private nativeLayer_: NativeLayer = NativeLayerImpl.getInstance();
 
-  // <if expr="chromeos_ash or chromeos_lacros">
+  // <if expr="is_chromeos">
   /**
    * Used to fetch information about Chrome OS local print destinations.
    */
@@ -240,16 +237,13 @@ export class DestinationStore extends EventTarget {
    */
   constructor(
       addListenerCallback:
-          (eventName: string,
-           listener:
-               (t: PrinterType,
-                p: LocalDestinationInfo[]|
-                ExtensionDestinationInfo[]) => void) => void) {
+          (eventName: string, listener: (p1: any, p2?: any) => void) => void) {
     super();
 
     this.destinationSearchStatus_ = new Map([
       [
-        PrinterType.EXTENSION_PRINTER, DestinationStorePrinterSearchStatus.START
+        PrinterType.EXTENSION_PRINTER,
+        DestinationStorePrinterSearchStatus.START,
       ],
       [PrinterType.LOCAL_PRINTER, DestinationStorePrinterSearchStatus.START],
     ]);
@@ -262,6 +256,15 @@ export class DestinationStore extends EventTarget {
         (type: PrinterType,
          printers: LocalDestinationInfo[]|ExtensionDestinationInfo[]) =>
             this.onPrintersAdded_(type, printers));
+
+    // <if expr="is_chromeos">
+    if (loadTimeData.getBoolean('isLocalPrinterObservingEnabled')) {
+      addListenerCallback(
+          'local-printers-updated',
+          (printers: LocalDestinationInfo[]) =>
+              this.onLocalPrintersUpdated_(printers));
+    }
+    // </if>
   }
 
   /**
@@ -286,11 +289,6 @@ export class DestinationStore extends EventTarget {
     return this.selectedDestination_;
   }
 
-  private isDestinationValid_(destination: (Destination|RecentDestination|
-                                            null)): boolean {
-    return !!destination && !!destination.id && !!destination.origin;
-  }
-
   private getPrinterTypeForRecentDestination_(destination: RecentDestination):
       PrinterType {
     if (isPdfPrinter(destination.id)) {
@@ -312,8 +310,8 @@ export class DestinationStore extends EventTarget {
    * will be automatically selected.
    * @param pdfPrinterDisabled Whether the PDF print destination is
    *     disabled in print preview.
-   * @param isDriveMounted Whether Google Drive is mounted. Only used
-        on Chrome OS.
+   * @param saveToDriveDisabled Whether the 'Save to Google Drive' destination
+   *     is disabled in print preview. Only used on Chrome OS.
    * @param systemDefaultDestinationId ID of the system default
    *     destination.
    * @param serializedDefaultDestinationSelectionRulesStr Serialized
@@ -322,11 +320,11 @@ export class DestinationStore extends EventTarget {
    */
   init(
       pdfPrinterDisabled: boolean,
-      // <if expr="chromeos_ash or chromeos_lacros">
-      isDriveMounted: boolean,
+      // <if expr="is_chromeos">
+      saveToDriveDisabled: boolean,
       // </if>
-      // <if expr="not chromeos_ash and not chromeos_lacros">
-      _isDriveMounted: boolean,
+      // <if expr="not is_chromeos">
+      _saveToDriveDisabled: boolean,
       // </if>
       systemDefaultDestinationId: string,
       serializedDefaultDestinationSelectionRulesStr: string|null,
@@ -336,10 +334,10 @@ export class DestinationStore extends EventTarget {
       const systemDefaultType = systemDefaultVirtual ?
           PrinterType.PDF_PRINTER :
           PrinterType.LOCAL_PRINTER;
-      // <if expr="not chromeos_ash and not chromeos_lacros">
+      // <if expr="not is_chromeos">
       const systemDefaultOrigin = DestinationOrigin.LOCAL;
       // </if>
-      // <if expr="chromeos_ash or chromeos_lacros">
+      // <if expr="is_chromeos">
       const systemDefaultOrigin = systemDefaultVirtual ?
           DestinationOrigin.LOCAL :
           DestinationOrigin.CROS;
@@ -364,8 +362,8 @@ export class DestinationStore extends EventTarget {
 
     this.pdfPrinterEnabled_ = !pdfPrinterDisabled;
     this.createLocalPdfPrintDestination_();
-    // <if expr="chromeos_ash or chromeos_lacros">
-    if (isDriveMounted) {
+    // <if expr="is_chromeos">
+    if (!saveToDriveDisabled) {
       this.createLocalDrivePrintDestination_();
     }
     // </if>
@@ -374,6 +372,11 @@ export class DestinationStore extends EventTarget {
     // destinationsInserted_ may never be called.
     if (this.typesToSearch_.size === 0) {
       this.tryToSelectInitialDestination_();
+      // <if expr="is_chromeos">
+      // Start observing local printers if there is no attempt to load
+      // destinations.
+      this.observeLocalPrinters_();
+      // </if>
       return;
     }
 
@@ -502,7 +505,7 @@ export class DestinationStore extends EventTarget {
     this.tracker_.removeAll();
   }
 
-  // <if expr="chromeos_ash or chromeos_lacros">
+  // <if expr="is_chromeos">
   /**
    * Attempts to find the EULA URL of the the destination ID.
    */
@@ -572,18 +575,44 @@ export class DestinationStore extends EventTarget {
     return new DestinationMatch(idRegExp, displayNameRegExp);
   }
 
-  /** @param Key identifying the destination to select */
+  /**
+   * This function is only invoked when the user selects a new destination via
+   * the UI. Programmatic selection of a destination should not use this
+   * function.
+   * @param Key identifying the destination to select
+   */
   selectDestinationByKey(key: string) {
-    assert(this.tryToSelectDestinationByKey_(key));
+    const success = this.tryToSelectDestinationByKey_(key);
+    assert(success);
+    // <if expr="is_chromeos">
+    if (success && this.selectedDestination_ &&
+        this.selectedDestination_.type !== PrinterType.PDF_PRINTER) {
+      this.selectedDestination_.printerManuallySelected = true;
+    }
+    // </if>
   }
 
   /**
-   * @param Destination to select.
+   * @param destination Destination to select.
+   * @param refreshDestination Set to true to allow the currently selected
+   *          destination to be re-selected.
    */
-  selectDestination(destination: Destination) {
+  selectDestination(
+      destination: Destination, refreshDestination: boolean = false) {
+    // <if expr="not is_chromeos">
+    assert(!refreshDestination, 'refreshDestination for CrOS only');
     if (destination === this.selectedDestination_) {
       return;
     }
+    // </if>
+    // <if expr="is_chromeos">
+    // Do not re-select the same destination unless explicitly requesting it to
+    // refetch the capabilities and reload the preview.
+    if (destination === this.selectedDestination_ && !refreshDestination) {
+      return;
+    }
+    // </if>
+
     if (destination === null) {
       this.selectedDestination_ = null;
       this.dispatchEvent(
@@ -591,7 +620,7 @@ export class DestinationStore extends EventTarget {
       return;
     }
 
-    // <if expr="chromeos_ash or chromeos_lacros">
+    // <if expr="is_chromeos">
     assert(
         !destination.isProvisional, 'Unable to select provisonal destinations');
     // </if>
@@ -610,14 +639,12 @@ export class DestinationStore extends EventTarget {
                   destination.origin, destination.id, caps),
               () => this.onGetCapabilitiesFail_(
                   destination.origin, destination.id));
-      MetricsContext.getPrinterCapabilities().record(
-          PrintPreviewInitializationEvents.FUNCTION_INITIATED);
     } else {
       this.sendSelectedDestinationUpdateEvent_();
     }
   }
 
-  // <if expr="chromeos_ash or chromeos_lacros">
+  // <if expr="is_chromeos">
   /**
    * Attempt to resolve the capabilities for a Chrome OS printer.
    */
@@ -701,8 +728,6 @@ export class DestinationStore extends EventTarget {
         type, DestinationStorePrinterSearchStatus.SEARCHING);
     this.nativeLayer_.getPrinters(type).then(
         () => this.onDestinationSearchDone_(type));
-    MetricsContext.getPrinters(type).record(
-        PrintPreviewInitializationEvents.FUNCTION_INITIATED);
   }
 
   /** Initiates loading of all known destination types. */
@@ -725,7 +750,7 @@ export class DestinationStore extends EventTarget {
     return this.destinationMap_.get(key);
   }
 
-  // <if expr="chromeos_ash or chromeos_lacros">
+  // <if expr="is_chromeos">
   /**
    * Removes the provisional destination with ID |provisionalId| from
    * |destinationMap_| and |destinations_|.
@@ -755,7 +780,7 @@ export class DestinationStore extends EventTarget {
    * Inserts multiple {@code destinations} to the data store and dispatches
    * single DESTINATIONS_INSERTED event.
    */
-  private insertDestinations_(destinations: (Destination|null)[]) {
+  private insertDestinations_(destinations: Array<Destination|null>) {
     let inserted = false;
     destinations.forEach(destination => {
       if (destination) {
@@ -842,7 +867,7 @@ export class DestinationStore extends EventTarget {
     }
   }
 
-  // <if expr="chromeos_ash or chromeos_lacros">
+  // <if expr="is_chromeos">
   /**
    * Creates a local Drive print destination.
    */
@@ -858,8 +883,6 @@ export class DestinationStore extends EventTarget {
    * @param type The type of printers that are done being retrieved.
    */
   private onDestinationSearchDone_(type: PrinterType) {
-    MetricsContext.getPrinters(type).record(
-        PrintPreviewInitializationEvents.FUNCTION_SUCCESSFUL);
     this.destinationSearchStatus_.set(
         type, DestinationStorePrinterSearchStatus.DONE);
     this.dispatchEvent(
@@ -870,6 +893,10 @@ export class DestinationStore extends EventTarget {
     } else if (this.typesToSearch_.size === 0) {
       this.tryToSelectInitialDestination_();
     }
+
+    // <if expr="is_chromeos">
+    this.observeLocalPrinters_();
+    // </if>
   }
 
   /**
@@ -886,8 +913,6 @@ export class DestinationStore extends EventTarget {
   private onCapabilitiesSet_(
       origin: DestinationOrigin, id: string,
       settingsInfo: CapabilitiesResponse) {
-    MetricsContext.getPrinterCapabilities().record(
-        PrintPreviewInitializationEvents.FUNCTION_SUCCESSFUL);
     let dest = null;
     const key = createDestinationKey(id, origin);
     dest = this.destinationMap_.get(key);
@@ -913,7 +938,7 @@ export class DestinationStore extends EventTarget {
       }
       dest.capabilities = settingsInfo.capabilities;
       this.updateDestination_(dest);
-      // <if expr="chromeos_ash or chromeos_lacros">
+      // <if expr="is_chromeos">
       // Start the fetch for the PPD EULA URL.
       this.fetchEulaUrl(dest.id);
       // </if>
@@ -929,8 +954,6 @@ export class DestinationStore extends EventTarget {
    */
   private onGetCapabilitiesFail_(
       _origin: DestinationOrigin, destinationId: string) {
-    MetricsContext.getPrinterCapabilities().record(
-        PrintPreviewInitializationEvents.FUNCTION_FAILED);
     console.warn(
         'Failed to get print capabilities for printer ' + destinationId);
     if (this.selectedDestination_ &&
@@ -954,6 +977,74 @@ export class DestinationStore extends EventTarget {
         (printer: LocalDestinationInfo|ExtensionDestinationInfo) =>
             parseDestination(type, printer)));
   }
+
+  // <if expr="is_chromeos">
+  private observeLocalPrinters_() {
+    if (!loadTimeData.getBoolean('isLocalPrinterObservingEnabled')) {
+      return;
+    }
+
+    this.nativeLayerCros_.observeLocalPrinters().then(
+        (printers: LocalDestinationInfo[]) =>
+            this.onLocalPrintersUpdated_(printers));
+  }
+
+  /**
+   * Inserts any new printers retrieved from the 'local-printers-updated' event.
+   * @param printerType The type of printer(s) added.
+   * @param printers Information about the printers that have been retrieved.
+   */
+  private onLocalPrintersUpdated_(printers: LocalDestinationInfo[]) {
+    if (!printers) {
+      return;
+    }
+
+    // The logic in insertDestinations_() ensures only new destinations are
+    // added to the store.
+    this.insertDestinations_(printers.map(
+        printer => parseDestination(PrinterType.LOCAL_PRINTER, printer)));
+
+    // Parse the printer status from the LocalDestinationInfo object.
+    for (const printer of printers) {
+      this.parsePrinterStatus(printer);
+    }
+  }
+
+  // Updates the printer status for an existing destination then fires an event
+  // for updating printer status icons and text.
+  private parsePrinterStatus(destinationInfo: LocalDestinationInfo): void {
+    const printerStatus = destinationInfo.printerStatus;
+    if (!printerStatus || !printerStatus.printerId) {
+      return;
+    }
+
+    const destinationKey = createDestinationKey(
+        destinationInfo.deviceName, DestinationOrigin.CROS);
+    const existingDestination = this.destinationMap_.get(destinationKey);
+    if (existingDestination === undefined) {
+      return;
+    }
+
+    // `nowOnline` captures the event where a previously offline printer
+    // becomes reachable. This will be used to trigger the destination to
+    // reload its preview.
+    const previousStatusReason = existingDestination.printerStatusReason;
+    const nextStatusReason = getStatusReasonFromPrinterStatus(printerStatus);
+    const nowOnline =
+        previousStatusReason === PrinterStatusReason.PRINTER_UNREACHABLE &&
+        (nextStatusReason !== PrinterStatusReason.PRINTER_UNREACHABLE &&
+         nextStatusReason !== PrinterStatusReason.UNKNOWN_REASON);
+
+    existingDestination.printerStatusReason = nextStatusReason;
+    this.dispatchEvent(new CustomEvent(
+        DestinationStoreEventType.DESTINATION_PRINTER_STATUS_UPDATE, {
+          detail: {
+            destinationKey: destinationKey,
+            nowOnline: nowOnline,
+          },
+        }));
+  }
+  // </if>
 }
 
 /**
@@ -1078,8 +1169,8 @@ const MEDIA_DISPLAY_NAMES_: {[key: string]: string} = {
   'NA_FANFOLD_EUR': 'FanFold European',
   'NA_FANFOLD_US': 'FanFold US',
   'NA_FOOLSCAP': 'FanFold German Legal',
-  'NA_GOVT_LEGAL': 'Government Legal',
-  'NA_GOVT_LETTER': 'Government Letter',
+  'NA_GOVT_LEGAL': '8x13',
+  'NA_GOVT_LETTER': '8x10',
   'NA_INDEX_3X5': 'Index 3x5',
   'NA_INDEX_4X6': 'Index 4x6',
   'NA_INDEX_4X6_EXT': 'Index 4x6 ext',
