@@ -1,18 +1,18 @@
-// Copyright 2020 The Chromium Authors. All rights reserved.
+// Copyright 2020 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-import 'chrome://resources/cr_elements/cr_button/cr_button.m.js';
-import 'chrome://resources/cr_elements/icons.m.js';
-import 'chrome://resources/cr_elements/hidden_style_css.m.js';
-import 'chrome://resources/cr_elements/shared_style_css.m.js';
+import 'chrome://resources/cr_elements/cr_button/cr_button.js';
+import 'chrome://resources/cr_elements/icons.html.js';
+import 'chrome://resources/cr_elements/cr_hidden_style.css.js';
+import 'chrome://resources/cr_elements/cr_shared_style.css.js';
 import './profile_card_menu.js';
-import './profile_picker_shared_css.js';
-import 'chrome://resources/cr_elements/cr_input/cr_input.m.js';
+import './profile_picker_shared.css.js';
+import 'chrome://resources/cr_elements/cr_input/cr_input.js';
 import 'chrome://resources/polymer/v3_0/paper-tooltip/paper-tooltip.js';
 
-import {CrInputElement} from 'chrome://resources/cr_elements/cr_input/cr_input.m.js';
-import {I18nMixin} from 'chrome://resources/js/i18n_mixin.js';
+import {CrInputElement} from 'chrome://resources/cr_elements/cr_input/cr_input.js';
+import {I18nMixin} from 'chrome://resources/cr_elements/i18n_mixin.js';
 import {PaperTooltipElement} from 'chrome://resources/polymer/v3_0/paper-tooltip/paper-tooltip.js';
 import {PolymerElement} from 'chrome://resources/polymer/v3_0/polymer/polymer_bundled.min.js';
 
@@ -110,6 +110,16 @@ export class ProfileCardElement extends ProfileCardElementBase {
         this.profileState.profilePath);
   }
 
+  private onNameInputPointerEnter_() {
+    this.dispatchEvent(new CustomEvent(
+        'toggle-drag', {composed: true, detail: {toggle: false}}));
+  }
+
+  private onNameInputPointerLeave_() {
+    this.dispatchEvent(new CustomEvent(
+        'toggle-drag', {composed: true, detail: {toggle: true}}));
+  }
+
   /**
    * Handler for when the profile name field is changed, then blurred.
    */
