@@ -4,9 +4,11 @@
 
 #include "chrome/browser/ui/webui/access_code_cast/access_code_cast_handler.h"
 
+#include "base/memory/raw_ptr.h"
 #include "base/ranges/algorithm.h"
 #include "base/test/bind.h"
 #include "base/test/gmock_callback_support.h"
+#include "base/test/metrics/histogram_tester.h"
 #include "base/test/mock_callback.h"
 #include "base/test/test_mock_time_task_runner.h"
 #include "base/test/test_simple_task_runner.h"
@@ -75,6 +77,8 @@ class MockPage : public access_code_cast::mojom::Page {
 };
 
 const char kEmail[] = "mock_email@gmail.com";
+constexpr char histogram[] =
+    "AccessCodeCast.Session.NewDeviceRouteCreationDuration";
 
 }  // namespace
 
@@ -216,7 +220,7 @@ class AccessCodeCastHandlerTest : public ChromeRenderViewHostTestHarness {
 
     EXPECT_CALL(*router(),
                 CreateRouteInternal(source.id(), cast_sink_1().sink().id(), _,
-                                    web_contents(), _, timeout, false));
+                                    web_contents(), _, timeout));
 
     handler()->CastToSink(mock_callback.Get());
   }
@@ -237,7 +241,7 @@ class AccessCodeCastHandlerTest : public ChromeRenderViewHostTestHarness {
     EXPECT_CALL(*router(),
                 CreateRouteInternal(source.id(), cast_sink_1().sink().id(),
                                     request.frame_origin, web_contents(), _,
-                                    base::Seconds(20), false));
+                                    base::Seconds(20)));
     handler()->CastToSink(mock_callback.Get());
   }
 
@@ -296,13 +300,13 @@ class AccessCodeCastHandlerTest : public ChromeRenderViewHostTestHarness {
     // Handler so MockMediaRouter will respond to requests to create a route.
     // Will construct a RouteRequestResult based on the set result code and
     // then call the handler's callback, which should call the page's callback.
-    ON_CALL(*router(), CreateRouteInternal(_, _, _, _, _, _, _))
+    ON_CALL(*router(), CreateRouteInternal(_, _, _, _, _, _))
         .WillByDefault([this](const MediaSource::Id& source_id,
                               const MediaSink::Id& sink_id,
                               const url::Origin& origin,
                               content::WebContents* web_contents,
                               MediaRouteResponseCallback& callback,
-                              base::TimeDelta timeout, bool incognito) {
+                              base::TimeDelta timeout) {
           std::unique_ptr<RouteRequestResult> result;
           if (result_code_ == mojom::RouteRequestResultCode::OK) {
             MediaSource source(source_id);
@@ -320,7 +324,7 @@ class AccessCodeCastHandlerTest : public ChromeRenderViewHostTestHarness {
 
   scoped_refptr<base::TestMockTimeTaskRunner> mock_time_task_runner_;
 
-  raw_ptr<MockMediaRouter> router_;
+  raw_ptr<MockMediaRouter, AcrossTasksDanglingUntriaged> router_;
   std::unique_ptr<LoggerImpl> logger_;
   signin::IdentityTestEnvironment identity_test_env_;
 
@@ -345,11 +349,12 @@ class AccessCodeCastHandlerTest : public ChromeRenderViewHostTestHarness {
   NiceMock<cast_channel::MockCastMessageHandler> message_handler_;
   std::unique_ptr<StrictMock<MockPage>> page_;
   std::unique_ptr<TestingProfileManager> profile_manager_;
-  raw_ptr<Profile> profile_;
+  raw_ptr<Profile, AcrossTasksDanglingUntriaged> profile_;
   std::unique_ptr<MockCastMediaSinkServiceImpl>
       mock_cast_media_sink_service_impl_;
   std::unique_ptr<MockWebContentsPresentationManager> presentation_manager_;
-  std::vector<MediaSinksObserver*> media_sinks_observers_;
+  std::vector<raw_ptr<MediaSinksObserver, VectorExperimental>>
+      media_sinks_observers_;
   mojom::RouteRequestResultCode result_code_ =
       mojom::RouteRequestResultCode::OK;
   MediaSinkInternal cast_sink_1_;
@@ -366,7 +371,7 @@ TEST_F(AccessCodeCastHandlerTest, OnSinkAddedResult) {
   EXPECT_CALL(mock_callback_failure,
               Run(AddSinkResultCode::CHANNEL_OPEN_ERROR));
   handler()->OnSinkAddedResult(AddSinkResultCode::CHANNEL_OPEN_ERROR,
-                               absl::nullopt);
+                               std::nullopt);
   EXPECT_FALSE(handler()->sink_id_.has_value());
 
   MockAddSinkCallback mock_callback_ok;
@@ -576,7 +581,7 @@ TEST_F(AccessCodeCastHandlerTest, ProfileSyncSuccess) {
           [](const std::string& access_code,
              AccessCodeCastSinkService::AddSinkResultCallback callback) {
             std::move(callback).Run(AddSinkResultCode::UNKNOWN_ERROR,
-                                    absl::nullopt);
+                                    std::nullopt);
           });
   EXPECT_CALL(*access_service(), DiscoverSink(_, _)).Times(1);
   handler()->AddSink(
@@ -585,4 +590,18 @@ TEST_F(AccessCodeCastHandlerTest, ProfileSyncSuccess) {
       mock_callback_success.Get());
 }
 
+// Demonstrates that adding a sink and successfully casting to it will trigger a
+// histogram.
+TEST_F(AccessCodeCastHandlerTest, SuccessfulAddAndCastMetric) {
+  base::HistogramTester histogram_tester;
+  histogram_tester.ExpectTotalCount(histogram, 0);
+
+  set_expected_cast_result(mojom::RouteRequestResultCode::OK);
+  MockCastToSinkCallback mock_cast_sink_callback;
+  EXPECT_CALL(mock_cast_sink_callback, Run(RouteRequestResultCode::OK));
+  StartDesktopMirroring(MediaSource::ForUnchosenDesktop(),
+                        mock_cast_sink_callback);
+  histogram_tester.ExpectTotalCount(histogram, 1);
+}
+
 }  // namespace media_router
