@@ -12,6 +12,7 @@
 
 #include "base/functional/bind.h"
 #include "base/functional/callback_helpers.h"
+#include "base/memory/raw_ptr.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_piece.h"
 #include "base/values.h"
@@ -102,7 +103,7 @@ class DeviceEmulatorMessageHandler::BluetoothObserver
   void DeviceRemoved(const dbus::ObjectPath& object_path) override;
 
  private:
-  DeviceEmulatorMessageHandler* owner_;
+  raw_ptr<DeviceEmulatorMessageHandler> owner_;
 };
 
 void DeviceEmulatorMessageHandler::BluetoothObserver::DeviceAdded(
@@ -152,7 +153,7 @@ class DeviceEmulatorMessageHandler::CrasAudioObserver
   }
 
  private:
-  DeviceEmulatorMessageHandler* owner_;
+  raw_ptr<DeviceEmulatorMessageHandler> owner_;
 };
 
 class DeviceEmulatorMessageHandler::PowerObserver
@@ -172,7 +173,7 @@ class DeviceEmulatorMessageHandler::PowerObserver
   void PowerChanged(const power_manager::PowerSupplyProperties& proto) override;
 
  private:
-  DeviceEmulatorMessageHandler* owner_;
+  raw_ptr<DeviceEmulatorMessageHandler> owner_;
 };
 
 void DeviceEmulatorMessageHandler::PowerObserver::PowerChanged(
@@ -425,7 +426,7 @@ void DeviceEmulatorMessageHandler::UpdatePowerSources(
     source->set_active_by_default(!dual_role);
     if (dual_role)
       props.set_supports_dual_role_devices(true);
-    absl::optional<int> port = val.GetDict().FindInt("port");
+    std::optional<int> port = val.GetDict().FindInt("port");
     CHECK(port.has_value());
     source->set_port(
         static_cast<power_manager::PowerSupplyProperties_PowerSource_Port>(
@@ -570,7 +571,7 @@ std::string DeviceEmulatorMessageHandler::CreateBluetoothDeviceFromListValue(
   CHECK(GetString(device_dict, "pairingAuthToken", &props.pairing_auth_token));
   CHECK(GetString(device_dict, "pairingAction", &props.pairing_action));
 
-  absl::optional<int> class_value = device_dict.FindInt("classValue");
+  std::optional<int> class_value = device_dict.FindInt("classValue");
   CHECK(class_value);
   props.device_class = *class_value;
 
