@@ -1,4 +1,4 @@
-// Copyright (c) 2019 The Chromium Authors. All rights reserved.
+// Copyright 2019 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,9 +6,9 @@
 
 #include <memory>
 
-#include "base/bind.h"
+#include "base/functional/bind.h"
 #include "base/metrics/histogram_macros.h"
-#include "base/threading/thread_task_runner_handle.h"
+#include "base/task/single_thread_task_runner.h"
 #include "base/time/time.h"
 #include "base/values.h"
 #include "chrome/browser/browser_switcher/alternative_browser_driver.h"
@@ -21,9 +21,9 @@
 #include "chrome/browser/ui/tabs/tab_strip_model.h"
 #include "chrome/browser/ui/webui/webui_util.h"
 #include "chrome/common/webui_url_constants.h"
+#include "chrome/grit/branded_strings.h"
 #include "chrome/grit/browser_switch_resources.h"
 #include "chrome/grit/browser_switch_resources_map.h"
-#include "chrome/grit/chromium_strings.h"
 #include "chrome/grit/generated_resources.h"
 #include "components/grit/components_resources.h"
 #include "content/public/browser/page_navigator.h"
@@ -66,20 +66,20 @@ bool IsLastTab(const Profile* profile) {
 //   "sitelist": ["example.com", ...],
 //   "greylist": ["example.net", ...]
 // }
-base::Value RuleSetToDict(const browser_switcher::RuleSet& ruleset) {
-  base::Value sitelist(base::Value::Type::LIST);
-  for (const auto& rule : ruleset.sitelist)
+base::Value::Dict RuleSetToDict(const browser_switcher::RuleSet& ruleset) {
+  base::Value::List sitelist;
+  for (const auto& rule : ruleset.sitelist) {
     sitelist.Append(rule->ToString());
+  }
 
-  base::Value greylist(base::Value::Type::LIST);
-  for (const auto& rule : ruleset.greylist)
+  base::Value::List greylist;
+  for (const auto& rule : ruleset.greylist) {
     greylist.Append(rule->ToString());
+  }
 
-  base::Value dict(base::Value::Type::DICTIONARY);
-  dict.SetKey("sitelist", std::move(sitelist));
-  dict.SetKey("greylist", std::move(greylist));
-
-  return dict;
+  return base::Value::Dict()
+      .Set("sitelist", std::move(sitelist))
+      .Set("greylist", std::move(greylist));
 }
 
 browser_switcher::BrowserSwitcherService* GetBrowserSwitcherService(
@@ -88,12 +88,9 @@ browser_switcher::BrowserSwitcherService* GetBrowserSwitcherService(
       web_ui->GetWebContents()->GetBrowserContext());
 }
 
-content::WebUIDataSource* CreateBrowserSwitchUIHTMLSource(
-    content::WebUI* web_ui) {
-  content::WebUIDataSource* source =
-      content::WebUIDataSource::Create(chrome::kChromeUIBrowserSwitchHost);
-
-  source->DisableTrustedTypesCSP();
+void CreateAndAddBrowserSwitchUIHTMLSource(content::WebUI* web_ui) {
+  content::WebUIDataSource* source = content::WebUIDataSource::CreateAndAdd(
+      Profile::FromWebUI(web_ui), chrome::kChromeUIBrowserSwitchHost);
 
   auto* service = GetBrowserSwitcherService(web_ui);
   source->AddInteger("launchDelay", service->prefs().GetDelay());
@@ -140,6 +137,14 @@ content::WebUIDataSource* CreateBrowserSwitchUIHTMLSource(
       {"urlCheckerTitle", IDS_ABOUT_BROWSER_SWITCH_INTERNALS_URL_CHECKER_TITLE},
       {"urlCheckerDesc", IDS_ABOUT_BROWSER_SWITCH_INTERNALS_URL_CHECKER_DESC},
       {"openBrowser", IDS_ABOUT_BROWSER_SWITCH_INTERNALS_OPEN_BROWSER},
+      {"openBrowserProtocolReason",
+       IDS_ABOUT_BROWSER_SWITCH_INTERNALS_OPEN_BROWSER_PROTOCOL_REASON},
+      {"openBrowserDefaultReason",
+       IDS_ABOUT_BROWSER_SWITCH_INTERNALS_OPEN_BROWSER_DEFAULT_REASON},
+      {"openBrowserRuleReason",
+       IDS_ABOUT_BROWSER_SWITCH_INTERNALS_OPEN_BROWSER_RULE_REASON},
+      {"openBrowserInvertRuleReason",
+       IDS_ABOUT_BROWSER_SWITCH_INTERNALS_OPEN_BROWSER_INVERT_RULE_REASON},
       {"invalidURL", IDS_ABOUT_BROWSER_SWITCH_INTERNALS_INVALID_URL},
       {"xmlTitle", IDS_ABOUT_BROWSER_SWITCH_INTERNALS_XML_TITLE},
       {"xmlDesc", IDS_ABOUT_BROWSER_SWITCH_INTERNALS_XML_DESC},
@@ -197,8 +202,6 @@ content::WebUIDataSource* CreateBrowserSwitchUIHTMLSource(
       "internals", IDR_BROWSER_SWITCH_INTERNALS_BROWSER_SWITCH_INTERNALS_HTML);
 
   source->UseStringsJs();
-
-  return source;
 }
 
 }  // namespace
@@ -233,14 +236,14 @@ class BrowserSwitchHandler : public content::WebUIMessageHandler {
   //
   // If it fails, the JavaScript promise is rejected. If it succeeds, the
   // JavaScript promise is not resolved, because we close the tab anyways.
-  void HandleLaunchAlternativeBrowserAndCloseTab(const base::ListValue* args);
+  void HandleLaunchAlternativeBrowserAndCloseTab(const base::Value::List& args);
 
   void OnLaunchFinished(base::TimeTicks start,
                         std::string callback_id,
                         bool success);
 
   // Navigates to the New Tab Page.
-  void HandleGotoNewTabPage(const base::ListValue* args);
+  void HandleGotoNewTabPage(const base::Value::List& args);
 
   // Resolves a promise with a JSON object with all the LBS rulesets, formatted
   // like this:
@@ -253,7 +256,7 @@ class BrowserSwitchHandler : public content::WebUIMessageHandler {
   //   "ieem": { "sitelist": [...], "greylist": [...] },
   //   "external": { "sitelist": [...], "greylist": [...] }
   // }
-  void HandleGetAllRulesets(const base::ListValue* args);
+  void HandleGetAllRulesets(const base::Value::List& args);
 
   // Resolves a promise with a JSON object describing the decision for a URL
   // (stay/go) + reason. The result is formatted like this:
@@ -263,7 +266,7 @@ class BrowserSwitchHandler : public content::WebUIMessageHandler {
   //   "reason": ("globally_disabled"|"protocol"|"sitelist"|...),
   //   "matching_rule": (string|undefined)
   // }
-  void HandleGetDecision(const base::ListValue* args);
+  void HandleGetDecision(const base::Value::List& args);
 
   // Resolves a promise with the time of the last policy fetch and next policy
   // fetch, as JS timestamps.
@@ -272,7 +275,7 @@ class BrowserSwitchHandler : public content::WebUIMessageHandler {
   //   "last_fetch": 123456789,
   //   "next_fetch": 234567890
   // }
-  void HandleGetTimestamps(const base::ListValue* args);
+  void HandleGetTimestamps(const base::Value::List& args);
 
   // Resolves a promise with the configured sitelist XML download URLs. The keys
   // are the name of the pref associated with the sitelist.
@@ -284,14 +287,14 @@ class BrowserSwitchHandler : public content::WebUIMessageHandler {
   //     "external_greylist_url": null
   //   }
   // }
-  void HandleGetRulesetSources(const base::ListValue* args);
+  void HandleGetRulesetSources(const base::Value::List& args);
 
   // Immediately re-download and apply XML rules.
-  void HandleRefreshXml(const base::ListValue* args);
+  void HandleRefreshXml(const base::Value::List& args);
 
   // Resolves a promise with the boolean value describing whether the feature
   // is enabled or not which is configured by BrowserSwitcherEnabled key
-  void HandleIsBrowserSwitchEnabled(const base::ListValue* args);
+  void HandleIsBrowserSwitchEnabled(const base::Value::List& args);
 
   base::CallbackListSubscription prefs_subscription_;
 
@@ -305,35 +308,35 @@ BrowserSwitchHandler::BrowserSwitchHandler() {}
 BrowserSwitchHandler::~BrowserSwitchHandler() = default;
 
 void BrowserSwitchHandler::RegisterMessages() {
-  web_ui()->RegisterDeprecatedMessageCallback(
+  web_ui()->RegisterMessageCallback(
       "launchAlternativeBrowserAndCloseTab",
       base::BindRepeating(
           &BrowserSwitchHandler::HandleLaunchAlternativeBrowserAndCloseTab,
           base::Unretained(this)));
-  web_ui()->RegisterDeprecatedMessageCallback(
+  web_ui()->RegisterMessageCallback(
       "gotoNewTabPage",
       base::BindRepeating(&BrowserSwitchHandler::HandleGotoNewTabPage,
                           base::Unretained(this)));
-  web_ui()->RegisterDeprecatedMessageCallback(
+  web_ui()->RegisterMessageCallback(
       "getAllRulesets",
       base::BindRepeating(&BrowserSwitchHandler::HandleGetAllRulesets,
                           base::Unretained(this)));
-  web_ui()->RegisterDeprecatedMessageCallback(
+  web_ui()->RegisterMessageCallback(
       "getDecision",
       base::BindRepeating(&BrowserSwitchHandler::HandleGetDecision,
                           base::Unretained(this)));
-  web_ui()->RegisterDeprecatedMessageCallback(
+  web_ui()->RegisterMessageCallback(
       "getTimestamps",
       base::BindRepeating(&BrowserSwitchHandler::HandleGetTimestamps,
                           base::Unretained(this)));
-  web_ui()->RegisterDeprecatedMessageCallback(
+  web_ui()->RegisterMessageCallback(
       "getRulesetSources",
       base::BindRepeating(&BrowserSwitchHandler::HandleGetRulesetSources,
                           base::Unretained(this)));
-  web_ui()->RegisterDeprecatedMessageCallback(
+  web_ui()->RegisterMessageCallback(
       "refreshXml", base::BindRepeating(&BrowserSwitchHandler::HandleRefreshXml,
                                         base::Unretained(this)));
-  web_ui()->RegisterDeprecatedMessageCallback(
+  web_ui()->RegisterMessageCallback(
       "isBrowserSwitcherEnabled",
       base::BindRepeating(&BrowserSwitchHandler::HandleIsBrowserSwitchEnabled,
                           base::Unretained(this)));
@@ -370,12 +373,11 @@ void BrowserSwitchHandler::SendDataChangedEvent() {
 }
 
 void BrowserSwitchHandler::HandleLaunchAlternativeBrowserAndCloseTab(
-    const base::ListValue* args) {
-  DCHECK(args);
+    const base::Value::List& args) {
   AllowJavascript();
 
-  std::string callback_id = args->GetListDeprecated()[0].GetString();
-  std::string url_spec = args->GetListDeprecated()[1].GetString();
+  std::string callback_id = args[0].GetString();
+  std::string url_spec = args[1].GetString();
   GURL url(url_spec);
 
   auto* service = GetBrowserSwitcherService(web_ui());
@@ -384,7 +386,7 @@ void BrowserSwitchHandler::HandleLaunchAlternativeBrowserAndCloseTab(
     // This URL shouldn't open in an alternative browser. Abort launch, because
     // something weird is going on (e.g. race condition from a new sitelist
     // being loaded).
-    RejectJavascriptCallback(args->GetListDeprecated()[0], base::Value());
+    RejectJavascriptCallback(args[0], base::Value());
     return;
   }
 
@@ -413,55 +415,44 @@ void BrowserSwitchHandler::OnLaunchFinished(base::TimeTicks start,
   if (service->prefs().KeepLastTab() && IsLastTab(profile)) {
     GotoNewTabPage(web_ui()->GetWebContents());
   } else {
-    base::ThreadTaskRunnerHandle::Get()->PostTask(
+    base::SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
         FROM_HERE, base::BindOnce(&content::WebContents::ClosePage,
                                   web_ui()->GetWebContents()->GetWeakPtr()));
   }
 }
 
-void BrowserSwitchHandler::HandleGotoNewTabPage(const base::ListValue* args) {
+void BrowserSwitchHandler::HandleGotoNewTabPage(const base::Value::List& args) {
   GotoNewTabPage(web_ui()->GetWebContents());
 }
 
-void BrowserSwitchHandler::HandleGetAllRulesets(const base::ListValue* args) {
-  DCHECK(args);
+void BrowserSwitchHandler::HandleGetAllRulesets(const base::Value::List& args) {
   AllowJavascript();
-
   auto* service = GetBrowserSwitcherService(web_ui());
-
-  base::Value retval(base::Value::Type::DICTIONARY);
-  auto gpo_dict = RuleSetToDict(service->prefs().GetRules());
-  retval.SetKey("gpo", std::move(gpo_dict));
-  auto ieem_dict = RuleSetToDict(*service->sitelist()->GetIeemSitelist());
-  retval.SetKey("ieem", std::move(ieem_dict));
-  auto external_sitelist_dict =
-      RuleSetToDict(*service->sitelist()->GetExternalSitelist());
-  retval.SetKey("external_sitelist", std::move(external_sitelist_dict));
-  auto external_greylist_dict =
-      RuleSetToDict(*service->sitelist()->GetExternalGreylist());
-  retval.SetKey("external_greylist", std::move(external_greylist_dict));
-
-  ResolveJavascriptCallback(args->GetListDeprecated()[0], retval);
+  auto retval =
+      base::Value::Dict()
+          .Set("gpo", RuleSetToDict(service->prefs().GetRules()))
+          .Set("ieem", RuleSetToDict(*service->sitelist()->GetIeemSitelist()))
+          .Set("external_sitelist",
+               RuleSetToDict(*service->sitelist()->GetExternalSitelist()))
+          .Set("external_greylist",
+               RuleSetToDict(*service->sitelist()->GetExternalGreylist()));
+  ResolveJavascriptCallback(args[0], retval);
 }
 
-void BrowserSwitchHandler::HandleGetDecision(const base::ListValue* args) {
-  DCHECK(args);
+void BrowserSwitchHandler::HandleGetDecision(const base::Value::List& args) {
   AllowJavascript();
 
-  GURL url = GURL(args->GetListDeprecated()[1].GetString());
+  GURL url = GURL(args[1].GetString());
   if (!url.is_valid()) {
-    RejectJavascriptCallback(args->GetListDeprecated()[0], base::Value());
+    RejectJavascriptCallback(args[0], base::Value());
     return;
   }
 
   auto* service = GetBrowserSwitcherService(web_ui());
   browser_switcher::Decision decision = service->sitelist()->GetDecision(url);
 
-  base::DictionaryValue retval;
-
   base::StringPiece action_name =
       (decision.action == browser_switcher::kStay) ? "stay" : "go";
-  retval.Set("action", std::make_unique<base::Value>(action_name));
 
   base::StringPiece reason_name;
   switch (decision.reason) {
@@ -481,72 +472,74 @@ void BrowserSwitchHandler::HandleGetDecision(const base::ListValue* args) {
       reason_name = "default";
       break;
   }
-  retval.Set("reason", std::make_unique<base::Value>(reason_name));
+
+  // clang-format off
+  auto retval =
+      base::Value::Dict()
+          .Set("action", action_name)
+          .Set("reason", reason_name);
+  // clang-format on
 
   if (decision.matching_rule) {
-    retval.Set("matching_rule", std::make_unique<base::Value>(
-                                    decision.matching_rule->ToString()));
+    retval.Set("matching_rule", decision.matching_rule->ToString());
   }
 
-  ResolveJavascriptCallback(args->GetListDeprecated()[0], retval);
+  ResolveJavascriptCallback(args[0], retval);
 }
 
-void BrowserSwitchHandler::HandleGetTimestamps(const base::ListValue* args) {
-  DCHECK(args);
+void BrowserSwitchHandler::HandleGetTimestamps(const base::Value::List& args) {
   AllowJavascript();
 
   auto* service = GetBrowserSwitcherService(web_ui());
   auto* downloader = service->sitelist_downloader();
 
   if (!downloader) {
-    ResolveJavascriptCallback(args->GetListDeprecated()[0], base::Value());
+    ResolveJavascriptCallback(args[0], base::Value());
     return;
   }
 
-  base::DictionaryValue retval;
-  retval.Set("last_fetch", std::make_unique<base::Value>(
-                               downloader->last_refresh_time().ToJsTime()));
-  retval.Set("next_fetch", std::make_unique<base::Value>(
-                               downloader->next_refresh_time().ToJsTime()));
+  auto retval =
+      base::Value::Dict()
+          .Set("last_fetch",
+               downloader->last_refresh_time().InMillisecondsFSinceUnixEpoch())
+          .Set("next_fetch",
+               downloader->next_refresh_time().InMillisecondsFSinceUnixEpoch());
 
-  ResolveJavascriptCallback(args->GetListDeprecated()[0], retval);
+  ResolveJavascriptCallback(args[0], retval);
 }
 
 void BrowserSwitchHandler::HandleGetRulesetSources(
-    const base::ListValue* args) {
-  DCHECK(args);
+    const base::Value::List& args) {
   AllowJavascript();
 
   auto* service = GetBrowserSwitcherService(web_ui());
   std::vector<browser_switcher::RulesetSource> sources =
       service->GetRulesetSources();
 
-  base::DictionaryValue retval;
+  base::Value::Dict retval;
   for (const auto& source : sources) {
     base::Value val;
     if (source.url.is_valid())
       val = base::Value(source.url.spec());
-    // |pref_name| is something like "browser_switcher.blah", so this will be in
-    // a nested object.
-    retval.SetKey(source.pref_name, std::move(val));
+    // |pref_name| is something like "browser_switcher.blah"; however path
+    // expansion is not expected on it as the JavaScript expects to see
+    // "browser_switcher.blah" as a key in the object, not a nested hierarchy.
+    retval.Set(source.pref_name, std::move(val));
   }
-  ResolveJavascriptCallback(args->GetListDeprecated()[0], retval);
+  ResolveJavascriptCallback(args[0], retval);
 }
 
-void BrowserSwitchHandler::HandleRefreshXml(const base::ListValue* args) {
-  DCHECK(args);
+void BrowserSwitchHandler::HandleRefreshXml(const base::Value::List& args) {
   auto* service = GetBrowserSwitcherService(web_ui());
   service->StartDownload(base::TimeDelta());
 }
 
 void BrowserSwitchHandler::HandleIsBrowserSwitchEnabled(
-    const base::ListValue* args) {
-  DCHECK(args);
+    const base::Value::List& args) {
   AllowJavascript();
 
   auto* service = GetBrowserSwitcherService(web_ui());
-  ResolveJavascriptCallback(args->GetListDeprecated()[0],
-                            base::Value(service->prefs().IsEnabled()));
+  ResolveJavascriptCallback(args[0], base::Value(service->prefs().IsEnabled()));
 }
 
 BrowserSwitchUI::BrowserSwitchUI(content::WebUI* web_ui)
@@ -554,7 +547,5 @@ BrowserSwitchUI::BrowserSwitchUI(content::WebUI* web_ui)
   web_ui->AddMessageHandler(std::make_unique<BrowserSwitchHandler>());
 
   // Set up the chrome://browser-switch source.
-  Profile* profile = Profile::FromWebUI(web_ui);
-  content::WebUIDataSource::Add(profile,
-                                CreateBrowserSwitchUIHTMLSource(web_ui));
+  CreateAndAddBrowserSwitchUIHTMLSource(web_ui);
 }
