@@ -5,10 +5,12 @@
 #include "chrome/browser/ui/webui/print_preview/print_preview_handler.h"
 
 #include <map>
+#include <optional>
 #include <string>
 #include <utility>
 #include <vector>
 
+#include "base/check.h"
 #include "base/containers/flat_set.h"
 #include "base/i18n/number_formatting.h"
 #include "base/json/json_writer.h"
@@ -19,20 +21,24 @@
 #include "base/strings/utf_string_conversions.h"
 #include "base/test/icu_test_util.h"
 #include "base/test/metrics/histogram_tester.h"
+#include "base/test/scoped_feature_list.h"
 #include "base/test/values_test_util.h"
 #include "base/values.h"
 #include "build/build_config.h"
 #include "build/chromeos_buildflags.h"
+#include "chrome/browser/printing/print_preview_dialog_controller.h"
 #include "chrome/browser/printing/print_test_utils.h"
 #include "chrome/browser/printing/print_view_manager.h"
 #include "chrome/browser/ui/webui/print_preview/fake_print_render_frame.h"
 #include "chrome/browser/ui/webui/print_preview/policy_settings.h"
 #include "chrome/browser/ui/webui/print_preview/print_preview_metrics.h"
 #include "chrome/browser/ui/webui/print_preview/print_preview_ui.h"
+#include "chrome/browser/ui/webui/print_preview/print_preview_utils.h"
 #include "chrome/browser/ui/webui/print_preview/printer_handler.h"
 #include "chrome/common/pref_names.h"
 #include "chrome/test/base/testing_browser_process.h"
 #include "chrome/test/base/testing_profile.h"
+#include "components/enterprise/buildflags/buildflags.h"
 #include "components/prefs/pref_service.h"
 #include "components/sync_preferences/testing_pref_service_syncable.h"
 #include "content/public/browser/render_frame_host.h"
@@ -42,12 +48,27 @@
 #include "content/public/test/navigation_simulator.h"
 #include "content/public/test/test_renderer_host.h"
 #include "content/public/test/test_web_ui.h"
+#include "printing/buildflags/buildflags.h"
 #include "printing/mojom/print.mojom.h"
+#include "printing/printing_features.h"
 #include "testing/gtest/include/gtest/gtest.h"
-#include "third_party/abseil-cpp/absl/types/optional.h"
+
+#if BUILDFLAG(ENABLE_PRINT_CONTENT_ANALYSIS)
+#include "chrome/browser/enterprise/connectors/common.h"
+#include "chrome/browser/enterprise/connectors/test/deep_scanning_test_utils.h"
+#include "chrome/browser/enterprise/connectors/test/fake_content_analysis_delegate.h"
+#include "chrome/browser/policy/dm_token_utils.h"
+
+#if BUILDFLAG(ENTERPRISE_LOCAL_CONTENT_ANALYSIS)
+#include "chrome/browser/enterprise/connectors/test/fake_content_analysis_sdk_manager.h"  // nogncheck
+#endif  // BUILDFLAG(ENTERPRISE_LOCAL_CONTENT_ANALYSIS)
+#endif  // BUILDFLAG(ENABLE_PRINT_CONTENT_ANALYSIS)
+
+#if BUILDFLAG(IS_CHROMEOS) || BUILDFLAG(ENABLE_PRINT_CONTENT_ANALYSIS)
+#include "chrome/test/base/testing_profile_manager.h"
+#endif
 
 #if BUILDFLAG(IS_CHROMEOS)
-#include "chrome/test/base/testing_profile_manager.h"
 #include "chromeos/crosapi/mojom/local_printer.mojom.h"
 #endif
 
@@ -59,7 +80,6 @@
 #include "chromeos/ash/components/login/login_state/login_state.h"
 #elif BUILDFLAG(IS_CHROMEOS_LACROS)
 #include "chrome/test/chromeos/printing/fake_local_printer_chromeos.h"
-#include "chromeos/crosapi/mojom/drive_integration_service.mojom.h"
 #endif
 
 namespace printing {
@@ -71,6 +91,11 @@ const char16_t kDummyInitiatorName16[] = u"TestInitiator";
 const char kEmptyPrinterName[] = "EmptyPrinter";
 const char kTestData[] = "abc";
 
+#if BUILDFLAG(ENTERPRISE_LOCAL_CONTENT_ANALYSIS)
+constexpr char kFakeDmToken[] = "fake-dm-token";
+constexpr char kCallbackId[] = "test-callback-id-1";
+#endif  // BUILDFLAG(ENTERPRISE_LOCAL_CONTENT_ANALYSIS)
+
 // Array of all mojom::PrinterTypes.
 constexpr mojom::PrinterType kAllTypes[] = {mojom::PrinterType::kExtension,
                                             mojom::PrinterType::kPdf,
@@ -119,7 +144,8 @@ PrinterInfo GetEmptyPrinterInfo() {
 }
 
 base::Value::Dict GetPrintPreviewTicket() {
-  base::Value::Dict print_ticket = GetPrintTicket(mojom::PrinterType::kLocal);
+  base::Value::Dict print_ticket =
+      test::GetPrintTicket(mojom::PrinterType::kLocal);
 
   // Make some modifications to match a preview print ticket.
   print_ticket.Set(kSettingPageRange, base::Value());
@@ -322,26 +348,9 @@ class TestLocalPrinter : public FakeLocalPrinter {
   }
 
  private:
-  absl::optional<crosapi::mojom::Policies> policies_;
+  std::optional<crosapi::mojom::Policies> policies_;
   std::vector<mojom::PrinterType> deny_list_;
 };
-
-class FakeDriveIntegrationService
-    : public crosapi::mojom::DriveIntegrationService {
- public:
-  FakeDriveIntegrationService() = default;
-  FakeDriveIntegrationService(const FakeDriveIntegrationService&) = delete;
-  FakeDriveIntegrationService& operator=(const FakeDriveIntegrationService&) =
-      delete;
-  ~FakeDriveIntegrationService() override = default;
-
-  void GetMountPointPath(GetMountPointPathCallback callback) override {
-    std::move(callback).Run(base::FilePath("/drive/path"));
-  }
-  void AddDriveIntegrationServiceObserver(
-      mojo::PendingRemote<crosapi::mojom::DriveIntegrationServiceObserver>)
-      override {}
-};
 #endif
 
 class TestPrintPreviewHandler : public PrintPreviewHandler {
@@ -379,9 +388,38 @@ class TestPrintPreviewHandler : public PrintPreviewHandler {
   int bad_messages_;
   base::flat_set<mojom::PrinterType> called_for_type_;
   std::unique_ptr<PrinterHandler> test_printer_handler_;
-  const raw_ptr<content::WebContents> initiator_;
+  const raw_ptr<content::WebContents, DanglingUntriaged> initiator_;
 };
 
+#if BUILDFLAG(ENABLE_PRINT_CONTENT_ANALYSIS)
+class TestPrintPreviewHandlerForContentAnalysis
+    : public TestPrintPreviewHandler {
+ public:
+  TestPrintPreviewHandlerForContentAnalysis(
+      std::unique_ptr<PrinterHandler> printer_handler,
+      content::WebContents* web_contents)
+      : TestPrintPreviewHandler(std::move(printer_handler), web_contents) {}
+
+  ~TestPrintPreviewHandlerForContentAnalysis() override = default;
+
+  bool print_called_after_scan() const { return print_called_after_scan_; }
+
+ private:
+  void FinishHandleDoPrint(UserActionBuckets user_action,
+                           base::Value::Dict settings,
+                           scoped_refptr<base::RefCountedMemory> data,
+                           const std::string& callback_id) override {
+    ASSERT_EQ(base::StringPiece(data->front_as<const char>(), data->size()),
+              kTestData);
+    print_called_after_scan_ = true;
+    PrintPreviewHandler::FinishHandleDoPrint(user_action, std::move(settings),
+                                             data, callback_id);
+  }
+
+  bool print_called_after_scan_ = false;
+};
+#endif  // BUILDFLAG(ENABLE_PRINT_CONTENT_ANALYSIS)
+
 }  // namespace
 
 class PrintPreviewHandlerTest : public testing::Test {
@@ -399,16 +437,25 @@ class PrintPreviewHandlerTest : public testing::Test {
   void AddToDenyList(const mojom::PrinterType& printer_type) {
     local_printer_.add_to_deny_list(printer_type);
   }
+#endif
 
   void SetProfileForInitialSettings(TestingProfile* profile) {
+    auto* dialog_controller = PrintPreviewDialogController::GetInstance();
+    CHECK(dialog_controller);
+    if (preview_web_contents_) {
+      dialog_controller->DisassociateWebContentsesForTesting(
+          preview_web_contents_.get());
+    }
+
     preview_web_contents_ = content::WebContents::Create(
         content::WebContents::CreateParams(profile));
     web_ui_->set_web_contents(preview_web_contents_.get());
+    dialog_controller->AssociateWebContentsesForTesting(
+        initiator_web_contents_.get(), preview_web_contents_.get());
   }
-#endif
 
   void SetUp() override {
-#if BUILDFLAG(IS_CHROMEOS)
+#if BUILDFLAG(IS_CHROMEOS) || BUILDFLAG(ENABLE_PRINT_CONTENT_ANALYSIS)
     ASSERT_TRUE(testing_profile_manager_.SetUp());
 #endif
 #if BUILDFLAG(IS_CHROMEOS_ASH)
@@ -417,6 +464,8 @@ class PrintPreviewHandlerTest : public testing::Test {
     ash::LoginState::Initialize();
     manager_ = crosapi::CreateCrosapiManagerWithTestRegistry();
 #endif
+
+    // Create the initiator.
     initiator_web_contents_ = content::WebContents::Create(
         content::WebContents::CreateParams(&profile_));
     content::WebContents* initiator = initiator_web_contents_.get();
@@ -424,20 +473,21 @@ class PrintPreviewHandlerTest : public testing::Test {
     // the print code will not bother to send IPCs to a non-live RenderFrame.
     content::NavigationSimulator::NavigateAndCommitFromDocument(
         GURL("about:blank"), initiator->GetPrimaryMainFrame());
-    preview_web_contents_ = content::WebContents::Create(
-        content::WebContents::CreateParams(&profile_));
+
+    // Create the print preview.
+    web_ui_ = std::make_unique<content::TestWebUI>();
+    SetProfileForInitialSettings(&profile_);
+
+    // Create the PrintViewManager and put it to work.
     PrintViewManager::CreateForWebContents(initiator);
     PrintViewManager::FromWebContents(initiator)->PrintPreviewNow(
         initiator->GetPrimaryMainFrame(), false);
-    web_ui_ = std::make_unique<content::TestWebUI>();
-    web_ui_->set_web_contents(preview_web_contents_.get());
 
-    printers_.push_back(GetSimplePrinterInfo(kDummyPrinterName, true));
+    printers_.push_back(GetSimplePrinterInfo(test::kPrinterName, true));
     auto printer_handler = CreatePrinterHandler(printers_);
     printer_handler_ = printer_handler.get();
 
-    auto preview_handler = std::make_unique<TestPrintPreviewHandler>(
-        std::move(printer_handler), initiator);
+    auto preview_handler = CreateHandler(std::move(printer_handler), initiator);
     handler_ = preview_handler.get();
     handler_->set_web_ui(web_ui());
 #if BUILDFLAG(IS_CHROMEOS_ASH)
@@ -445,7 +495,6 @@ class PrintPreviewHandlerTest : public testing::Test {
 #elif BUILDFLAG(IS_CHROMEOS_LACROS)
     handler_->local_printer_ = &local_printer_;
     handler_->local_printer_version_ = crosapi::mojom::LocalPrinter::Version_;
-    handler_->drive_integration_service_ = &drive_integration_service_;
 #endif
 
     auto preview_ui = std::make_unique<FakePrintPreviewUI>(
@@ -461,6 +510,11 @@ class PrintPreviewHandlerTest : public testing::Test {
 #endif
     PrintViewManager::FromWebContents(initiator_web_contents_.get())
         ->PrintPreviewDone();
+
+    auto* dialog_controller = PrintPreviewDialogController::GetInstance();
+    CHECK(dialog_controller);
+    dialog_controller->DisassociateWebContentsesForTesting(
+        preview_web_contents_.get());
   }
 
 #if BUILDFLAG(IS_CHROMEOS)
@@ -472,6 +526,13 @@ class PrintPreviewHandlerTest : public testing::Test {
     return std::make_unique<TestPrinterHandler>(printers);
   }
 
+  virtual std::unique_ptr<TestPrintPreviewHandler> CreateHandler(
+      std ::unique_ptr<TestPrinterHandler> printer_handler,
+      content::WebContents* web_contents) {
+    return std::make_unique<TestPrintPreviewHandler>(std::move(printer_handler),
+                                                     web_contents);
+  }
+
   void Initialize() { InitializeWithLocale("en"); }
 
   void InitializeWithLocale(const std::string& locale) {
@@ -588,7 +649,7 @@ class PrintPreviewHandlerTest : public testing::Test {
   void ValidateInitialSettingsValuePolicy(
       const content::TestWebUI::CallData& data,
       const std::string& policy_name,
-      absl::optional<base::Value> expected_policy_value) {
+      std::optional<base::Value> expected_policy_value) {
     CheckWebUIResponse(data, "test-callback-id-0", true);
     const base::Value::Dict& settings = data.arg3()->GetDict();
 
@@ -608,8 +669,8 @@ class PrintPreviewHandlerTest : public testing::Test {
   void ValidateInitialSettingsAllowedDefaultModePolicy(
       const content::TestWebUI::CallData& data,
       const std::string& policy_name,
-      absl::optional<base::Value> expected_allowed_mode,
-      absl::optional<base::Value> expected_default_mode) {
+      std::optional<base::Value> expected_allowed_mode,
+      std::optional<base::Value> expected_default_mode) {
     CheckWebUIResponse(data, "test-callback-id-0", true);
     const base::Value::Dict& settings = data.arg3()->GetDict();
 
@@ -692,9 +753,10 @@ class PrintPreviewHandlerTest : public testing::Test {
   }
   content::TestWebUI* web_ui() { return web_ui_.get(); }
   TestPrintPreviewHandler* handler() { return handler_; }
+  TestingProfile* profile() { return &profile_; }
   TestPrinterHandler* printer_handler() { return printer_handler_; }
   std::vector<PrinterInfo>& printers() { return printers_; }
-#if BUILDFLAG(IS_CHROMEOS)
+#if BUILDFLAG(IS_CHROMEOS) || BUILDFLAG(ENABLE_PRINT_CONTENT_ANALYSIS)
   TestingProfileManager* testing_profile_manager() {
     return &testing_profile_manager_;
   }
@@ -702,7 +764,7 @@ class PrintPreviewHandlerTest : public testing::Test {
 
  private:
   content::BrowserTaskEnvironment task_environment_;
-#if BUILDFLAG(IS_CHROMEOS)
+#if BUILDFLAG(IS_CHROMEOS) || BUILDFLAG(ENABLE_PRINT_CONTENT_ANALYSIS)
   TestingProfileManager testing_profile_manager_{
       TestingBrowserProcess::GetGlobal()};
 #endif
@@ -711,7 +773,6 @@ class PrintPreviewHandlerTest : public testing::Test {
   std::unique_ptr<crosapi::CrosapiManager> manager_;
 #elif BUILDFLAG(IS_CHROMEOS_LACROS)
   TestLocalPrinter local_printer_;
-  FakeDriveIntegrationService drive_integration_service_;
 #endif
   TestingProfile profile_;
   std::unique_ptr<content::TestWebUI> web_ui_;
@@ -727,7 +788,7 @@ TEST_F(PrintPreviewHandlerTest, InitialSettingsSimple) {
   Initialize();
 
   // Verify initial settings were sent.
-  ValidateInitialSettings(*web_ui()->call_data().back(), kDummyPrinterName,
+  ValidateInitialSettings(*web_ui()->call_data().back(), test::kPrinterName,
                           kDummyInitiatorName);
 }
 
@@ -736,8 +797,8 @@ TEST_F(PrintPreviewHandlerTest, InitialSettingsHiLocale) {
 
   // Verify initial settings were sent for Hindi.
   ValidateInitialSettingsForLocale(*web_ui()->call_data().back(),
-                                   kDummyPrinterName, kDummyInitiatorName, "hi",
-                                   ",", ".");
+                                   test::kPrinterName, kDummyInitiatorName,
+                                   "hi", ",", ".");
 }
 
 TEST_F(PrintPreviewHandlerTest, InitialSettingsRuLocale) {
@@ -745,22 +806,22 @@ TEST_F(PrintPreviewHandlerTest, InitialSettingsRuLocale) {
 
   // Verify initial settings were sent for Russian.
   ValidateInitialSettingsForLocale(*web_ui()->call_data().back(),
-                                   kDummyPrinterName, kDummyInitiatorName, "ru",
-                                   "\xC2\xA0", ",");
+                                   test::kPrinterName, kDummyInitiatorName,
+                                   "ru", "\xC2\xA0", ",");
 }
 
 TEST_F(PrintPreviewHandlerTest, InitialSettingsNoPolicies) {
   Initialize();
   ValidateInitialSettingsAllowedDefaultModePolicy(*web_ui()->call_data().back(),
-                                                  "headerFooter", absl::nullopt,
-                                                  absl::nullopt);
+                                                  "headerFooter", std::nullopt,
+                                                  std::nullopt);
   ValidateInitialSettingsAllowedDefaultModePolicy(*web_ui()->call_data().back(),
-                                                  "cssBackground",
-                                                  absl::nullopt, absl::nullopt);
+                                                  "cssBackground", std::nullopt,
+                                                  std::nullopt);
   ValidateInitialSettingsAllowedDefaultModePolicy(
-      *web_ui()->call_data().back(), "mediaSize", absl::nullopt, absl::nullopt);
+      *web_ui()->call_data().back(), "mediaSize", std::nullopt, std::nullopt);
   ValidateInitialSettingsValuePolicy(*web_ui()->call_data().back(), "sheets",
-                                     absl::nullopt);
+                                     std::nullopt);
 }
 
 #if BUILDFLAG(IS_CHROMEOS)
@@ -768,19 +829,19 @@ TEST_F(PrintPreviewHandlerTest, InitialSettingsNoAsh) {
   DisableAshChrome();
   Initialize();
   // Verify initial settings were sent.
-  ValidateInitialSettings(*web_ui()->call_data().back(), kDummyPrinterName,
+  ValidateInitialSettings(*web_ui()->call_data().back(), test::kPrinterName,
                           kDummyInitiatorName);
   // Verify policy settings are empty.
   ValidateInitialSettingsAllowedDefaultModePolicy(*web_ui()->call_data().back(),
-                                                  "headerFooter", absl::nullopt,
-                                                  absl::nullopt);
+                                                  "headerFooter", std::nullopt,
+                                                  std::nullopt);
   ValidateInitialSettingsAllowedDefaultModePolicy(*web_ui()->call_data().back(),
-                                                  "cssBackground",
-                                                  absl::nullopt, absl::nullopt);
+                                                  "cssBackground", std::nullopt,
+                                                  std::nullopt);
   ValidateInitialSettingsAllowedDefaultModePolicy(
-      *web_ui()->call_data().back(), "mediaSize", absl::nullopt, absl::nullopt);
+      *web_ui()->call_data().back(), "mediaSize", std::nullopt, std::nullopt);
   ValidateInitialSettingsValuePolicy(*web_ui()->call_data().back(), "sheets",
-                                     absl::nullopt);
+                                     std::nullopt);
 }
 #endif
 
@@ -798,7 +859,7 @@ TEST_F(PrintPreviewHandlerTest, InitialSettingsRestrictHeaderFooterEnabled) {
   Initialize();
   ValidateInitialSettingsAllowedDefaultModePolicy(
       *web_ui()->call_data().back(), "headerFooter", base::Value(true),
-      absl::nullopt);
+      std::nullopt);
 }
 
 TEST_F(PrintPreviewHandlerTest, InitialSettingsRestrictHeaderFooterDisabled) {
@@ -815,7 +876,7 @@ TEST_F(PrintPreviewHandlerTest, InitialSettingsRestrictHeaderFooterDisabled) {
   Initialize();
   ValidateInitialSettingsAllowedDefaultModePolicy(
       *web_ui()->call_data().back(), "headerFooter", base::Value(false),
-      absl::nullopt);
+      std::nullopt);
 }
 
 TEST_F(PrintPreviewHandlerTest, InitialSettingsEnableHeaderFooter) {
@@ -830,7 +891,7 @@ TEST_F(PrintPreviewHandlerTest, InitialSettingsEnableHeaderFooter) {
 #endif
   Initialize();
   ValidateInitialSettingsAllowedDefaultModePolicy(*web_ui()->call_data().back(),
-                                                  "headerFooter", absl::nullopt,
+                                                  "headerFooter", std::nullopt,
                                                   base::Value(true));
 }
 
@@ -846,7 +907,7 @@ TEST_F(PrintPreviewHandlerTest, InitialSettingsDisableHeaderFooter) {
 #endif
   Initialize();
   ValidateInitialSettingsAllowedDefaultModePolicy(*web_ui()->call_data().back(),
-                                                  "headerFooter", absl::nullopt,
+                                                  "headerFooter", std::nullopt,
                                                   base::Value(false));
 }
 
@@ -862,9 +923,9 @@ TEST_F(PrintPreviewHandlerTest,
   prefs()->SetInteger(prefs::kPrintingAllowedBackgroundGraphicsModes, 1);
 #endif
   Initialize();
-  ValidateInitialSettingsAllowedDefaultModePolicy(
-      *web_ui()->call_data().back(), "cssBackground", base::Value(1),
-      absl::nullopt);
+  ValidateInitialSettingsAllowedDefaultModePolicy(*web_ui()->call_data().back(),
+                                                  "cssBackground",
+                                                  base::Value(1), std::nullopt);
 }
 
 TEST_F(PrintPreviewHandlerTest,
@@ -879,9 +940,9 @@ TEST_F(PrintPreviewHandlerTest,
   prefs()->SetInteger(prefs::kPrintingAllowedBackgroundGraphicsModes, 2);
 #endif
   Initialize();
-  ValidateInitialSettingsAllowedDefaultModePolicy(
-      *web_ui()->call_data().back(), "cssBackground", base::Value(2),
-      absl::nullopt);
+  ValidateInitialSettingsAllowedDefaultModePolicy(*web_ui()->call_data().back(),
+                                                  "cssBackground",
+                                                  base::Value(2), std::nullopt);
 }
 
 TEST_F(PrintPreviewHandlerTest, InitialSettingsEnableBackgroundGraphics) {
@@ -895,9 +956,9 @@ TEST_F(PrintPreviewHandlerTest, InitialSettingsEnableBackgroundGraphics) {
   prefs()->SetInteger(prefs::kPrintingBackgroundGraphicsDefault, 1);
 #endif
   Initialize();
-  ValidateInitialSettingsAllowedDefaultModePolicy(
-      *web_ui()->call_data().back(), "cssBackground", absl::nullopt,
-      base::Value(1));
+  ValidateInitialSettingsAllowedDefaultModePolicy(*web_ui()->call_data().back(),
+                                                  "cssBackground", std::nullopt,
+                                                  base::Value(1));
 }
 
 TEST_F(PrintPreviewHandlerTest, InitialSettingsDisableBackgroundGraphics) {
@@ -911,9 +972,9 @@ TEST_F(PrintPreviewHandlerTest, InitialSettingsDisableBackgroundGraphics) {
   prefs()->SetInteger(prefs::kPrintingBackgroundGraphicsDefault, 2);
 #endif
   Initialize();
-  ValidateInitialSettingsAllowedDefaultModePolicy(
-      *web_ui()->call_data().back(), "cssBackground", absl::nullopt,
-      base::Value(2));
+  ValidateInitialSettingsAllowedDefaultModePolicy(*web_ui()->call_data().back(),
+                                                  "cssBackground", std::nullopt,
+                                                  base::Value(2));
 }
 
 TEST_F(PrintPreviewHandlerTest, InitialSettingsDefaultPaperSizeName) {
@@ -939,7 +1000,7 @@ TEST_F(PrintPreviewHandlerTest, InitialSettingsDefaultPaperSizeName) {
   Initialize();
 
   ValidateInitialSettingsAllowedDefaultModePolicy(
-      *web_ui()->call_data().back(), "mediaSize", absl::nullopt,
+      *web_ui()->call_data().back(), "mediaSize", std::nullopt,
       base::test::ParseJson(kExpectedInitialSettingsPolicy));
 }
 
@@ -970,7 +1031,7 @@ TEST_F(PrintPreviewHandlerTest, InitialSettingsDefaultPaperSizeCustomSize) {
   Initialize();
 
   ValidateInitialSettingsAllowedDefaultModePolicy(
-      *web_ui()->call_data().back(), "mediaSize", absl::nullopt,
+      *web_ui()->call_data().back(), "mediaSize", std::nullopt,
       base::test::ParseJson(kExpectedInitialSettingsPolicy));
 }
 
@@ -1000,7 +1061,7 @@ TEST_F(PrintPreviewHandlerTest, InitialSettingsEnableColorAndMonochrome) {
 #endif
   Initialize();
   ValidateInitialSettingsAllowedDefaultModePolicy(
-      *web_ui()->call_data().back(), "color", base::Value(3), absl::nullopt);
+      *web_ui()->call_data().back(), "color", base::Value(3), std::nullopt);
 }
 
 TEST_F(PrintPreviewHandlerTest, InitialSettingsDefaultColor) {
@@ -1014,7 +1075,7 @@ TEST_F(PrintPreviewHandlerTest, InitialSettingsDefaultColor) {
 #endif
   Initialize();
   ValidateInitialSettingsAllowedDefaultModePolicy(
-      *web_ui()->call_data().back(), "color", absl::nullopt, base::Value(2));
+      *web_ui()->call_data().back(), "color", std::nullopt, base::Value(2));
 }
 
 TEST_F(PrintPreviewHandlerTest, InitialSettingsEnableSimplexAndDuplex) {
@@ -1028,7 +1089,7 @@ TEST_F(PrintPreviewHandlerTest, InitialSettingsEnableSimplexAndDuplex) {
 #endif
   Initialize();
   ValidateInitialSettingsAllowedDefaultModePolicy(
-      *web_ui()->call_data().back(), "duplex", base::Value(7), absl::nullopt);
+      *web_ui()->call_data().back(), "duplex", base::Value(7), std::nullopt);
 }
 
 TEST_F(PrintPreviewHandlerTest, InitialSettingsDefaultSimplex) {
@@ -1043,7 +1104,7 @@ TEST_F(PrintPreviewHandlerTest, InitialSettingsDefaultSimplex) {
 #endif
   Initialize();
   ValidateInitialSettingsAllowedDefaultModePolicy(
-      *web_ui()->call_data().back(), "duplex", absl::nullopt, base::Value(1));
+      *web_ui()->call_data().back(), "duplex", std::nullopt, base::Value(1));
 }
 
 TEST_F(PrintPreviewHandlerTest, InitialSettingsRestrictPin) {
@@ -1057,7 +1118,7 @@ TEST_F(PrintPreviewHandlerTest, InitialSettingsRestrictPin) {
 #endif
   Initialize();
   ValidateInitialSettingsAllowedDefaultModePolicy(
-      *web_ui()->call_data().back(), "pin", base::Value(1), absl::nullopt);
+      *web_ui()->call_data().back(), "pin", base::Value(1), std::nullopt);
 }
 
 TEST_F(PrintPreviewHandlerTest, InitialSettingsDefaultNoPin) {
@@ -1071,11 +1132,12 @@ TEST_F(PrintPreviewHandlerTest, InitialSettingsDefaultNoPin) {
 #endif
   Initialize();
   ValidateInitialSettingsAllowedDefaultModePolicy(
-      *web_ui()->call_data().back(), "pin", absl::nullopt, base::Value(2));
+      *web_ui()->call_data().back(), "pin", std::nullopt, base::Value(2));
 }
 #endif  // BUILDFLAG(IS_CHROMEOS)
 
 TEST_F(PrintPreviewHandlerTest, GetPrinters) {
+  base::HistogramTester histogram_tester;
   Initialize();
 
   // Check all three printer types that implement
@@ -1098,6 +1160,10 @@ TEST_F(PrintPreviewHandlerTest, GetPrinters) {
     const content::TestWebUI::CallData& data = *web_ui()->call_data().back();
     CheckWebUIResponse(data, callback_id_in, true);
   }
+  histogram_tester.ExpectTotalCount("PrintPreview.GetPrintersTime.Extension",
+                                    1);
+  histogram_tester.ExpectTotalCount("PrintPreview.GetPrintersTime.Local", 1);
+  histogram_tester.ExpectTotalCount("PrintPreview.GetPrintersTime.PDF", 0);
 }
 
 // Validates the 'printing.printer_type_deny_list' pref by placing the extension
@@ -1156,7 +1222,7 @@ TEST_F(PrintPreviewHandlerTest, GetPrinterCapabilities) {
     std::string callback_id_in =
         "test-callback-id-" + base::NumberToString(i + 1);
     handler()->reset_calls();
-    SendGetPrinterCapabilities(type, callback_id_in, kDummyPrinterName);
+    SendGetPrinterCapabilities(type, callback_id_in, test::kPrinterName);
     EXPECT_TRUE(handler()->CalledOnlyForType(type));
 
     // Start with 1 call from initial settings, then add 1 more for each loop
@@ -1210,7 +1276,7 @@ TEST_F(PrintPreviewHandlerTest, GetNoDenyListPrinterCapabilities) {
     std::string callback_id_in =
         "test-callback-id-" + base::NumberToString(i + 1);
     handler()->reset_calls();
-    SendGetPrinterCapabilities(type, callback_id_in, kDummyPrinterName);
+    SendGetPrinterCapabilities(type, callback_id_in, test::kPrinterName);
 
     const bool is_allowed_type = type == mojom::PrinterType::kExtension;
     EXPECT_EQ(is_allowed_type, handler()->CalledOnlyForType(type));
@@ -1223,6 +1289,64 @@ TEST_F(PrintPreviewHandlerTest, GetNoDenyListPrinterCapabilities) {
   }
 }
 
+TEST_F(PrintPreviewHandlerTest, GetPrinterCapabilitiesContinuousMedia) {
+  // Add two media sizes to our printer - one with discrete sizes and one with
+  // continuous feed.  The continuous feed media should get filtered out when
+  // the capabilities are retrieved.
+  base::Value::Dict media_1;
+  media_1.Set("width_microns", 100);
+  media_1.Set("height_microns", 200);
+  base::Value::Dict media_2;
+  media_2.Set("width_microns", 300);
+  media_2.Set("is_continuous_feed", true);
+  // After filtering, the expected media will just have the discrete media.
+  base::Value::List expected_media;
+  expected_media.Append(media_1.Clone());
+
+  base::Value::List option_list;
+  option_list.Append(std::move(media_1));
+  option_list.Append(std::move(media_2));
+  base::Value::Dict media_size;
+  media_size.Set("option", std::move(option_list));
+  base::Value::Dict printer;
+  printer.Set("media_size", std::move(media_size));
+  base::Value::Dict cdd;
+  cdd.Set("printer", std::move(printer));
+
+  ASSERT_EQ(1u, printers().size());
+  printers()[0].capabilities.Set(kSettingCapabilities, std::move(cdd));
+  printer_handler()->SetPrinters(printers());
+
+  const base::Value::Dict* capabilities =
+      printers()[0].capabilities.FindDict(kSettingCapabilities);
+  ASSERT_TRUE(capabilities);
+  const base::Value::List* options = GetMediaSizeOptionsFromCdd(*capabilities);
+
+  ASSERT_TRUE(options);
+  EXPECT_EQ(2u, options->size());
+
+  // Initial settings first to enable javascript.
+  Initialize();
+
+  std::string callback_id_in = "test-callback-id";
+  handler()->reset_calls();
+  SendGetPrinterCapabilities(mojom::PrinterType::kLocal, callback_id_in,
+                             test::kPrinterName);
+  ASSERT_EQ(2u, web_ui()->call_data().size());
+
+  // Validate printer capabilities only has one media type (the continuous feed
+  // option should get filtered out).
+  const content::TestWebUI::CallData& data = *web_ui()->call_data().back();
+  CheckWebUIResponse(data, callback_id_in, true);
+  ASSERT_TRUE(data.arg3()->is_dict());
+  const base::Value::Dict& settings = data.arg3()->GetDict();
+  capabilities = settings.FindDict(kSettingCapabilities);
+  ASSERT_TRUE(capabilities);
+  options = GetMediaSizeOptionsFromCdd(*capabilities);
+  ASSERT_TRUE(options);
+  EXPECT_EQ(expected_media, *options);
+}
+
 TEST_F(PrintPreviewHandlerTest, Print) {
   Initialize();
 
@@ -1246,11 +1370,11 @@ TEST_F(PrintPreviewHandlerTest, Print) {
     std::string print_callback_id =
         "test-callback-id-" + base::NumberToString(2 * (i + 1));
     print_args.Append(print_callback_id);
-    base::Value print_ticket(GetPrintTicket(type));
+    base::Value print_ticket(test::GetPrintTicket(type));
     std::string json;
     base::JSONWriter::Write(print_ticket, &json);
     print_args.Append(json);
-    handler()->HandlePrint(print_args);
+    handler()->HandleDoPrint(print_args);
 
     CheckHistograms(histograms, type);
 
@@ -1310,11 +1434,11 @@ TEST_F(PrintPreviewHandlerTest, SendPreviewUpdates) {
   const base::Value::Dict& preview_params = print_render_frame.GetSettings();
 
   // Read the preview UI ID and request ID
-  absl::optional<int> request_value = preview_params.FindInt(kPreviewRequestID);
+  std::optional<int> request_value = preview_params.FindInt(kPreviewRequestID);
   ASSERT_TRUE(request_value.has_value());
   int preview_request_id = request_value.value();
 
-  absl::optional<int> ui_value = preview_params.FindInt(kPreviewUIID);
+  std::optional<int> ui_value = preview_params.FindInt(kPreviewUIID);
   ASSERT_TRUE(ui_value.has_value());
   int preview_ui_id = ui_value.value();
 
@@ -1332,7 +1456,8 @@ TEST_F(PrintPreviewHandlerTest, SendPreviewUpdates) {
   layout.Set(kSettingPrintableAreaWidth, 578);
   layout.Set(kSettingPrintableAreaHeight, 734);
   handler()->SendPageLayoutReady(std::move(layout),
-                                 /*has_custom_page_size_style,=*/false,
+                                 /*all_pages_have_custom_size,=*/false,
+                                 /*all_pages_have_custom_orientation,=*/false,
                                  preview_request_id);
 
   // Verify that page-layout-ready webUI event was fired.
@@ -1357,7 +1482,8 @@ TEST_F(PrintPreviewHandlerTest, SendPreviewUpdates) {
   // Check that there are no new web UI messages sent.
   size_t message_count = web_ui()->call_data().size();
   handler()->SendPageLayoutReady(base::Value::Dict(),
-                                 /*has_custom_page_size_style,=*/false,
+                                 /*all_pages_have_custom_size,=*/false,
+                                 /*all_pages_have_custom_orientation,=*/false,
                                  preview_request_id);
   EXPECT_EQ(message_count, web_ui()->call_data().size());
   handler()->SendPageCountReady(1, -1, 0);
@@ -1456,7 +1582,7 @@ TEST_F(PrintPreviewHandlerFailingTest, GetPrinterCapabilities) {
     std::string callback_id_in =
         "test-callback-id-" + base::NumberToString(i + 1);
     args.Append(callback_id_in);
-    args.Append(kDummyPrinterName);
+    args.Append(test::kPrinterName);
     args.Append(static_cast<int>(type));
     handler()->HandleGetPrinterCapabilities(args);
     EXPECT_TRUE(handler()->CalledOnlyForType(type));
@@ -1474,4 +1600,111 @@ TEST_F(PrintPreviewHandlerFailingTest, GetPrinterCapabilities) {
   }
 }
 
+#if BUILDFLAG(ENTERPRISE_LOCAL_CONTENT_ANALYSIS)
+class ContentAnalysisPrintPreviewHandlerTest
+    : public PrintPreviewHandlerTest,
+      public testing::WithParamInterface<bool> {
+ public:
+  void SetUp() override {
+    enterprise_connectors::ContentAnalysisDelegate::SetFactoryForTesting(
+        base::BindRepeating(
+            &enterprise_connectors::test::FakeContentAnalysisDelegate::Create,
+            run_loop_.QuitClosure(),
+            base::BindRepeating(
+                &ContentAnalysisPrintPreviewHandlerTest::ScanningResponse,
+                base::Unretained(this)),
+            kFakeDmToken));
+
+    enterprise_connectors::ContentAnalysisDelegate::DisableUIForTesting();
+    PrintPreviewHandlerTest::SetUp();
+
+    // Set the policy that enables local content analysis for print.
+    enterprise_connectors::test::SetAnalysisConnector(
+        profile()->GetPrefs(), enterprise_connectors::AnalysisConnector::PRINT,
+        R"({
+          "service_provider": "local_system_agent",
+          "enable": [ {"url_list": ["*"], "tags": ["dlp"]} ],
+          "block_until_verdict": 1
+        })");
+  }
+
+  void TearDown() override {
+    PrintPreviewHandlerTest::TearDown();
+    enterprise_connectors::ContentAnalysisDelegate::EnableUIAfterTesting();
+  }
+
+  std::unique_ptr<TestPrintPreviewHandler> CreateHandler(
+      std ::unique_ptr<TestPrinterHandler> printer_handler,
+      content::WebContents* web_contents) override {
+    return std::make_unique<TestPrintPreviewHandlerForContentAnalysis>(
+        std::move(printer_handler), web_contents);
+  }
+
+  bool scanning_allows_print() const { return GetParam(); }
+
+  enterprise_connectors::ContentAnalysisResponse ScanningResponse(
+      const std::string& content,
+      const base::FilePath& path) {
+    enterprise_connectors::ContentAnalysisResponse response;
+
+    auto* result = response.add_results();
+    result->set_tag("dlp");
+    result->set_status(
+        enterprise_connectors::ContentAnalysisResponse::Result::SUCCESS);
+
+    if (!scanning_allows_print()) {
+      auto* rule = result->add_triggered_rules();
+      rule->set_rule_name("blocking_rule_name");
+      rule->set_action(enterprise_connectors::TriggeredRule::BLOCK);
+    }
+
+    return response;
+  }
+
+  void WaitForScan() { run_loop_.Run(); }
+
+ private:
+  base::RunLoop run_loop_;
+
+  // This installs a fake SDK manager that creates fake SDK clients when
+  // its GetClient() method is called. This is needed so that calls to
+  // ContentAnalysisSdkManager::Get()->GetClient() do not fail.
+  enterprise_connectors::FakeContentAnalysisSdkManager sdk_manager_;
+};
+
+TEST_P(ContentAnalysisPrintPreviewHandlerTest, LocalScanBeforePrinting) {
+  TestingProfile* main_profile =
+      testing_profile_manager()->CreateTestingProfile("Main Profile",
+                                                      /*is_main_profile=*/true);
+  SetProfileForInitialSettings(main_profile);
+  Initialize();
+
+  base::Value::List print_args;
+  print_args.Append(kCallbackId);
+  base::Value print_ticket(test::GetPrintTicket(kAllTypes[0]));
+  std::string json;
+  base::JSONWriter::Write(print_ticket, &json);
+  print_args.Append(json);
+
+  handler()->HandleDoPrint(print_args);
+  WaitForScan();
+
+  auto* print_preview_handler =
+      static_cast<TestPrintPreviewHandlerForContentAnalysis*>(handler());
+
+  // Check if printing was allowed or not depending on the parameter.
+  ASSERT_EQ(print_preview_handler->print_called_after_scan(),
+            scanning_allows_print());
+
+  // Verify that printing went through or not based on the parameter.
+  const content::TestWebUI::CallData& data = *web_ui()->call_data().back();
+  CheckWebUIResponse(data, kCallbackId, scanning_allows_print());
+}
+
+INSTANTIATE_TEST_SUITE_P(All,
+                         ContentAnalysisPrintPreviewHandlerTest,
+                         /*scanning_allows_print=*/testing::Bool());
+
+#endif  // BUILDFLAG(ENTERPRISE_LOCAL_CONTENT_ANALYSIS)
+
 }  // namespace printing
