@@ -79,7 +79,7 @@ void DefaultBrowserHandler::SetAsDefaultBrowser(const base::Value::List& args) {
 
   default_browser_worker_->StartSetAsDefault(
       base::BindOnce(&DefaultBrowserHandler::OnDefaultBrowserWorkerFinished,
-                     weak_ptr_factory_.GetWeakPtr(), absl::nullopt));
+                     weak_ptr_factory_.GetWeakPtr(), std::nullopt));
 
   // If the user attempted to make Chrome the default browser, notify
   // them when this changes.
@@ -89,7 +89,7 @@ void DefaultBrowserHandler::SetAsDefaultBrowser(const base::Value::List& args) {
 void DefaultBrowserHandler::OnDefaultBrowserSettingChange() {
   default_browser_worker_->StartCheckIsDefault(
       base::BindOnce(&DefaultBrowserHandler::OnDefaultBrowserWorkerFinished,
-                     weak_ptr_factory_.GetWeakPtr(), absl::nullopt));
+                     weak_ptr_factory_.GetWeakPtr(), std::nullopt));
 }
 
 void DefaultBrowserHandler::RecordSetAsDefaultUMA() {
@@ -98,7 +98,7 @@ void DefaultBrowserHandler::RecordSetAsDefaultUMA() {
 }
 
 void DefaultBrowserHandler::OnDefaultBrowserWorkerFinished(
-    const absl::optional<std::string>& js_callback_id,
+    const std::optional<std::string>& js_callback_id,
     shell_integration::DefaultWebClientState state) {
   if (state == shell_integration::IS_DEFAULT) {
     // Notify the user in the future if Chrome ceases to be the user's chosen
