@@ -1,12 +1,10 @@
-// Copyright 2018 The Chromium Authors. All rights reserved.
+// Copyright 2018 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/ui/webui/version/version_handler_win.h"
 
-#include "base/bind.h"
-#include "base/strings/stringprintf.h"
-#include "base/task/post_task.h"
+#include "base/functional/bind.h"
 #include "base/task/thread_pool.h"
 #include "base/win/windows_version.h"
 #include "chrome/browser/ui/webui/version/version_util_win.h"
@@ -17,7 +15,7 @@ VersionHandlerWindows::VersionHandlerWindows() {}
 VersionHandlerWindows::~VersionHandlerWindows() {}
 
 void VersionHandlerWindows::HandleRequestVersionInfo(
-    const base::ListValue* args) {
+    const base::Value::List& args) {
   VersionHandler::HandleRequestVersionInfo(args);
 
   // Start the asynchronous load of the versions.
@@ -29,8 +27,7 @@ void VersionHandlerWindows::HandleRequestVersionInfo(
 }
 
 void VersionHandlerWindows::OnVersion(const std::string& version) {
-  base::Value arg(version);
-  CallJavascriptFunction("returnOsVersion", arg);
+  FireWebUIListener("return-os-version", base::Value(version));
 }
 
 // static
