@@ -1,4 +1,4 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,20 +10,19 @@
 #include <string>
 #include <vector>
 
-#include "base/compiler_specific.h"
 #include "base/files/file_path.h"
 #include "base/synchronization/lock.h"
+#include "base/task/sequenced_task_runner.h"
 #include "build/branding_buildflags.h"
 #include "build/build_config.h"
 #include "chrome/common/buildflags.h"
 #include "components/nacl/common/buildflags.h"
 #include "content/public/common/content_client.h"
-#include "pdf/buildflags.h"
 #include "ppapi/buildflags/buildflags.h"
 
-#if BUILDFLAG(ENABLE_PLUGINS)
-#include "content/public/common/pepper_plugin_info.h"
-#endif
+#if BUILDFLAG(ENABLE_NACL)
+#include "content/public/common/content_plugin_info.h"
+#endif  // BUILDFLAG(ENABLE_NACL)
 
 namespace embedder_support {
 class OriginTrialPolicyImpl;
@@ -38,13 +37,8 @@ class ChromeContentClient : public content::ContentClient {
   static const base::FilePath::CharType kNotPresent[];
 #endif
 
-#if BUILDFLAG(ENABLE_NACL)
-  static const base::FilePath::CharType kNaClPluginFileName[];
-#endif
-
-  static const char kPDFExtensionPluginName[];
-  static const char kPDFInternalPluginName[];
-  static const base::FilePath::CharType kPDFPluginPath[];
+  static const base::FilePath::CharType kPDFExtensionPluginPath[];
+  static const base::FilePath::CharType kPDFInternalPluginPath[];
 
   ChromeContentClient();
   ~ChromeContentClient() override;
@@ -55,33 +49,14 @@ class ChromeContentClient : public content::ContentClient {
   // the split DLL.
 #if BUILDFLAG(ENABLE_NACL)
   static void SetNaClEntryFunctions(
-      content::PepperPluginInfo::GetInterfaceFunc get_interface,
-      content::PepperPluginInfo::PPP_InitializeModuleFunc initialize_module,
-      content::PepperPluginInfo::PPP_ShutdownModuleFunc shutdown_module);
-#endif
-
-#if BUILDFLAG(ENABLE_PLUGINS) && BUILDFLAG(ENABLE_PDF)
-  static void SetPDFEntryFunctions(
-      content::PepperPluginInfo::GetInterfaceFunc get_interface,
-      content::PepperPluginInfo::PPP_InitializeModuleFunc initialize_module,
-      content::PepperPluginInfo::PPP_ShutdownModuleFunc shutdown_module);
-#endif
-
-#if BUILDFLAG(ENABLE_PLUGINS)
-  // This returns the most recent plugin based on the plugin versions. In the
-  // event of a tie, a debug plugin will be considered more recent than a
-  // non-debug plugin.
-  // It does not make sense to call this on a vector that contains more than one
-  // plugin type. This function may return a nullptr if given an empty vector.
-  // The method is only visible for testing purposes.
-  static content::PepperPluginInfo* FindMostRecentPlugin(
-      const std::vector<std::unique_ptr<content::PepperPluginInfo>>& plugins);
+      content::ContentPluginInfo::GetInterfaceFunc get_interface,
+      content::ContentPluginInfo::PPP_InitializeModuleFunc initialize_module,
+      content::ContentPluginInfo::PPP_ShutdownModuleFunc shutdown_module);
 #endif
 
   void SetActiveURL(const GURL& url, std::string top_origin) override;
   void SetGpuInfo(const gpu::GPUInfo& gpu_info) override;
-  void AddPepperPlugins(
-      std::vector<content::PepperPluginInfo>* plugins) override;
+  void AddPlugins(std::vector<content::ContentPluginInfo>* plugins) override;
   void AddContentDecryptionModules(
       std::vector<content::CdmInfo>* cdms,
       std::vector<media::CdmHostFilePath>* cdm_host_file_paths) override;
@@ -93,17 +68,13 @@ class ChromeContentClient : public content::ContentClient {
       int resource_id,
       ui::ResourceScaleFactor scale_factor) override;
   base::RefCountedMemory* GetDataResourceBytes(int resource_id) override;
+  std::string GetDataResourceString(int resource_id) override;
   gfx::Image& GetNativeImageNamed(int resource_id) override;
-#if defined(OS_MAC)
-  base::FilePath GetChildProcessPath(
-      int child_flags,
-      const base::FilePath& helpers_path) override;
-#endif  // OS_MAC
   std::string GetProcessTypeNameInEnglish(int type) override;
   blink::OriginTrialPolicy* GetOriginTrialPolicy() override;
-#if defined(OS_ANDROID)
+#if BUILDFLAG(IS_ANDROID)
   media::MediaDrmBridgeClient* GetMediaDrmBridgeClient() override;
-#endif  // OS_ANDROID
+#endif  // BUILDFLAG(IS_ANDROID)
   void ExposeInterfacesToBrowser(
       scoped_refptr<base::SequencedTaskRunner> io_task_runner,
       mojo::BinderMap* binders) override;
