@@ -1,11 +1,11 @@
-// Copyright 2015 The Chromium Authors. All rights reserved.
+// Copyright 2015 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #ifndef CHROME_RENDERER_PLUGINS_NON_LOADABLE_PLUGIN_PLACEHOLDER_H_
 #define CHROME_RENDERER_PLUGINS_NON_LOADABLE_PLUGIN_PLACEHOLDER_H_
 
-#include "base/macros.h"
+#include "ppapi/buildflags/buildflags.h"
 
 namespace base {
 class FilePath;
@@ -25,6 +25,11 @@ class PluginPlaceholder;
 
 class NonLoadablePluginPlaceholder {
  public:
+  NonLoadablePluginPlaceholder() = delete;
+  NonLoadablePluginPlaceholder(const NonLoadablePluginPlaceholder&) = delete;
+  NonLoadablePluginPlaceholder& operator=(const NonLoadablePluginPlaceholder&) =
+      delete;
+
   // Creates a non-loadable plugin placeholder for platforms without plugins.
   static plugins::PluginPlaceholder* CreateNotSupportedPlugin(
       content::RenderFrame* render_frame,
@@ -34,12 +39,11 @@ class NonLoadablePluginPlaceholder {
       content::RenderFrame* render_frame,
       const blink::WebPluginParams& params);
 
+#if BUILDFLAG(ENABLE_PLUGINS)
   static plugins::PluginPlaceholder* CreateErrorPlugin(
       content::RenderFrame* render_frame,
       const base::FilePath& file_path);
-
- private:
-  DISALLOW_IMPLICIT_CONSTRUCTORS(NonLoadablePluginPlaceholder);
+#endif
 };
 
 #endif  // CHROME_RENDERER_PLUGINS_NON_LOADABLE_PLUGIN_PLACEHOLDER_H_
