@@ -1,21 +1,28 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include <tuple>
 
-#include "base/bind.h"
-#include "base/macros.h"
+#include "base/base_paths.h"
+#include "base/files/file.h"
+#include "base/files/file_path.h"
+#include "base/files/file_util.h"
+#include "base/functional/bind.h"
+#include "base/memory/raw_ptr.h"
+#include "base/path_service.h"
 #include "base/run_loop.h"
+#include "base/test/scoped_feature_list.h"
 #include "base/time/time.h"
+#include "base/types/cxx23_to_underlying.h"
 #include "chrome/common/chrome_isolated_world_ids.h"
 #include "chrome/test/base/chrome_render_view_test.h"
 #include "components/translate/content/common/translate.mojom.h"
 #include "components/translate/content/renderer/translate_agent.h"
 #include "components/translate/core/common/translate_constants.h"
+#include "components/translate/core/common/translate_util.h"
+#include "content/public/common/url_constants.h"
 #include "content/public/renderer/render_frame.h"
-#include "content/public/renderer/render_view.h"
-#include "extensions/common/constants.h"
 #include "mojo/public/cpp/bindings/pending_receiver.h"
 #include "mojo/public/cpp/bindings/pending_remote.h"
 #include "mojo/public/cpp/bindings/receiver_set.h"
@@ -30,6 +37,12 @@ using testing::Return;
 
 namespace {
 
+std::string UpdateGURLScheme(GURL url, const char scheme[]) {
+  GURL::Replacements replacements;
+  replacements.SetSchemeStr(scheme);
+  return url.ReplaceComponents(replacements).spec();
+}
+
 class FakeContentTranslateDriver
     : public translate::mojom::ContentTranslateDriver {
  public:
@@ -46,36 +59,56 @@ class FakeContentTranslateDriver
   void RegisterPage(
       mojo::PendingRemote<translate::mojom::TranslateAgent> translate_agent,
       const translate::LanguageDetectionDetails& details,
-      bool page_level_translation_critiera_met) override {
+      bool page_level_translation_criteria_met) override {
     called_new_page_ = true;
     details_ = details;
-    page_level_translation_critiera_met_ = page_level_translation_critiera_met;
+    page_level_translation_criteria_met_ = page_level_translation_criteria_met;
   }
   void GetLanguageDetectionModel(
       GetLanguageDetectionModelCallback callback) override {}
 
   void ResetNewPageValues() {
     called_new_page_ = false;
-    details_ = base::nullopt;
-    page_level_translation_critiera_met_ = false;
+    details_ = std::nullopt;
+    page_level_translation_criteria_met_ = false;
   }
 
   bool called_new_page_ = false;
-  bool page_level_translation_critiera_met_ = false;
-  base::Optional<translate::LanguageDetectionDetails> details_;
+  bool page_level_translation_criteria_met_ = false;
+  std::optional<translate::LanguageDetectionDetails> details_;
 
  private:
   mojo::ReceiverSet<translate::mojom::ContentTranslateDriver> receivers_;
 };
 
+// Load the model file at the provided file path.
+base::File LoadModelFile(const base::FilePath& model_file_path) {
+  if (!base::PathExists(model_file_path))
+    return base::File();
+
+  return base::File(model_file_path,
+                    base::File::FLAG_OPEN | base::File::FLAG_READ);
+}
+
+base::FilePath model_file_path() {
+  base::FilePath source_root_dir;
+  base::PathService::Get(base::DIR_SRC_TEST_DATA_ROOT, &source_root_dir);
+  return source_root_dir.AppendASCII("components")
+      .AppendASCII("test")
+      .AppendASCII("data")
+      .AppendASCII("translate")
+      .AppendASCII("valid_model.tflite");
+}
+
 }  // namespace
 
 class TestTranslateAgent : public translate::TranslateAgent {
  public:
   explicit TestTranslateAgent(content::RenderFrame* render_frame)
-      : translate::TranslateAgent(render_frame,
-                                  ISOLATED_WORLD_ID_TRANSLATE,
-                                  extensions::kExtensionScheme) {}
+      : translate::TranslateAgent(render_frame, ISOLATED_WORLD_ID_TRANSLATE) {}
+
+  TestTranslateAgent(const TestTranslateAgent&) = delete;
+  TestTranslateAgent& operator=(const TestTranslateAgent&) = delete;
 
   base::TimeDelta AdjustDelay(int delayInMs) override {
     // Just returns base::TimeDelta() which has initial value 0.
@@ -89,8 +122,8 @@ class TestTranslateAgent : public translate::TranslateAgent {
     // Reset result values firstly.
     page_translated_ = false;
     trans_result_cancelled_ = false;
-    trans_result_original_lang_ = base::nullopt;
-    trans_result_translated_lang_ = base::nullopt;
+    trans_result_source_lang_ = std::nullopt;
+    trans_result_translated_lang_ = std::nullopt;
     trans_result_error_type_ = translate::TranslateErrors::NONE;
 
     // Will get new result values via OnPageTranslated.
@@ -99,13 +132,13 @@ class TestTranslateAgent : public translate::TranslateAgent {
                                   base::Unretained(this)));
   }
 
-  bool GetPageTranslatedResult(std::string* original_lang,
+  bool GetPageTranslatedResult(std::string* source_lang,
                                std::string* target_lang,
-                               translate::TranslateErrors::Type* error) {
+                               translate::TranslateErrors* error) {
     if (!page_translated_)
       return false;
-    if (original_lang)
-      *original_lang = *trans_result_original_lang_;
+    if (source_lang)
+      *source_lang = *trans_result_source_lang_;
     if (target_lang)
       *target_lang = *trans_result_translated_lang_;
     if (error)
@@ -117,7 +150,7 @@ class TestTranslateAgent : public translate::TranslateAgent {
   MOCK_METHOD0(IsTranslateLibReady, bool());
   MOCK_METHOD0(HasTranslationFinished, bool());
   MOCK_METHOD0(HasTranslationFailed, bool());
-  MOCK_METHOD0(GetOriginalPageLanguage, std::string());
+  MOCK_METHOD0(GetPageSourceLanguage, std::string());
   MOCK_METHOD0(GetErrorCode, int64_t());
   MOCK_METHOD0(StartTranslation, bool());
   MOCK_METHOD1(ExecuteScript, void(const std::string&));
@@ -129,57 +162,58 @@ class TestTranslateAgent : public translate::TranslateAgent {
 
  private:
   void OnPageTranslated(bool cancelled,
-                        const std::string& original_lang,
+                        const std::string& source_lang,
                         const std::string& translated_lang,
-                        translate::TranslateErrors::Type error_type) {
+                        translate::TranslateErrors error_type) {
     page_translated_ = true;
     trans_result_cancelled_ = cancelled;
-    trans_result_original_lang_ = original_lang;
+    trans_result_source_lang_ = source_lang;
     trans_result_translated_lang_ = translated_lang;
     trans_result_error_type_ = error_type;
   }
 
   bool page_translated_;
   bool trans_result_cancelled_;
-  base::Optional<std::string> trans_result_original_lang_;
-  base::Optional<std::string> trans_result_translated_lang_;
-  translate::TranslateErrors::Type trans_result_error_type_;
-
-  DISALLOW_COPY_AND_ASSIGN(TestTranslateAgent);
+  std::optional<std::string> trans_result_source_lang_;
+  std::optional<std::string> trans_result_translated_lang_;
+  translate::TranslateErrors trans_result_error_type_;
 };
 
 class TranslateAgentBrowserTest : public ChromeRenderViewTest {
  public:
   TranslateAgentBrowserTest() : translate_agent_(nullptr) {}
 
+  TranslateAgentBrowserTest(const TranslateAgentBrowserTest&) = delete;
+  TranslateAgentBrowserTest& operator=(const TranslateAgentBrowserTest&) =
+      delete;
+
  protected:
   void SetUp() override {
     ChromeRenderViewTest::SetUp();
-    translate_agent_ = new TestTranslateAgent(view_->GetMainRenderFrame());
+    scoped_feature_list_.InitAndEnableFeature(
+        translate::kTFLiteLanguageDetectionEnabled);
+    translate_agent_ = new TestTranslateAgent(GetMainRenderFrame());
 
-    view_->GetMainRenderFrame()
-        ->GetBrowserInterfaceBroker()
-        ->SetBinderForTesting(
-            translate::mojom::ContentTranslateDriver::Name_,
-            base::BindRepeating(&FakeContentTranslateDriver::BindHandle,
-                                base::Unretained(&fake_translate_driver_)));
+    GetMainRenderFrame()->GetBrowserInterfaceBroker()->SetBinderForTesting(
+        translate::mojom::ContentTranslateDriver::Name_,
+        base::BindRepeating(&FakeContentTranslateDriver::BindHandle,
+                            base::Unretained(&fake_translate_driver_)));
+    base::File model_file = LoadModelFile(model_file_path());
+    translate_agent_->SeedLanguageDetectionModelForTesting(
+        std::move(model_file));
   }
 
   void TearDown() override {
-    view_->GetMainRenderFrame()
-        ->GetBrowserInterfaceBroker()
-        ->SetBinderForTesting(translate::mojom::ContentTranslateDriver::Name_,
-                              {});
+    GetMainRenderFrame()->GetBrowserInterfaceBroker()->SetBinderForTesting(
+        translate::mojom::ContentTranslateDriver::Name_, {});
 
     delete translate_agent_;
     ChromeRenderViewTest::TearDown();
   }
 
-  TestTranslateAgent* translate_agent_;
+  raw_ptr<TestTranslateAgent, DanglingUntriaged> translate_agent_;
   FakeContentTranslateDriver fake_translate_driver_;
-
- private:
-  DISALLOW_COPY_AND_ASSIGN(TranslateAgentBrowserTest);
+  base::test::ScopedFeatureList scoped_feature_list_;
 };
 
 // Tests that the browser gets notified of the translation failure if the
@@ -198,12 +232,13 @@ TEST_F(TranslateAgentBrowserTest, TranslateLibNeverReady) {
 
   EXPECT_CALL(*translate_agent_, GetErrorCode())
       .Times(AtLeast(5))
-      .WillRepeatedly(Return(translate::TranslateErrors::NONE));
+      .WillRepeatedly(
+          Return(base::to_underlying(translate::TranslateErrors::NONE)));
 
   translate_agent_->TranslatePage("en", "fr", std::string());
   base::RunLoop().RunUntilIdle();
 
-  translate::TranslateErrors::Type error;
+  translate::TranslateErrors error;
   ASSERT_TRUE(
       translate_agent_->GetPageTranslatedResult(nullptr, nullptr, &error));
   EXPECT_EQ(translate::TranslateErrors::TRANSLATION_TIMEOUT, error);
@@ -223,7 +258,7 @@ TEST_F(TranslateAgentBrowserTest, TranslateSuccess) {
       .WillOnce(Return(true));
 
   EXPECT_CALL(*translate_agent_, GetErrorCode())
-      .WillOnce(Return(translate::TranslateErrors::NONE));
+      .WillOnce(Return(base::to_underlying(translate::TranslateErrors::NONE)));
 
   EXPECT_CALL(*translate_agent_, StartTranslation()).WillOnce(Return(true));
 
@@ -238,17 +273,17 @@ TEST_F(TranslateAgentBrowserTest, TranslateSuccess) {
   // V8 call for performance monitoring should be ignored.
   EXPECT_CALL(*translate_agent_, ExecuteScriptAndGetDoubleResult(_)).Times(3);
 
-  std::string original_lang("en");
+  std::string source_lang("en");
   std::string target_lang("fr");
-  translate_agent_->TranslatePage(original_lang, target_lang, std::string());
+  translate_agent_->TranslatePage(source_lang, target_lang, std::string());
   base::RunLoop().RunUntilIdle();
 
-  std::string received_original_lang;
+  std::string received_source_lang;
   std::string received_target_lang;
-  translate::TranslateErrors::Type error;
+  translate::TranslateErrors error;
   ASSERT_TRUE(translate_agent_->GetPageTranslatedResult(
-      &received_original_lang, &received_target_lang, &error));
-  EXPECT_EQ(original_lang, received_original_lang);
+      &received_source_lang, &received_target_lang, &error));
+  EXPECT_EQ(source_lang, received_source_lang);
   EXPECT_EQ(target_lang, received_target_lang);
   EXPECT_EQ(translate::TranslateErrors::NONE, error);
 }
@@ -278,7 +313,8 @@ TEST_F(TranslateAgentBrowserTest, TranslateFailure) {
       .WillRepeatedly(Return(false));
 
   EXPECT_CALL(*translate_agent_, GetErrorCode())
-      .WillOnce(Return(translate::TranslateErrors::TRANSLATION_ERROR));
+      .WillOnce(Return(
+          base::to_underlying(translate::TranslateErrors::TRANSLATION_ERROR)));
 
   // V8 call for performance monitoring should be ignored.
   EXPECT_CALL(*translate_agent_, ExecuteScriptAndGetDoubleResult(_)).Times(2);
@@ -286,7 +322,7 @@ TEST_F(TranslateAgentBrowserTest, TranslateFailure) {
   translate_agent_->TranslatePage("en", "fr", std::string());
   base::RunLoop().RunUntilIdle();
 
-  translate::TranslateErrors::Type error;
+  translate::TranslateErrors error;
   ASSERT_TRUE(
       translate_agent_->GetPageTranslatedResult(nullptr, nullptr, &error));
   EXPECT_EQ(translate::TranslateErrors::TRANSLATION_ERROR, error);
@@ -303,7 +339,7 @@ TEST_F(TranslateAgentBrowserTest, UndefinedSourceLang) {
 
   EXPECT_CALL(*translate_agent_, IsTranslateLibReady()).WillOnce(Return(true));
 
-  EXPECT_CALL(*translate_agent_, GetOriginalPageLanguage())
+  EXPECT_CALL(*translate_agent_, GetPageSourceLanguage())
       .WillOnce(Return("de"));
 
   EXPECT_CALL(*translate_agent_, StartTranslation()).WillOnce(Return(true));
@@ -320,12 +356,12 @@ TEST_F(TranslateAgentBrowserTest, UndefinedSourceLang) {
                                   std::string());
   base::RunLoop().RunUntilIdle();
 
-  translate::TranslateErrors::Type error;
-  std::string original_lang;
+  translate::TranslateErrors error;
+  std::string source_lang;
   std::string target_lang;
-  ASSERT_TRUE(translate_agent_->GetPageTranslatedResult(&original_lang,
+  ASSERT_TRUE(translate_agent_->GetPageTranslatedResult(&source_lang,
                                                         &target_lang, &error));
-  EXPECT_EQ("de", original_lang);
+  EXPECT_EQ("de", source_lang);
   EXPECT_EQ("fr", target_lang);
   EXPECT_EQ(translate::TranslateErrors::NONE, error);
 }
@@ -351,20 +387,20 @@ TEST_F(TranslateAgentBrowserTest, MultipleSimilarTranslations) {
   // V8 call for performance monitoring should be ignored.
   EXPECT_CALL(*translate_agent_, ExecuteScriptAndGetDoubleResult(_)).Times(3);
 
-  std::string original_lang("en");
+  std::string source_lang("en");
   std::string target_lang("fr");
-  translate_agent_->TranslatePage(original_lang, target_lang, std::string());
+  translate_agent_->TranslatePage(source_lang, target_lang, std::string());
   // While this is running call again TranslatePage to make sure noting bad
   // happens.
-  translate_agent_->TranslatePage(original_lang, target_lang, std::string());
+  translate_agent_->TranslatePage(source_lang, target_lang, std::string());
   base::RunLoop().RunUntilIdle();
 
-  std::string received_original_lang;
+  std::string received_source_lang;
   std::string received_target_lang;
-  translate::TranslateErrors::Type error;
+  translate::TranslateErrors error;
   ASSERT_TRUE(translate_agent_->GetPageTranslatedResult(
-      &received_original_lang, &received_target_lang, &error));
-  EXPECT_EQ(original_lang, received_original_lang);
+      &received_source_lang, &received_target_lang, &error));
+  EXPECT_EQ(source_lang, received_source_lang);
   EXPECT_EQ(target_lang, received_target_lang);
   EXPECT_EQ(translate::TranslateErrors::NONE, error);
 }
@@ -386,21 +422,20 @@ TEST_F(TranslateAgentBrowserTest, MultipleDifferentTranslations) {
   // V8 call for performance monitoring should be ignored.
   EXPECT_CALL(*translate_agent_, ExecuteScriptAndGetDoubleResult(_)).Times(5);
 
-  std::string original_lang("en");
+  std::string source_lang("en");
   std::string target_lang("fr");
-  translate_agent_->TranslatePage(original_lang, target_lang, std::string());
+  translate_agent_->TranslatePage(source_lang, target_lang, std::string());
   // While this is running call again TranslatePage with a new target lang.
   std::string new_target_lang("de");
-  translate_agent_->TranslatePage(original_lang, new_target_lang,
-                                  std::string());
+  translate_agent_->TranslatePage(source_lang, new_target_lang, std::string());
   base::RunLoop().RunUntilIdle();
 
-  std::string received_original_lang;
+  std::string received_source_lang;
   std::string received_target_lang;
-  translate::TranslateErrors::Type error;
+  translate::TranslateErrors error;
   ASSERT_TRUE(translate_agent_->GetPageTranslatedResult(
-      &received_original_lang, &received_target_lang, &error));
-  EXPECT_EQ(original_lang, received_original_lang);
+      &received_source_lang, &received_target_lang, &error));
+  EXPECT_EQ(source_lang, received_source_lang);
   EXPECT_EQ(new_target_lang, received_target_lang);
   EXPECT_EQ(translate::TranslateErrors::NONE, error);
 }
@@ -412,7 +447,7 @@ TEST_F(TranslateAgentBrowserTest, TranslatablePage) {
 
   base::RunLoop().RunUntilIdle();
   ASSERT_TRUE(fake_translate_driver_.called_new_page_);
-  EXPECT_TRUE(fake_translate_driver_.page_level_translation_critiera_met_)
+  EXPECT_TRUE(fake_translate_driver_.page_level_translation_criteria_met_)
       << "Page should be translatable.";
   fake_translate_driver_.ResetNewPageValues();
 
@@ -423,7 +458,7 @@ TEST_F(TranslateAgentBrowserTest, TranslatablePage) {
 
   base::RunLoop().RunUntilIdle();
   ASSERT_TRUE(fake_translate_driver_.called_new_page_);
-  EXPECT_FALSE(fake_translate_driver_.page_level_translation_critiera_met_)
+  EXPECT_FALSE(fake_translate_driver_.page_level_translation_criteria_met_)
       << "Page should not be translatable.";
   fake_translate_driver_.ResetNewPageValues();
 
@@ -434,7 +469,7 @@ TEST_F(TranslateAgentBrowserTest, TranslatablePage) {
 
   base::RunLoop().RunUntilIdle();
   ASSERT_TRUE(fake_translate_driver_.called_new_page_);
-  EXPECT_FALSE(fake_translate_driver_.page_level_translation_critiera_met_)
+  EXPECT_FALSE(fake_translate_driver_.page_level_translation_criteria_met_)
       << "Page should not be translatable.";
 }
 
@@ -443,7 +478,8 @@ TEST_F(TranslateAgentBrowserTest, TranslatablePage) {
 TEST_F(TranslateAgentBrowserTest, LanguageMetaTag) {
   LoadHTML(
       "<html><head><meta http-equiv=\"content-language\" content=\"es\">"
-      "</head><body>A random page with random content.</body></html>");
+      "</head><body>A</body></html>");
+  //   "</head><body>Esta página está en español.</body></html>");
 
   base::RunLoop().RunUntilIdle();
   ASSERT_TRUE(fake_translate_driver_.called_new_page_);
@@ -454,7 +490,7 @@ TEST_F(TranslateAgentBrowserTest, LanguageMetaTag) {
   LoadHTML(
       "<html><head><meta http-equiv=\"content-language\" "
       "content=\" fr , es,en \">"
-      "</head><body>A random page with random content.</body></html>");
+      "</head><body>Cette page est en français.</body></html>");
 
   base::RunLoop().RunUntilIdle();
   ASSERT_TRUE(fake_translate_driver_.called_new_page_);
@@ -466,7 +502,7 @@ TEST_F(TranslateAgentBrowserTest, LanguageMetaTag) {
 TEST_F(TranslateAgentBrowserTest, LanguageMetaTagCase) {
   LoadHTML(
       "<html><head><meta http-equiv=\"Content-Language\" content=\"es\">"
-      "</head><body>A random page with random content.</body></html>");
+      "</head><body>E</body></html>");
   base::RunLoop().RunUntilIdle();
   ASSERT_TRUE(fake_translate_driver_.called_new_page_);
   EXPECT_EQ("es", fake_translate_driver_.details_->adopted_language);
@@ -476,7 +512,7 @@ TEST_F(TranslateAgentBrowserTest, LanguageMetaTagCase) {
   LoadHTML(
       "<html><head><meta http-equiv=\"Content-Language\" "
       "content=\" fr , es,en \">"
-      "</head><body>A random page with random content.</body></html>");
+      "</head><body>Cette page est en français.</body></html>");
   base::RunLoop().RunUntilIdle();
   ASSERT_TRUE(fake_translate_driver_.called_new_page_);
   EXPECT_EQ("fr", fake_translate_driver_.details_->adopted_language);
@@ -496,7 +532,7 @@ TEST_F(TranslateAgentBrowserTest, LanguageCommonMistakesAreCorrected) {
 
   LoadHTML(
       "<html><head><meta http-equiv='Content-Language' content='ZH_tw'>"
-      "</head><body>A random page with random content.</body></html>");
+      "</head><body>A</body></html>");
   base::RunLoop().RunUntilIdle();
   ASSERT_TRUE(fake_translate_driver_.called_new_page_);
   EXPECT_EQ("zh-TW", fake_translate_driver_.details_->adopted_language);
@@ -506,7 +542,7 @@ TEST_F(TranslateAgentBrowserTest, LanguageCommonMistakesAreCorrected) {
 TEST_F(TranslateAgentBrowserTest, BackToTranslatablePage) {
   LoadHTML(
       "<html><head><meta http-equiv=\"content-language\" content=\"es\">"
-      "</head><body>This page is in Spanish.</body></html>");
+      "</head><body>E</body></html>");
   base::RunLoop().RunUntilIdle();
   ASSERT_TRUE(fake_translate_driver_.called_new_page_);
   EXPECT_EQ("es", fake_translate_driver_.details_->adopted_language);
@@ -516,7 +552,7 @@ TEST_F(TranslateAgentBrowserTest, BackToTranslatablePage) {
 
   LoadHTML(
       "<html><head><meta http-equiv=\"content-language\" content=\"fr\">"
-      "</head><body>This page is in French.</body></html>");
+      "</head><body>E</body></html>");
   base::RunLoop().RunUntilIdle();
   ASSERT_TRUE(fake_translate_driver_.called_new_page_);
   EXPECT_EQ("fr", fake_translate_driver_.details_->adopted_language);
@@ -524,10 +560,37 @@ TEST_F(TranslateAgentBrowserTest, BackToTranslatablePage) {
 
   GoBack(GURL("data:text/html;charset=utf-8,<html><head>"
               "<meta http-equiv=\"content-language\" content=\"es\">"
-              "</head><body>This page is in Spanish.</body></html>"),
+              "</head><body>E</body></html>"),
          back_state);
 
   base::RunLoop().RunUntilIdle();
   ASSERT_TRUE(fake_translate_driver_.called_new_page_);
   EXPECT_EQ("es", fake_translate_driver_.details_->adopted_language);
 }
+
+TEST_F(TranslateAgentBrowserTest, UnsupportedTranslateSchemes) {
+  GURL url("https://foo.com");
+  LoadHTMLWithUrlOverride(
+      "<html><body>A random page with random content.</body></html>",
+      UpdateGURLScheme(url, content::kChromeUIScheme).c_str());
+
+  base::RunLoop().RunUntilIdle();
+  ASSERT_FALSE(fake_translate_driver_.called_new_page_);
+  EXPECT_FALSE(fake_translate_driver_.page_level_translation_criteria_met_);
+
+  LoadHTMLWithUrlOverride(
+      "<html><body>A random page with random content.</body></html>",
+      url::kAboutBlankURL);
+
+  base::RunLoop().RunUntilIdle();
+  ASSERT_FALSE(fake_translate_driver_.called_new_page_);
+  EXPECT_FALSE(fake_translate_driver_.page_level_translation_criteria_met_);
+
+  LoadHTMLWithUrlOverride(
+      "<html><body>A random page with random content.</body></html>",
+      UpdateGURLScheme(url, content::kChromeDevToolsScheme).c_str());
+
+  base::RunLoop().RunUntilIdle();
+  ASSERT_FALSE(fake_translate_driver_.called_new_page_);
+  EXPECT_FALSE(fake_translate_driver_.page_level_translation_criteria_met_);
+}
