@@ -1,4 +1,4 @@
-// Copyright 2020 The Chromium Authors. All rights reserved.
+// Copyright 2020 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -11,7 +11,7 @@
 namespace account_manager {
 
 // Generates a Gaia ID from |raw_email| and creates an account.
-account_manager::Account CreateTestGaiaAccount(const std::string& raw_email);
+Account CreateTestGaiaAccount(const std::string& raw_email);
 
 }  // namespace account_manager
 
