@@ -1,4 +1,4 @@
-// Copyright 2017 The Chromium Authors. All rights reserved.
+// Copyright 2017 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,22 +7,26 @@
 
 #include <map>
 #include <memory>
+#include <optional>
 #include <string>
 #include <utility>
 #include <vector>
 
-#include "base/optional.h"
-#include "base/strings/string16.h"
+#include "base/gtest_prod_util.h"
+#include "base/memory/raw_ptr.h"
 #include "build/build_config.h"
 #include "components/autofill/core/browser/autofill_client.h"
-#include "components/autofill/core/browser/autofill_metrics.h"
 #include "components/autofill/core/browser/data_model/credit_card.h"
+#include "components/autofill/core/browser/form_data_importer.h"
 #include "components/autofill/core/browser/form_structure.h"
-#include "components/autofill/core/browser/payments/credit_card_save_strike_database.h"
+#include "components/autofill/core/browser/metrics/autofill_metrics.h"
+#include "components/autofill/core/browser/metrics/payments/credit_card_save_metrics.h"
 #include "components/autofill/core/browser/payments/legal_message_line.h"
-#include "components/autofill/core/browser/payments/local_card_migration_strike_database.h"
-#include "components/autofill/core/browser/payments/payments_client.h"
+#include "components/autofill/core/browser/payments/payments_network_interface.h"
 #include "components/autofill/core/browser/personal_data_manager.h"
+#include "components/autofill/core/browser/strike_databases/payments/credit_card_save_strike_database.h"
+#include "components/autofill/core/browser/strike_databases/payments/cvc_storage_strike_database.h"
+#include "components/autofill/core/browser/strike_databases/payments/local_card_migration_strike_database.h"
 #include "url/origin.h"
 
 class SaveCardOfferObserver;
@@ -82,7 +86,7 @@ class CreditCardSaveManager {
   // particular actions occur.
   class ObserverForTest {
    public:
-    virtual ~ObserverForTest() {}
+    virtual ~ObserverForTest() = default;
     virtual void OnOfferLocalSave() {}
     virtual void OnOfferUploadSave() {}
     virtual void OnDecideToRequestUploadSave() {}
@@ -94,34 +98,56 @@ class CreditCardSaveManager {
   };
 
   // The parameters should outlive the CreditCardSaveManager.
-  CreditCardSaveManager(AutofillClient* client,
-                        payments::PaymentsClient* payments_client,
-                        const std::string& app_locale,
-                        PersonalDataManager* personal_data_manager);
+  CreditCardSaveManager(
+      AutofillClient* client,
+      const std::string& app_locale,
+      PersonalDataManager* personal_data_manager);
+
+  CreditCardSaveManager(const CreditCardSaveManager&) = delete;
+  CreditCardSaveManager& operator=(const CreditCardSaveManager&) = delete;
+
   virtual ~CreditCardSaveManager();
 
-  // Begins the process to offer local credit card save to the user.
-  // If |has_non_focusable_field| is true, the save is triggered by a form that
-  // has non_focusable fields.
-  // If |from_dynamic_change_form| is true, the save is triggered by a dynamic
-  // change form.
-  void AttemptToOfferCardLocalSave(bool from_dynamic_change_form,
-                                   bool has_non_focusable_field,
-                                   const CreditCard& card);
+  // Begins the process to offer local credit card save to the user. Returns
+  // true if the prompt is shown.
+  virtual bool AttemptToOfferCardLocalSave(const CreditCard& card);
+
+  // Begins the process to offer local CVC save to the user. Returns true if the
+  // prompt is shown. `card` is the credit card extracted from the form.
+  virtual bool AttemptToOfferCvcLocalSave(const CreditCard& card);
+
+  // Returns true if CVC local or upload save should be offered to the user.
+  // `card` is the credit card extracted from the form. It refers to the
+  // CVC-only save. If card is unknown we will offer to save the the card
+  // including it's CVC. `credit_card_import_type` is the credit card type
+  // extracted from the form.
+  // TODO(crbug.com/1450749): Update param after resolving duplicate local and
+  // server card issue.
+  virtual bool ShouldOfferCvcSave(
+      const CreditCard& card,
+      FormDataImporter::CreditCardImportType credit_card_import_type,
+      bool is_credit_card_upstream_enabled);
+
+  // Check and attempt to offer if CVC or card local or upload save should be
+  // offered to the user. `card` is the credit card extracted from the form.
+  // `credit_card_import_type` is the credit card type extracted from the form.
+  virtual bool ProceedWithSavingIfApplicable(
+      const FormStructure& submitted_form,
+      const CreditCard& card,
+      FormDataImporter::CreditCardImportType credit_card_import_type,
+      bool is_credit_card_upstream_enabled);
 
   // Begins the process to offer upload credit card save to the user if the
   // imported card passes all requirements and Google Payments approves.
-  // If |has_non_focusable_field| is true, the save is triggered by a form that
-  // has non-focusable fields. if |uploading_local_card| is true, the card being
-  // offered for upload is already a local card on the device. If
-  // |from_dynamic_change_form| is true, the save is triggered by a dynamic
-  // change form.
+  // If |uploading_local_card| is true, the card being
+  // offered for upload is already a local card on the device.
   void AttemptToOfferCardUploadSave(const FormStructure& submitted_form,
-                                    bool from_dynamic_change_form,
-                                    bool has_non_focusable_field,
                                     const CreditCard& card,
                                     const bool uploading_local_card);
 
+  // Begins the process to offer server CVC save to the user.
+  virtual void AttemptToOfferCvcUploadSave(const CreditCard& card);
+
   // Returns true if all the conditions for enabling the upload of credit card
   // are satisfied.
   virtual bool IsCreditCardUploadEnabled();
@@ -129,13 +155,28 @@ class CreditCardSaveManager {
   // For testing.
   void SetAppLocale(std::string app_locale) { app_locale_ = app_locale; }
 
+  // Set Autofill address profiles that are only preliminarily imported.
+  // A preliminary import may happen when the address is found in the same
+  // form as a credit card that is currently processed by the manager.
+  void SetPreliminarilyImportedAutofillProfile(
+      const std::vector<AutofillProfile>& profiles) {
+    preliminarily_imported_address_profiles_ = profiles;
+  }
+
+  // Clear the preliminarily imported Autofill address profiles.
+  void ClearPreliminarilyImportedAutofillProfile() {
+    preliminarily_imported_address_profiles_.clear();
+  }
+
  protected:
   // Returns the result of an upload request. If |result| ==
-  // |AutofillClient::SUCCESS|, clears strikes for the saved card. Additionally,
-  // |server_id| may, optionally, contain the opaque identifier for the card on
-  // the server. Exposed for testing.
-  virtual void OnDidUploadCard(AutofillClient::PaymentsRpcResult result,
-                               const std::string& server_id);
+  // |AutofillClient::PaymentsRpcResult::kSuccess|, clears strikes for the saved
+  // card. Additionally, |server_id| may, optionally, contain the opaque
+  // identifier for the card on the server. Exposed for testing.
+  virtual void OnDidUploadCard(
+      AutofillClient::PaymentsRpcResult result,
+      const payments::PaymentsNetworkInterface::UploadCardResponseDetails&
+          upload_card_response_details);
 
  private:
   friend class CreditCardSaveManagerTest;
@@ -156,20 +197,27 @@ class CreditCardSaveManager {
   // Returns the CreditCardSaveStrikeDatabase for |client_|.
   CreditCardSaveStrikeDatabase* GetCreditCardSaveStrikeDatabase();
 
-#if !defined(OS_ANDROID) && !defined(OS_IOS)
+  // Returns the CvcStorageStrikeDatabase for `client_`.
+  CvcStorageStrikeDatabase* GetCvcStorageStrikeDatabase();
+
+  // Query the CvcStorageStrikeDatabase to check if the offer-to-save prompt for
+  // this CVC should be blocked.
+  bool DetermineAndLogCvcSaveStrikeDatabaseBlockDecision();
+
+#if !BUILDFLAG(IS_ANDROID) && !BUILDFLAG(IS_IOS)
   // Returns the GetLocalCardMigrationStrikeDatabase for |client_|.
   LocalCardMigrationStrikeDatabase* GetLocalCardMigrationStrikeDatabase();
-#endif  // !defined(OS_ANDROID) && !defined(OS_IOS)
+#endif  // !BUILDFLAG(IS_ANDROID) && !BUILDFLAG(IS_IOS)
 
   // Returns the legal message retrieved from Payments. On failure or not
   // meeting Payments's conditions for upload, |legal_message| will contain
   // nullptr. |supported_card_bin_ranges| is a list of BIN prefix ranges which
-  // are supoorted, with the first and second number in the pair being the start
+  // are supported, with the first and second number in the pair being the start
   // and end of the range.
   void OnDidGetUploadDetails(
       AutofillClient::PaymentsRpcResult result,
-      const base::string16& context_token,
-      std::unique_ptr<base::Value> legal_message,
+      const std::u16string& context_token,
+      std::unique_ptr<base::Value::Dict> legal_message,
       std::vector<std::pair<int, int>> supported_card_bin_ranges);
 
   // Logs the number of strikes that a card had when save succeeded.
@@ -185,7 +233,7 @@ class CreditCardSaveManager {
   // contain countries.
   void SetProfilesForCreditCardUpload(
       const CreditCard& card,
-      payments::PaymentsClient::UploadRequestDetails* upload_request);
+      payments::PaymentsNetworkInterface::UploadRequestDetails* upload_request);
 
   // Analyzes the decisions made while importing address profile and credit card
   // data in preparation for upload credit card save, in order to determine what
@@ -196,6 +244,10 @@ class CreditCardSaveManager {
   // decision.
   void OfferCardLocalSave();
 
+  // Offers local CVC save once `AttemptToOfferCvcLocalSave()` decides it should
+  // be allowed.
+  void OfferCvcLocalSave();
+
   // Offers credit card upload if Payments has allowed offering to save and the
   // Autofill StrikeSystem has made its decision.
   void OfferCardUploadSave();
@@ -206,6 +258,11 @@ class CreditCardSaveManager {
   void OnUserDidDecideOnLocalSave(
       AutofillClient::SaveCardOfferUserDecision user_decision);
 
+  // Called once the user makes a decision with respect to the local CVC
+  // offer-to-save prompt.
+  void OnUserDidDecideOnCvcLocalSave(
+      AutofillClient::SaveCardOfferUserDecision user_decision);
+
   // Called once the user makes a decision with respect to the credit card
   // upload offer-to-save prompt.
   // If accepted:
@@ -222,18 +279,25 @@ class CreditCardSaveManager {
       const AutofillClient::UserProvidedCardDetails&
           user_provided_card_details);
 
-#if defined(OS_ANDROID) || defined(OS_IOS)
+  // Called once the user makes a decision with respect to the server CVC
+  // offer-to-save prompt.
+  void OnUserDidDecideOnCvcUploadSave(
+      AutofillClient::SaveCardOfferUserDecision user_decision,
+      const AutofillClient::UserProvidedCardDetails&
+          user_provided_card_details);
+
+#if BUILDFLAG(IS_ANDROID) || BUILDFLAG(IS_IOS)
   // Upload the card details with the user provided cardholder_name.
   // Only relevant for mobile as fix flow is two steps on mobile compared to
   // one step on desktop.
-  void OnUserDidAcceptAccountNameFixFlow(const base::string16& cardholder_name);
+  void OnUserDidAcceptAccountNameFixFlow(const std::u16string& cardholder_name);
 
   // Upload the card details with the user provided expiration date month and
   // year. Only relevant for mobile as fix flow is two steps on mobile compared
   // to one step on desktop.
-  void OnUserDidAcceptExpirationDateFixFlow(const base::string16& month,
-                                            const base::string16& year);
-#endif  // defined(OS_ANDROID) || defined(OS_IOS)
+  void OnUserDidAcceptExpirationDateFixFlow(const std::u16string& month,
+                                            const std::u16string& year);
+#endif  // BUILDFLAG(IS_ANDROID) || BUILDFLAG(IS_IOS)
 
   // Helper function that calls SendUploadCardRequest by setting
   // UserProvidedCardDetails.
@@ -245,14 +309,15 @@ class CreditCardSaveManager {
   // if the user has accepted the prompt.
   void OnDidGetUploadRiskData(const std::string& risk_data);
 
-  // Finalizes the upload request and calls PaymentsClient::UploadCard().
+  // Finalizes the upload request and calls
+  // PaymentsNetworkInterface::UploadCard().
   void SendUploadCardRequest();
 
   // Called when the user ignored or declined the credit card save prompt. Logs
   // a strike for the given card in order to help deter future offers to save,
   // provided that save was actually offered to the user.
   void OnUserDidIgnoreOrDeclineSave(
-      const base::string16& card_last_four_digits);
+      const std::u16string& card_last_four_digits);
 
   // Used for browsertests. Gives the |observer_for_testing_| a notification
   // a strike change has been made.
@@ -262,14 +327,16 @@ class CreditCardSaveManager {
   // |found_cvc_field_|, |found_value_in_cvc_field_| and
   // |found_cvc_value_in_non_cvc_field_|. Only called when a valid CVC was NOT
   // found.
-  AutofillMetrics::CardUploadDecisionMetric GetCVCCardUploadDecisionMetric()
-      const;
+  autofill_metrics::CardUploadDecision GetCVCCardUploadDecisionMetric() const;
 
   // Logs the card upload decisions in UKM and UMA.
   // |upload_decision_metrics| is a bitmask of
   // |AutofillMetrics::CardUploadDecisionMetric|.
   void LogCardUploadDecisions(int upload_decision_metrics);
 
+  // Logs the card upload decisions bitmask to chrome://autofill-internals.
+  void LogCardUploadDecisionsToAutofillInternals(int upload_decision_metrics);
+
   // Logs the reason why expiration date was explicitly requested.
   void LogSaveCardRequestExpirationDateReasonMetric();
 
@@ -278,25 +345,21 @@ class CreditCardSaveManager {
     observer_for_testing_ = observer;
   }
 
-  AutofillClient* const client_;
-
-  // Handles Payments service requests.
-  // Owned by AutofillManager.
-  payments::PaymentsClient* payments_client_;
+  const raw_ptr<AutofillClient> client_;
 
   std::string app_locale_;
 
   // The personal data manager, used to save and load personal data to/from the
-  // web database.  This is overridden by the AutofillManagerTest.
+  // web database.  This is overridden by the BrowserAutofillManagerTest.
   // Weak reference.
-  // May be NULL.  NULL indicates OTR.
-  PersonalDataManager* personal_data_manager_;
+  raw_ptr<PersonalDataManager> personal_data_manager_;
 
-  // The credit card to be saved if local credit card save is accepted.
-  CreditCard local_card_save_candidate_;
+  // The credit card to be saved if local credit card or local or server CVC
+  // save is accepted.
+  CreditCard card_save_candidate_;
 
   // Collected information about a pending upload request.
-  payments::PaymentsClient::UploadRequestDetails upload_request_;
+  payments::PaymentsNetworkInterface::UploadRequestDetails upload_request_;
 
   // A bitmask of |AutofillMetrics::CardUploadDecisionMetric| representing the
   // decisions made when determining if credit card upload save should be
@@ -304,8 +367,8 @@ class CreditCardSaveManager {
   int upload_decision_metrics_ = 0;
 
   // |true| if the offer-to-save bubble/infobar should pop-up, |false| if not.
-  // Will be base::nullopt until data has been retrieved from the StrikeSystem.
-  base::Optional<bool> show_save_prompt_;
+  // Will be std::nullopt until data has been retrieved from the StrikeSystem.
+  std::optional<bool> show_save_prompt_;
 
   // |true| if the card being offered for upload is already a local card on the
   // device; |false| otherwise.
@@ -333,14 +396,6 @@ class CreditCardSaveManager {
   // determined to be a CVC field via heuristics has a valid CVC |value|.
   bool found_cvc_value_in_non_cvc_field_ = false;
 
-  // |has_non_focusable_field_| is |true| if there exists a field that
-  // |is_focusable| is false.
-  bool has_non_focusable_field_ = false;
-
-  // |from_dynamic_change_form_| is |true| values imported from dynamic change
-  // form.
-  bool from_dynamic_change_form_ = false;
-
   // The origin of the top level frame from which a form is uploaded.
   url::Origin pending_upload_request_origin_;
 
@@ -350,13 +405,20 @@ class CreditCardSaveManager {
   std::unique_ptr<CreditCardSaveStrikeDatabase>
       credit_card_save_strike_database_;
 
-#if !defined(OS_ANDROID) && !defined(OS_IOS)
+  std::unique_ptr<CvcStorageStrikeDatabase> cvc_storage_strike_database_;
+
+  // Profiles that are only preliminarily imported. Those profiles are used
+  // during a card import to determine the name and country for storing a new
+  // card.
+  std::vector<AutofillProfile> preliminarily_imported_address_profiles_;
+
+#if !BUILDFLAG(IS_ANDROID) && !BUILDFLAG(IS_IOS)
   std::unique_ptr<LocalCardMigrationStrikeDatabase>
       local_card_migration_strike_database_;
-#endif  // !defined(OS_ANDROID) && !defined(OS_IOS)
+#endif  // !BUILDFLAG(IS_ANDROID) && !BUILDFLAG(IS_IOS)
 
   // May be null.
-  ObserverForTest* observer_for_testing_ = nullptr;
+  raw_ptr<ObserverForTest> observer_for_testing_ = nullptr;
 
   base::WeakPtrFactory<CreditCardSaveManager> weak_ptr_factory_{this};
 
@@ -372,8 +434,6 @@ class CreditCardSaveManager {
   FRIEND_TEST_ALL_PREFIXES(
       CreditCardSaveManagerTest,
       UploadCreditCard_WalletSyncTransportNotEnabled_ShouldRequestExpirationDate);
-
-  DISALLOW_COPY_AND_ASSIGN(CreditCardSaveManager);
 };
 
 }  // namespace autofill
